//
// Module Name:		CollectionViewer.cc
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Author:		John Porter
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	This is a simple Motif-based application for viewing the
//	contents of a Collection.
//	There are two windows: one for viewing a Collection and its
//	associated info, and one for viewing a Document and its info.
//	The main window of the application is a Collection window.
//
//	There are five main activities, each with a keyboard shortcut.
//	Key	Window		Action
//	------- ----------- 	-----------
//	Ctrl-O	Collection	Open a Collection (dialog)
//	Ctrl-X	Collection	Exit the application
//	Ctrl-V	Collection	View the selected Document
//	Ctrl-H	Document	Highlight the selected Annotation
//	Ctrl-C	Document	Close the Document View window
//
//	Also note that the "default action" (double-clicking or
//	pressing <Enter>) in the Document List behaves the same
//	as Ctrl-V, and the "default action" in the Annotation List
//	behaves the same as Ctrl-H.
//
// Limitations:
//	Currently, no means is provided for viewing the attributes of
//	an Annotation, generally.  The first attribute of an Annotation
//	is displayed in the Annotation List, but any other attributes
//	are not visible.
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//          1998-06-24  porter   Initial version
//

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <strstream.h>

#include <Xm/XmAll.h>

#include "cmcPersistentCollection.h"
#include "cmcAttribute.h"
#include "cmcAttributeValue.h"
#include "cmcStringValue.h"
#include "cmcSequenceValue.h"
#include "cmcRecordFile.h"


const int stringbufsize = 1024;
char* attribute_value( Attribute& attr ) {
  static char buf[stringbufsize];
  memset( buf, 0, stringbufsize );
  ostrstream ostr( buf, stringbufsize );
  ostr << attr.GetValue();

  char* p = strchr( buf, ' ' );
  if ( p ) {
    char type;
    int count;
    if ( sscanf( buf, "%c%d", &type, &count ) ) {
      p++;
      p[ count ] = '\0';
      return p;
    }
  }
  return buf;
}


class PrintfLabel;
class DirectoryW;
class DirectorySelectionDialog;
class CollectionSelectionDialog;
class CollectionWindow;
class DocumentsListFrame;
class CollectionPulldownMenu;
class DocumentPulldownMenu;
class DocumentWindow;

//
// declare globals:
//
CollectionWindow* collection_window;
DocumentWindow* document_window;
DirectoryW* directoryw;
DirectorySelectionDialog* dir_sel_dlg;
CollectionSelectionDialog* coll_sel_dlg;





// a nice wrapper for xmLabel; allows to printf into it.
class PrintfLabel {
  protected:
    Widget w;

  public:
    PrintfLabel() : w(0) { }; // only for use by derived classes.

    PrintfLabel( Widget parent, char* msg = "" ) {
      XmString str = XmStringCreateLocalized( msg );
      w = XtVaCreateManagedWidget( "message_label",
        xmLabelWidgetClass, parent,
        XmNalignment, XmALIGNMENT_BEGINNING,
        XmNlabelString, str,
        NULL );
      XmStringFree( str );
    };

    Widget root_widget() {
      return w;
    };

    void printf( char* fmt, ... ) {
      char msgbuf[0x4000];
      va_list args;
      va_start(args,fmt);
      vsprintf( msgbuf, fmt, args );
      va_end(args);

      XmString str = XmStringCreateLocalized( msgbuf );
      XtVaSetValues( w, XmNlabelString, str, NULL );
      XmStringFree( str );
    };

};


//
// this is usable directly, if your button doesn't need to do anything.
// if you want to have a callback, derive a class which implements the
// callback() method.
//
class PrintfButton : public PrintfLabel {
  protected:
    virtual void callback() { }; // do nothing, by default.

    static void cb( Widget w, XtPointer cd, XtPointer cbs ) {
      PrintfButton* self = (PrintfButton*) cd;
      self->callback(); // (XmPushButtonCallbackStruct*) cbs );
    };

  public:
    PrintfButton( Widget parent, char* msg = 0 ) {
      XmString str = XmStringCreateLocalized( msg ? msg : "" );
      w = XtVaCreateManagedWidget( "message_label",
        xmPushButtonWidgetClass, parent,
        XmNlabelString, str,
        NULL );
      XmStringFree( str );
      XtAddCallback( w, XmNactivateCallback, cb, this );
    };

    void set_accelerator( char* text, char* definition, char* mnem = 0 ) {
      XmString str = XmStringCreateLocalized( text );
      XtVaSetValues( w,
        XmNaccelerator, definition,
        XmNacceleratorText, str,
        NULL );
      XmStringFree( str );
      if ( mnem ) {
        XtVaSetValues( w,
          XtVaTypedArg, XmNmnemonic, XmRString, mnem, strlen(mnem)+1,
          NULL );
      }
    };
};



class TextArea {
  protected:
    Widget sw;
    Widget w;

  public:
    TextArea( Widget parent ) {
      Arg args[16]; int n = 0;
      XtSetArg( args[n], XmNrows, 16 ); n++;
      XtSetArg( args[n], XmNcolumns, 80 ); n++;
      XtSetArg( args[n], XmNeditMode, XmMULTI_LINE_EDIT ); n++;
      XtSetArg( args[n], XmNscrollHorizontal, True ); n++;
      XtSetArg( args[n], XmNscrollVertical, True ); n++;
      XtSetArg( args[n], XmNwordWrap, False ); n++;
      XtSetArg( args[n], XmNeditable, False ); n++;
      w = XmCreateScrolledText( parent, "text_area", args, n );
      for ( sw = w; !XmIsScrolledWindow(sw); sw = XtParent(sw) ) ;
      XtManageChild( w );
    };

    Widget root_widget() {
      return sw;
    };

    void set_string( char* str ) {
      XmTextSetString( w, str );
    };

    void printf( char* fmt, ... ) {
      char msgbuf[0x4000];
      va_list args;
      va_start(args,fmt);
      vsprintf( msgbuf, fmt, args );
      va_end(args);

      XmTextPosition pos = XmTextGetInsertionPosition( w );
      XmTextInsert( w, pos, msgbuf );
      pos += strlen(msgbuf);
      XmTextSetInsertionPosition( w, pos );
      XmTextShowPosition( w, pos );
    };

    void add_highlight( int start, int end ) {
      XmTextSetHighlight( w, start, end, XmHIGHLIGHT_SELECTED );
    };

    void no_highlights() {
      XmTextSetHighlight( w, 0, XmTextGetLastPosition(w), XmHIGHLIGHT_NORMAL );
    };
};



#include <dirent.h>


class DirectoryW {
  protected:
    char dir_name[1024];
    char* coll_names[1024];
    int n_coll_names;

  friend class CollectionSelectionDialog;

    void load_directory( DIR* dp ) {
      struct dirent* d;
      n_coll_names = 0;
      while ( d=readdir(dp), d ) {
        char* ext = strstr( d->d_name, ".idx" );
        if ( ext && !strcmp( ext, ".idx" ) ) {
          *ext = '\0';
          coll_names[n_coll_names] = new char[ strlen(d->d_name) + 1 ];
          strcpy( coll_names[n_coll_names], d->d_name );
          n_coll_names++;
        }
      }
      coll_names[n_coll_names] = NULL;
      closedir(dp);
    };

  public:

    char* name() { return dir_name; };

    // returns boolean: success
    int set_directory( char* dir );

    DirectoryW() : n_coll_names(0) {
    };
};


class DirectorySelectionDialog {
  protected:
    Widget dlg;

    static void cancel_cb( Widget w, XtPointer cd, XtPointer cbs ) {
      DirectorySelectionDialog* self = (DirectorySelectionDialog*) cd;
      XtUnmanageChild( self->dlg );
    };

    static void ok_cb( Widget w, XtPointer cd, XtPointer cbs ) {
      DirectorySelectionDialog* self = (DirectorySelectionDialog*) cd;
       XmFileSelectionBoxCallbackStruct* sbcbs = 
      (XmFileSelectionBoxCallbackStruct*)  cbs;
      char* value = 0;
      if ( XmStringGetLtoR( sbcbs->value, XmFONTLIST_DEFAULT_TAG, &value ) ) {
        directoryw->set_directory( value ); // might fail...
        XtFree( value );
      }
      XtUnmanageChild( self->dlg );
    };

  public:
    DirectorySelectionDialog( Widget parent ) {
      dlg = XmCreateFileSelectionDialog( parent, "directory_selection", 0, 0 );
      XtVaSetValues( XtParent(dlg), XmNtitle, "Select Directory", NULL );
      XtVaSetValues( dlg,
        XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
        XmNfileTypeMask, XmFILE_DIRECTORY,
        NULL );
      XtAddCallback( dlg, XmNokCallback, ok_cb, this );
      XtAddCallback( dlg, XmNcancelCallback, cancel_cb, this );
    };

    void popup() {
      XtManageChild( dlg );
    };
};


class AttributesListFrame;

// base class for CollectionWindow and DocumentWindow.
class StandardWindow {
  protected:
    Widget form;
    Widget menubar;
    Widget panedw;
    PrintfLabel* message_area;

  public:
    PrintfLabel& message() { return *message_area; };

    StandardWindow( Widget shell ) {
      form = XtVaCreateManagedWidget( "form",
        xmFormWidgetClass, shell,
        XmNwidth, 300,
        NULL );

      menubar = XmCreateMenuBar( form, "main_menubar", NULL, 0 );
      XtManageChild( menubar );

      XtVaSetValues( menubar,
        XmNtopAttachment, XmATTACH_FORM,
        XmNleftAttachment, XmATTACH_FORM,
        XmNrightAttachment, XmATTACH_FORM,
        XmNbottomAttachment, XmATTACH_NONE,
        NULL );

      message_area = new PrintfLabel( form );
      XtVaSetValues( message_area->root_widget(),
        XmNtopAttachment, XmATTACH_NONE,
        XmNleftAttachment, XmATTACH_FORM,
        XmNrightAttachment, XmATTACH_FORM,
        XmNbottomAttachment, XmATTACH_FORM,
        NULL );

      panedw = XtVaCreateManagedWidget( "dw_panedw",
        xmPanedWindowWidgetClass, form,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopWidget, menubar,
        XmNleftAttachment, XmATTACH_FORM,
        XmNrightAttachment, XmATTACH_FORM,
        XmNbottomAttachment, XmATTACH_WIDGET,
        XmNbottomWidget, message_area->root_widget(),
        NULL );
    };
};

class CollectionWindow : public StandardWindow {
  protected:
    PersistentCollection* collection;
    char collection_name[1024];
    char document_id[1024];
    Document* pD;
    DocumentsListFrame* docs_list_frame;
    AttributesListFrame* attributes_list_frame;
    CollectionPulldownMenu* collection_pulldown;
    DocumentPulldownMenu* document_pulldown;

  public:
    char* name() { return collection_name; };

    CollectionWindow( Widget shell );

    ~CollectionWindow() {
      delete pD;
    };


    void set_collection( char* name );      // sets collection_name
    void unset_collection();

    void show_document( char* id, Boolean show_data );
    void show_document_data();
};



class CollectionSelectionDialog {
  protected:
    Widget dlg;

    static void cb( Widget w, XtPointer cd, XtPointer cbs ) {
      CollectionSelectionDialog* self = (CollectionSelectionDialog*) cd;
      XmSelectionBoxCallbackStruct* sbcbs = (XmSelectionBoxCallbackStruct*) cbs;
      if ( sbcbs->reason == XmCR_OK ) {
        char* value = 0;
        if ( XmStringGetLtoR( sbcbs->value, XmFONTLIST_DEFAULT_TAG, &value ) ) {
	  // JP3+
	  cerr << "Collection Selected: " << value << endl;
	  // JP3-
          collection_window->set_collection( value );
          XtFree( value );
        }
      }
    };

  public:
    void popup( ) {
      XmString* strings = new XmString[ directoryw->n_coll_names ];
      int i;
      for ( i=0; i < directoryw->n_coll_names; i++ ) {
        strings[i] = XmStringCreateLocalized( directoryw->coll_names[i] );
      }
      XtVaSetValues( dlg,
        XmNlistItems, strings,
        XmNlistItemCount, i,
        NULL );
      for ( i=0; i < directoryw->n_coll_names; i++ ) {
        XmStringFree( strings[i] );
      }
      delete[] strings;
      XtManageChild( dlg );
    };

    CollectionSelectionDialog( Widget parent ) {
      dlg = XmCreateSelectionDialog( parent, "collection_selection", NULL, 0 );
      XtVaSetValues( XtParent(dlg), XmNtitle, "Select Collection", NULL );
      XtUnmanageChild( XmSelectionBoxGetChild( dlg, XmDIALOG_APPLY_BUTTON ) );
      XtUnmanageChild( XmSelectionBoxGetChild( dlg, XmDIALOG_TEXT ) );
      XtUnmanageChild( XmSelectionBoxGetChild( dlg, XmDIALOG_SELECTION_LABEL ));
      XtVaSetValues( dlg,
        XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
        XmNmustMatch, True,
        NULL );
      XtAddCallback( dlg, XmNokCallback, cb, this );
    };
};





//
// class Frame is a plain wrapper for xmFrameWidgetClass,
// but it is created with a title label child, and that
// label is made via class PrintfLabel.
// This makes it easy to write to the title of a frame!
// You do it by getting the PrintfLabel via method title_label().
//
class Frame {
  protected:
    Widget frame;
    PrintfLabel* title_lbl;

  public:
    // the widget to use for e.g. setting attachments:
    Widget root_widget() { return frame; };

    // the widget to use as a parent for another widget:
    Widget as_parent() { return frame; };

    PrintfLabel& title_label() { return *title_lbl; };

    void set_sensitive( Boolean sens ) {
      XtSetSensitive( frame, sens );
    };

    Frame( Widget parent, char* initial_title ) {
      frame = XtVaCreateManagedWidget( "flab_frame",
        xmFrameWidgetClass, parent,
        XmNshadowType, XmSHADOW_ETCHED_IN,
        NULL );

      title_lbl = new PrintfLabel( frame, initial_title );
      XtVaSetValues( title_lbl->root_widget(),
        XmNchildType, XmFRAME_TITLE_CHILD,
        NULL );
    };
};




class ListFrame : public Frame {
  protected:
    Widget list;

    static void _cb( Widget w, XtPointer cd, XtPointer cbs ) {
      ListFrame* self = (ListFrame*) cd;
      XmListCallbackStruct* lcbs = (XmListCallbackStruct*) cbs;

      XmStringContext context;
      if ( XmStringInitContext( &context, lcbs->item ) ) {
        char buf[1024];
        char* p = buf;
        char* text;
        XmStringCharSet tag;
        XmStringDirection dir;
        Boolean sep;
        while ( XmStringGetNextSegment( context, &text, &tag, &dir, &sep ) ) {
          p += strlen( strcpy( p, text ) );
          if ( sep ) {
            *p++ = '\n';
            *p = '\0';
          }
          XtFree( text );
        }
        XmStringFreeContext( context );

        // List positions start at 1, so adjust to normal C conventions...
        self->callback( lcbs->item_position - 1, 
                   buf, lcbs->reason == XmCR_DEFAULT_ACTION );
      }
      else {
        fprintf(stderr,"Error in XmStringInitContext()\n");
      }
    };

    // by default, does nothing.
    virtual void callback( int pos, char* string, Boolean doubleclick ) { };

  public:

    void clear() {
      XmListDeleteAllItems( list );
    };

    void append_item( XmString item ) {
      XmListAddItemUnselected( list, item, 0 );
    };

    void append_string( char* item ) {
      XmString str = XmStringCreateLocalized( item );
      XmListAddItemUnselected( list, str, 0 );
      XmStringFree( str );
    };

    void append_printf( char* fmt, ... ) {
      char buf[0x4000];
      va_list args;
      va_start(args,fmt);
      vsprintf( buf, fmt, args );
      va_end(args);

      // this makes a compound string; the input string is parsed at 
      // newlines, and each part becomes a segment in the xmstring.
      XmString item = XmStringCreateLtoR( buf, XmFONTLIST_DEFAULT_TAG );
      XmListAddItemUnselected( list, item, 0 );
      XmStringFree( item );
    };

    ListFrame( Widget parent, char* title, int init_size = 5 )
    : Frame(parent,title)
    {
      Arg args[8]; int n = 0;
      XtSetArg( args[n], XmNscrollingPolicy, XmAUTOMATIC ); n++;
      XtSetArg( args[n], XmNscrollBarDisplayPolicy, XmSTATIC ); n++;
      XtSetArg( args[n], XmNlistSizePolicy, XmCONSTANT ); n++;
      XtSetArg( args[n], XmNselectionPolicy, XmBROWSE_SELECT ); n++;
      XtSetArg( args[n], XmNvisibleItemCount, init_size ); n++;
      list = XmCreateScrolledList( frame, "lf_list", args, n );
      XtManageChild( list );
      XtAddCallback( list, XmNbrowseSelectionCallback, _cb, this );
      XtAddCallback( list, XmNdefaultActionCallback,   _cb, this );
    };
};


class DocumentsListFrame : public ListFrame {
  protected:
    void callback( int pos, char* doc_id, Boolean display_it ) {
      collection_window->show_document( doc_id, display_it );
    };

  public:
    DocumentsListFrame( Widget parent ) : ListFrame(parent,"Documents",12) { };

    void fill( PersistentCollection* collection );
};


class AttributesListFrame : public ListFrame {
  protected:
    void callback( int pos, char* str, Boolean display_it ) {
      // Can you think of anything you might want to do with an Attribute?
    };

  public:
    AttributesListFrame( Widget parent ) : ListFrame(parent,"Attributes") { };

    void fill( Set<Attribute>& attributes );
};


class AnnotationsListFrame : public ListFrame {
  protected:
    void callback( int pos, char* str, Boolean display_it );

  public:
    AnnotationsListFrame( Widget parent ) : ListFrame(parent,"Attributes") { };

    void fill( AnnotationSet& annotations );
};



class PulldownMenu {
  protected:
    Widget pulldown;

  public:
    Widget widget() { return pulldown; };

    PulldownMenu( Widget menubar, char* label, char mnem ) {
      pulldown = XmCreatePulldownMenu( menubar, "pulldown", NULL, 0 );
      XtVaSetValues( pulldown, XmNtearOffModel, XmTEAR_OFF_ENABLED, NULL );
      XmString str = XmStringCreateSimple( label );
      XtVaCreateManagedWidget( "coll_cascade",
        xmCascadeButtonWidgetClass, menubar,
        XmNlabelString, str,
        XmNmnemonic, mnem,
        XmNsubMenuId, pulldown,
        NULL );
      XmStringFree( str );
    };

    void add_separator() {
      XtVaCreateManagedWidget( "sep", xmSeparatorGadgetClass, pulldown, 0 );
    };
};


class CollectionPulldownMenu : public PulldownMenu {
  protected:
    class OpenButton : public PrintfButton {
      protected:
        void callback() { coll_sel_dlg->popup(); };
      public:
        OpenButton(Widget p) : PrintfButton( p, "Open..." ) {
          set_accelerator( "Ctrl+O", "Ctrl<Key>O", "O" );
        };
    };

    class OpenDirButton : public PrintfButton {
      protected:
        void callback() { dir_sel_dlg->popup(); };
      public:
        OpenDirButton(Widget p) : PrintfButton( p, "Directory..." ) { };
    };

    class ExitButton : public PrintfButton {
      protected:
        void callback() { exit(0); };
      public:
        ExitButton(Widget p) : PrintfButton( p, "Exit" ) {
          set_accelerator( "Ctrl+x", "Ctrl<Key>x", "x" );
        };
    };

    PrintfButton* open_btn;
    PrintfButton* odir_btn;
    PrintfButton* exit_btn;

  public:
    CollectionPulldownMenu( Widget menubar ) :
      PulldownMenu( menubar, "Collection", 'C' )
    {
      open_btn = new OpenButton( pulldown );
      odir_btn = new OpenDirButton( pulldown );
      add_separator();
      exit_btn = new ExitButton( pulldown );
    };
};


class DocumentPulldownMenu : public PulldownMenu {
  protected:
    class DataButton : public PrintfButton {
      protected:
        void callback() { collection_window->show_document_data(); };
      public:
        DataButton(Widget p) : PrintfButton( p, "View..." ) {
          set_accelerator( "Ctrl+V", "Ctrl<Key>V", "V" );
        };
    };

    PrintfButton* data_btn;

  public:
    PrintfButton& doc_data_button()    { return *data_btn; };

    void sensitive( Boolean sens ) {
      XtSetSensitive( pulldown, sens );
    };

    DocumentPulldownMenu( Widget menubar ) :
      PulldownMenu( menubar, "Document", 'D' )
    {
      data_btn = new DataButton( pulldown );
    };
};



// a nice wrapper for TopLevelShell.
// it's printf-enabled, for writing to the title bar.
class TopLevelWindow {
  protected:
    Widget sh;

  public:
    Widget shell() { return sh; };
    void popup() { XtPopup( sh, XtGrabNone ); };
    void popdown() { XtPopdown( sh ); };

    TopLevelWindow( Widget p ) {
      sh = XtVaCreatePopupShell("toplevelsh", topLevelShellWidgetClass, p,0);
      XtVaSetValues( sh, XmNdeleteResponse, XmUNMAP, NULL );
    };

    void printf( char* fmt, ... ) {
      char msgbuf[0x4000];
      va_list args;
      va_start(args,fmt);
      vsprintf( msgbuf, fmt, args );
      va_end(args);
      XtVaSetValues( sh, XmNtitle, msgbuf, NULL );
    };
};


    class DocPulldownMenu;
    class AnnotationPulldownMenu;

// order of inheritance is critical here.
class DocumentWindow : public TopLevelWindow, public StandardWindow {
  protected:
    Document* document;
    DocPulldownMenu* doc_pulldown;
    AnnotationPulldownMenu* anno_pulldown;
    TextArea* text_area;
    AttributesListFrame* attributes_list_frame;
    AnnotationsListFrame* annotations_list_frame;
    int anno_index;

  public:
    DocumentWindow( Widget parent );

    void display_document( Document* pD ) {
      document = pD;
      printf( "Document %s", (char*)(pD->GetId()) );
      text_area->set_string( (char*)(pD->GetRawData()->ConvertToString()) );
      attributes_list_frame->fill( pD->GetAttributes() );
      annotations_list_frame->fill( pD->GetAnnotations() );
      XtManageChild( form );
      popup();
      XmProcessTraversal( annotations_list_frame->root_widget(), 
        XmTRAVERSE_CURRENT ); // might not work
    };

    void mark_annotation() {
      if (anno_index != -1) {
	  Set<Span>& spans = document->GetAnnotations().Nth(anno_index).GetSpans();
	  int n_spans = spans.Length();
	  text_area->no_highlights();
	  for ( int s = 0; s < n_spans; s++ ) {
	      Span& span = spans.Nth(s);
	      text_area->add_highlight( span.GetStart(), span.GetEnd() );
	  }
      }
    };

    void show_annotation( int index, Boolean mark_it ) {
      anno_index = index;
      Annotation& ann = document->GetAnnotations().Nth( anno_index );
      Set<Attribute>& attributes = ann.GetAttributes();
      message_area->printf( "%d Spans, %d Attributes", 
        ann.GetSpans().Length(),
        attributes.Length() 
      );
      if ( mark_it ) mark_annotation();
    };
};


class DocPulldownMenu : public PulldownMenu {
    class CloseButton : public PrintfButton {
      protected:
        void callback() { document_window->popdown(); };
      public:
        CloseButton(Widget p) : PrintfButton( p, "Close" ) {
          set_accelerator( "Ctrl+C", "Ctrl<Key>C", "C" );
        };
    };

    PrintfButton* close_btn;

  public:
    DocPulldownMenu( Widget menubar ) :
      PulldownMenu( menubar, "Document", 'D' )
    {
      add_separator();
      close_btn = new CloseButton( pulldown );
    };
};


class AnnotationPulldownMenu : public PulldownMenu {
    class HighlightButton : public PrintfButton {
      protected:
        void callback() { document_window->mark_annotation(); };
      public:
        HighlightButton(Widget p) : PrintfButton( p, "Highlight" ) {
          set_accelerator( "Ctrl+H", "Ctrl<Key>H", "H" );
        };
    };

    PrintfButton* mark_btn;

  public:
    AnnotationPulldownMenu( Widget menubar ) :
      PulldownMenu( menubar, "Annotation", 'A' )
    {
      mark_btn = new HighlightButton( pulldown );
    };
};






CollectionWindow::CollectionWindow( Widget shell ) : 
  StandardWindow( shell ),
  collection(NULL)
{
  pD = NULL;
  collection_name[0] = '\0';
  document_id[0] = '\0';

  collection_pulldown = new CollectionPulldownMenu( menubar );
  document_pulldown = new DocumentPulldownMenu( menubar );

  docs_list_frame = new DocumentsListFrame( panedw );
  attributes_list_frame = new AttributesListFrame( panedw );

  document_window = new DocumentWindow( shell );

  // these two dialogs are global (for now, anyway):
   dir_sel_dlg = new  DirectorySelectionDialog( shell );
  coll_sel_dlg = new CollectionSelectionDialog( shell );
};



int DirectoryW::set_directory( char* dir ) {
      DIR* dp = opendir( dir );
      if ( ! dp ) {
        if ( collection_window ) {
          collection_window->message().printf(strerror(errno));
        }
        else {
          // as might happen for a directory specified on the cmdline
          perror(dir); 
        }
        return 0;
      }

      strcpy( dir_name, dir );
      // JP3+
      cerr << "Setting record file directory to: " << dir_name <<endl;
      // JP3-
      SetRecordFileDirectory( dir_name );

      if ( collection_window ) { // false if processing dir from cmdline
        collection_window->unset_collection();
      }

      load_directory( dp );

      if ( n_coll_names == 0 ) {
        if ( collection_window ) {
          collection_window->message().printf(
            "Warning: directory '%s' contains no collections",dir); 
        }
        else {
          // as might happen for a directory specified on the cmdline
          fprintf(stderr,
            "Warning: directory '%s' contains no collections\n",dir); 
        }
      }
      else {
        if ( collection_window ) {
          collection_window->message().printf("Ok");
        }
      }
      return 1;
    };

    void CollectionWindow::show_document_data() {
      if ( document_window ) {
        document_window->display_document( pD );
      }
    };

    void CollectionWindow::unset_collection() {
      if ( document_pulldown ) {
        document_pulldown->sensitive( False );
      }
      if ( docs_list_frame ) {
        docs_list_frame->clear();
        docs_list_frame->set_sensitive( False );
      }
      message_area->printf("No collection");
    };

    void CollectionWindow::set_collection( char* name ) {
      PersistentCollection* new_coll = NULL;
      try {
         new_coll = new PersistentCollection( name );
      }
      catch(...) {
         message_area->printf( "Collection not found" );
         new_coll = NULL;
      }
      if ( ! new_coll ) {
        return;
      }

      delete collection;
      collection = new_coll;
      strcpy( collection_name, name );

      document_pulldown->sensitive( False );

      //
      // fill the attributes list:
      //
      if ( attributes_list_frame ) {
        attributes_list_frame->fill( collection->GetAttributes() );
      }

      //
      // fill the document list:
      //
      if ( docs_list_frame ) {
        docs_list_frame->fill( collection );
        XmProcessTraversal( docs_list_frame->root_widget(), 
          XmTRAVERSE_CURRENT );
      }
    };

void CollectionWindow::show_document( char* doc_id, Boolean show_data ) {
      if ( ! collection ) {
        message_area->printf("No collection loaded\n" );
        return;
      }
      Document* new_pD = collection->GetDocument( doc_id );  
      if ( ! new_pD ) {
        message_area->printf("No such document '%s'\n", doc_id );
        return;
      }
      delete pD;
      pD = new_pD;

      strcpy( document_id, doc_id );

      if ( document_pulldown ) {
        message_area->printf( "%s -- %d attributes, %d annotations",
          document_id,
          pD->GetAttributes().Length(),
          pD->GetAnnotations().Length()
        );

        document_pulldown->sensitive( True );
      }

      if ( show_data ) {
        show_document_data();
      }
    };

    DocumentWindow::DocumentWindow( Widget parent ) : 
      TopLevelWindow( parent ), 
      StandardWindow( sh ) 
    {
      doc_pulldown = new DocPulldownMenu( menubar );
      anno_pulldown = new AnnotationPulldownMenu( menubar );

      text_area = new TextArea( panedw );
      annotations_list_frame = new AnnotationsListFrame( panedw );
      attributes_list_frame = new AttributesListFrame( panedw );
      anno_index = -1;
    };


    void DocumentsListFrame::fill( PersistentCollection* collection ) {
        int n_docs = collection->Length();

        set_sensitive( False );
        clear();
        collection_window->message().printf("Loading %s...", 
          collection->GetName() );
        title_label().printf("%s (%d documents)", 
          collection->GetName(), n_docs );

        int i;
        for ( i=0; i < n_docs; i++ ) {
          Document* tpD = ( i
            ? collection->NextDocument()
            : collection->FirstDocument()
            );
          append_string( (char*)(tpD->GetId()) );
          delete tpD;
        }
        set_sensitive( True );
        collection_window->message().printf("%s", collection->GetName() );
    };



    void AttributesListFrame::fill( Set<Attribute>& attributes ) {
      int n = attributes.Length();
      title_label().printf("%d attributes", n );
      clear();

      for ( int i=0; i < n; i++ ) {
        Attribute& attr = attributes.Nth(i);
        append_printf( "%s: %s", attr.GetName(), attribute_value( attr ) );
      }
    };

    void AnnotationsListFrame::fill( AnnotationSet& annotations ) {
      int n = annotations.Length();
      title_label().printf("%d annotations", n );
      clear();

      //
      // Note: Annotation IDs are not unique.
      //
      for ( int i=0; i < n; i++ ) {
        Annotation& ann = annotations.Nth(i);
        Attribute& att = ann.GetAttributes().Nth(0);
        append_printf( "%10s %4s -- %5s: %s",
          ann.GetType(),
          ann.GetId(),
          att.GetName(),
          attribute_value( att )
        );
      }
    };

void AnnotationsListFrame::callback( int pos, char* str, Boolean display_it ) {
  document_window->show_annotation( pos, display_it );
};


int main( int argc, char** argv ) {
  XtAppContext app_context;

  // this should be a try/catch:
  if ( ! GetRecordFileDirectory() ) {
    fprintf(stderr,"Error: unable to determine data directory!\n");
    exit(1);
  }

  char* start_dir = (argc > 1) ? argv[1] : (char*)GetRecordFileDirectory();
  directoryw = new DirectoryW();
  if ( ! directoryw->set_directory( start_dir ) ) {
    return 1;
  }

  Widget app_shell = XtVaAppInitialize( &app_context, "Demo", NULL, 0,
    &argc, argv, NULL, NULL );
  XtVaSetValues( app_shell, XmNtitle, "Demo", NULL );

  collection_window = new CollectionWindow( app_shell );

  XtRealizeWidget( app_shell );

  collection_window->unset_collection();

  XtAppMainLoop( app_context );

  return 0;
};


