//
// Module Name:		cmcAnnotation.cc
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Provide code for the Annotation class.  An Annotation represents
//	some annotation on a document, including identification of sections
//	of the document to which it applies, a type, and attributes.
//
// Function List:
//	Annotation::Annotation() - there are 2 types of constructors herein
//	Annotation::~Annotation() - a destructor
//	Annotation::GetId() - Get the ID component of the Annotation
//	Annotation::SetId() - Set the ID component of the Annotation
//	Annotation::GetType() - Get the Type component of the Annotation
//	Annotation::GetSpans() - Get the Spans component of the Annotation
//	Annotation::Copy() - Make a copy of the Annotation
//	Annotation::WriteToStream() - Write a text version of the Annotation 
//			to a stream
//	Annotation::IsChanged() - Test if the Annotation has been changed
//	Annotation::ClearChanged() - Clear the changed indicator
//	Annotation::GetClass() - Get the class label of this
//	Annotation::CastToAnnotation() - Downcast function, in two flavors
//	ostream &operator<<() - Write out an Annotation to a stream
//
// Other Modules Called:
//	Set<> - used to keep spans in an Annotation
//	Span - used to track parts of a document
//	Attribute - used to represent attributes of the Annotation
//	AttributedObject - Annotation is a subclass of AttributedObject
//	Misc - Annotation uses CMC miscellaneous functions
//	Error - error exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           11/17/95    jd_2     Initial version
//

#include <stddef.h>			// to get NULL
#include <string.h>			// to get string functions
#include <iostream.h>			// to allow stream I/O

#include "cmcAnnotation.h"		// this operates on Annotations
#include "cmcSet.h"			// this uses Sets as parms
#include "cmcSpan.h"			// Annotations use Spans
#include "cmcAttribute.h"		// Annotations have Attributes
#include "cmcAttributedObject.h"	// Annotation is a AttributedObject
#include "cmcMisc.h"			// to get string read/write functions
#include "cmcError.h"			// contains exception classes

// ClassLabel is the string name of this class
const char *Annotation::ClassLabel = "Annotation";

//
// Function Name:	Annotation::Annotation()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Construct an Annotation given its pieces
//
// Parameters:		NewType - Type of the new annotation
//			NewSpans - Set of spans in the Annotation
//			NewAttributes - Attributes of the Annotation
//
// Return Value:	None
//
Annotation::Annotation(const char *NewType, 
			const Set<Span> &NewSpans, 
			const Set<Attribute> &NewAttributes) : 
				AttributedObject(NewAttributes),
				Id(NULL),
				Type(NULL)
{
    try
    {
    	int i;			// a loop index 

    	// copy in Type
    	if((Type = new char[strlen(NewType) + 1]) == NULL)
	    throw LogiconErrors::MemoryAllocation(
		"Annotation::Annotation(char*, Set<Span>, set<Attribute>)");
    	strcpy(Type, NewType);

    	// copy in spans
    	for(i = 0; i < NewSpans.Length(); ++i)
    	{
	    // allocate a span and check it out
	    Span *pSpan = new Span(NewSpans.Nth(i).GetStart(),
				   NewSpans.Nth(i).GetEnd());
	    if(pSpan == NULL)
	    	throw LogiconErrors::MemoryAllocation(
		    "Annotation::Annotation(char*, Set<Span>, set<Attribute>)");

	    // add in the span
	    Spans.Push(pSpan);
    	}

	// clear any changed flags set in this processing
	Spans.ClearChanged();
    }
    catch(...)
    {
	// clean up anything that got allocated
	if(Type != NULL)
	    delete[] Type;

	// throw on the exception
	throw;
    }
}




//
// Function Name:	Annotation::Annotation()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Construct an Annotation by reading it from an input
//			stream.  The format of the Annotation from the
//			stream is:
//	
//				<Id, in ReadString format>
//				<Type, in ReadString format>
//				<SpanCount> blank
//				<Span0> blank ... blank <SpanN> blank
//				<AttributeCount> blank
//				<AttrName0> blank <AttrValue0> blank
//				...
//				<AttrNameN> blank <AttrValueN> blank
//
// Parameters:		Input - the stream to read from
//
// Return Value:	None
//
Annotation::Annotation(istream &Input) : Id(NULL), Type(NULL)
{
    try
    {
    	int count = 0;			// count of stuff
    	int i;				// loop index
    	char ch;			// for input

    	Id = ReadString(Input);
    	Type = ReadString(Input);

    	// read spans
    	Input >> count;
    	Input.get(ch);

	// if there was a read failure...
	if(!Input)
	    throw LogiconErrors::Read("Annotation::Annotation(istream&)");

    	for(i = 0; i < count; ++i)
    	{
	    char ch;		// to hold input

	    // construct and test the span
	    Span *pSpan = new Span(Input);
	    if(pSpan == NULL)
		throw LogiconErrors::MemoryAllocation(
			"Annotation::Annotation(istream&)");		

	    // add in the span
	    Spans.Push(pSpan);
	    Input.get(ch);
    	}

    	// read attributes
    	Input >> count;
    	Input.get(ch);

	// if there was a read failure...
	if(!Input)
	    throw LogiconErrors::Read("Annotation::Annotation(istream&)");

	for(i = 0; i < count; ++i)
    	{
	    char *AttrName = ReadString(Input);
	    AttributeValue *pValue = ReadAttributeValue(Input);
	    PutAttribute(AttrName, *pValue);
	    delete[] AttrName;
	    delete pValue;
    	}

	// clear any changed flags set in this processing
	ClearAttributesChanged();
	Spans.ClearChanged();
    }
    catch(...)
    {
	if(Id != NULL)
	    delete[] Id;
	if(Type != NULL)
	    delete[] Type;
	throw;
    }
}



//
// Function Name:	Annotation::~Annotation()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Destruct an Annotation by freeing all allocated memory
//			within it.
//
// Parameters:		None
//
// Return Value:	None
//
Annotation::~Annotation()
{
    if(Type != NULL)
    	delete[] Type;
    if(Id != NULL)
	delete[] Id;
}




//
// Function Name:	Annotation::GetId()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Return the Id component of an Annotation.
//
// Parameters:		None
//
// Return Value:	The Id component
//
const char *Annotation::GetId() const
{
    return Id;
}


//
// Function Name:	Annotation::SetId()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Set the Id component of the Annotation to a new value.
//
// Parameters:		NewId - the value to set Id to
//
// Return Value:	None
//
void Annotation::SetId(const char *NewId)
{
    // trash old Id
    if(Id != NULL)
    {
	delete[] Id;
	Id = NULL;
    }
   
    // copy in new Id
    if(NewId == NULL)
	Id = NULL;
    else
    {
    	if((Id = new char[strlen(NewId) + 1]) == NULL)
	    throw LogiconErrors::MemoryAllocation("Annotation::SetId(char *)");

    	strcpy(Id, NewId);
    }
}


//
// Function Name:	Annotation::GetType()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Return the Type component of an Annotation
//
// Parameters:		None
//
// Return Value:	The type component
//
const char *Annotation::GetType() const
{
    return Type;
}


//
// Function Name:	Annotation::GetSpans()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Get the Spans component of the Annotation
//
// Parameters:		None
//
// Return Value:	the Spans component
//
Set<Span> &Annotation::GetSpans()
{
    return Spans;
}



//
// Function Name:	Annotation::Copy()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Create a copy of an Annotation.
//
// Parameters:		None
//
// Return Value:	A new Annotation, a copy of the current annotation
//
Annotation *Annotation::Copy() const
{
    Annotation *pCopy = new Annotation(Type, Spans, GetAttributes());

    if(pCopy == NULL)
	throw LogiconErrors::MemoryAllocation("Annotation::Copy(void)");
    try
    {
    	pCopy->SetId(Id);
    }
    catch(...)
    {
	delete pCopy;
    }

    return(pCopy);
}



//
// Function Name:	Annotation::IsChanged()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Test whether the Annotation has been changed since
//			the last time it was created or ClearChanged() was
//			called.
//
// Parameters:		None
//
// Return Value:	1 if changed
//			0 if not
//
int Annotation::IsChanged() const
{
    return Spans.IsChanged() || IsAttributesChanged();
}



//
// Function Name:	Annotation::ClearChanged()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Clear all indicators that show that the Annotation
//			has been changed.
//
// Parameters:		None
//
// Return Value:	None
//
void Annotation::ClearChanged()
{
    Spans.ClearChanged();
    ClearAttributesChanged();
}




//
// Function Name:	Annotation::WriteToStream()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Write a text version of the Annotation to an output
//			stream.  The format is as described for
//			Annotation::Annotation(istream&) above.  This is
//			separate from operator<< so that a subclass could
//			override it.
//
// Parameters:		Output - the stream to write to
//
// Return Value:	None
//
void Annotation::WriteToStream(ostream &Output) const
{
    int i;			// loop index

    WriteString(Id, Output);
    WriteString(Type, Output);
    Output << Spans.Length() << ' ';
    for(i = 0; i < Spans.Length(); ++i)
	Output << Spans.Nth(i) << ' ';
    Output << GetAttributes().Length() << ' ';
    for(i = 0; i < GetAttributes().Length(); ++i)
    {
	WriteString(GetAttributes().Nth(i).GetName(), Output);
	Output << GetAttributes().Nth(i).GetValue();
    }

    // if there was a write failure...
    if(!Output)
	throw LogiconErrors::Write(
			"Annotation::WriteToStream(ostream &Output)");
}


//
// Function Name:	operator<<
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Write an Annotation to an output stream in standard
//			C++ style.  This is not a member of Annotation but 
//			is affiliated with it.
//
// Parameters:		Output - The stream to write to
//			Data - The Annotation to write
//
// Return Value:	The Output stream after it has been written
//
ostream &operator<<(ostream &Output, Annotation &Data)
{
    Data.WriteToStream(Output);
    return Output;
}




//
// Function Name:	Annotation::GetClass()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *Annotation::GetClass() const
{
    return ClassLabel;
}



//
// Function Name:	Annotation::CastToAnnotation()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
Annotation &Annotation::CastToAnnotation()
{
    return *this;
}




//
// Function Name:	Annotation::CastToAnnotation()
//
// CSCI/CSC/CSU:	CMC/Annotation
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const Annotation &Annotation::CastToAnnotation() const
{
    return *this;
}


