//
// Module Name:		cmcAnnotationReference.cc
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Includes all code for AnnotationReference member functions.  An
//	AnnotationReference references an Annotation by keeping its
//	Collection name, Document ID, and Annotation ID.  It is a pretty
//	straight-forward implementation that lists these three items.
//
// Function List:
//	AnnotationReference::AnotationReference() - this contains 3 constructors
//	AnnotationReference::~AnnotationReference() - a destructor
//	AnnotationReference::GetCollectionName() - Get CollectionName component
//	AnnotationReference::GetDocumentId() - Get the DocumentId component
//	AnnotationReference::GetAnnotationId() - Get the AnnotationId component
//	AnnotationReference::TypeOf() - Determine the type of ObjectReference 
//				that this is
//	AnnotationReference::Copy() - Produce a copy of this
//	AnnotationReference::operator==() - Test for equivalence with another 
//				ObjectReference
//	AnnotationReference::Write() - Write this to a stream
//	AnnotationReference::GetClass() - Get the name of this class
//	AnnotationReference::Cleanup() - Clean up memory used by this
//	AnnotationReference::CastToAnnotationReference() - Downcast function
//
// Other Modules Called:
//	AttributeValueType - Includes possible types of AttributeValues and
//			ObjectReferences
//	Document - Used in the constructor
//	Annotation - Used in the constructor
//	Misc - Used to read/write strings
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#include <string.h>			// for string functions
#include <iostream.h>			// for I/O functions

#include "cmcAnnotationReference.h"	// includes for this
#include "cmcAttributeValueType.h"	// return of TypeOf() function
#include "cmcDocument.h"		// used in constructor
#include "cmcAnnotation.h"		// used in constructor
#include "cmcMisc.h"			// used for string I/O
#include "cmcError.h"			// used for exceptions

// ClassLabel is the string name of this class
const char *AnnotationReference::ClassLabel = "AnnotationReference";


//
// Function Name:	AnnotationReference::AnnotationReference()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Constructor given the things to reference.
//
// Parameters:		ReferencedDocument - the Document containing the 
//				referenced Annotation
//			ReferencedAnnotation - the referenced Annotation
//
// Return Value:	None
//
AnnotationReference::AnnotationReference(Document &ReferencedDocument, 
		Annotation &ReferencedAnnotation) :
			CollectionName(NULL),
			DocumentId(NULL),
			AnnotationId(NULL)
{
    try
    {
	// allocate space
    	CollectionName = 
		new char[strlen(ReferencedDocument.GetParent().GetName()) + 1];
    	DocumentId = new char[strlen(ReferencedDocument.GetId()) + 1];
    	AnnotationId = new char[strlen(ReferencedAnnotation.GetId()) + 1];

	// test the allocations
	if(CollectionName == NULL || DocumentId == NULL || AnnotationId == NULL)
	    throw LogiconErrors::MemoryAllocation(
  "AnnotationReference::AnnotationReference(Document&, Annotation&)");

	// copy strings
    	strcpy(CollectionName, ReferencedDocument.GetParent().GetName());
    	strcpy(DocumentId, ReferencedDocument.GetId());
    	strcpy(AnnotationId, ReferencedAnnotation.GetId());
    }
    catch(...)
    {
	Cleanup();

	throw;
    }
}



//
// Function Name:	AnnotationReference::AnnotationReference()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Constructor given various ID's and names to store.
//
// Parameters:		NewCollectionName - The name of the Collection
//			NewDocumentId - The ID of the Document
//			NewAnnotationId - The ID of the Annotation
//
// Return Value:	None
//
AnnotationReference::AnnotationReference(const char *NewCollectionName, 
					 const char *NewDocumentId, 
					 const char *NewAnnotationId) :
			CollectionName(NULL),
			DocumentId(NULL),
			AnnotationId(NULL)
{
    try
    {
	// allocate space
    	CollectionName = 
		new char[strlen(NewCollectionName) + 1];
    	DocumentId = new char[strlen(NewDocumentId) + 1];
    	AnnotationId = new char[strlen(NewAnnotationId) + 1];

	// test the allocations
	if(CollectionName == NULL || DocumentId == NULL || AnnotationId == NULL)
	    throw LogiconErrors::MemoryAllocation(
	     "AnnotationReference::AnnotationReference(char*, char*, char*)");

	// copy strings
    	strcpy(CollectionName, NewCollectionName);
    	strcpy(DocumentId, NewDocumentId);
    	strcpy(AnnotationId, NewAnnotationId);
    }
    catch(...)
    {
	Cleanup();

	throw;
    }
}



//
// Function Name:	AnnotationReference::AnnotationReference()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Constructor given a stream containing the
//			input ID's and names.  The stream data will
//			in the following format:
//
//				ANNOTATION_REFERENCE (an AttributeValueType)
//				CollectionName (in ReadString() format)
//				DocumentId (in ReadString() format)
//				AnnotationId (in ReadString() format)
//
// Parameters:		Input - The input stream
//
// Return Value:	None
//
AnnotationReference::AnnotationReference(istream &Input) :
			CollectionName(NULL),
			DocumentId(NULL),
			AnnotationId(NULL)
{
    try
    {
    	AttributeValueType Filler;		// to read fille stuff

    	Input >> Filler;
    	CollectionName = ReadString(Input);
    	DocumentId = ReadString(Input);
    	AnnotationId = ReadString(Input);

	// check for Input format
	if(!Input)
	    throw LogiconErrors::Read(
		"AnnotationReference::AnnotationReference(istream &)");
    }
    catch(...)
    {
	Cleanup();

	throw;
    }
}


//
// Function Name:	AnnotationReference::~AnnotationReference()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Destructor that frees all allocated storage.
//
// Parameters:		None
//
// Return Value:	None
//
AnnotationReference::~AnnotationReference()
{
    Cleanup();
}



//
// Function Name:	AnnotationReference::GetCollectionName()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Return the CollectionName component.
//
// Parameters:		None
//
// Return Value:	The CollectionName
//
const char *AnnotationReference::GetCollectionName() const
{
    return CollectionName;
}



//
// Function Name:	AnnotationReference::GetDocumentId()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Return the DocumentId component.
//
// Parameters:		None
//
// Return Value:	The DocumentId component.
//
const char *AnnotationReference::GetDocumentId() const
{
    return DocumentId;
}



//
// Function Name:	AnnotationReference::GetAnnotationId()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Return the AnnotationId component.
//
// Parameters:		None
//
// Return Value:	The AnnotationId component.
//
const char *AnnotationReference::GetAnnotationId() const
{
    return AnnotationId;
}



//
// Function Name:	AnnotationReference::TypeOf()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Return the type of ObjectReference that this is.
//
// Parameters:		None
//
// Return Value:	ANNOTATION_REFERENCE
//
AttributeValueType AnnotationReference::TypeOf() const
{
    return ANNOTATION_REFERENCE;
}



//
// Function Name:	AnnotationReference::Copy()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Produce a duplicate of this AnnotationReference.
//
// Parameters:		None
//
// Return Value:	A pointer to that duplicate.
//
ObjectReference *AnnotationReference::Copy() const
{
    ObjectReference *pCopy = new AnnotationReference(CollectionName, 
						     DocumentId, 
						     AnnotationId);

    if(pCopy == NULL)
	throw LogiconErrors::MemoryAllocation("AnnotationReference::Copy()");

    return pCopy;
}



//
// Function Name:	AnnotationReference::operator==
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Test whether this is equal to another ObjectReference.
//
// Parameters:		Target - the ObjectReference to test against
//
// Return Value:	1 if equal
//			0 if not
//
int AnnotationReference::operator==(const ObjectReference &Target) const
{
    int Results;			// results of this

    // if the target is the correct type...
    if(Target.TypeOf() == ANNOTATION_REFERENCE)
    {
	// compare the elements
	Results = (strcmp(((AnnotationReference *)(&Target))->CollectionName,
				CollectionName) == 0) &&
		(strcmp(((AnnotationReference *)(&Target))->DocumentId,
				DocumentId) == 0) &&
		(strcmp(((AnnotationReference *)(&Target))->AnnotationId,
				AnnotationId) == 0);
    }
    else
	Results = 0;

    return Results;
}



//
// Function Name:	AnnotationReference::Write()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Write this to a stream in the format described for
//			AnnotationReference(istream&).
//
// Parameters:		Output - the stream to write to
//
// Return Value:	None
//
void AnnotationReference::Write(ostream &Output) const
{
    Output << TypeOf();
    WriteString(CollectionName, Output);
    WriteString(DocumentId, Output);
    WriteString(AnnotationId, Output);

    // check output results
    if(!Output)
	throw LogiconErrors::Write("AnnotationReference::Write(ostream &)");
}




//
// Function Name:	AnnotationReference::GetClass()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *AnnotationReference::GetClass() const
{
    return ClassLabel;
}



//
// Function Name:	AnnotationReference::Cleanup()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Clean out all memory used by this.
//
// Parameters:		None
//
// Return Value:	None
//
void AnnotationReference::Cleanup()
{
    if(CollectionName != NULL)
    {
	delete[] CollectionName;
	CollectionName = NULL;
    }
    if(DocumentId != NULL)
    {
	delete[] DocumentId;
	DocumentId = NULL;
    }
    if(AnnotationId != NULL)
    {	
	delete[] AnnotationId;
	AnnotationId = NULL;
    }
}



//
// Function Name:	AnnotationReference::CastToAnnotationReference()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
AnnotationReference &AnnotationReference::CastToAnnotationReference()
{
    return *this;
}




//
// Function Name:	AnnotationReference::CastToAnnotationReference()
//
// CSCI/CSC/CSU:	CMC/AnnotationReference
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const AnnotationReference &AnnotationReference::CastToAnnotationReference() 
			const
{
    return *this;
}


