//
// Module Name:		cmcAnnotationSet.cc
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Provide code for the AnnotationSet class.  An AnnotationSet
//	maintains a set of Annotations, providing functions to act
//	on a specified subset of those Annotations.
//
// Function List:
//	AnnotationSet::AnnotationSet() - there are 3 types of constructors here
//	AnnotationSet::~AnnotationSet() - class destructor
//	AnnotationSet::AddAnnotation() - Add an annotation to the set
//	AnnotationSet::RemoveAnnotation() - Remove an annotation from the set
//	AnnotationSet::GetAnnotation() - Get a specified annotation 
//	AnnotationSet::Length() - Find the number of annotations in the set
//	AnnotationSet::Nth() - Return the Nth annotation in the set
//	AnnotationSet::SelectAnnotations() - get specified annotations
//	AnnotationSet::DeleteAnnotations() - Delete specified annotations
//	AnnotationSet::AnnotationsAt() - Get annotations at some position
//	AnnotationSet::NextAnnotations() - Get annotations after some position
//	AnnotationSet::MergeAnnotations() - Merge two AnnotationSets
//	AnnotationSet::Copy() - Copy an AnnotationSet
//	AnnotationSet::WriteToStream() - Write text version to stream
//	AnnotationSet::IsChanged() - Test if this has been recently changed
//	AnnotationSet::ClearChanged() - Clear the changed indicator for this
//	operator<< - Output the AnnotationSet
//	operator>> - Input the AnnotationSet
//	AnnotationSet::GetClass() - get the class label of this
//	AnnotationSet::CastToAnnotationSet() - Downcast function
//
// Other Modules Called:
//	Set<> - To hold Annotations and Attributes
//	Annotation - The items processed
//	Attribute - To give characteristics of desired Annotations
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           11/17/95    jd_2     Initial version
//

#include <stddef.h>			// used to get NULL
#include <strstream.h>			// used for string I/O functions
#include <stdlib.h>			// used for C standard functions
#include <string.h>			// used for strlen() functions

#include "cmcAnnotationSet.h"		// includes declaration of this class
#include "cmcSet.h"			// used to keep Annotations
#include "cmcAnnotation.h"		// things kept herein
#include "cmcAttribute.h"		// allows search of specific Annotations
#include "cmcError.h"			// used for exception classes



// ClassLabel is the string name of this class
const char *AnnotationSet::ClassLabel = "AnnotationSet";


//
// Function Name:	AnnotationSet::AnnotationSet()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Construct an empty AnnotationSet
//
// Parameters:		None
//
// Return Value:	None
//
AnnotationSet::AnnotationSet() : LastAnnotationId(0), Changed(0)
{
}




//
// Function Name:	AnnotationSet::AnnotationSet()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Construct an AnnotationSet by reading its contents
//			from an input stream.  The format in the stream
//			is as follows:
//
//				<count of annotations> blank
//				<Annotation 0> ... <Annotation N>
//
// Parameters:		Input - The stream to read from
//
// Return Value:	None
//
AnnotationSet::AnnotationSet(istream &Input) : Changed(0)
{
    Input >> *this;
    if(!Input)
	throw LogiconErrors::Read("AnnotationSet::AnnotationSet(istream&)");
}




//
// Function Name:	AnnotationSet::AnnotationSet()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Copy constructor.
//
// Parameters:		Base - AnnotationSet to copy.
//
// Return Value:	None
//
AnnotationSet::AnnotationSet(const AnnotationSet &Base) : 
		LastAnnotationId(Base.LastAnnotationId),
		Changed(Base.Changed)
{
    // for all Annotations in Base
    for(int i = 0; i < Base.Members.Length(); ++i)
	Members.Push(Base.Members.Nth(i).Copy());
}



//
// Function Name:	AnnotationSet::~AnnotationSet()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Destruct an AnnotationSet and its members.
//
// Parameters:		None
//
// Return Value:	None
//
AnnotationSet::~AnnotationSet()
{
}



//
// Function Name:	AnnotationSet::AddAnnotation()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Add a new Annotation to the AnnotationSet, assigning
//			the Annotation a new ID if necessary.  If the
//			Annotation has an ID which matches an Annotation
//			already in the set, replace the existing 
//			Annotation with the new one.
//
// Parameters:		NewAnnotation - The new annotation
//
// Return Value:	ID of the new Annotation
//
const char *AnnotationSet::AddAnnotation(Annotation &NewAnnotation)
{
    // if the Annotation has an ID...
    if(NewAnnotation.GetId() != NULL)
    {
	// if the ID is greater than any gotten thus far...
	int NewId = atoi(NewAnnotation.GetId());
	if(NewId >= LastAnnotationId)
	{
	    LastAnnotationId = NewId + 1;
	}
	else
	{
	    // look for it in the list
	    for(int i = 0; i < Members.Length(); ++i)
	    {
	    	// if this is a match...
	    	if(strcmp(Members.Nth(i).GetId(), NewAnnotation.GetId()) 
					== 0)
	    	{
		    // delete the old one
		    Members.Delete(i);
		    break;
	    	}
	    }
	}
    }
    else
    {
    	ostrstream IdBuffer;		// to write to the annotation id

	// Assign a new AnnotationId
    	IdBuffer << LastAnnotationId << '\0';

	char *Id = IdBuffer.str();		// string version of the ID

	// set the ID
    	NewAnnotation.SetId(Id);
    	++LastAnnotationId;

	delete[] Id;
    }

    // Add in the new Annotation
    Members.Push(NewAnnotation.Copy());

    return NewAnnotation.GetId();
}



//
// Function Name:	AnnotationSet::RemoveAnnotation()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Remove an Annotation from the AnnotationSet
//
// Parameters:		Id - ID of the Annotation to remove
//
// Return Value:	None
//
void AnnotationSet::RemoveAnnotation(const char *Id)
{
    int Offset = 0;			// offset of item to examine
    int Found = 0;			// flag indicating if it is found

    // walk through all annotations, looking for the ones to delete
    while(Offset < Members.Length())
    {
	if(strcmp(Members.Nth(Offset).GetId(), Id) == 0)
	{
	    Members.Delete(Offset);
	    Found = 1;
	}
	else
	    ++Offset;
    }

    // if not found...
    if(!Found)
	throw LogiconErrors::NotFound(
			"AnnotationSet::RemoveAnnotations(char *)");
}



//
// Function Name:	AnnotationSet::GetAnnotation()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Get a specified Annotation from the set
//
// Parameters:		Id - Id of Annotation to get
//
// Return Value:	The Annotation, or NULL if not found
//
Annotation &AnnotationSet::GetAnnotation(const char *Id) const
{
    Annotation *pResult = NULL;		// results of this function

    // find the annotation in question
    for(int i = 0; i < Members.Length() && pResult == NULL; ++i)
	if(strcmp(Members.Nth(i).GetId(), Id) == 0)
	    pResult = &Members.Nth(i);

    // if not found...
    if(pResult == NULL)
	throw LogiconErrors::NotFound(
			"AnnotationSet::GetAnnotation(const char *)");

    return *pResult;
}



//
// Function Name:	AnnotationSet::Length()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Get the number of Annotations in the set
//
// Parameters:		None
//
// Return Value:	Integer count of annotations.
//
int AnnotationSet::Length() const
{
    return Members.Length();
}



//
// Function Name:	AnnotationSet::Nth()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Get the Nth Annotation in the set
//
// Parameters:		N - The Annotation to get
//
// Return Value:	An Annotation, or NULL if it is not available
//
Annotation &AnnotationSet::Nth(int N) const
{
    return Members.Nth(N);
}



//
// Function Name:	AnnotationSet::SelectAnnotations()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Get all Annotations from the set that match a set
//			of criteria.  An Annotation matches the criteria if
//			it is of the specified type and has all of the
//			specified Attributes with the specified values.
//
// Parameters:		Type - Type of Annotation of interest
//			Constraints - Attributes that the Annotation must have
//
// Return Value:	AnnotationSet of Annotations that match the criteria
//
AnnotationSet *AnnotationSet::SelectAnnotations(const char *Type, 
			Set<Attribute> &Constraints) const
{
    AnnotationSet *pResults = new AnnotationSet;	// result of this

    // test allocation
    if(pResults == NULL)
	throw LogiconErrors::MemoryAllocation(
		"AnnotationSet::SelectAnnotations(char*, Set<Attribute>)");

    try
    {
    	// do for all Annotations
    	for(int i = 0; i < Members.Length(); ++i)
    	{
	    // if this matches the constraints...
	    if((Type == NULL || strcmp(Members.Nth(i).GetType(), Type) == 0)
			&& Members.Nth(i).HasAttributes(Constraints))
	    {
	    	pResults->AddAnnotation(Members.Nth(i));
	    }
    	}

    	return pResults;
    }
    catch(...)
    {
	delete pResults;

	throw;
    }
}



//
// Function Name:	AnnotationSet::DeleteAnnotations()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Delete all Annotations that match specified criteria.
//
// Parameters:		Type - The type of Annotations that should be deleted
//			Constraints - Attributes that deleted Annotations 
//				should have
//
// Return Value:	None
//
void AnnotationSet::DeleteAnnotations(const char *Type, 
				      Set<Attribute> &Constraints)
{
    // do for all Annotations
    int Offset = 0;			// offset to examine

    while(Offset < Members.Length())
    {
	// if this matches the constraints...
	if((Type == NULL || strcmp(Members.Nth(Offset).GetType(), Type) == 0)
		&& Members.Nth(Offset).HasAttributes(Constraints))
	{
	    Members.Delete(Offset);
	}
	else
	    ++Offset;
    }
}




//
// Function Name:	AnnotationSet::AnnotationsAt()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Find all Annotations that have at least one span that
//			starts at a given position.
//
// Parameters:		Position - Position of interest
//
// Return Value:	An AnnotationSet containing all matching
//			Annotations.
//
AnnotationSet *AnnotationSet::AnnotationsAt(int Position) const
{
    AnnotationSet *pResults = new AnnotationSet;	// results set

    // test allocation
    if(pResults == NULL)
	throw LogiconErrors::MemoryAllocation(
			"AnnotationSet::AnnotationsAt(int)");
    try
    {
    	// for all Annotations...
    	for(int i = 0; i < Members.Length(); ++i)
    	{
	    // for all Spans in the Annotation...
	    for(int j = 0; j < Members.Nth(i).GetSpans().Length(); ++ j)
	    {
	    	if(Members.Nth(i).GetSpans().Nth(j).GetStart() == Position)
	    	{
	    	    pResults->AddAnnotation(Members.Nth(i));
		    break;
	    	}
	    }
    	}

    	return pResults;
    }
    catch(...)
    {
	// clean-up allocated memory
	delete pResults;
	throw;
    }
}



//
// Function Name:	AnnotationSet::NextAnnotations()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Find all Annotations from the set that have at
//			least one span that starts at the first location
//			following Position after which at least one
//			Annotation starts.
//
// Parameters:		Position - Position of interest
//
// Return Value:	An AnnotationSet containing all matching
//			Annotations.
//
AnnotationSet *AnnotationSet::NextAnnotations(int Position) const
{
    int ChosenPosition = -1;		// position to use
    AnnotationSet *pResults = NULL;	// AnnotationSet to use

    try
    {
    	// for all Annotations...
    	for(int i = 0; i < Members.Length(); ++i)
    	{
	    int CurrentChosen = -1;	// position to use for this Annotation

	    // for all spans within it...
	    for(int j = 0; j < Members.Nth(i).GetSpans().Length(); ++j)
	    {
	    	// if this is the best choice for this Annotation...
	    	int Start = Members.Nth(i).GetSpans().Nth(j).GetStart();
	    	if(Start >= Position 
			&& (CurrentChosen == -1 || Start < CurrentChosen))
	    	{
		    CurrentChosen = Start;
	    	}
	    }

	    // if this is the best one found yet...
	    if(CurrentChosen != -1 
		&& (ChosenPosition == -1 || CurrentChosen < ChosenPosition))
	    {
	    	// if there are results, trash them
	    	if(pResults != NULL)
	    	{
		    delete pResults;
	    	}

	    	// generate new results
	    	if((pResults = new AnnotationSet) == NULL)
		    throw LogiconErrors::MemoryAllocation(
			"AnnotationSet::NextAnnotations(int)");

	    	// note new position
	    	ChosenPosition = CurrentChosen;
	    }

	    // if this is as good as any found yet...
	    if(CurrentChosen != -1 && CurrentChosen == ChosenPosition)
	    {
	    	pResults->AddAnnotation(Members.Nth(i));
	    }
    	}

    	// if there are no results, create an empty one
    	if(pResults == NULL)
	    if((pResults = new AnnotationSet) == NULL)
	    	throw LogiconErrors::MemoryAllocation(
			"AnnotationSet::NextAnnotations(int)");
		
    	return pResults;
    }
    catch(...)
    {
	// clean up stuff
	if(pResults != NULL)
	    delete pResults;
	throw;
    }
}



//
// Function Name:	AnnotationSet::MergeAnnotations()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Merge two AnnotationSets.
//
// Parameters:		NewAnnotations - AnnotationSet to merge into current
//				one
//
// Return Value:	An AnnotationSet containing the merged Annotations
//
AnnotationSet *AnnotationSet::MergeAnnotations(AnnotationSet &NewAnnotations) 
			const
{
    AnnotationSet *pResults = new AnnotationSet;	// results of merger
    Annotation *pNew = NULL;				// the annotation to add

    // test allocation
    if(pResults == NULL)
	throw LogiconErrors::MemoryAllocation(
		"AnnotationSet::MergeAnnotations(AnnotationSet)");

    try
    {
    	int i;						// loop index

    	// do for all Annotations locally
    	for(i = 0; i < Members.Length(); ++i)
     	{
	    pNew = Members.Nth(i).Copy();
	    pNew->SetId(NULL);
	    pResults->AddAnnotation(*pNew);
	    delete pNew;
    	}

    	// for all new Annotations...
    	for(i=0; i<NewAnnotations.Members.Length(); ++i)
    	{
	    pNew = NewAnnotations.Members.Nth(i).Copy();
	    pNew->SetId(NULL);
	    pResults->AddAnnotation(*pNew);
	    delete pNew;
    	}
  
    	return pResults;
    }
    catch(...)
    {
	// cleanup stuff
	delete pResults;
	if(pNew != NULL)
	    delete pNew;
	throw;
    }
}



//
// Function Name:	AnnotationSet::Copy()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Produce a copy of an AnnotationSet
//
// Parameters:		None
//
// Return Value:	the copy
//
AnnotationSet *AnnotationSet::Copy() const
{
    AnnotationSet *pResults = new AnnotationSet;	// result list

    // test allocation
    if(pResults == NULL)
	throw LogiconErrors::MemoryAllocation("AnnotationSet::Copy()");

    try
    {
    	// Add in all annotations
    	for(int i = 0; i < Length(); ++i)
    	{
	    pResults->AddAnnotation(Members.Nth(i));
    	}

    	return pResults;
    }
    catch(...)
    {
	delete pResults;

	throw;
    }
}




//
// Function Name:	AnnotationSet::IsChanged()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Test whether this has been changed since it was
//			constructed or ClearChanged was called.
//
// Parameters:		None
//
// Return Value:	1 if it has been changed
//			0 if not
//
int AnnotationSet::IsChanged() const
{
    // check if the members have been changed
    if(Members.IsChanged())
    {
    	// this violates the const-ness of the Changed parameter
	// Changed in this case is set only as an optimization
	((AnnotationSet*)this)->Changed = 1;
    }
    else
    {
    	// check all Annotations...
    	for(int i = 0; !Changed && i < Members.Length(); ++i)
	    if(Members.Nth(i).IsChanged())
	    {
	    	// this violates the const-ness of the Changed parameter
	    	// Changed in this case is set only as an optimization
	    	((AnnotationSet*)this)->Changed = 1;
	    }
    }

    return Changed;
}




//
// Function Name:	AnnotationSet::ClearChanged()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Clear the changed indicators of this and its member
//			objects.
//
// Parameters:		None
//
// Return Value:	None
//
void AnnotationSet::ClearChanged()
{
    Changed = 0;

    // clear all Annotations changed
    for(int i = 0; i < Members.Length(); ++i)
	Members.Nth(i).ClearChanged();

    Members.ClearChanged();
}





//
// Function Name:	AnnotationSet::WriteToStream()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Write a text version of the AnnotationSet to an
//			output stream.  The format is documented in
//			Annotation::Annotation(istream&).
//
//			This is separate from operator<< so that a
//			subclass could override it.
//
// Parameters:		Output - Stream to write to
//
// Return Value:	None
//
void AnnotationSet::WriteToStream(ostream &Output) const
{
    // output the length
    Output << Members.Length() << ' ';

    // output the data
    for(int i = 0; i < Members.Length(); ++i)
    {
	Output << Members.Nth(i);
    }

    // check the output
    if(!Output)
	throw LogiconErrors::Write("AnnotationSet::WriteToStream(ostream&)");
}


//
// Function Name:	operator<<
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Write an AnnotationSet to a stream in standard C++
//			style.
//
// Parameters:		Output - Stream to write to
//			Data - AnnotationSet to write
//
// Return Value:	Output, after it has been written to
//
ostream &operator<<(ostream &Output, const AnnotationSet &Data)
{
    Data.WriteToStream(Output);
    return Output;
}




//
// Function Name:	operator>>
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Read in an AnnotationSet from a stream.
//
// Parameters:		Input - the stream
//			Data - AnnotationSet to read
//
// Return Value:	The Input stream
//
istream &operator>>(istream &Input, AnnotationSet &Data)
{
    int count;		// number of annotations in set
    char ch;		// to skip stuff

    // get count
    Input >> count;

    // skip following blank
    Input.get(ch);

    // only continue if input is okay
    if(Input)
    {
    	// read in all annotations
    	for(int i = 0; i < count; ++i)
    	{
	    Annotation *pAnnotation = new Annotation(Input);

	    if(pAnnotation == NULL)
	    	throw LogiconErrors::MemoryAllocation(
			"operator>>(istream&, AnnotationSet&)");

	    Data.Members.Push(pAnnotation);

	    // update the AnnotationId if necessary
	    int AnnotationId = atoi(pAnnotation->GetId());
	    if(AnnotationId > Data.LastAnnotationId)
		Data.LastAnnotationId = AnnotationId;
    	}
    } 
    
    return Input; 
} 


// 
// Function Name:	AnnotationSet::GetClass() 
// 
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *AnnotationSet::GetClass() const
{
    return ClassLabel;
}



//
// Function Name:	AnnotationSet::CastToAnnotationSet()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
AnnotationSet &AnnotationSet::CastToAnnotationSet()
{
    return *this;
}




//
// Function Name:	AnnotationSet::CastToAnnotationSet()
//
// CSCI/CSC/CSU:	CMC/AnnotationSet
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const AnnotationSet &AnnotationSet::CastToAnnotationSet() const
{
    return *this;
}


