//
// Module Name:		cmcAttribute.cc
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Contain code for the Attribute class.  An Attribute maintains a
//	name-value pair that describes some characteristic of an object.
//
// Function List:
//	Attribute::Attribute() - There are two constructors herein
//	Attribute::~Attribute() - A destructor
//	Attribute::GetName() - Get the Attribute's name
//	Attribute::GetValue() - Get the Attribute's value
//	Attribute::WriteToStream() - Write the Attribute to an output stream
//	Attribute::IsChanged() - Test if this has been recently changed
//	Attribute::ClearChanged() - Clear the changed indicator for this
//	operator<< - Write out the Attribute
//	Attribute::GetClass() - get the class label for this
//	Attribute::CastToAttribute() - Downcast function
//
// Other Modules Called:
//	AttributeValue - The value of the Attribute
//	Misc - For miscellaneous support functions
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           11/17/95    jd_2     Initial version
//

#include <iostream.h>			// for I/O
#include <string.h>			// for basic string classes

#include "cmcAttribute.h"		// includes declaration of this class
#include "cmcAttributeValue.h"		// for type of the class
#include "cmcMisc.h"			// for string I/O functions
#include "cmcError.h"			// for exception classes

// ClassLabel is the string name of this class
const char *Attribute::ClassLabel = "Attribute";

//
// Function Name:	Attribute::Attribute()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Construct an Attribute given its name and value
//
// Parameters:		NewName - name of the Attribute
//			NewValue - value of the Attribute
//
// Return Value:	None
//
Attribute::Attribute(const char *NewName, const AttributeValue &NewValue) :
	Name(NULL), pValue(NULL)
{
    try
    {
    	Name = new char[strlen(NewName) + 1];
    	if(Name == NULL)
	    throw LogiconErrors::MemoryAllocation(
		"Attribute::Attribute(char *, AttributeValue&)");
    	strcpy(Name, NewName);
    	pValue = NewValue.Copy();
    }
    catch(...)
    {
	// cleanup allocated memory
	if(Name != NULL)
	    delete[] Name;
	if(pValue != NULL)
	    delete pValue;

	throw;
    }
}



//
// Function Name:	Attribute::Attribute()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Construct an Attribute by reading it from an input
//			stream.  The format of it is as follows:
//
//				<Name (in ReadString() format)>
//				<AttributeValue>
//
// Parameters:		Input - the input stream
//
// Return Value:	None
//
Attribute::Attribute(istream &Input) : Name(NULL), pValue(NULL)
{
    try
    {
    	Name = ReadString(Input);
    	pValue = ReadAttributeValue(Input);
    }
    catch(...)
    {
	// cleanup allocated memory
	if(Name != NULL)
	    delete[] Name;
	if(pValue != NULL)
	    delete pValue;

	throw;
    }
}



//
// Function Name:	Attribute::~Attribute()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Destruct an Attribute by freeing its components.
//
// Parameters:		None
//
// Return Value:	None
//
Attribute::~Attribute()
{
    if(Name != NULL)
    	delete[] Name;
    if(pValue != NULL)
    	delete pValue;
}


//
// Function Name:	Attribute::GetName()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Return the Name component of the Attribute
//
// Parameters:		None
//
// Return Value:	The Attribute's name
//
const char *Attribute::GetName() const
{
    return Name;
}


//
// Function Name:	Attribute::GetValue()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Return the pValue component of the Attribute
//
// Parameters:		None
//
// Return Value:	An AttributeValue pointer indicating the value.
//
AttributeValue &Attribute::GetValue() const
{
    return *pValue;
}



//
// Function Name:	Attribute::WriteToStream()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Write a text version of the Attribute to an output
//			stream.  The format is described under
//			Attribute::Attribute(istream&). 
//
//			This is separate from operator<< because we
//			might want to make this virtual in a future
//			implementation.
//
// Parameters:		Output - Stream to write to
//
// Return Value:	None
//
void Attribute::WriteToStream(ostream &Output) const
{
    WriteString(Name, Output);
    Output << *pValue;
}




//
// Function Name:	Attribute::Copy()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Return a copy of the Attribute.
//
// Parameters:		None
//
// Return Value:	A new Attribute that is a copy of the current
//			Attribute.
//
Attribute *Attribute::Copy() const
{
    Attribute *pCopy = new Attribute(Name, *pValue);

    // test it out
    if(pCopy == NULL)
	throw LogiconErrors::MemoryAllocation("Attribute::Copy()");

    return pCopy;
}



//
// Function Name:	Attribute::IsChanged()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Test whether this has been changed since it was
//			constructed or ClearChanged was called.
//
// Parameters:		None
//
// Return Value:	1 if it has been changed
//			0 if not
//
int Attribute::IsChanged() const
{
    if(pValue != NULL)
    	return pValue->IsChanged();
    else
	return 0;
}



//
// Function Name:	Attribute::ClearChanged()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Clear the changed indicators of this and its member
//			objects.
//
// Parameters:		None
//
// Return Value:	None
//
void Attribute::ClearChanged() 
{
    if(pValue != NULL)
    	pValue->ClearChanged();
}




//
// Function Name:	Attribute::operator<<
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Write out the Attribute.  Provide a standard
//			C++ style output function.
//
// Parameters:		Output - Stream to write to
//			Data - Attribute to write
//
// Return Value:	The Output stream
//
ostream &operator<<(ostream &Output, Attribute &Data)
{
    Data.WriteToStream(Output);
    return(Output);
}





//
// Function Name:	Attribute::GetClass()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *Attribute::GetClass() const
{
    return ClassLabel;
}



//
// Function Name:	Attribute::CastToAttribute()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
Attribute &Attribute::CastToAttribute()
{
    return *this;
}




//
// Function Name:	Attribute::CastToAttribute()
//
// CSCI/CSC/CSU:	CMC/Attribute
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const Attribute &Attribute::CastToAttribute() const
{
    return *this;
}


