//
// Module Name:		cmcAttributeReference.cc
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for member functions of AttributeReference.  An AttributeReference
//	is a type of ObjectReference that references a Document Attribute.  
//	It does this by maintaining the name of the Collection containing
//	the Document, the ID of the Document, and the name of the Attribute.
//
// Function List:
//	AttributeReference::AttributeReference() - This has 3 constructors
//	AttributeReference::~AttributeReference() - The destructor
//	AttributeReference::GetCollectionName() - Return the CollectionName 
//				component
//	AttributeReference::GetDocumentId() - Return the DocumentId component
//	AttributeReference::GetAttributeName() - Return AttributeName component
//	AttributeReference::TypeOf() - Return the type of ObjectReference that 
//				this is
//	AttributeReference::Copy() - Produce a copy of this
//	AttributeReference::operator==() - Test equality of this to another 
//				ObjectReference
//	AttributeReference::Write() - Write this to a stream
//	AttributeReference::GetClass() - Get the text name of this class
//	AttributeReference::Cleanup() - Free memory used by this
//	AttributeReference::CastToAttributeReference() - Downcast function
//
// Other Modules Called:
//	AttributeValueType - Possible types of ObjectReferences
//	Document - Used in a constructor
//	Attribute - Used in a constructor
//	Misc - Used to read/write strings
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#include <string.h>			// for basic string functions
#include <iostream.h>			// for I/O

#include "cmcAttributeReference.h"	// includes declaration of this class
#include "cmcAttributeValueType.h"	// for TypeOf() return value
#include "cmcDocument.h"		// used in a constructor
#include "cmcAttribute.h"		// used in a constructor
#include "cmcMisc.h"			// for string I/O
#include "cmcError.h"			// for exception classes

// ClassLabel is the string name of this class
const char *AttributeReference::ClassLabel = "AttributeReference";


//
// Function Name:	AttributeReference::AttributeReference()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Construct an AttributeReference given the name of
//			the Attribute to reference and the Document in which
//			it exists.
//
// Parameters:		ReferencedDocument - The Document in question
//			Name - the Attribute's name
//
// Return Value:	None
//
AttributeReference::AttributeReference(Document &ReferencedDocument, 
					const char *Name) :
				CollectionName(NULL),
				DocumentId(NULL),
				AttributeName(NULL)
{
    try
    {
    	CollectionName = 
		new char[strlen(ReferencedDocument.GetParent().GetName()) + 1];
    	DocumentId = new char[strlen(ReferencedDocument.GetId()) + 1];
    	AttributeName = new char[strlen(Name) + 1];

	if(CollectionName == NULL || DocumentId == NULL || 
			AttributeName == NULL)
	    throw LogiconErrors::MemoryAllocation(
		"AttributeReference::AttributeReference(Document&, char*)");

    	strcpy(CollectionName, ReferencedDocument.GetParent().GetName());
    	strcpy(DocumentId, ReferencedDocument.GetId());
    	strcpy(AttributeName, Name);
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}




//
// Function Name:	AttributeReference::AttributeReference()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Construct an AttributeReference given the names and
//			ID's that are parts of it.
//
// Parameters:		NewCollectionName - Value for CollectionName component
//			NewDocumentId - Value for DocumentId component
//			NewAttributeName - Value for AttributeName component
//
// Return Value:	None
//
AttributeReference::AttributeReference(const char *NewCollectionName, 
	const char *NewDocumentId, 
	const char *NewAttributeName) :
				CollectionName(NULL),
				DocumentId(NULL),
				AttributeName(NULL)
{
    try
    {
    	CollectionName = 
		new char[strlen(NewCollectionName) + 1];
    	DocumentId = new char[strlen(NewDocumentId) + 1];
    	AttributeName = new char[strlen(NewAttributeName) + 1];

	if(CollectionName == NULL || DocumentId == NULL || 
			AttributeName == NULL)
	    throw LogiconErrors::MemoryAllocation(
		"AttributeReference::AttributeReference(char*, char*, char*)");

    	strcpy(CollectionName, NewCollectionName);
    	strcpy(DocumentId, NewDocumentId);
    	strcpy(AttributeName, NewAttributeName);
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}




//
// Function Name:	AttributeReference::AttributeReference()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Construct an AttributeReference by reading its
//			elements in from a stream in the following format:
//
//				ATTRIBUTE_REFERENCE (an AttributeValueType)
//				CollectionName (in ReadString() format)
//				DocumentId (in ReadString() format)
//				AttributeName (in ReadString() format)
//
// Parameters:		Input - the Stream
//
// Return Value:	None
//
AttributeReference::AttributeReference(istream &Input) :
				CollectionName(NULL),
				DocumentId(NULL),
				AttributeName(NULL)
{
    try
    {
    	AttributeValueType Filler;		// to hold throw-away data

    	Input >> Filler;

    	CollectionName = ReadString(Input);
    	DocumentId = ReadString(Input);
    	AttributeName = ReadString(Input);

	if(!Input)
	    throw LogiconErrors::Read(
			"AttributeReference::AttributeReference(istream &)");
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}


//
// Function Name:	AttributeReference::~AttributeReference()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Destruct an AttributeReference by freeing its
//			allocated memory.
//
// Parameters:		None
//
// Return Value:	None
//
AttributeReference::~AttributeReference()
{
    Cleanup();
}



//
// Function Name:	AttributeReference::GetCollectionName()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Get the CollectionName component.
//
// Parameters:		None
//
// Return Value:	CollectionName component value.
//
const char *AttributeReference::GetCollectionName() const
{
    return CollectionName;
}



//
// Function Name:	AttributeReference::GetDocumentId()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Get the DocumentId component.
//
// Parameters:		None
//
// Return Value:	The DocumentId component.
//
const char *AttributeReference::GetDocumentId() const
{
    return DocumentId;
}



//
// Function Name:	AttributeReference::GetAttributeName()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Get the AttributeName component.
//
// Parameters:		None
//
// Return Value:	The AttributeName component.
//
const char *AttributeReference::GetAttributeName() const
{
    return AttributeName;
}



//
// Function Name:	AttributeReference::TypeOf()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Get the type of this thing.
//
// Parameters:		None
//
// Return Value:	ATTRIBUTE_REFERENCE
//
AttributeValueType AttributeReference::TypeOf() const
{
    return ATTRIBUTE_REFERENCE;
}




//
// Function Name:	AttributeReference::Copy()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Produce a copy of this AttributeReference.
//
// Parameters:		None
//
// Return Value:	Pointer to the new copy
//
ObjectReference *AttributeReference::Copy() const
{
    ObjectReference *pCopy = new AttributeReference(CollectionName, 
						    DocumentId, 
						    AttributeName);

    // check allocation
    if(pCopy == NULL)
	throw LogiconErrors::MemoryAllocation("AttributeReference::Copy()");

    return pCopy;
}



//
// Function Name:	AttributeReference::operator==
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Test the equality of this against another 
//			ObjectReference.
//
// Parameters:		Target - the ObjectReference to test
//
// Return Value:	0 if not equal
//			1 if equal
//
int AttributeReference::operator==(const ObjectReference &Target) const
{
    int Results;			// Results of this

    // if target is correct type
    if(Target.TypeOf() == ATTRIBUTE_REFERENCE)
    {
	// test data within it
	Results = (strcmp(CollectionName, 
		   ((AttributeReference*)(&Target))->CollectionName) == 0) &&
		(strcmp(DocumentId, 
			((AttributeReference*)(&Target))->DocumentId) == 0) &&
		(strcmp(AttributeName, 
			((AttributeReference*)(&Target))->AttributeName) == 0); 
    }
    else
	Results = 0;

    return Results;
}


//
// Function Name:	AttributeReference::Write()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Write this to a stream in the format described for
//			AttributeReference(istream&).
//
// Parameters:		Output - The stream
//
// Return Value:	None
//
void AttributeReference::Write(ostream &Output) const
{
    Output << ATTRIBUTE_REFERENCE;
    WriteString(CollectionName, Output);
    WriteString(DocumentId, Output);
    WriteString(AttributeName, Output);

    if(!Output)
	throw LogiconErrors::Write("AttributeReference::Write(ostream&)");
}




//
// Function Name:	AttributeReference::GetClass()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *AttributeReference::GetClass() const
{
    return ClassLabel;
}



//
// Function Name:	AttributeReference::Cleanup()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Cleanup allocated memory
//
// Parameters:		None
//
// Return Value:	None
//
void AttributeReference::Cleanup()
{
    if(CollectionName != NULL)
    	delete[] CollectionName;
    if(DocumentId != NULL)
    	delete[] DocumentId;
    if(AttributeName != NULL)
    	delete[] AttributeName;
}



//
// Function Name:	AttributeReference::CastToAttributeReference()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
AttributeReference &AttributeReference::CastToAttributeReference()
{
    return *this;
}




//
// Function Name:	AttributeReference::CastToAttributeReference()
//
// CSCI/CSC/CSU:	CMC/AttributeReference
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const AttributeReference &AttributeReference::CastToAttributeReference() const
{
    return *this;
}


