//
// Module Name:		cmcAttributeValue.cc
//
// CSCI/CSC/CSU:	CMC/AttributeValue
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Provide functions for the AttributeValue class.  AttributeValue
//	is a virtual class that serves as a parent class for the various
//	types of supported values of Attributes.
//
// Function List:
//	AttributeValue::operator!= - Test for non-equality of an AttributeValue
//	ReadAttributeValue() - Read an AttributeValue from a stream without
//		knowing going in what its specific type is
//	operator<< - Output an AttributeValue
//	AttributeValue::IsChanged() - Test if this has been recently changed
//	AttributeValue::ClearChanged() - Clear the changed indicator for this
//	AttributeValue::CastToSequenceValue() - default downcast function
//	AttributeValue::CastToStringValue() - default downcast function
//	AttributeValue::CastToObjectReferenceValue() - default downcast function
//	AttributeValue::CastToAttributeValue() - downcast function
//
// Other Modules Called:
//	StringValue - A type of AttributeValue
//	SequenceValue - A type of AttributeValue
//	ObjectReferenceValue - A type of AttributeValue
//	AttributeValueType - Enumeration of types of AttributeValues
//	Error - Used for exceptions
//
//	Note: Normally, code for a class would not know of any subclasses
//	of the class.  However, the ReadAttributeValue function needs to
//	know about all types of AttributeValues in order to be able to
//	read them from the stream.
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           11/17/95    jd_2     Initial version
//

#include <iostream.h>			// for I/O
#include <stddef.h>			// for NULL

#include "cmcAttributeValue.h"		// contains declaration of this class
#include "cmcStringValue.h"		// a subclass of this
#include "cmcSequenceValue.h"		// a subclass of this
#include "cmcObjectReferenceVal.h"	// a subclass of this
#include "cmcAttributeValueType.h"	// types of various subclasses
#include "cmcError.h"			// for exception classes

// ClassLabel identifies this class
const char *AttributeValue::ClassLabel = "AttributeValue";


//
// Function Name:	AttributeValue::operator!=
//
// CSCI/CSC/CSU:	cmc/AttributeValue
//
// Description:		Test whether an AttributeValue is not equal to 
//			another AttributeValue.
//
// Parameters:		Comparer - The AttributeValue to test the current
//				one against
//
// Return Value:	0 if they are equal
//			1 if not
//
int AttributeValue::operator!=(const AttributeValue &Comparer) const
{
    if(operator==(Comparer))
	return 0;
    else
	return 1;
}




//
// Function Name:	ReadAttributeValue()
//
// CSCI/CSC/CSU:	cmc/AttributeValue
//
// Description:		Read an AttributeValue from an input stream.  This
//			accepts any type of AttributeValue.  The format
//			from the stream is:
//
//				<the AttributeValue>
//
//			Note: This assumes that the first element of any 
//			AttributeValue is its type.
//
// Parameters:		Input - Stream to read from
//
// Return Value:	The read AttributeValue
//			NULL on error
//
AttributeValue *ReadAttributeValue(istream &Input)
{
    AttributeValueType Type;		// Type of AttributeValue read in
    AttributeValue *pValue;		// the value

    // Get the type, but do not take it permanently from Input
    streampos StartPos = Input.tellg();
    Input >> Type;
    Input.seekg(StartPos);

    // if reads failed...
    if(!Input)
	throw LogiconErrors::Read("ReadAttributeValue(istream &)");

    // do based on that type
    switch(Type)
    {
	case STRING:
	    pValue = new StringValue(Input);
	    break;
	case SEQUENCE:
	    pValue = new SequenceValue(Input);
	    break;
	case COLLECTION_REFERENCE:
	case DOCUMENT_REFERENCE:
	case ANNOTATION_REFERENCE:
	case ATTRIBUTE_REFERENCE:
	    pValue = new ObjectReferenceValue(Type, Input);
	    break;
	default:
	    throw LogiconErrors::Format("ReadAttributeValue(istream&)");
	    break;
    }

    return(pValue);
}




//
// Function Name:	AttributeValue::IsChanged()
//
// CSCI/CSC/CSU:	CMC/AttributeValue
//
// Description:		Test whether this has been changed since it was
//			constructed or ClearChanged was called.
//
// Parameters:		None
//
// Return Value:	1 if it has been changed
//			0 if not
//
int AttributeValue::IsChanged() const
{
    return 0;
}



//
// Function Name:	AttributeValue::ClearChanged()
//
// CSCI/CSC/CSU:	CMC/AttributeValue
//
// Description:		Clear the changed indicators of this and its member
//			objects.
//
// Parameters:		None
//
// Return Value:	None
//
void AttributeValue::ClearChanged()
{
}








//
// Function Name:	operator<<
//
// CSCI/CSC/CSU:	cmc/AttributeValue
//
// Description:		Output an AttributeValue to an output stream in a
//			standard C++ output style.  This uses the virtual
//			function AttributeValue::WriteToStream() to write
//			out any type of AttributeValue.
//
// Parameters:		Output - Stream to write to
//			Value - Value to write
//
// Return Value:	The stream written to
//
ostream &operator<<(ostream &Output, const AttributeValue &Value)
{
    Value.WriteToStream(Output);
    return Output;
}




//
// Function Name:	AttributeValue::CastToSequenceValue()
//
// CSCI/CSC/CSU:	cmc/AttributeValue
//
// Description:		Downcast this to a SequenceValue.  This is the
//			default behavior, which causes an error.  SequenceValue
//			replaces this to act appropriately.
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
SequenceValue &AttributeValue::CastToSequenceValue()
{
    throw LogiconErrors::InvalidType("AttributeValue::CastToSequenceValue()");
   
    // this is here to satisfy the compiler
    return *(SequenceValue*)NULL;
}




//
// Function Name:	AttributeValue::CastToSequenceValue()
//
// CSCI/CSC/CSU:	cmc/AttributeValue
//
// Description:		Downcast this to a SequenceValue when this is
//			const.  This is the default behavior, which 
//			causes an error.  SequenceValue replaces this 
//			to act appropriately.
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
const SequenceValue &AttributeValue::CastToSequenceValue() const
{
    throw LogiconErrors::InvalidType("AttributeValue::CastToSequenceValue()");
   
    // this is here to satisfy the compiler
    return *(const SequenceValue*)NULL;
}


//
// Function Name:	AttributeValue::CastToStringValue()
//
// CSCI/CSC/CSU:	cmc/AttributeValue
//
// Description:		Downcast this to a StringValue.  This is the
//			default behavior, which causes an error.  StringValue
//			replaces this to act appropriately.
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
StringValue &AttributeValue::CastToStringValue()
{
    throw LogiconErrors::InvalidType("AttributeValue::CastToStringValue()");
   
    // this is here to satisfy the compiler
    return *(StringValue*)NULL;
}




//
// Function Name:	AttributeValue::CastToStringValue()
//
// CSCI/CSC/CSU:	cmc/AttributeValue
//
// Description:		Downcast this to a StringValue when this is
//			const.  This is the default behavior, which 
//			causes an error.  StringValue replaces this 
//			to act appropriately.
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
const StringValue &AttributeValue::CastToStringValue() const
{
    throw LogiconErrors::InvalidType("AttributeValue::CastToStringValue()");
   
    // this is here to satisfy the compiler
    return *(const StringValue*)NULL;
}


//
// Function Name:	AttributeValue::CastToObjectReferenceValue()
//
// CSCI/CSC/CSU:	cmc/AttributeValue
//
// Description:		Downcast this to a ObjectReferenceValue.  This is the
//			default behavior, which causes an error.  
//			ObjectReferenceValue replaces this to act appropriately.
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
ObjectReferenceValue &AttributeValue::CastToObjectReferenceValue()
{
    throw LogiconErrors::InvalidType(
		"AttributeValue::CastToObjectReferenceValue()");
   
    // this is here to satisfy the compiler
    return *(ObjectReferenceValue*)NULL;
}




//
// Function Name:	AttributeValue::CastToObjectReferenceValue()
//
// CSCI/CSC/CSU:	cmc/AttributeValue
//
// Description:		Downcast this to a ObjectReferenceValue when this is
//			const.  This is the default behavior, which 
//			causes an error.  ObjectReferenceValue replaces this 
//			to act appropriately.
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
const ObjectReferenceValue &AttributeValue::CastToObjectReferenceValue() const
{
    throw LogiconErrors::InvalidType(
		"AttributeValue::CastToObjectReferenceValue()");
   
    // this is here to satisfy the compiler
    return *(const ObjectReferenceValue*)NULL;
}





//
// Function Name:	AttributeValue::CastToAttributeValue()
//
// CSCI/CSC/CSU:	CMC/AttributeValue
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
AttributeValue &AttributeValue::CastToAttributeValue()
{
    return *this;
}




//
// Function Name:	AttributeValue::CastToAttributeValue()
//
// CSCI/CSC/CSU:	CMC/AttributeValue
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const AttributeValue &AttributeValue::CastToAttributeValue() const
{
    return *this;
}


