//
// Module Name:		cmcAttributedObject.cc
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	This file contains the code for the AttributedObject class.  
//	AttributedObject represents objects with attributes.  It is
//	primarily meant to be inherited from.
//
// Function List:
//	AttributedObject::AttributedObject() - there are 2 constructors
//	AttributedObject::~AttributedObject() - destructor
//	AttributedObject::PutAttribute() - Add an attribute to the object
//	AttributedObject::GetAttribute() - Get an attribute from the object
//	AttributedObject::GetAttributes() - Get all Attributes of the object
//	AttributedObject::HasAttributes() - Test whether an object has a set 
//			of Attributes
//	AttributedObject::IsAttributesChanged() - Test if this has been 
//			recently changed
//	AttributedObject::ClearAttributesChanged() - Clear the changed 
//			indicator for this
//	AttributedObject::CastToAttributedObject() - Downcast function
//	AttributedObject::GetClass() - Get class label of this
//
// Other Modules Called:
//	Attribute - represents Attributes of the object
//	AttributeValue - Value of those attributes
//	Set<> - Container for the Attribute
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           11/17/95    jd_2     Initial version
//

#include <stddef.h>			// for NULL
#include <string.h>			// for string functions

#include "cmcAttributedObject.h"	// contains declaration for this class
#include "cmcError.h"			// contains exception classes


// the following is the class label for this
const char *AttributedObject::ClassLabel = "AttributedObject";

//
// Function Name:	AttributedObject::AttributedObject()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Construct an AttributedObject with no Attributes.
//
// Parameters:		None
//
// Return Value:	None
//
AttributedObject::AttributedObject() : Changed(0)
{
}



//
// Function Name:	AttributedObject::AttributedObject()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Construct an AttributedObject given a list of
//			attributes to go in it.
//
// Parameters:		BaseAttributes - Attributes to store in it
//
// Return Value:	None
//
AttributedObject::AttributedObject(const Set<Attribute> &BaseAttributes) :
	Changed(0)
{
    // copy over all attributes
    for(int i = 0; i < BaseAttributes.Length(); ++i)
	Attributes.Push(BaseAttributes.Nth(i).Copy());

    // clear the changed flag
    Attributes.ClearChanged();
}




//
// Function Name:	AttributedObject::~AttributedObject()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Destruct an AttributedObject.
//
// Parameters:		None
//
// Return Value:	None
//
AttributedObject::~AttributedObject()
{
}



//
// Function Name:	AttributedObject::PutAttribute()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Add an Attribute to an AttributedObject.
//
// Parameters:		Name - name of the Attribute
//			Value - Value of the Attribute
//
// Return Value:	None
//
void AttributedObject::PutAttribute(const char *Name, 
				    const AttributeValue &Value)
{
    int i;						// loop index
    Attribute *pAttribute = new Attribute(Name, Value);	// attribute to put

    // check allocation before anything else
    if(pAttribute == NULL)
	throw LogiconErrors::MemoryAllocation(
		"AttributedObject::PutAttribute(char*, AttributeValue&)");
    
    // walk through all attributes looking for it
    for(i = 0; 
	i < Attributes.Length() 
		&& strcmp(Attributes.Nth(i).GetName(),Name) != 0;
	++i)
    {
    }

    if(i < Attributes.Length())
    {
	// it is already present: delete it's value
	Attributes.Delete(i);
    }

    // add it in
    Attributes.Push(pAttribute);
}



//
// Function Name:	AttributedObject::GetAttribute()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Get an Attribute given its name
//
// Parameters:		Name - Name of the Attribute
//
// Return Value:	The Value of the Attribute
//			NULL if not found
//
const AttributeValue *AttributedObject::GetAttribute(const char *Name) const
{
    int i;			// loop index

    // walk through all attributes looking for it
    for(i = 0; 
	i < Attributes.Length() 
		&& strcmp(Attributes.Nth(i).GetName(), Name) != 0;
	++i)
    {
    }

    if(i < Attributes.Length())
	return &Attributes.Nth(i).GetValue();
    else
	return NULL;
}



//
// Function Name:	AttributedObject::GetAttributes()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Return the Attributes component
//
// Parameters:		None
//
// Return Value:	Attributes
//
Set<Attribute> &AttributedObject::GetAttributes(void) 
{
    return Attributes;
}



//
// Function Name:	AttributedObject::GetAttributes()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Return the Attributes component: constant version.
//
// Parameters:		None
//
// Return Value:	Attributes
//
const Set<Attribute> &AttributedObject::GetAttributes(void) const
{
    return Attributes;
}




//
// Function Name:	AttributedObject::IsAttributesChanged()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Test whether this has been changed since it was
//			constructed or ClearChanged was called.
//
// Parameters:		None
//
// Return Value:	1 if it has been changed
//			0 if not
//
int AttributedObject::IsAttributesChanged() const
{
    // check to see if the Attributes array has changed
    if(Attributes.IsChanged())
    {
	// the following casts away const-ness of Changed
	// this is as an optimization only and does not affect
	//		the conceptual const-ness of Changed
	((AttributedObject*)this)->Changed = 1;
    }

    // check on all of Attributes to see if they have changed...
    for(int i = 0; !Changed && i < Attributes.Length(); ++i)
	if(Attributes.Nth(i).IsChanged())
	{
	    // the following casts away const-ness of Changed
	    // this is as an optimization only and does not affect
	    //		the conceptual const-ness of Changed
	    ((AttributedObject*)this)->Changed = 1;
	}

    return Changed;
}




//
// Function Name:	AttributedObject::ClearAttributesChanged()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Clear the changed indicators of this and its member
//			objects.
//
// Parameters:		None
//
// Return Value:	None
//
void AttributedObject::ClearAttributesChanged()
{
    // clear the local Changed flag
    Changed = 0;

    // clear changed for all Attributes
    for(int i = 0; i < Attributes.Length(); ++i)
	Attributes.Nth(i).ClearChanged();
    Attributes.ClearChanged();
}




//
// Function Name:	AttributedObject::HasAttributes()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Test whether an AttributedObject has a set of
//			Attributes with specified values.
//
// Parameters:		Constraints - The Attributes to look for
//
// Return Value:	1 if they are all present
//			0 otherwise
//
int AttributedObject::HasAttributes(Set<Attribute> &Constraints) const
{
    int Match = 1;              // flag to test for match
 
    // do for all constraints
    for(int i = 0; i < Constraints.Length() && Match; ++i)
    {
	const AttributeValue *pValue 
			= GetAttribute(Constraints.Nth(i).GetName());
 
        // if the Attribute was not found...
        if(pValue == NULL || *pValue != Constraints.Nth(i).GetValue())
            Match = 0;
    }

    return Match;
}




//
// Function Name:	AttributedObject::CastToAttributedObject()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
AttributedObject &AttributedObject::CastToAttributedObject()
{
    return *this;
}




//
// Function Name:	AttributedObject::CastToAttributedObject()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const AttributedObject &AttributedObject::CastToAttributedObject() const
{
    return *this;
}



//
// Function Name:	AttributedObject::GetClass()
//
// CSCI/CSC/CSU:	CMC/AttributedObject
//
// Description:		Get class of this.
//
// Parameters:		None
//
// Return Value:	AttributedObject::ClassLabel
//
const char *AttributedObject::GetClass() const
{
    return ClassLabel;
}
