//
// Module Name:		cmcByteSequence.cc
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Contain the code for the ByteSequence class.  A ByteSequence
//	represents the text of a Document.
//
// Function List:
//	ByteSequence::ByteSequence() - There are 2 constructors here
//	ByteSequence::~ByteSequence() - A destructor
//	ByteSequence::Length() - Return the length of the ByteSequence
//	ByteSequence::ConvertToString() - Return a string representation of 
//			the ByteSequence
//	ByteSequence::Copy() - Produce a copy of the ByteSequence
//	ByteSequence::WriteToStream() - Write a text version of the Sequence 
//			to a stream
//	operator<< - Write a ByteSequence to a stream
//	ByteSequence::GetClass() - Get class label of this
//	ByteSequence::CastToByteSequence() - Downcast function
//
// Other Modules Called:
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           11/17/95    jd_2     Initial version
//

#include <string.h>			// basic string functions

#include "cmcByteSequence.h"		// contains declarations of this
#include "cmcError.h"			// for exception classes

// ClassLabel is the string name of this class
const char *ByteSequence::ClassLabel = "ByteSequence";


//
// Function Name:	ByteSequence::ByteSequence()
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Description:		Construct a ByteSequence given its text and
//			length.
//
// Parameters:		NewText - text of the sequence
//			NewLength - length of text.  If -1, calculate the
//				length based on the strlen() of NewText
//
// Return Value:	None
//
ByteSequence::ByteSequence(const char *NewText, int NewLength) :
	Text(NULL), TextLength(0)
{
    try
    {
    	// get the length if need be
    	if(NewLength < 0)
	    NewLength = strlen(NewText);

    	// allocate space for the string
    	if((Text = new char[NewLength + 1]) == NULL)
	    throw LogiconErrors::MemoryAllocation(
			"ByteSequence::ByteSequence(char *, int)");

    	// copy over the string
    	memcpy(Text, NewText, NewLength);
    	Text[NewLength] = '\0';

    	TextLength = NewLength;
    }
    catch(...)
    {
	if(Text != NULL)
	    delete[] Text;
	throw;
    }
}


//
// Function Name:	ByteSequence::ByteSequence()
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Description:		Construct a ByteSequence from a representation of it
//			on a stream.  The format of this is:
//
//				<length> space <text>
//
// Parameters:		Input - stream to read from
//
// Return Value:	None
//
ByteSequence::ByteSequence(istream &Input) : Text(NULL)
{
    try
    {
    	char ch;			// character to read in

    	// get text length
    	Input >> TextLength;
    	Input.get(ch);

	// check that all was read correctly
	if(!Input)
	    throw LogiconErrors::Format("ByteSequence::ByteSequence(istream&)");

    	// get text
    	Text = new char[TextLength + 1];	
    	Input.read(Text, TextLength);
    	Text[TextLength] = '\0';

	// check that all was read correctly
	if(!Input)
	    throw LogiconErrors::Format("ByteSequence::ByteSequence(istream&)");
    }
    catch(...)
    {
	if(Text != NULL)
	    delete[] Text;
	throw;
    }
}


//
// Function Name:	ByteSequence::~ByteSequence()
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Description:		Destruct a ByteSequence by freeing its text.
//
// Parameters:		None
//
// Return Value:	None
//
ByteSequence::~ByteSequence()
{
    if(Text != NULL)
    	delete[] Text;
}



//
// Function Name:	ByteSequence::Length()
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Description:		Return the TextLength component of the ByteSequence
//
// Parameters:		None
//
// Return Value:	Length of the byte sequence
//
int ByteSequence::Length() const
{
    return TextLength;
}



//
// Function Name:	ByteSequence::ConvertToString()
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Description:		Convert the ByteSequence to string by returning its
//			Text component.
//
// Parameters:		None
//
// Return Value:	Text of the ByteSequence
//
const char *ByteSequence::ConvertToString() const
{
    return Text;
}



//
// Function Name:	ByteSequence::Copy()
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Description:		Produce a copy of the ByteSequence.
//
// Parameters:		None
//
// Return Value:	A copy ByteSequence
//
ByteSequence *ByteSequence::Copy() const
{
    ByteSequence *pCopy = new ByteSequence(Text, TextLength);

    if(pCopy == NULL)
	throw LogiconErrors::MemoryAllocation("ByteSequence::Copy()");

    return pCopy;
}


//
// Function Name:	ByteSequence::WriteToStream()
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Description:		Write out a ByteSequence to a stream in the format
//			described under ByteSequence::ByteSequence(istream&).
//
// Parameters:		Output - The output stream
//
// Return Value:	None
//
void ByteSequence::WriteToStream(ostream &Output) const
{
    Output << TextLength << ' ';
    Output.write(Text, TextLength);

    if(!Output)
	throw LogiconErrors::Write("ByteSequence::WriteToStream(ostream &)");
}



//
// Function Name:	operator<<
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Description:		Write out a ByteSequence in a standard C++ output
//			function.
//
// Parameters:		Output - The output stream
//			Data - ByteSequence to write out
//
// Return Value:	The Output stream
//
ostream &operator<<(ostream &Output, const ByteSequence &Data)
{
    Data.WriteToStream(Output);
    return Output;
}




//
// Function Name:	ByteSequence::GetClass()
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *ByteSequence::GetClass() const
{
    return ClassLabel;
}



//
// Function Name:	ByteSequence::CastToByteSequence()
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
ByteSequence &ByteSequence::CastToByteSequence()
{
    return *this;
}




//
// Function Name:	ByteSequence::CastToByteSequence()
//
// CSCI/CSC/CSU:	CMC/ByteSequence
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const ByteSequence &ByteSequence::CastToByteSequence() const
{
    return *this;
}


