//
// Module Name:		cmcCBinding.cc
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	This is the C binding for the Document Manager.  It includes
//	C++ functions that are callable by C programs that will run the
//	various DocumentManager functions of the manager.
//
// Function List:
//	All of the TIPSTER DocumentManager functions.  For details on
//	these, or more details on the calling formats, etc, see the
//	Tipster Architecture Design Document.  This code is based on
//	version 1.52 of that document.
//
// Other Modules Called:
//	The following are the CMC classes invoked for this.  These are the
//	C++ classes corresponding to the Tipster Document manager classes.
//		Annotation, AnnotationReference, AnnotationSet,
//		Attribute, AttributeReference, AttributeValue,
//		AttributeValueType, AttributedObject, ByteSequence,
//		Collection, CollectionReference, Document, DocumentReference,
//		ObjectReference, ObjectReferenceValue, PersistentObject,
//		Set, Span, StringValue
//
//	In addition, the following are called used for various reasons:
//
//	DocumentManagerObject - Provides base class and virtual destructor for 
//		all Tipster classes
//	TipsterCSet - Wrapper around Sets, needed for generic operations
//	PersistentCollection - Type of Collection with persistent existance
//	VolatileCollection - A Collection that is volatile
//	
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#include <string.h>			// for string functions
#include <stddef.h>			// for NULL

#include "cmcCBinding.h"		// declaration of all these functions
#include "cmcPersistentCollection.h"	// to construct one of these
#include "cmcVolatileCollection.h"	// to construct one of these
#include "cmcSecSurrogate.h"		// to construct a set of these
#include "cmcError.h"			// for error exceptions

static void HandleError(LogiconErrors::TipsterError &);	
					// handle an error exception


extern "C" {



//
// Function Name:	tip_Free()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_Free() 
//			Tipster function.
//
// Parameters:		O - the item to free
//
// Return Value:	None
//
void tip_Free(tip_DocumentManagerObject O)
{
    try
    {
    	delete O;
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}

//
// Function Name:	tip_AddAnnotation()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_AddAnnotation() 
//			Tipster function.
//
// Parameters:		D - The Document to add to
//			A - The Annotation to add
//
// Return Value:	The ID of the new Annotation
//
tip_string tip_AddAnnotation(tip_Document D, tip_Annotation A)
{
    try
    {
    	return (tip_string)D->CastToDocument().
			AddAnnotation(A->CastToAnnotation());
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_Annotate()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_Annotate() 
//			Tipster function.
//
// Parameters:		D - The Document to Annotate
//			S - The Annotator name
//
// Return Value:	None
//
void tip_Annotate(tip_Document D, tip_string S)
{
    try
    {
    	D->CastToDocument().Annotate(S);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}

//
// Function Name:	tip_AnnotateCollection()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_AnnotateCollection() 
//			Tipster function.
//
// Parameters:		C1 - The Collection to Annotate
//			C2 - The Collection of Documents that need to be 
//				Annotated
//			S - The Annotator name
//
// Return Value:	None
//
void tip_AnnotateCollection(tip_Collection C1, tip_Collection C2,
		tip_string S)
{
    try
    {
    	C1->CastToCollection().AnnotateCollection(C2->CastToCollection(), S);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}

//
// Function Name:	tip_AnnotationsAt()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_AnnotationsAt() 
//			Tipster function.
//
// Parameters:		O - The Document or AnnotationSet to search
//			I - The location for those Annotations
//
// Return Value:	The AnnotationSet of Annotations there
//
tip_AnnotationSet tip_AnnotationsAt(tip_DocumentManagerObject O, tip_integer I)
{
    try
    {
    	if(strcmp(O->GetClass(), Document::ClassLabel) == 0)
	    return O->CastToDocument().AnnotationsAt(I);
    	else if(strcmp(O->GetClass(), AnnotationSet::ClassLabel) == 0)
	    return O->CastToAnnotationSet().AnnotationsAt(I);
    	else 
    	{
	    // handle error
	    return NULL;
    	}
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return(NULL);
    }
}

//
// Function Name:	tip_Close()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_Close() 
//			Tipster function.
//
// Parameters:		O - The object to close
//
// Return Value:	None
//
void tip_Close(tip_PersistentObject O)
{
    try
    {
    	delete O;
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}



//
// Function Name:	tip_ConvertToString()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_ConvertToString() 
//			Tipster function.
//
// Parameters:		B - The ByteSequence to convert
//
// Return Value:	The string
//
tip_string tip_ConvertToString(tip_ByteSequence B)
{
    try
    {
    	return (tip_string)B->CastToByteSequence().ConvertToString();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}



//
// Function Name:	tip_CopyBareDocument()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CopyBareDocument() 
//			Tipster function.
//
// Parameters:		C - The Collection in which to place the Document
//			D - The Document
//
// Return Value:	The new Document
//
tip_Document tip_CopyBareDocument(tip_Collection C, tip_Document D)
{
    try
    {
    	return (tip_Document)D->CastToDocument().
			CopyBare(C->CastToCollection());
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CopyDocument()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CopyDocument() 
//			Tipster function.
//
// Parameters:		C - The Collection in which the Document should occur
//			D - The Document in question
//
// Return Value:	The new Document
//
tip_Document tip_CopyDocument(tip_Collection C, tip_Document D)
{
    try
    {
    	return (tip_Document)D->CastToDocument().
			CopyDocument(C->CastToCollection());
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateAnnotation()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateAnnotation() 
//			Tipster function.
//
// Parameters:		S - the Type of the Annotation
//			SS - The Spans of the Annotation
//			AS - The Attributes of the Annotation
//
// Return Value:	The new Annotation
//
tip_Annotation tip_CreateAnnotation(tip_string S, tip_SpanSet SS,
		tip_AttributeSet AS)
{
    try
    {
    	return (tip_Annotation)
		(new Annotation(S, SS->GetSpanSet(), AS->GetAttributeSet()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateAnnotationReference()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateAnnotationReference() 
//			Tipster function.
//
// Parameters:		D - The Document in which the Annotation occurs
//			A - The Annotation in question
//
// Return Value:	The new AnnotationReference
//
tip_AnnotationReference tip_CreateAnnotationReference(tip_Document D,
		tip_Annotation A)
{
    try
    {
    	return (tip_AnnotationReference)
		(new AnnotationReference(D->CastToDocument(), 
					 A->CastToAnnotation()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateAnnotationSet()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateAnnotationSet() 
//			Tipster function.
//
// Parameters:		None
//
// Return Value:	The new AnnotationSet
//
tip_AnnotationSet tip_CreateAnnotationSet()
{
    try
    {
    	return new AnnotationSet;
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateAttribute()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateAttribute() 
//			Tipster function.
//
// Parameters:		S - The name of the Attribute
//			V - The Value of the Attribute
//
// Return Value:	The new Attribute
//
tip_Attribute tip_CreateAttribute(tip_string S, tip_AttributeValue V)
{
    try
    {
    	return new Attribute(S, V->CastToAttributeValue());
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateAttributeReference()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateAttributeReference()
//			Tipster function.
//
// Parameters:		D - The Document containing the Attribute
//			S - The Attribute name
//
// Return Value:	The new AttributeReference
//
tip_AttributeReference tip_CreateAttributeReference(tip_Document D,
	tip_string S)
{
    try
    {
    	return (tip_AttributeReference)
			(new AttributeReference(D->CastToDocument(), S));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}



//
// Function Name:	tip_CreateAttributeSet()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateAttributeSet() 
//			Tipster function.
//
// Parameters:		None
//
// Return Value:	The new AttributeSet
//
tip_AttributeSet tip_CreateAttributeSet()
{
    try
    {
    	return new TipsterCSet(new Set<Attribute>);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateAttributeValueSet()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateAttributeValueSet() 
//			Tipster function.
//
// Parameters:		None
//
// Return Value:	The new AttributeValues
//
tip_AttributeValueSet tip_CreateAttributeValueSet()
{
    try
    {
    	return new TipsterCSet(new Set<AttributeValue>);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateByteSequence()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateByteSequence() 
//			Tipster function.
//
// Parameters:		S - The String in the ByteSequence
//
// Return Value:	The new ByteSequence
//
tip_ByteSequence tip_CreateByteSequence(tip_string S)
{
    try
    {
    	return new ByteSequence(S);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateBinaryByteSequence()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateBinaryByteSequence() 
//			Tipster function.
//
// Parameters:		S - The String in the ByteSequence
//			L - The length of the string
//
// Return Value:	The new ByteSequence
//
tip_ByteSequence tip_CreateBinaryByteSequence(tip_string S, tip_integer L)
{
    try
    {
    	return new ByteSequence(S, L);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}


//
// Function Name:	tip_CreateCollection()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateCollection() 
//			Tipster function.
//
// Parameters:		S - The Name of the Collection
//			AS - The AttributeSet of the Collection
//
// Return Value:	A new Collection
//
tip_Collection tip_CreateCollection(tip_string S, tip_AttributeSet AS)
{
    try
    {
    	return (tip_Collection)(new PersistentCollection(S, 
					AS->GetAttributeSet()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}



//
// Function Name:	tip_CreateCollectionReference()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateCollectionReference()
//			Tipster function.
//
// Parameters:		C - The Collection to reference
//
// Return Value:	The new CollectionReference
//
tip_CollectionReference tip_CreateCollectionReference(tip_Collection C)
{
    try
    {
    	return (tip_CollectionReference)
		(new CollectionReference(C->CastToCollection()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateDocument()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateDocument() 
//			Tipster function.
//
// Parameters:		C - The Collection for the Document
//			S - the ExternalId of the Document
//			B - The RawData of the Document
//			AnnS - The AnnotationSet of the Document
//			AttS - The Attributes of the Document
//
// Return Value:	The Document created
//
tip_Document tip_CreateDocument(tip_Collection C, tip_string S,
	tip_ByteSequence B, tip_AnnotationSet AnnS, tip_AttributeSet AttS)
{
    try
    {
    	if(B == NULL)
    	    return (tip_Document)(new Document(C->CastToCollection(), S, NULL, 
			AnnS->CastToAnnotationSet(), AttS->GetAttributeSet()));
    	else
    	    return (tip_Document)(new Document(C->CastToCollection(), S, 
		&(B->CastToByteSequence()), AnnS->CastToAnnotationSet(), 
		AttS->GetAttributeSet()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}


//
// Function Name:	tip_CreateDetectionNeed()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateDetectionNeed()
//			Tipster function.
//
// Parameters:		C - The Collection for the Document
//			S - the ExternalId of the Document
//			B - The RawData of the Document
//			AnnS - The AnnotationSet of the Document
//			AttS - The Attributes of the Document
//
// Return Value:	The Document created
//
tip_DetectionNeed tip_CreateDetectionNeed(tip_Collection C, tip_string S,
	tip_ByteSequence B, tip_AnnotationSet AnnS, tip_AttributeSet AttS)
{
    try
    {
    	if(B == NULL)
    	    return (tip_DetectionNeed)
		(new DetectionNeed(C->CastToCollection(), S, NULL, 
			AnnS->CastToAnnotationSet(), AttS->GetAttributeSet()));
    	else
    	    return (tip_DetectionNeed)
		(new DetectionNeed(C->CastToCollection(), S, 
		   &(B->CastToByteSequence()), AnnS->CastToAnnotationSet(), 
		   AttS->GetAttributeSet()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}


//
// Function Name:	tip_CreateDetectionNeedCollection()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the 
//			tip_CreateDetectionNeedCollection() Tipster function.
//
// Parameters:		S - The Name of the DetectionNeedCollection
//			AS - The AttributeSet of the DetectionNeedCollection
//
// Return Value:	A new DetectionNeedCollection
//
tip_DetectionNeedCollection tip_CreateDetectionNeedCollection(
			tip_string S, tip_AttributeSet AS)
{
    try
    {
    	return (tip_DetectionNeedCollection)
	    (new PersistentDetectionNeedCollection(S, AS->GetAttributeSet()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}


//
// Function Name:	tip_CreateDetectionQuery()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateDetectionQuery()
//			Tipster function.
//
// Parameters:		C - The Collection for the Document
//			S - the ExternalId of the Document
//			B - The RawData of the Document
//			AnnS - The AnnotationSet of the Document
//			AttS - The Attributes of the Document
//			DN - The DetectionNeed for this
//
// Return Value:	The Document created
//
tip_DetectionNeed tip_CreateDetectionQuery(tip_Collection C, tip_string S,
	tip_ByteSequence B, tip_AnnotationSet AnnS, tip_AttributeSet AttS,
	tip_DetectionNeed DN)
{
    try
    {
    	if(B == NULL)
    	    return (tip_DetectionQuery)
		(new DetectionQuery(C->CastToCollection(), S, NULL, 
			AnnS->CastToAnnotationSet(), AttS->GetAttributeSet(),
			DN->CastToDetectionNeed()));
    	else
    	    return (tip_DetectionQuery)
		(new DetectionQuery(C->CastToCollection(), S, 
		   &(B->CastToByteSequence()), AnnS->CastToAnnotationSet(), 
		   AttS->GetAttributeSet(),
		   DN->CastToDetectionNeed()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}



//
// Function Name:	tip_GetDetectionNeed()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		C binding for DetectionQuery::GetDetectionNeed()
//
// Parameters:
//
// Return Value:
//
tip_DocumentReference tip_GetDetectionNeed(tip_DetectionQuery Query)
{
    return (tip_DocumentReference)
		&Query->CastToDetectionQuery().GetDetectionNeed();
}


//
// Function Name:	tip_SetDetectionNeed()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		C binding for DetectionQuery::SetDetectionNeed()
//
// Parameters:
//
// Return Value:
//
void tip_SetDetectionNeed(tip_DetectionQuery Query, tip_DocumentReference Ref)
{
    Query->CastToDetectionQuery().SetDetectionNeed(
				Ref->CastToDocumentReference());
}


//
// Function Name:	tip_CreateDocumentCollectionIndexSet()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the 
//			tip_CreateDocumentCollectionIndexSet() TIPSTER function.
//
// Parameters:		None
//
// Return Value:	A new DocumentCollectionIndexSet.
//
tip_DocumentCollectionIndexSet tip_CreateDocumentCollectionIndexSet()
{
    try
    {
    	return new TipsterCSet(new Set<SecSurrogate>);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}





//
// Function Name:	tip_CreateDocumentReference()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateDocumentReference() 
//			Tipster function.
//
// Parameters:		D - The Document to reference
//
// Return Value:	The new DocumentReference
//
tip_DocumentReference tip_CreateDocumentReference(tip_Document D)
{
    try
    {
    	return (tip_DocumentReference)(new DocumentReference(
						D->CastToDocument()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateDocumentReferenceFromSpecification()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the 
//			tip_CreateDocumentReferenceFromSpecification() 
//			Tipster function.
//
// Parameters:		CollectionName - Name of Collection
//			DocumentId - The Document's ID
//
// Return Value:	New DocumentReference
//
tip_DocumentReference tip_CreateDocumentReferenceFromSpecification(
	tip_string CollectionName, tip_string DocumentId)
{
    try
    {
    	return (tip_DocumentReference)
		(new DocumentReference(CollectionName, DocumentId));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}



//
// Function Name:	tip_CreateQueryCollectionIndexSet()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the 
//			tip_CreateQueryCollectionIndexSet() TIPSTER function.
//
// Parameters:		None
//
// Return Value:	A new QueryCollectionIndexSet.
//
tip_QueryCollectionIndexSet tip_CreateQueryCollectionIndexSet()
{
    try
    {
    	return new TipsterCSet(new Set<SecSurrogate>);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}




//
// Function Name:	tip_CreateSpan()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateSpan() 
//			Tipster function.
//
// Parameters:		I1 - The Span's start
//			I2 - The Span's end
//
// Return Value:	The new span
//
tip_Span tip_CreateSpan(tip_integer I1, tip_integer I2)
{
    try
    {
    	return new Span(I1, I2);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateSpanSet()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateSpanSet() 
//			Tipster function.
//
// Parameters:		None
//
// Return Value:	The Set
//
tip_SpanSet tip_CreateSpanSet()
{
    try
    {
    	return new TipsterCSet(new Set<Span>);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateVolatileCollection()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateVolatileCollection() 
//			Tipster function.
//
// Parameters:		A - Attributes on the Collection
//
// Return Value:	The Collection
//
tip_Collection tip_CreateVolatileCollection(tip_AttributeSet A)
{
    try
    {
    	return (tip_Collection)(new VolatileCollection(A->GetAttributeSet()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_CreateVolatileDetectionNeedCollection()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the 
//			tip_CreateVolatileDetectionNeedCollection() Tipster 
//			function.
//
// Parameters:		A - Attributes on the DetectionNeedCollection
//
// Return Value:	The DetectionNeedCollection
//
tip_DetectionNeedCollection 
		tip_CreateVolatileDetectionNeedCollection(tip_AttributeSet A)
{
    try
    {
    	return (tip_DetectionNeedCollection)
		(new VolatileDetectionNeedCollection(A->GetAttributeSet()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}



//
// Function Name:	tip_CreatestringSet()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreatestringSet() 
//			Tipster function.
//
// Parameters:		None
//
// Return Value:	The set of strings
//
tip_stringSet tip_CreatestringSet()
{
    try
    {
    	return new TipsterCSet(new Set<char*>);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_DeleteAnnotations()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_DeleteAnnotations() 
//			Tipster function.
//
// Parameters:		O - The thing to delete Annotations from
//			S - The type of Annotations to delete
//			AS - The Attributes that must be on deleted Annotations
//
// Return Value:	None
//
void tip_DeleteAnnotations(tip_DocumentManagerObject O, tip_string S, 
		tip_AttributeSet AS)
{
    try
    {
    	if(strcmp(O->GetClass(), Document::ClassLabel) == 0)
	    O->CastToDocument().DeleteAnnotations(S, AS->GetAttributeSet());
    	else if(strcmp(O->GetClass(), AnnotationSet::ClassLabel) == 0)
	    O->CastToAnnotationSet().DeleteAnnotations(S, 
						AS->GetAttributeSet());
    	else
    	{
	    // handle error
    	}
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}

//
// Function Name:	tip_DestroyCollection()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_DestroyCollection() 
//			Tipster function.
//
// Parameters:		S - The Collection to destroy
//
// Return Value:	None
//
void tip_DestroyCollection(tip_string S)
{
    try
    {
    	PersistentCollection::Destroy(S);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}



//
// Function Name:	tip_FirstDocument()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_FirstDocument() 
//			Tipster function.
//
// Parameters:		C - The Collection to get from
//
// Return Value:	The Document
//
tip_Document tip_FirstDocument(tip_Collection C)
{
    try
    {
    	return (tip_Document)C->CastToCollection().FirstDocument();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetAnnotation()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetAnnotation() 
//			Tipster function.
//
// Parameters:		O - The Document or AnnotationSet to get it from
//			S - The ID of the Annotation to get
//
// Return Value:	The Annotation
//
tip_Annotation tip_GetAnnotation(tip_DocumentManagerObject O, tip_string S)
{
    try
    {
    	if(strcmp(O->GetClass(), Document::ClassLabel) == 0)
	    return (tip_Annotation)&O->CastToDocument().GetAnnotation(S);
	else if(strcmp(O->GetClass(), AnnotationSet::ClassLabel) == 0)
	    return (tip_Annotation)&O->CastToAnnotationSet().GetAnnotation(S);
    	else 
    	{
	    // handle error
	    return NULL;
    	}
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetAnnotationId()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetAnnotationId() 
//			Tipster function.
//
// Parameters:		R - The reference to get it from
//
// Return Value:	The Annotation's ID
//
tip_string tip_GetAnnotationId(tip_AnnotationReference R)
{
    try
    {
    	return (tip_string)R->CastToAnnotationReference().GetAnnotationId();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetAnnotations()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetAnnotations() 
//			Tipster function.
//
// Parameters:		D - The Document to get the Set from
//
// Return Value:	The AnnotationSet
//
tip_AnnotationSet tip_GetAnnotations(tip_Document D)
{
    try
    {
    	return &D->CastToDocument().GetAnnotations();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetAttribute()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetAttribute() 
//			Tipster function.
//
// Parameters:		O - The object to get value from
//			S - The Attribute to get
//
// Return Value:	The Attribute value
//
tip_AttributeValue tip_GetAttribute(tip_AttributedObject O, tip_string S)
{
    try
    {
    	return (tip_AttributeValue)O->CastToAttributedObject().GetAttribute(S);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetAttributeName()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetAttributeName() 
//			Tipster function.
//
// Parameters:		R - The thing to get name from
//
// Return Value:	The Attribute Name
//
tip_string tip_GetAttributeName(tip_AttributeReference R)
{
    try
    {
    	return (tip_string)R->CastToAttributeReference().GetAttributeName();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetAttributes()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetAttributes() 
//			Tipster function.
//
// Parameters:		O - The Object with the Attributes
//
// Return Value:	The set of Attributes
//
tip_AttributeSet tip_GetAttributes(tip_AttributedObject O)
{
    try
    {
    	return new TipsterCSet(O->CastToAttributedObject().GetAttributes());
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetByExternalId()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetByExternalId() 
//			Tipster function.
//
// Parameters:		C - The Collection with the Document
//			S - The ExternalId
//
// Return Value:	The Document
//
tip_Document tip_GetByExternalId(tip_Collection C, tip_string S)
{
    try
    {
    	return (tip_Document)C->CastToCollection().GetByExternalId(S);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetCollectionName()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetCollectionName() 
//			Tipster function.
//
// Parameters:		O - The Object to get name from
//
// Return Value:	The CollectionName
//
tip_string tip_GetCollectionName(tip_DocumentManagerObject O)
{
    try
    {
    	if(strcmp(O->GetClass(), AnnotationReference::ClassLabel) == 0)
	    return (tip_string)O->CastToAnnotationReference().
							GetCollectionName();
    	else if(strcmp(O->GetClass(), AttributeReference::ClassLabel) == 0)
	    return (tip_string)O->CastToAttributeReference().
							GetCollectionName();
    	else if(strcmp(O->GetClass(), DocumentReference::ClassLabel) == 0)
	    return (tip_string)O->CastToDocumentReference().GetCollectionName();
    	else if(strcmp(O->GetClass(), CollectionReference::ClassLabel) == 0)
	    return (tip_string)O->CastToCollectionReference().
							GetCollectionName();
    	else 
    	{
	    // handle error
	    return NULL;
    	}
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}


//
// Function Name:	tip_GetDocument()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetDocument() 
//			Tipster function.
//
// Parameters:		C - The Collection to get from
//			S - The ID of the Document
//
// Return Value:	The Document
//
tip_Document tip_GetDocument(tip_Collection C, tip_string S)
{
    try
    {
	return (tip_Document)C->CastToCollection().GetDocument(S);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}


//
// Function Name:	tip_GetDocumentId()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetDocumentId() 
//			Tipster function.
//
// Parameters:		O - The Object to get it from
//
// Return Value:	The Document ID
//
tip_string tip_GetDocumentId(tip_DocumentManagerObject O)
{
    try
    {
    	if(strcmp(O->GetClass(), DocumentReference::ClassLabel) == 0)
	    return (tip_string)O->CastToDocumentReference().GetDocumentId();
    	else if(strcmp(O->GetClass(), AnnotationReference::ClassLabel) == 0)
	    return (tip_string)O->CastToAnnotationReference().GetDocumentId();
	else if(strcmp(O->GetClass(), AttributeReference::ClassLabel) == 0)
	    return (tip_string)O->CastToAttributeReference().GetDocumentId();
	else 
    	{
	    // handle error
	    return NULL;
    	}
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}


//
// Function Name:	tip_GetEnd()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetEnd() 
//			Tipster function.
//
// Parameters:		S - The Span to get it from
//
// Return Value:	The End
//
tip_integer tip_GetEnd(tip_Span S)
{
    try
    {
    	return S->CastToSpan().GetEnd();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}



//
// Function Name:	tip_GetExternalId()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetExternalId() 
//			Tipster function.
//
// Parameters:		D - The Document to get it from
//
// Return Value:	The ExternalId
//
tip_string tip_GetExternalId(tip_Document D)
{
    try
    {
    	return (tip_string)D->CastToDocument().GetExternalId();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}



//
// Function Name:	tip_GetId()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetId() 
//			Tipster function.
//
// Parameters:		O - The Object to get the ID from
//
// Return Value:	The ID
//
tip_string tip_GetId(tip_DocumentManagerObject O)
{
    try
    {
    	if(strcmp(O->GetClass(), Document::ClassLabel) == 0)
	    return (tip_string)O->CastToDocument().GetId();
	else if(strcmp(O->GetClass(), Annotation::ClassLabel) == 0)
	    return (tip_string)O->CastToAnnotation().GetId();
	else 
	{
	    // handle error
	    cout << "Type not found: is " << O->GetClass() << "\n";
	    return NULL;
    	}
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetName()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetName() 
//			Tipster function.
//
// Parameters:		O - The Object with the name
//
// Return Value:	The Name
//
tip_string tip_GetName(tip_PersistentObject O)
{
    try
    {
	if(strcmp(O->GetClass(), Attribute::ClassLabel) == 0)
	    return (tip_string)O->CastToAttribute().GetName();
	else
    	    return (tip_string)O->CastToPersistentObject().GetName();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetOwner()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetOwner() 
//			Tipster function.
//
// Parameters:		C - The Collection to get it from
//
// Return Value:	The owner
//
tip_string tip_GetOwner(tip_Collection C)
{
    try
    {
	return (tip_string)C->CastToCollection().GetOwner();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetParent()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetParent() 
//			Tipster function.
//
// Parameters:		D - The Document to get it from
//
// Return Value:	The parent
//
tip_Collection tip_GetParent(tip_Document D)
{
    try
    {
	return (tip_Collection)(&D->CastToDocument().GetParent());
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetRawData()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetRawData() 
//			Tipster function.
//
// Parameters:		D - The Document to get it from
//
// Return Value:	The RawData component
//
tip_ByteSequence tip_GetRawData(tip_Document D)
{
    try
    {
	return (tip_ByteSequence)D->CastToDocument().GetRawData();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetSpans()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetSpans() 
//			Tipster function.
//
// Parameters:		A - Thing to get spans from
//
// Return Value:	The spans
//
tip_SpanSet tip_GetSpans(tip_Annotation A)
{
    try
    {
	return new TipsterCSet(A->CastToAnnotation().GetSpans());
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetStart()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetStart() 
//			Tipster function.
//
// Parameters:		S - The Span in which the Start is
//
// Return Value:	The start offset
//
tip_integer tip_GetStart(tip_Span S)
{
    try
    {
	return S->CastToSpan().GetStart();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return -1;
    }
}

//
// Function Name:	tip_GetType()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetType() 
//			Tipster function.
//
// Parameters:		A - Annotation to get Type of
//
// Return Value:	The Type value
//
tip_string tip_GetType(tip_Annotation A)
{
    try
    {
	return (tip_string)A->CastToAnnotation().GetType();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_GetValue()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetValue() 
//			Tipster function.
//
// Parameters:		O - Item to get Value from
//
// Return Value:	Value gotten
//
void *tip_GetValue(tip_DocumentManagerObject O)
{
    try
    {
	if(strcmp(O->GetClass(), StringValue::ClassLabel) == 0)
	    return (void *)O->CastToStringValue().GetValue();
	else if(strcmp(O->GetClass(), SequenceValue::ClassLabel) == 0)
	    return (void *)new TipsterCSet(O->CastToSequenceValue().GetValue());
	else if(strcmp(O->GetClass(), ObjectReferenceValue::ClassLabel) == 0)
	    return (void *)&O->CastToObjectReferenceValue().GetValue();
    	else if(strcmp(O->GetClass(), Attribute::ClassLabel) == 0)
	    return (void *)&O->CastToAttribute().GetValue();
    	else 
    	{
	    // handle error
	    return NULL;
    	}
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_Length()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_Length() 
//			Tipster function.
//
// Parameters:		O - Thing to get length of
//
// Return Value:	Length of thing
//
tip_integer tip_Length(tip_DocumentManagerObject O)
{
    try
    {
	if(strcmp(O->GetClass(), Collection::ClassLabel) == 0)
	    return O->CastToCollection().Length();
	else if(strcmp(O->GetClass(), TipsterCSet::ClassLabel) == 0)
	    return O->CastToTipsterCSet().Length();
	else if(strcmp(O->GetClass(), AnnotationSet::ClassLabel) == 0)
	    return O->CastToAnnotationSet().Length();
	else if(strcmp(O->GetClass(), ByteSequence::ClassLabel) == 0)
	    return O->CastToByteSequence().Length();
	else
	    return -1;
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return -1;
    }
}

//
// Function Name:	tip_MergeAnnotations()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_MergeAnnotations() 
//			Tipster function.
//
// Parameters:		S1, S2 - Sets to merge
//
// Return Value:	AnnotationSet of merged stuff
//
tip_AnnotationSet tip_MergeAnnotations(tip_AnnotationSet S1, 
			tip_AnnotationSet S2)
{
    try
    {
    	return S1->CastToAnnotationSet().MergeAnnotations(
						S2->CastToAnnotationSet());
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_NextAnnotations()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_NextAnnotations() 
//			Tipster function.
//
// Parameters:		O - Document or AnnotationSet to get from
//			I - Location to get next from
//
// Return Value:	Annotations that are next
//
tip_AnnotationSet tip_NextAnnotations(tip_DocumentManagerObject O, 
			tip_integer I)
{
    try
    {
	if(strcmp(O->GetClass(), Document::ClassLabel) == 0)
	    return O->CastToDocument().NextAnnotations(I);
	else if(strcmp(O->GetClass(), AnnotationSet::ClassLabel) == 0)
	    return O->CastToAnnotationSet().NextAnnotations(I);
	else 
    	{
	    // Handle error
	    return NULL;
    	}
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_NextDocument()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_NextDocument() 
//			Tipster function.
//
// Parameters:		C - Collection to get from
//
// Return Value:	Document gotten
//
tip_Document tip_NextDocument(tip_Collection C)
{
    try
    {
	return (tip_Document)C->CastToCollection().NextDocument();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_Nth()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_Nth() 
//			Tipster function.
//
// Parameters:		S - Set to get item from
//			I - N of item to get
//
// Return Value:	Nth item extracted
//
void *tip_Nth(tip_DocumentManagerObject O, tip_integer I)
{
    try
    {
	if(strcmp(O->GetClass(), TipsterCSet::ClassLabel) == 0)
	    return O->CastToTipsterCSet().Nth(I);
	else if(strcmp(O->GetClass(), AnnotationSet::ClassLabel) == 0)
	    return (void*)&O->CastToAnnotationSet().Nth(I);
	else
	    return NULL;
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
} 

//
// Function Name:	tip_OpenCollection()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_OpenCollection() 
//			Tipster function.
//
// Parameters:		S - Name of Collection
//
// Return Value:	Opened Collection
//
tip_Collection tip_OpenCollection(tip_string S)
{
    try
    {
	return (tip_Collection)(new PersistentCollection(S));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_Pop()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_Pop() 
//			Tipster function.
//
// Parameters:		S - Set to pop from
//
// Return Value:	Item popped out
//
void *tip_Pop(tip_Set S)
{
    try
    {
	return S->CastToTipsterCSet().Pop();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_Push()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_Push() 
//			Tipster function.
//
// Parameters:		S - Set to add to
//			pData - Thing to add
//
// Return Value:	None
//
void tip_Push(tip_Set S, void *pData)
{
    try
    {
	S->CastToTipsterCSet().Push(pData);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}

//
// Function Name:	tip_PutAttribute()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_PutAttribute() 
//			Tipster function.
//
// Parameters:		O - Thing to add Attribute to
//			S - Name of the Attribute
//			V - Value of the Attribute
//
// Return Value:	None
//
void tip_PutAttribute(tip_AttributedObject O, tip_string S, 
			tip_AttributeValue V)
{
    try
    {
	O->CastToAttributedObject().PutAttribute(S, V->CastToAttributeValue());
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}

//
// Function Name:	tip_ReadSGML()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_ReadSGML() 
//			Tipster function.
//
// Parameters:		S1 - SGML Document
//			C - Collection to put it into
//			S2 - Its external ID
//
// Return Value:	Document read in
//
tip_Document tip_ReadSGML(tip_string S1, tip_Collection C, tip_string S2)
{
    try
    {
	return (tip_Document)(new Document(S1, C->CastToCollection(), S2));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}



//
// Function Name:	tip_RemoveAnnotation()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_RemoveAnnotation() 
//			Tipster function.
//
// Parameters:		O - Object to remove Annotation from
//			S - ID of Annotation to remove
//
// Return Value:	None
//
void tip_RemoveAnnotation(tip_DocumentManagerObject O, tip_string S)
{
    try
    {
    	if(strcmp(O->GetClass(), Document::ClassLabel) == 0)
	    O->CastToDocument().RemoveAnnotation(S);
    	else if(strcmp(O->GetClass(), AnnotationSet::ClassLabel) == 0)
	    O->CastToAnnotationSet().RemoveAnnotation(S);
    	else
    	{
	    // handle error
    	}
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}

//
// Function Name:	tip_RemoveDocument()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_RemoveDocument() 
//			Tipster function.
//
// Parameters:		C - Collection to remove from
//			S - ID of Document to remove
//
// Return Value:	None
//
void tip_RemoveDocument(tip_Collection C, tip_string S)
{
    try
    {
    	C->CastToCollection().RemoveDocument(S);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}

//
// Function Name:	tip_SelectAnnotations()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_SelectAnnotations() 
//			Tipster function.
//
// Parameters:		O - The Object to select from
//			S - The type of Annotations to get
//			AS - Attributes that selected Annotations should have
//
// Return Value:	The selected Annotations
//
tip_AnnotationSet tip_SelectAnnotations(tip_DocumentManagerObject O, 
		tip_string S, tip_AttributeSet AS)
{
    try
    {
    	if(strcmp(O->GetClass(), Document::ClassLabel) == 0)
	    return O->CastToDocument().SelectAnnotations(S, 
						AS->GetAttributeSet());
    	else if(strcmp(O->GetClass(), AnnotationSet::ClassLabel) == 0)
	    return O->CastToAnnotationSet().SelectAnnotations(S, 
						AS->GetAttributeSet());
    	else 
    	{
	    // handle error
	    return NULL;
    	}
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}

//
// Function Name:	tip_SetExternalId()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_SetExternalId() 
//			Tipster function.
//
// Parameters:		D - The Document to set ID of
//			S - The ID to set
//
// Return Value:	None
//
void tip_SetExternalId(tip_Document D, tip_string S)
{
    try
    {
    	D->CastToDocument().SetExternalId(S);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}

//
// Function Name:	tip_SetOwner()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_SetOwner() 
//			Tipster function.
//
// Parameters:		C - The Collection to set owner of
//			O - The owner to set
//
// Return Value:	None
//
void tip_SetOwner(tip_Collection C, tip_string O)
{
    try
    {
	C->CastToCollection().SetOwner(O);
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}


//
// Function Name:	tip_Sync()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_Sync() 
//			Tipster function.
//
// Parameters:		O - The object to sync
//
// Return Value:	None
//
void tip_Sync(tip_PersistentObject O)
{
    try
    {
	if(strcmp(O->GetClass(), Document::ClassLabel) == 0)
	    O->CastToDocument().Sync();
	else 
    	    O->CastToPersistentObject().Sync();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
    }
}



//
// Function Name:	tip_TypeOf()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_TypeOf() 
//			Tipster function.
//
// Parameters:		V - The AttributeValue to get type of
//
// Return Value:	Type of the AttributeValue
//
tip_AttributeValueType tip_TypeOf(tip_AttributeValue V)
{
    try
    {
	return V->CastToAttributeValue().TypeOf();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return STRING;
    }
}



//
// Function Name:	tip_WriteSGML()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_WriteSGML() 
//			Tipster function.
//
// Parameters:		D - The Document to write
//			A - The Annotations to put in it in SGML form
//			S - The order of Annotations to use
//
// Return Value:	the SGML version of the Document
//
tip_string tip_WriteSGML(tip_Document D, tip_AnnotationSet A,
		tip_stringSet S)
{
    try
    {
    	return D->CastToDocument().WriteSGML(A->CastToAnnotationSet(), 
					 S->GetStringSet());
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}




//
// Function Name:	tip_GetClass()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_GetClass()
//			function.
//
// Parameters:		O - Object to get class of
//
// Return Value:	the class label of O
//
tip_string tip_GetClass(tip_DocumentManagerObject O)
{
    try
    {
    	return (tip_string)O->GetClass();
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}



//
// Function Name:	tip_CreateStringValue()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateStringValue()
//			function.
//
// Parameters:		O - Object to get class of
//
// Return Value:	the class label of O
//
tip_AttributeValue tip_CreateStringValue(tip_string S)
{
    try
    {
    	return (tip_AttributeValue)(new StringValue(S));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}


//
// Function Name:	tip_CreateSequenceValue()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the tip_CreateSequenceValue()
//			function.
//
// Parameters:		O - Object to get class of
//
// Return Value:	the class label of O
//
tip_AttributeValue tip_CreateSequenceValue(tip_AttributeValueSet S)
{
    try
    {
    	return (tip_AttributeValue)
			(new SequenceValue(S->GetAttributeValueSet()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}


//
// Function Name:	tip_CreateObjectReferenceValue()
//
// CSCI/CSC/CSU:	CMC/CBinding
//
// Description:		Function binding for the 
//			tip_CreateObjectReferenceValue() function.
//
// Parameters:		O - Object to get class of
//
// Return Value:	the class label of O
//
tip_AttributeValue tip_CreateObjectReferenceValue(tip_ObjectReference O)
{
    try
    {
    	return (tip_AttributeValue)
		(new ObjectReferenceValue(O->CastToObjectReference()));
    }
    catch(LogiconErrors::TipsterError &Error)
    {
	HandleError(Error);
	return NULL;
    }
}




}



// grish
void HandleError(LogiconErrors::TipsterError &Error)
{
    cout << "Got error " << Error.GetLabel() << " found in " 
			<< Error.GetLocation() << "\n";
}
