//
// Module Name:		cmcCollection.cc
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	This contains the member functions for Collection.  A Collection keeps
//	a set of Documents.  Collection is an abstract class identifying the
//	operations of a Collection with some defaults.
//
// Function List:
//	Collection::Collection() - This has 3 constructors
//	Collection::~Collection() - Destructor
//	Collection::SetOwner() - Set the owner of the Collection
//	Collection::GetOwner() - Get the owner of the Collection
//	Collection::GetClass() - Get the class label for this
//	Collection::CastToCollection() - Downcast function for this
//	Collection::CastToAttributedObject() - Another downcast function 
//
// Other Modules Called:
//	Set - Collections use Sets for Attributes
//	Attribute - Collections are AttributedObjects and thus have Attributes
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#include <string.h>			// for basic string functions

#include "cmcCollection.h"		// declaration of this class
#include "cmcSet.h"			// sets of stuff are in the Collection
#include "cmcAttribute.h"		// to get Attributes of this
#include "cmcError.h"			// for exception classes

// ClassLabel is the string name of this class
const char *Collection::ClassLabel = "Collection";


//
// Function Name:	Collection::Collection()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Construct a Collection given its Attributes and name.
//
// Parameters:		NewName - The Collection's name
//			NewAttributes - The Collection's Attributes
//
// Return Value:	None
//
Collection::Collection(const char *NewName, const Set<Attribute> &NewAttributes)
		: PersistentObject(NewName),
	  	  AttributedObject(NewAttributes),
		  Owner(NULL)
{
}


//
// Function Name:	Collection::Collection()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Construct a Collection given its name.
//
// Parameters:		NewName - The Collection's name
//
// Return Value:	None
//
Collection::Collection(const char *NewName) : PersistentObject(NewName),
		  Owner(NULL)
{
}



//
// Function Name:	Collection::Collection()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Construct a Collection given its Attributes and name.
//
// Parameters:		NewAttributes - The Collection's Attributes
//
// Return Value:	None
//
Collection::Collection(const Set<Attribute> &NewAttributes) :
	PersistentObject(),
	AttributedObject(NewAttributes),
	Owner(NULL)
{
}


//
// Function Name:	Collection::Collection()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Construct a Collection given nothing.
//
// Parameters:		None
//
// Return Value:	None
//
Collection::Collection(void) : 
	PersistentObject(NULL), 
	AttributedObject(), 
	Owner(NULL)
{
}



//
// Function Name:	Collection::~Collection()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Destruct a Collection by deleting its owner name.
//
// Parameters:		None
//
// Return Value:	None
//
Collection::~Collection(void)
{
    if(Owner != NULL)
	delete[] Owner;
}


//
// Function Name:	Collection::SetOwner()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Set the Owner component of the Collection.
//
// Parameters:		NewOwner - Owner to set
//
// Return Value:	None
//
void Collection::SetOwner(const char *NewOwner)
{
    // delete old owner
    if(Owner != NULL)
    {
	delete[] Owner;
	Owner = NULL;
    }

    // allocate and copy in owner
    if((Owner = new char[strlen(NewOwner) + 1]) == NULL)
	throw LogiconErrors::MemoryAllocation("Collection::SetOwner(char *)");

    strcpy(Owner, NewOwner);
}



//
// Function Name:	Collection::GetOwner()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Get the owner of the Collection
//
// Parameters:		None
//
// Return Value:	The Owner component
//
const char *Collection::GetOwner(void) const
{
    return Owner;
}





//
// Function Name:	Collection::GetClass()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *Collection::GetClass() const
{
    return ClassLabel;
}



//
// Function Name:	Collection::CastToCollection()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
Collection &Collection::CastToCollection()
{
    return *this;
}




//
// Function Name:	Collection::CastToCollection()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const Collection &Collection::CastToCollection() const
{
    return *this;
}



//
// Function Name:	Collection::CastToAttributedObject()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Downcast this to the current type.  This is needed
//			because Collection multiply-inherits from both
//			AttributedObject and PersistentObject, and the
//			downcast from PersistentObject overrides the
//			downcast from AttributedObject.
//
// Parameters:		None
//
// Return Value:	*this
//
AttributedObject &Collection::CastToAttributedObject()
{
    return *this;
}




//
// Function Name:	Collection::CastToAttributedObject()
//
// CSCI/CSC/CSU:	CMC/Collection
//
// Description:		Downcast this to the current type.  This is needed
//			because Collection multiply-inherits from both
//			AttributedObject and PersistentObject, and the
//			downcast from PersistentObject overrides the
//			downcast from AttributedObject.
//
// Parameters:		None
//
// Return Value:	*this
//
const AttributedObject &Collection::CastToAttributedObject() const
{
    return *this;
}
