//
// Module Name:		cmcCollectionReference.cc
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for CollectionReference.  A CollectionReference references
//	a Collection by keeping its name.
//
// Function List:
//	CollectionReference::CollectionReference() - This has 3 constructors
//	CollectionReference::~CollectionReference() - A destructor
//	CollectionReference::GetCollectionName() - Get name of the Collection
//	CollectionReference::TypeOf() - Get type of ObjectReference
//	CollectionReference::Copy() - Produce a copy of this
//	CollectionReference::operator==() - Test equality against another 
//				ObjectReference
//	CollectionReference::Write() - Write this to a stream
//	CollectionReference::GetClass() - Get the class label of this
//	CollectionReference::CastToCollectionReference() - Downcast function
//
// Other Modules Called:
//	AttributeValueType - Contains types of ObjectReferences
//	Collection - To use in the constructor
//	Misc - To read and write strings
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#include <iostream.h>			// for I/O
#include <string.h>			// for basic string operations

#include "cmcCollectionReference.h"	// for declaration of this class
#include "cmcAttributeValueType.h"	// for TypeOf() return value
#include "cmcCollection.h"		// for a constructor
#include "cmcMisc.h"			// for string I/O
#include "cmcError.h"			// for exception classes

// ClassLabel is the string name of this class
const char *CollectionReference::ClassLabel = "CollectionReference";


//
// Function Name:	CollectionReference::CollectionReference()
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Construct a Collection reference given the
//			referenced Collection.
//
// Parameters:		Collection - The Collection in question
//
// Return Value:	None
//
CollectionReference::CollectionReference(Collection &Collection)
{
    if((CollectionName = new char[strlen(Collection.GetName()) + 1]) == NULL)
	throw LogiconErrors::MemoryAllocation(
		"CollectionReference::CollectionReference(Collection&)");
    strcpy(CollectionName, Collection.GetName());
}



//
// Function Name:	CollectionReference::CollectionReference()
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Construct a Collection reference given the name to
//			go in it.
//
// Parameters:		NewCollectionName - The name in question
//
// Return Value:	None
//
CollectionReference::CollectionReference(const char *NewCollectionName)
{
    if((CollectionName = new char[strlen(NewCollectionName) + 1]) == NULL)
	throw LogiconErrors::MemoryAllocation(
		"CollectionReference::CollectionReference(char*)");
    strcpy(CollectionName, NewCollectionName);
}



//
// Function Name:	CollectionReference::CollectionReference()
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Construct a CollectionReference given an input
//			stream to it.  The format in the stream is:
//
//				COLLECTION_REFERENCE (an AttributeValueType)
//				CollectionName (in ReadString() format)
//
// Parameters:		Input - the stream in question
//
// Return Value:	None
//
CollectionReference::CollectionReference(istream &Input) : CollectionName(NULL)
{
    AttributeValueType Filler;		// to hold throw-away data

    Input >> Filler;

    if(!Input)
	throw LogiconErrors::Read(
		"CollectionReference::CollectionReference(istream&)");

    CollectionName = ReadString(Input);
}


//
// Function Name:	CollectionReference::~CollectionReference()
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Destructor: destroy the contained name.
//
// Parameters:		None
//
// Return Value:	None
//
CollectionReference::~CollectionReference()
{
    if(CollectionName != NULL)
    	delete[] CollectionName;
}



//
// Function Name:	CollectionReference::GetCollectionName()
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Get the CollectionName component.
//
// Parameters:		None
//
// Return Value:	The CollectionName.
//
const char *CollectionReference::GetCollectionName() const
{
    return CollectionName;
}



//
// Function Name:	CollectionReference::TypeOf()
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Get the type of ObjectReference that this is.
//
// Parameters:		None
//
// Return Value:	COLLECTION_REFERENCE
//
AttributeValueType CollectionReference::TypeOf() const
{
    return COLLECTION_REFERENCE;
}



//
// Function Name:	CollectionReference::Copy()
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Produce a copy of this CollectionReference.
//
// Parameters:		None
//
// Return Value:	The copy.
//
ObjectReference *CollectionReference::Copy() const
{
    CollectionReference *pCopy = new CollectionReference(CollectionName);

    if(pCopy == NULL)
	throw LogiconErrors::MemoryAllocation("CollectionReference::Copy()");

    return pCopy;
}



//
// Function Name:	CollectionReference::operator==
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Test whether this is equal to another ObjectReference.
//
// Parameters:		Target - the ObjectReference to test against
//
// Return Value:	1 if equal
//			0 if not
//
int CollectionReference::operator==(const ObjectReference &Target) const
{
    int Results;		// results of this

    // if is of the correct type...
    if(Target.TypeOf() == COLLECTION_REFERENCE)
    {
	Results = (strcmp(CollectionName,
		((CollectionReference *)(&Target))->CollectionName) == 0);
    }
    else
	Results = 0;

    return Results;
}



//
// Function Name:	CollectionReference::Write()
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Write out a CollectionReference to a stream in the
//			format described under CollectionReference(istream&).
//
// Parameters:		Output - Stream to write to
//
// Return Value:	None
//
void CollectionReference::Write(ostream &Output) const
{
    Output << COLLECTION_REFERENCE;

    if(!Output)
	throw LogiconErrors::Write("CollectionReference::Write(ostream&)");

    WriteString(CollectionName, Output);
}




//
// Function Name:	CollectionReference::GetClass()
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *CollectionReference::GetClass() const
{
    return ClassLabel;
}



//
// Function Name:	CollectionReference::CastToCollectionReference()
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
CollectionReference &CollectionReference::CastToCollectionReference()
{
    return *this;
}




//
// Function Name:	CollectionReference::CastToCollectionReference()
//
// CSCI/CSC/CSU:	CMC/CollectionReference
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const CollectionReference &CollectionReference::CastToCollectionReference() 
			const
{
    return *this;
}


