//
//
// Module Name:         cmcDetectionQuery.cc - The PRIDES representation of a
//                             Tipster Detection Query
//
// CSCI/CSC/CSU:        CMC/DetectionQuery
//
// Author:		Kenneth Kolda
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:  DetectionQuery is a derived class from the CMC class
//               Document which holds a system-dependent but collection-
//               independent query.
//
// Function List:
//   DetectionQuery(Collection&, Document&, DetectionNeed&) - Constructor
//         from a Document
//   DetectionQuery::DetectionQuery(Collection&, char*, ...) - Constructor from
//	   elements
//   DetectionQuery::DetectionQuery(DetectionQuery&) - Copy constructor
//   ~DetectionQuery() - Destructor
//   DocumentReference* GetSourceNeed() - Returns a reference to the Source
//         Detection Need
// + RetrievalQuery* FormRetrievalQuery(DocumentCollectionIndex*) -
//         Forms a RetrievalQuery object based on given DCI
// + RoutingQuery* FormRoutingQuery(DocumentCollectionIndex*) - Forms a
//         RoutingQuery (Profile) object based on given DCI
//   DetectionQuery::CopyDocument(Collection&) - Copy to another
//   DetectionQuery::CopyBare(Collection&) - Copy to another without 
//	   Annotations or Attributes
//   DetectionQuery &CastToDetectionQuery() - Downcast function, in 2 flavors
//
// + = caller responsible for destructing return value
//
// Other Modules Called:
//
//
///////////////////////////////////////////////////////////////////////////////
//
// Maintenance History
// Version     Date      Userid     Description of Changes
//=============================================================================
// 1.0         12/05/95  kolda      Initial version
//
///////////////////////////////////////////////////////////////////////////////

#include "cmcDocument.h"
#include "cmcDocumentReference.h"
#include "cmcDetectionNeed.h"
#include "cmcDetectionQuery.h"
#include "cmcObjectReferenceVal.h"
#include "cmcError.h"

// #include "prides.h"


// the following is an attribute name for the DetectionNeed of this
static const char *DetectionNeedAttribute = "BaseDetectionNeed";


//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  DetectionQuery
//
// Description:    Constructor
//
// CSCI/CSC/CSU:   CMC/DetectionQuery
//
// Parameters:     Document& Base - Base Document to Copy
//                 DetectionNeed* Source - DetectionNeed this Query is Based on
//
// Return Value:   None
//
// Caller Destructs Return Value:  N/A
//
//////////////////////////////////////////////////////////////////////////////
DetectionQuery::DetectionQuery(Document& Base) :
                    Document(Base)
{
    const AttributeValue *pValue;	// value of the desired attribute

    // if the reference attribute is NULL...
    if((pValue = GetAttribute(DetectionNeedAttribute)) == NULL)
    {
	// document is not a DetectionQuery: raise an exception
	throw LogiconErrors::InvalidType(
			"DetectionQuery::DetectionQuery(Document&)");
    }

    // get the SourceDetectionNeed value
    SourceDetectionNeed = pValue->CastToObjectReferenceValue().GetValue().
				CastToDocumentReference();
}

 
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  DetectionQuery
//
// Description:    Constructor
//
// CSCI/CSC/CSU:   CMC/DetectionQuery
//
// Parameters:     Collection &NewParent - Parent Collection for this Query
//                 char* NewExternalId - External ID for this Query
//                 ByteSequence* NewRawData - The Actual text of the Query
//                 AnnotationSet &NewAnnotations - Annotation for the Query
//                 Set<Attribute> &NewAttributes - Attributes of the Query
//		   DetectionNeed &BaseNeed - Base DetectionNeed of the Query
//
// Return Value:   None
//
// Caller Destructs Return Value:  N/A
//
//////////////////////////////////////////////////////////////////////////////
DetectionQuery::DetectionQuery(Collection& NewParent, const char* NewExternalId,
                             const ByteSequence* NewRawData,
                             AnnotationSet& NewAnnotations,
                             Set<Attribute>& NewAttributes,
			     DetectionNeed &BaseNeed) :
                             Document(NewParent, NewExternalId, NewRawData,
                                      NewAnnotations, NewAttributes),
			     SourceDetectionNeed(BaseNeed)
{
    PutAttribute(DetectionNeedAttribute, 
		 ObjectReferenceValue(SourceDetectionNeed));
}



//
// Function Name:	DetectionQuery::DetectionQuery(Collection&, char*, ...)
//
// CSCI/CSC/CSU:	CMC/DetectionQuery
//
// Description:		Construct a new DetectionQuery given its elements
//			and given its BaseNeed as a DocumentReference
//
// Parameters:		Collection &NewParent - Parent Collection for this Query
//                 	char* NewExternalId - External ID for this Query
//                 	ByteSequence* NewRawData - The Actual text of the Query
//                 	AnnotationSet &NewAnnotations - Annotation for the Query
//                 	Set<Attribute> &NewAttributes - Attributes of the Query
//		   	DocumentReference &BaseNeed - Base DetectionNeed 
//					of the Query
//
// Return Value:	None
//
DetectionQuery::DetectionQuery(Collection& NewParent, const char* NewExternalId,
                             const ByteSequence* NewRawData,
                             AnnotationSet& NewAnnotations,
                             Set<Attribute>& NewAttributes,
			     DocumentReference &BaseNeed) :
                             Document(NewParent, NewExternalId, NewRawData,
                                      NewAnnotations, NewAttributes),
			     SourceDetectionNeed(BaseNeed)
{
    PutAttribute(DetectionNeedAttribute, 
		 ObjectReferenceValue(SourceDetectionNeed));
}





//
// Function Name:	DetectionQuery::DetectionQuery(DetectionQuery&)
//
// CSCI/CSC/CSU:	CMC/DetectionQuery
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
DetectionQuery::DetectionQuery(const DetectionQuery &Base) :
			    Document(Base),
			    SourceDetectionNeed(Base.SourceDetectionNeed)
{
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  ~DetectionQuery
//
// Description:    Destructor
//
// CSCI/CSC/CSU:   CMC/DetectionQuery
//
// Parameters:     None
//
// Return Value:   None
//
// Caller Destructs Return Value:  N/A
//
//////////////////////////////////////////////////////////////////////////////

DetectionQuery::~DetectionQuery(void)
{
   Cleanup();
}




//
// Function Name:	DetectionQuery::CopyDocument(Collection&)
//
// CSCI/CSC/CSU:	CMC/DetectionQuery
//
// Description:		Create a new copy of the DetectionQuery
//
// Parameters:		NewParent - collection in which to place new copy
//
// Return Value:	The new copy
//
Document *DetectionQuery::CopyDocument(Collection &NewParent)
{
    return new DetectionQuery(NewParent, GetExternalId(), GetRawData(),
		GetAnnotations(), GetAttributes(), SourceDetectionNeed);
}



//
// Function Name:	DetectionQuery::CopyBare(Collection&)
//
// CSCI/CSC/CSU:	CMC/DetectionQuery
//
// Description:		Create a new copy of the DetectionQuery without
//			Annotations or Attributes
//
// Parameters:		NewParent - collection in which to place new copy
//
// Return Value:	The new copy
//
Document *DetectionQuery::CopyBare(Collection &NewParent)
{
    AnnotationSet NewAnnotations;		// the Annotations to use
    Set<Attribute> AttributeSet;		// the Attributes to use

    return new DetectionQuery(NewParent, GetExternalId(), GetRawData(),
		NewAnnotations, AttributeSet, SourceDetectionNeed);
}



//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  Cleanup
//
// Description:    Destroys the objects components
//
// CSCI/CSC/CSU:   CMC/DetectionQuery
//
// Parameters:     None
//
// Return Value:   None
//
// Caller Destructs Return Value:  N/A
//
//////////////////////////////////////////////////////////////////////////////

void DetectionQuery::Cleanup(void)
{
}



//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  GetDetectionNeed
//
// Description:    Returns a reference to the Source Detection Need
//
// CSCI/CSC/CSU:   CMC/DetectionQuery
//
// Parameters:     None
//
// Return Value:   Pointer to a reference to the Source DetectionNeed
//
// Caller Destructs Return Value:  No
//
//////////////////////////////////////////////////////////////////////////////

const DocumentReference& DetectionQuery::GetDetectionNeed(void) const
{
   return SourceDetectionNeed;
}



//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  SetDetectionNeed
//
// Description:    Set the Source Detection Need
//
// CSCI/CSC/CSU:   CMC/DetectionQuery
//
// Parameters:     NewNeed - value to set it to
//
// Return Value:   None
//
// Caller Destructs Return Value:  N/A
//
//////////////////////////////////////////////////////////////////////////////

void DetectionQuery::SetDetectionNeed(DocumentReference &NewNeed)
{
    SourceDetectionNeed = NewNeed;
}


#if 0
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  FormRoutingQuery
//
// Description:    Calls the Tipster FormRoutingQuery function to generate
//                 a profile for this Detection Query
//
// CSCI/CSC/CSU:   CMC/DetectionQuery
//
// Parameters:     DocumentCollectionIndex* Index - The DCI which the
//                       Routing Query will be based upon
//
// Return Value:   Pointer to a Routing Query for this Detection Query
//
// Caller Destructs Return Value:  Yes
//
//////////////////////////////////////////////////////////////////////////////

RoutingQuery* DetectionQuery::FormRoutingQuery(DocumentCollectionIndex& Index)
{
   return NULL;
}
#endif




//
// Function Name:	DetectionQuery::CastToDetectionQuery()
//
// CSCI/CSC/CSU:	CMC/DetectionQuery
//
// Description:		Downcast function.
//
// Parameters:		None
//
// Return Value:	*this
//
DetectionQuery &DetectionQuery::CastToDetectionQuery()
{
    return *this;
}






//
// Function Name:	DetectionQuery::CastToDetectionQuery()
//
// CSCI/CSC/CSU:	CMC/DetectionQuery
//
// Description:		Downcast function.
//
// Parameters:		None
//
// Return Value:	*this
//
const DetectionQuery &DetectionQuery::CastToDetectionQuery() const
{
    return *this;
}




