//
// Module Name:		cmcDocument.cc
//
// CSCI/CSC/CSU:	CMC/Document
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	This contains the code for the Document class.  Document represents
//	the Documents stored in the system.
//
// Function List:
//	Document::Document() - This contains 4 constructors
//	Document::~Document() - Destructor
//	Document::Annotate() - Annotate a Document with an Annotator
//	Document::WriteSGML() - Write out an SGML version of the Document
//	Document::GetParent() - Get the Parent Collection of the Document
//	Document::CopyBare() - Copy the Document without Attributes, Annotations
//	Document::CopyDocument() - Copy the Document
//	Document::GetId() - Get the internal ID of the Document
//	Document::SetId() - Set the internal ID of the Document
//	Document::GetExternalId() - Get the external ID of the Document
//	Document::SetExternalId() - Set the external ID of the Document
//	Document::GetOldExternalId() - Get the external ID of the Document 
//			before it was changed
//	Document::ClearOldExternalId() - Forget about any old external ID's
//	Document::GetRawData() - Get the text of the Document
//	Document::GetAnnotations() - Get the Annotations of the Document
//	Document::AddAnnotation() - Add an Annotation to the Document
//	Document::RemoveAnnotation() - Remove an Annotation from the Document
//	Document::GetAnnotation() - Get an Annotation from the Document
//	Document::SelectAnnotations() - Select a set of Document Annotations
//	Document::DeleteAnnotations() - Delete a set of Document Annotations
//	Document::AnnotationsAt() - Find a set of Annotations at some given 
//			point
//	Document::NextAnnotations() - Find Annotations after a given point
//	Document::Sync() - Synchronize a Document to its parent Collection
//	Document::WriteToStream() - Write a text version of the Document to a 
//			stream
//	Document::Copy() - Produce a copy of the Document
//	Document::GetClass() - Get the class label of this
//	Document::PutAttribute() - Add an attribute to the Document
//	Document::Cleanup() - free all memory used by this
//	Document::CastToDocument() - Downcast function
//	
// Other Modules Called:
//	AttributedObject - Document is an AttributedObject
//	Collection - Documents have parent Collections
//	Annotation - To represent Annotations on a Document
//	AnnotationSet - To hold a Document's Annotations
//	ByteSequence - To represent a Document's text
//	Set<> - To hold pieces of a Document
//	Misc - For string writing
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           11/17/95    jd_2     Initial version
//

#include <iostream.h>			// for I/O
#include <stddef.h>			// for NULL
#include <stdlib.h>			// for basic C functions
#include <string.h>			// for string functions

#include "cmcDocument.h"		// for declaration of this
#include "cmcMisc.h"			// for string I/O
#include "cmcCollection.h"		// for a constructor
#include "cmcByteSequence.h"		// to hold Document text
#include "cmcAnnotationSet.h"		// to hold Document Annotations
#include "cmcSet.h"			// to receive Document Attributes
#include "cmcAttribute.h"		// to keep a Document Attribute
#include "cmcError.h"			// for exception classes

// ClassLabel is the string name of this class
const char *Document::ClassLabel = "Document";

//
// Function Name:	Document::Document()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Construct a Document given its pieces.
//
// Parameters:		NewParent - Collection to contain this
//			NewExternalId - External ID for the Document
//			pNewRawData - Raw data of the Document: can be NULL
//			NewAnnotations - Annotations of the Document
//			NewAttributes - Attributes of the Document
//
// Return Value:	None
//
Document::Document(Collection &NewParent, const char *NewExternalId,
	const ByteSequence *pNewRawData, AnnotationSet &NewAnnotations,
	Set<Attribute> &NewAttributes) : 
		AttributedObject(NewAttributes),
		Parent(NewParent), 
		Id(NULL),
		ExternalId(NULL),
		pRawData(pNewRawData == NULL ? NULL : pNewRawData->Copy()), 
		Annotations(NewAnnotations), 
		OldExternalId(NULL),
		Changed(0) 
{
    try
    {
    	if(NewExternalId != NULL)
    	{
    	    // allocate and store ExternalId
    	    if((ExternalId = new char[strlen(NewExternalId) + 1]) == NULL)
		throw LogiconErrors::MemoryAllocation(
		    "Document::Document(Collection& char*, ByteSequence*, ...");

    	    strcpy(ExternalId, NewExternalId);

    	    // store it as old as well
   	    if((OldExternalId = new char[strlen(ExternalId) + 1]) == NULL)
		throw LogiconErrors::MemoryAllocation(
		    "Document::Document(Collection& char*, ByteSequence*, ...");
    	    strcpy(OldExternalId, NewExternalId);
    	}
    	else
    	{
	    ExternalId = NULL;
	    OldExternalId = NULL;
    	}

    	// store this to the Parent
    	Parent.Store(*this);
    }
    catch(...)
    {
	Cleanup();
	throw;	
    }
}


//
// Function Name:	Document::Document()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Construct a Document by reading it in from an input
//			stream in the following format:
//
//				<ID in ReadString() format>
//				<ExternalId in ReadString() format>
//				<RawData flag - 1 if is RawData, 0 otherwise>
//				<RawData - only if flag was 1>
//				<Annotations in AnnotationSet format>
//				<count of Attributes> blank
//				<Attribute name 0> <AttributeValue 0>
//				... 
//				<Attribute name N> <AttributeValue N>
//
// Parameters:		NewParent - Parent Collection for this
//			Input - stream to read from
//
// Return Value:	None
//
Document::Document(Collection &NewParent, istream &Input) : 
	AttributedObject(),
	Parent(NewParent),
	Id(NULL), 
	ExternalId(NULL), 
	pRawData(NULL), 
	Annotations(),
	OldExternalId(NULL),
	Changed(0)
{
    try
    {
    	// get data
    	Id = ReadString(Input);
    	ExternalId = ReadString(Input);

    	if(ExternalId == NULL)
	    OldExternalId = NULL;
    	else
    	{
    	    // store it as old as well
    	    if((OldExternalId = new char[strlen(ExternalId) + 1]) == NULL)
		throw LogiconErrors::MemoryAllocation(
			"Document::Document(Collection&, istream&)");
    	    strcpy(OldExternalId, ExternalId);
    	}

    	// get RawData
    	char flag;			// indicates if there is raw data
    	Input >> flag;
	if(!Input)
	    throw LogiconErrors::Read(
			"Document::Document(Collection&, istream&)");
			
    	if(flag == '0')
	    pRawData = NULL;
    	else
	    pRawData = new ByteSequence(Input);

    	// put AnnotationSet
    	Input >> Annotations;

    	// put Attributes
    	int count;			// count of attributes

    	Input >> count;
	if(!Input)
	    throw LogiconErrors::Read(
			"Document::Document(Collection&, istream&)");
    	for(int i = 0; i < count; ++i)
    	{
	    char *Name;		// attribute name
	    AttributeValue *pValue;	// attribute value

	    Name = ReadString(Input);
	    pValue = ReadAttributeValue(Input);
	    PutAttribute(Name, *pValue);
	    delete[] Name;
	    delete pValue;
    	}

	// clear changed flags set during this
	Annotations.ClearChanged();
	ClearAttributesChanged();
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}



//
// Function Name:	Document::Document()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Construct a Document from an SGML representation of
//			it, recording in SGML tags as Annotations
//
// Parameters:		SgmlText - The SGML Document
//			NewParent - Collection to store it in
//			NewExternalId - External ID of the Document
//
// Return Value:
//
Document::Document(const char *SgmlText, Collection &NewParent,
		const char *NewExternalId) : Parent(NewParent), Changed(0)
{
    // TBD: Document(SgmlText)
}





//
// Function Name:	Document::Document()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Copy constructor for a Document.
//
// Parameters:		Base - the document to base this on
//
// Return Value:	None
//
Document::Document(const Document &Base) : 
	AttributedObject(Base.GetAttributes()),
	Parent(Base.Parent),
	Id(NULL),
	ExternalId(NULL),
	pRawData(NULL),
	Annotations(Base.Annotations),
	OldExternalId(NULL),
	Changed(Base.Changed)
{
    try
    {
    	// copy ID's
    	if(Base.Id != NULL)
    	{
	    if((Id = new char[strlen(Base.Id) + 1]) == NULL)
		throw LogiconErrors::MemoryAllocation(
			"Document::Document(Document&)");
	    strcpy(Id, Base.Id);
    	}
    	else
	    Id = NULL;

    	if(Base.ExternalId != NULL)
    	{
	    if((ExternalId = new char[strlen(Base.ExternalId) + 1]) == NULL)
		throw LogiconErrors::MemoryAllocation(
			"Document::Document(Document&)");
	    strcpy(ExternalId, Base.ExternalId);
    	}
    	else
	    ExternalId = NULL;

    	if(Base.OldExternalId != NULL)
    	{
	    if((OldExternalId = new char[strlen(Base.OldExternalId) + 1]) 
				== NULL)
		throw LogiconErrors::MemoryAllocation(
			"Document::Document(Document&)");
	    strcpy(OldExternalId, Base.OldExternalId);
    	}
    	else
	    OldExternalId = NULL;

    	// duplicate RawData
    	if(Base.pRawData != NULL)
	    pRawData = Base.pRawData->Copy();
    	else
	    pRawData = NULL;
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}



//
// Function Name:	Document::~Document()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Destruct a Document, destructing all of its pieces.
//
// Parameters:		None
//
// Return Value:	None
//
Document::~Document()
{
    Cleanup();
}




//
// Function Name:	Document::Annotate()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Run an Annotator to annotate a Document.
//
// Parameters:		AnnotatorName - Name of the Annotator
//
// Return Value:	None
//
void Document::Annotate(const char *AnnotatorName)
{
    // TBD: Annotate(AnnotatorName)
}



//
// Function Name:	Document::WriteSGML()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Write a Document, representing a set of Annotations as
//			SGML tags.
//
// Parameters:		NewAnnotations - the Annotations to apply to it
//			AnnotationPrecedence - Order of importance of 
//				Annotations
//
// Return Value:	SGML representation of Document
//
char *Document::WriteSGML(AnnotationSet &NewAnnotations,
		Set<char*> &AnnotationPrecedence) const
{
    // TBD: WriteSGML
    return NULL;
}


//
// Function Name:	Document::GetParent()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Return the pParent component of a Document.
//
// Parameters:		None
//
// Return Value:	The Parent Collection
//
Collection &Document::GetParent() const
{
    return Parent;
}



//
// Function Name:	Document::CopyBare()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Create a persistent copy of the Document without
//			Attributes or Annotations.
//
// Parameters:		NewParent - Collection to copy to
//
// Return Value:	The new Document.
//
Document *Document::CopyBare(Collection &NewParent)
{
    AnnotationSet NewAnnotations;		// an empty AnnotationSet
    Set<Attribute> NewAttributes;		// an empty Attribute set

    return new Document(NewParent, ExternalId, pRawData, NewAnnotations,
		NewAttributes);
}



//
// Function Name:	Document::CopyDocument()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Create a persistent copy of the Document.
//
// Parameters:		NewParent - Collection to copy to
//
// Return Value:	The new Document.
//
Document *Document::CopyDocument(Collection &NewParent)
{
    return new Document(NewParent, ExternalId, pRawData, Annotations,
		GetAttributes());
}





//
// Function Name:	Document::GetId()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Get the internal ID of a Document.
//
// Parameters:		None
//
// Return Value:	The internal ID
//
const char *Document::GetId() const
{
    return Id;
}


//
// Function Name:	Document::SetId()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Set the internal ID of a Document
//
// Parameters:		NewId - new value for the ID
//
// Return Value:	None
//
void Document::SetId(const char *NewId)
{
    char *IdValue;		// to hold the ID

    // store the new ID value: do this first in case there is a problem
    if(NewId != NULL)
    {
	if((IdValue = new char[strlen(NewId) + 1]) == NULL)
	    throw LogiconErrors::MemoryAllocation("Document::SetId(char *)");
    	strcpy(IdValue, NewId);
    }
    else
	IdValue = NULL;
	
    // delete old Id
    if(Id != NULL)
	delete[] Id;

    // store Id
    Id = IdValue;

    // note this is a change
    Changed = 1;
}


//
// Function Name:	Document::GetExternalId()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Get the ExternalId component of a Document
//
// Parameters:		None
//
// Return Value:	The ExternalId component
//
const char *Document::GetExternalId() const
{
    return ExternalId;
}



//
// Function Name:	Document::SetExternalId()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Set the ExternalId component of a Document.  If
//			there is no old ExternalId stored, store the
//			current ExternalId as an old ExternalId.
//
// Parameters:		NewId - new value to set for the ExternalId
//
// Return Value:	None
//
void Document::SetExternalId(const char *NewId) 
{
    char *IdValue;			// to hold the new value

    if(NewId != NULL)
    {
	if((IdValue = new char[strlen(NewId) + 1]) == NULL)
	    throw LogiconErrors::MemoryAllocation(
			"Document::SetExternalId(char*)");
	strcpy(IdValue, NewId);
    }
    else
	IdValue = NULL;

    // if necessary, save the old one
    if(OldExternalId == NULL && ExternalId != NULL)
    {
	OldExternalId = ExternalId;
    }
    else if(ExternalId != NULL)
    {
	delete[] ExternalId;
    }

    ExternalId = IdValue;

    // note this is a change
    Changed = 1;
}



//
// Function Name:	Document::GetOldExternalId()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Get the OldExternalId component of a Document.
//
// Parameters:		None
//
// Return Value:	The OldExternalId
//
const char *Document::GetOldExternalId() const
{
    return OldExternalId;
}



//
// Function Name:	Document::ClearOldExternalId()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Clear out the OldExternalId component of the Document.
//
// Parameters:		None
//
// Return Value:	None
//
void Document::ClearOldExternalId()
{
    char *IdValue;			// to hold new OldExternalId value

    if(ExternalId == NULL)
	IdValue = NULL;
    else
    {
	IdValue = new char[strlen(ExternalId) + 1];
	strcpy(IdValue, ExternalId);
    }

    // out with the old...
    if(OldExternalId != NULL)
    {
	delete[] OldExternalId;
    }

    // and in with the new
    OldExternalId = IdValue;
}



//
// Function Name:	Document::GetRawData()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Get the raw data (aka, the text) of a Document.
//			If the Document does not have this stored,
//			retrieve it from the Document's base Collection.
//
// Parameters:		None
//
// Return Value:	The raw data of the Document as a ByteSequence
//
const ByteSequence *Document::GetRawData() const
{
    return pRawData;
}



//
// Function Name:	Document::GetAnnotations()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Get the Annotations component of a Document
//
// Parameters:		None
//
// Return Value:	The Annotations component.
//
AnnotationSet &Document::GetAnnotations()
{
    return Annotations;
}



//
// Function Name:	Document::GetAnnotations()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Get the Annotations component of a constant Document
//
// Parameters:		None
//
// Return Value:	The Annotations component.
//
const AnnotationSet &Document::GetAnnotations() const
{
    return Annotations;
}





//
// Function Name:	Document::AddAnnotation()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Add an Annotation to the Document.
//
// Parameters:		NewAnnotation - The Annotation to add.
//
// Return Value:	None
//
const char *Document::AddAnnotation(Annotation &NewAnnotation)
{
    return Annotations.AddAnnotation(NewAnnotation);
}




//
// Function Name:	Document::RemoveAnnotation()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Remove a specified Annotation from the Document
//
// Parameters:		AnnotationId - ID of the Annotation to remove
//
// Return Value:	None
//
void Document::RemoveAnnotation(const char *AnnotationId)
{
    Annotations.RemoveAnnotation(AnnotationId);
}



//
// Function Name:	Document::GetAnnotation()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		get an Annotation from the Document
//
// Parameters:		AnnotationId - ID of the Annotation to get
//
// Return Value:	The gotten Annotation
//
Annotation &Document::GetAnnotation(const char *AnnotationId) const
{
    return Annotations.GetAnnotation(AnnotationId);
}



//
// Function Name:	Document::SelectAnnotations()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Select a set of Annotations from the Document
//			according to Annotation type and attributes which
//			the Annotations should have.
//
// Parameters:		Type - Type of the Annotations to get
//			Constraints - Attributes that the chosen Annotations
//				must have
//
// Return Value:	The set of selected Annotations
//
AnnotationSet *Document::SelectAnnotations(const char *Type, 
		Set<Attribute> &Constraints) const
{
    return Annotations.SelectAnnotations(Type, Constraints);
}



//
// Function Name:	Document::DeleteAnnotations()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Remove all Annotations of a given type matching a
//			set of constraints from the Document.
//
// Parameters:		Type - Type of Annotations to remove
//			Constraints - Attributes which removed Annotations
//				should possess
//
// Return Value:	None
//
void Document::DeleteAnnotations(const char *Type, Set<Attribute> &Constraints)
{
    Annotations.DeleteAnnotations(Type, Constraints);
}



//
// Function Name:	Document::AnnotationsAt
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Find all Annotations that start at a given position
//
// Parameters:		Position - the specified starting position
//
// Return Value:	The set of Annotations at that position
//
AnnotationSet *Document::AnnotationsAt(int Position) const
{
    return Annotations.AnnotationsAt(Position);
}



//
// Function Name:	Document::NextAnnotations()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Find the set of Annotations that start on the first
//			position following Position that starts any
//			Annotation.
//
// Parameters:		Position - the Position in question
//
// Return Value:	The set of Annotations from that position
//
AnnotationSet *Document::NextAnnotations(int Position) const
{
    return Annotations.NextAnnotations(Position);
}






//
// Function Name:	Document::Sync()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Synchronize changes to a Document to its persistent
//			version kept in its parent Collection.
//
// Parameters:		None
//
// Return Value:	None
//
void Document::Sync()
{
    if(Changed || IsAttributesChanged() || Annotations.IsChanged())
    {
    	Parent.Update(*this);
	Parent.Sync();
	ClearAttributesChanged();
	Annotations.ClearChanged();
	Changed = 0;
    }
}



//
// Function Name:	Document::WriteToStream()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Write a Document to an output stream in the
//			format identified under 
//			Document::Document(Collection *, istream&).
//
// Parameters:		Output - The stream to write to
//
// Return Value:	None
//
void Document::WriteToStream(ostream &Output) const
{
    // put in id's
    WriteString(Id, Output);
    WriteString(ExternalId, Output);

    // put RawData
    if(pRawData == NULL)
	Output << '0';
    else
	Output << '1' << *pRawData;
    Output << ' ';

    // put AnnotationSet
    Output << Annotations << ' ';

    // put Attributes
    Output << GetAttributes().Length() << ' ';
    for(int i = 0; i < GetAttributes().Length(); ++i)
    {
	WriteString(GetAttributes().Nth(i).GetName(), Output);
	Output << ' ' << GetAttributes().Nth(i).GetValue() << ' ';
    }

    // check Output status
    if(!Output)
	throw LogiconErrors::Write("Document::WriteToStream(ostream &)");
}




//
// Function Name:	Document::Copy()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Create a copy of a Document.  Unlike the
//			constructor version of Copy found above, this
//			does nothing with Collections: it simply provides
//			a new version of the Document.
//
// Parameters:		None
//
// Return Value:	A new Document.
//
Document *Document::Copy() const
{
    Document *pCopy = new Document(*this);	// the copy

    if(pCopy == NULL)
	throw LogiconErrors::MemoryAllocation("Document::Copy()");

    return pCopy;
}




//
// Function Name:	Document::GetClass()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *Document::GetClass() const
{
    return ClassLabel;
}





//
// Function Name:	Document::Cleanup()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Cleanup allocated memory
//
// Parameters:		None
//
// Return Value:	None
//
void Document::Cleanup()
{
    if(ExternalId != NULL)
	delete[] ExternalId;
    if(OldExternalId != NULL)
	delete[] OldExternalId;
    if(Id != NULL)
	delete[] Id;
    if(pRawData != NULL)
	delete pRawData;
}



//
// Function Name:	Document::CastToDocument()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
Document &Document::CastToDocument()
{
    return *this;
}




//
// Function Name:	Document::CastToDocument()
//
// CSCI/CSC/CSU:	CMC/Document
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const Document &Document::CastToDocument() const
{
    return *this;
}


