//
// Module Name:		cmcDocumentId.cc
//
// CSCI/CSC/CSU:	CMC/DocumentId
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Contains the code for DocumentId.  A DocumentId stores the internal
//	ID of a Document.  The primary purpose of this class is to break
//	the ID into its three components: collection name, volume number,
//	and offset in volume.
//
// Function List:
//	DocumentId::DocumentId() - this has 2 contructors
//	DocumentId::~DocumentId() - Destructor
//	DocumentId::GetIdText() - Get the string version of the entire ID
//	DocumentId::GetCollectionName() - Get the CollectionName part of the ID
//	DocumentId::GetVolume() - Get the Volume part of the ID
//	DocumentId::GetOffset() - Get the Offset part of the ID
//	DocumentId::GetTime() - Get the Time part of the ID
//	DocumentId::Cleanup() - Free memory allocated for this
//	Error - Used for exceptions
//
// Other Modules Called:
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#include <string.h>			// for string operations
#include <iostream.h>			// for I/O
#include <strstream.h>			// for string I/O

#include "cmcDocumentId.h"		// for declaration of this
#include "cmcError.h"			// for exception classes


//
// Function Name:	DocumentId::DocumentId()
//
// CSCI/CSC/CSU:	CMC/DocumentId
//
// Description:		Construct a DocumentId given the elements that
//			go into it.
//
// Parameters:		NewVolume - The Id's Volume number
//			NewOffset - The Offset of the ID
//			NewTime - The Time of the ID
//
// Return Value:	None
//
DocumentId::DocumentId(int NewVolume, int NewOffset, int NewTime) : 
				IdText(NULL), 
				Volume(NewVolume),
				Offset(NewOffset),
				Time(NewTime)
{
    try
    {
    	// construct the complete ID
    	ostrstream Output;			// output stream for the ID

    	Output << Volume << '.' << Offset << '.' << Time << '\0';

	if(!Output)
	    throw LogiconErrors::Write(
			"DocumentId::DocumentId(char*, int, int)");
    	IdText = Output.str();
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}




//
// Function Name:	DocumentId::DocumentId()
//
// CSCI/CSC/CSU:	CMC/DocumentId
//
// Description:		Construct a DocumentId given its text representation.
//
// Parameters:		NewIdText - The text representation
//
// Return Value:	None
//
DocumentId::DocumentId(const char *NewIdText) : 
	IdText(NULL)
{
    try
    {
    	if(NewIdText == NULL)
    	{
	    IdText = NULL;
	    Volume = Offset = -1;
    	}
    	else
    	{
    	    // store input parameters
    	    if((IdText = new char[strlen(NewIdText) + 1]) == NULL)
		throw LogiconErrors::MemoryAllocation(
				"DocumentId::DocumentId(char *)");

    	    strcpy(IdText, NewIdText);

    	    // get the other pieces of it
    	    istrstream Input(NewIdText, strlen(NewIdText));	// input stream
    	    char ch;						// filler
   
    	    // get in the stuff
    	    if(!(Input >> Volume >> ch >> Offset >> ch >> Time))
		throw LogiconErrors::Format("DocumentId::DocumentId(char *)");
    	}
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}




//
// Function Name:	DocumentId::~DocumentId()
//
// CSCI/CSC/CSU:	CMC/DocumentId
//
// Description:		Destructor.
//
// Parameters:		None
//
// Return Value:	None
//
DocumentId::~DocumentId()
{
    Cleanup();
}



//
// Function Name:	DocumentId::GetIdText()
//
// CSCI/CSC/CSU:	CMC/DocumentId
//
// Description:		Get the string representation of the ID
//
// Parameters:		None
//
// Return Value:	The IdText component
//
const char *DocumentId::GetIdText() const
{
    return IdText;
}




//
// Function Name:	DocumentId::GetVolume()
//
// CSCI/CSC/CSU:	CMC/DocumentId
//
// Description:		Get the Volume number of this.
//
// Parameters:		None
//
// Return Value:	The Volume component
//
int DocumentId::GetVolume() const
{
    return Volume;
}



//
// Function Name:	DocumentId::GetOffset()
//
// CSCI/CSC/CSU:	CMC/DocumentId
//
// Description:		Get the Offset of the Id.
//
// Parameters:		None
//
// Return Value:	The Offset component.
//
int DocumentId::GetOffset() const
{
    return Offset;
}




//
// Function Name:	DocumentId::GetTime()
//
// CSCI/CSC/CSU:	CMC/DocumentId
//
// Description:		Get the Time component of the DocumentId.
//
// Parameters:		None
//
// Return Value:	The Time component
//
int DocumentId::GetTime() const
{
    return Time;
}



//
// Function Name:	DocumentId::Cleanup()
//
// CSCI/CSC/CSU:	CMC/DocumentId
//
// Description:		Clean out all allocated memory
//
// Parameters:		None
//
// Return Value:	None
//
void DocumentId::Cleanup()
{
    if(IdText != NULL)
    {
	delete[] IdText;
	IdText = NULL;
    }
}
