//
// Module Name:		cmcDocumentReference.cc
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for the DocumentReference class.  A DocumentReference is a
//	type of ObjectReference that references Documents.  It maintains
//	the Document's ID and the name of its Collection as member
//	variable.
//
// Function List:
//	DocumentReference::DocumentReference() - This contains 5 constructors
//	DocumentReference::~DocumentReference() - A destructor
//	DocumentReference::GetCollectionName() - Return the referenced 
//			Document's Collection's name
//	DocumentReference::GetDocumentId() - Return the referenced Document's ID
// 	DocumentReference::operator=(DocumentReference&) - Assignment operator
//	DocumentReference::TypeOf() - Return the type of ObjectReference that 
//			this is
//	DocumentReference::Copy() - Produce a copy of this
//	DocumentReference::operator==() - Test equality against another 
//			ObjectReference
//	DocumentReference::Write() - Write this out to stream
//	DocumentReference::GetClass() - Get the class label of this
//	DocumentReference::Cleanup() - Free memory used by this
//	DocumentReference::CastToDocumentReference() - Downcast function
//
// Other Modules Called:
//	AttributeValueType - Types of ObjectReferences
//	Document - To use in the Constructor
//	Misc - To get ReadString() and WriteString()
//	Error - Used for exceptions
//
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#include <string.h>			// for basic string functions
#include <iostream.h>			// for I/O

#include "cmcDocumentReference.h"	// for declaration of this
#include "cmcAttributeValueType.h"	// for TypeOf() return value
#include "cmcDocument.h"		// for a constructor
#include "cmcMisc.h"			// for string I/O
#include "cmcError.h"			// for exception classes

// ClassLabel is the string name of this class
const char *DocumentReference::ClassLabel = "DocumentReference";


//
// Function Name:	DocumentReference::DocumentReference()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Construct a DocumentReference given the referenced
//			Document.
//
// Parameters:		pDocument - The referenced Document
//
// Return Value:	None
//
DocumentReference::DocumentReference(Document &ReferencedDocument) :
	CollectionName(NULL),
	DocumentId(NULL)
{
    try
    {
	// allocate memory
    	CollectionName = new char[
		strlen(ReferencedDocument.GetParent().GetName()) + 1];
    	DocumentId = new char[strlen(ReferencedDocument.GetId()) + 1];

	// check allocation
	if(CollectionName == NULL || DocumentId == NULL)
	    throw LogiconErrors::MemoryAllocation(
		"DocumentReference::DocumentReference(Document&)");

	// make copies
    	strcpy(CollectionName, ReferencedDocument.GetParent().GetName());
    	strcpy(DocumentId, ReferencedDocument.GetId());
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}



//
// Function Name:	DocumentReference::DocumentReference()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Construct a DocumentReference given its
//			CollectionName and DocumentId components
//
// Parameters:		NewCollectionName - New value for CollectionName
//			NewDocumentId - New value for Document ID
//
// Return Value:	None
//
DocumentReference::DocumentReference(const char *NewCollectionName, 
			const char *NewDocumentId)
{
    try
    {
	// allocate memory
    	CollectionName = new char[strlen(NewCollectionName) + 1];
    	DocumentId = new char[strlen(NewDocumentId) + 1];

	// check allocation
	if(CollectionName == NULL || DocumentId == NULL)
	    throw LogiconErrors::MemoryAllocation(
		"DocumentReference::DocumentReference(Document&)");

	// make copies
    	strcpy(CollectionName, NewCollectionName);
    	strcpy(DocumentId, NewDocumentId);
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}




//
// Function Name:	DocumentReference::DocumentReference()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Construct a DocumentReference from a representation
//			of it in an input stream.  This is in the following
//			format:
//
//				DOCUMENT_REFERENCE (an AttributeValueType)
//				CollectionName (in ReadString() format)
//				DocumentId (in ReadString() format)
//
// Parameters:		Input - The input stream
//
// Return Value:	None
//
DocumentReference::DocumentReference(istream &Input) :
	CollectionName(NULL), 
	DocumentId(NULL)
{
    try
    {
    	AttributeValueType Filler;		// throw-away data

    	Input >> Filler;
    	CollectionName = ReadString(Input);
    	DocumentId = ReadString(Input);

    	if(!Input)
	    throw LogiconErrors::Read(
			"DocumentReference::DocumentReference(istream&)");
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}



//
// Function Name:	DocumentReference::DocumentReference()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Copy constructor.
//
// Parameters:		Base - The DocumentReference to copy from
//
// Return Value:	None
//
DocumentReference::DocumentReference(const DocumentReference &Base) :
	CollectionName(NULL),
	DocumentId(NULL)
{
    try
    {
	// Copy CollectionName
	if(Base.CollectionName != NULL)
	{
	    // allocate memory
    	    if((CollectionName = new char[strlen(Base.CollectionName) + 1])
			== NULL)
	    	throw LogiconErrors::MemoryAllocation(
		    "DocumentReference::DocumentReference(DocumentReference&)");

	    // copy it in
    	    strcpy(CollectionName, Base.CollectionName);
	}

	// copy DocumentId
	if(Base.DocumentId != NULL)
	{
	    // allocate memory
    	    if((DocumentId = new char[strlen(Base.DocumentId) + 1]) == NULL)
	    	throw LogiconErrors::MemoryAllocation(
		    "DocumentReference::DocumentReference(DocumentReference&)");

	    // copy it in
    	    strcpy(DocumentId, Base.DocumentId);
	}
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}




//
// Function Name:	DocumentReference::DocumentReference()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Construct empty DocumentReference
//
// Parameters:		None
//
// Return Value:	None
//
DocumentReference::DocumentReference() : CollectionName(NULL), DocumentId(NULL)
{
}




//
// Function Name:	DocumentReference::~DocumentReference()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Destruct a DocumentReference.
//
// Parameters:		None
//
// Return Value:	None
//
DocumentReference::~DocumentReference()
{
    Cleanup();
}



//
// Function Name:	DocumentReference::GetCollectionName()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Get the name of the referenced Document's Collection
//
// Parameters:		None
//
// Return Value:	The CollectionName component
//
const char *DocumentReference::GetCollectionName() const
{
    return CollectionName;
}



//
// Function Name:	DocumentReference::GetDocumentId()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Get the referenced Document's ID
//
// Parameters:		None
//
// Return Value:	The DocumentId component
//
const char *DocumentReference::GetDocumentId() const
{
    return DocumentId;
}


//
// Function Name:	DocumentReference::TypeOf()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Get the type of ObjectReference that this is.
//
// Parameters:		None
//
// Return Value:	DOCUMENT_REFERENCE
//
AttributeValueType DocumentReference::TypeOf() const
{
    return DOCUMENT_REFERENCE;
}



//
// Function Name:	DocumentReference::Copy()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Produce a copy of this.
//
// Parameters:		None
//
// Return Value:	the copy
//
ObjectReference *DocumentReference::Copy() const
{
    DocumentReference *pCopy = new DocumentReference(CollectionName, 
						DocumentId);

    // check allocation
    if(pCopy == NULL)
	throw LogiconErrors::MemoryAllocation("DocumentReference::Copy()");

    return pCopy;
}



//
// Function Name:	DocumentReference::operator==
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Test if this is equal to another ObjectReference.
//
// Parameters:		Target - The thing to test against
//
// Return Value:	1 if equal
//			0 if not
//
int DocumentReference::operator==(const ObjectReference &Target) const
{
    int Results;		// results of this

    // if of the correct type...
    if(Target.TypeOf() == DOCUMENT_REFERENCE)
    {
	Results = (strcmp(DocumentId,
			((DocumentReference *)(&Target))->DocumentId) == 0) &&
		(strcmp(CollectionName,
			((DocumentReference *)(&Target))->CollectionName) == 0);
    }
    else
	Results = 0;

    return Results;
}



//
// Function Name:	DocumentReference::operator=(DocumentReference&)
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Copy operator.
//
// Parameters:		Base - Thing to copy
//
// Return Value:	The value of this
//
DocumentReference &DocumentReference::operator=(const DocumentReference &Base)
{
    // clean out old stuff
    Cleanup();
    CollectionName = NULL;
    DocumentId = NULL;

    try
    {
	// Copy CollectionName
	if(Base.CollectionName != NULL)
	{
	    // allocate memory
    	    if((CollectionName = new char[strlen(Base.CollectionName) + 1])
			== NULL)
	    	throw LogiconErrors::MemoryAllocation(
		    "DocumentReference::operator=(DocumentReference&)");

	    // copy it in
    	    strcpy(CollectionName, Base.CollectionName);
	}

	// copy DocumentId
	if(Base.DocumentId != NULL)
	{
	    // allocate memory
    	    if((DocumentId = new char[strlen(Base.DocumentId) + 1]) == NULL)
	    	throw LogiconErrors::MemoryAllocation(
		    "DocumentReference::operator=(DocumentReference&)");

	    // copy it in
    	    strcpy(DocumentId, Base.DocumentId);
	}
    }
    catch(...)
    {
	Cleanup();
	throw;
    }

    return *this;
}





//
// Function Name:	DocumentReference::Write()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Write a DocumentReference to a stream in the format
//			indicated under DocumentReference(istream&)
//
// Parameters:		Output - The stream to write to
//
// Return Value:	None
//
void DocumentReference::Write(ostream &Output) const
{
    Output << DOCUMENT_REFERENCE;
    WriteString(CollectionName, Output);
    WriteString(DocumentId, Output);

    // check status
    if(!Output)
	throw LogiconErrors::Write("DocumentReference::Write(ostream&)");
}




//
// Function Name:	DocumentReference::GetClass()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *DocumentReference::GetClass() const
{
    return ClassLabel;
}



//
// Function Name:	DocumentReference::Cleanup()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Clean up allocated memory.
//
// Parameters:		None
//
// Return Value:	None
//
void DocumentReference::Cleanup()
{
    if(CollectionName != NULL)
    	delete[] CollectionName;
    if(DocumentId != NULL)
    	delete[] DocumentId;
}



//
// Function Name:	DocumentReference::CastToDocumentReference()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
DocumentReference &DocumentReference::CastToDocumentReference()
{
    return *this;
}




//
// Function Name:	DocumentReference::CastToDocumentReference()
//
// CSCI/CSC/CSU:	CMC/DocumentReference
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const DocumentReference &DocumentReference::CastToDocumentReference() const
{
    return *this;
}


