//
// Module Name:		cmcError.cc
//
// CSCI/CSC/CSU:	CMC/Error
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Contains all code for Error classes.  These are the classes that
//	can be thrown as exceptions in the Document Manager.
//
//	The base class for all of these is TipsterError.  A TipsterError
//	keeps a location string indicating where the error was encountered.
//	In addition, it has a virtual function GetLabel() to get the
//	label of any particular error.
//
//	All other errors are subclasses of TipsterError.
//
// Function List:
//	TipsterError::TipsterError() - Construct a TipsterError
//	TipsterError::~TipsterError() - Destruct a TipsterError
//	TipsterError::GetLocation() - Get the Location component 
//	X:X() - For all subclasses X of TipsterError, there is a constructor
//			and a copy constructor
//	X:GetLabel() - For all subclasses X of TipsterError, there is a
//		GetLabel() function that returns the name of the subclass
//
// Other Modules Called:
//
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           1/2/96      jd_2     Initial version
//


#include <string.h>		// for standard string functions

#include "cmcError.h"		// for declarations of these functions

//
// Function Name:	TipsterError::TipsterError()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Construct a TipsterError.
//
// Parameters:		NewLocation - Name of function that encountered the
//				error
//
// Return Value:	None
//
LogiconErrors::TipsterError::TipsterError(const char *NewLocation)
	: Location(NULL)
{
    // store NewLocation
    Location = new char[strlen(NewLocation) + 1];
    strcpy(Location, NewLocation);
}



//
// Function Name:	TipsterError::~TipsterError()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Destructor for a TipsterError.
//
// Parameters:		None
//
// Return Value:	None
//
LogiconErrors::TipsterError::~TipsterError()
{
    if(Location != NULL)
	delete[] Location;
}



//
// Function Name:	TipsterError::GetLocation()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Return the Location component of the TipsterError
//
// Parameters:		None
//
// Return Value:	The error string
//
const char *LogiconErrors::TipsterError::GetLocation() const
{
    return Location;
}


// This is the label for the type of error
const char *LogiconErrors::MemoryAllocation::Label = "MemoryAllocation";

//
// Function Name:	MemoryAllocation::MemoryAllocation()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::MemoryAllocation::MemoryAllocation(const char *NewLocation)
	: TipsterError(NewLocation)
{
}




//
// Function Name:	MemoryAllocation::MemoryAllocation()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::MemoryAllocation::MemoryAllocation(const MemoryAllocation &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	MemoryAllocation::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::MemoryAllocation::GetLabel() const
{
    return Label;
}



// This is the label for the type of error
const char *LogiconErrors::OutOfBounds::Label = "OutOfBounds";

//
// Function Name:	OutOfBounds::OutOfBounds()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::OutOfBounds::OutOfBounds(const char *NewLocation)
	: TipsterError(NewLocation)
{
}


//
// Function Name:	OutOfBounds::OutOfBounds()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::OutOfBounds::OutOfBounds(const OutOfBounds &Base)
	: TipsterError(Base.GetLocation())
{
}



//
// Function Name:	OutOfBounds::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::OutOfBounds::GetLabel() const
{
    return Label;
}


// This is the label for the type of error
const char *LogiconErrors::NotFound::Label = "NotFound";

//
// Function Name:	NotFound::NotFound()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::NotFound::NotFound(const char *NewLocation)
	: TipsterError(NewLocation)
{
}



//
// Function Name:	NotFound::NotFound()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::NotFound::NotFound(const NotFound &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	NotFound::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::NotFound::GetLabel() const
{
    return Label;
}



// This is the label for the type of error
const char *LogiconErrors::AlreadyPresent::Label = "AlreadyPresent";

//
// Function Name:	AlreadyPresent::AlreadyPresent()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::AlreadyPresent::AlreadyPresent(const char *NewLocation)
	: TipsterError(NewLocation)
{
}



//
// Function Name:	AlreadyPresent::AlreadyPresent()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::AlreadyPresent::AlreadyPresent(const AlreadyPresent &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	AlreadyPresent::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::AlreadyPresent::GetLabel() const
{
    return Label;
}


// This is the label for the type of error
const char *LogiconErrors::InvalidObject::Label = "InvalidObject";

//
// Function Name:	InvalidObject::InvalidObject()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::InvalidObject::InvalidObject(const char *NewLocation)
	: TipsterError(NewLocation)
{
}



//
// Function Name:	InvalidObject::InvalidObject()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::InvalidObject::InvalidObject(const InvalidObject &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	InvalidObject::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::InvalidObject::GetLabel() const
{
    return Label;
}


// This is the label for the type of error
const char *LogiconErrors::InvalidType::Label = "InvalidType";

//
// Function Name:	InvalidType::InvalidType()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::InvalidType::InvalidType(const char *NewLocation) 
	: TipsterError(NewLocation)
{
}



//
// Function Name:	InvalidType::InvalidType()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::InvalidType::InvalidType(const InvalidType &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	InvalidType::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::InvalidType::GetLabel() const
{
    return Label;
}




// This is the label for the type of error
const char *LogiconErrors::Format::Label = "Format";

//
// Function Name:	Format::Format()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::Format::Format(const char *NewLocation)
	: TipsterError(NewLocation)
{
}



//
// Function Name:	Format::Format()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::Format::Format(const Format &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	Format::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::Format::GetLabel() const
{
    return Label;
}





// This is the label for the type of error
const char *LogiconErrors::Write::Label = "Write";

//
// Function Name:	Write::Write()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::Write::Write(const char *NewLocation)
	: TipsterError(NewLocation)
{
}



//
// Function Name:	Write::Write()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::Write::Write(const Write &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	Write::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::Write::GetLabel() const
{
    return Label;
}



// This is the label for the type of error
const char *LogiconErrors::Open::Label = "Open";

//
// Function Name:	Open::Open()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::Open::Open(const char *NewLocation)
	: TipsterError(NewLocation)
{
}



//
// Function Name:	Open::Open()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::Open::Open(const Open &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	Open::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::Open::GetLabel() const
{
    return Label;
}



// This is the label for the type of error
const char *LogiconErrors::Read::Label = "Read";

//
// Function Name:	Read::Read()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::Read::Read(const char *NewLocation)
	: TipsterError(NewLocation)
{
}



//
// Function Name:	Read::Read()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::Read::Read(const Read &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	Read::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::Read::GetLabel() const
{
    return Label;
}



// This is the label for the type of error
const char *LogiconErrors::Seek::Label = "Seek";

//
// Function Name:	Seek::Seek()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::Seek::Seek(const char *NewLocation)
	: TipsterError(NewLocation)
{
}



//
// Function Name:	Seek::Seek()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::Seek::Seek(const Seek &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	Seek::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::Seek::GetLabel() const
{
    return Label;
}



// This is the label for the type of error
const char *LogiconErrors::Lock::Label = "Lock";

//
// Function Name:	Lock::Lock()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::Lock::Lock(const char *NewLocation)
	: TipsterError(NewLocation)
{
}



//
// Function Name:	Lock::Lock()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::Lock::Lock(const Lock &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	Lock::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::Lock::GetLabel() const
{
    return Label;
}



// This is the label for the type of error
const char *LogiconErrors::Parameter::Label = "Parameter";

//
// Function Name:	Parameter::Parameter()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::Parameter::Parameter(const char *NewLocation)
	: TipsterError(NewLocation)
{
}



//
// Function Name:	Parameter::Parameter()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::Parameter::Parameter(const Parameter &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	Parameter::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::Parameter::GetLabel() const
{
    return Label;
}



// This is the label for the type of error
const char *LogiconErrors::Delete::Label = "Delete";

//
// Function Name:	Delete::Delete()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Constructor.
//
// Parameters:		NewLocation - name of function encountering error
//
// Return Value:	None
//
LogiconErrors::Delete::Delete(const char *NewLocation)
	: TipsterError(NewLocation)
{
}



//
// Function Name:	Delete::Delete()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Copy constructor
//
// Parameters:		Base - Thing to copy
//
// Return Value:	None
//
LogiconErrors::Delete::Delete(const Delete &Base)
	: TipsterError(Base.GetLocation())
{
}


//
// Function Name:	Delete::GetLabel()
//
// CSCI/CSC/CSU:	CMC/Error
//
// Description:		Get label for this.
//
// Parameters:		None
//
// Return Value:	Label string for this error
//
const char *LogiconErrors::Delete::GetLabel() const
{
    return Label;
}




