//
// Module Name:		cmcIndex.cc
//
// CSCI/CSC/CSU:	CMC/Index
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for the Index class.  An Index maps a set of strings to their
//	Location records, indicating their locations in a Collection.  It
//	is used primarily to map Document external ID's to their locations
//	in Collections.  For more details in the implementation of an Index,
//	see IMPLEMENTATION NOTES below.
//
// Function List:
//	Index::Index() - There are 2 constructors to this
//	Index::~Index() - Destructor
//	Index::Destroy() - Destroy an Index
//	Index::Find() - Find an item in the Index
//	Index::Remove() - Remove an item from the Index
//	Index::AddId() - Add an item to the Index
//	Index::Sync() - Synchronize the Index with its data storage
//	Index::FindIndexBlock() - Find the block for an Id in the Index
//	Index::GetFilename() - Get the filename to hold the Index
//	Index::SetIndexRoot() - Set the Index's root block
//	Index::GetIndexFile() - Get the TreeFile component
//	Index::CreateHeader() - Create the header of this and write it out
//	Index::Display() - Do a debug display of the Index
//	Index::CheckInBlock() - Add the block to the allocation list
//	Index::CheckOutBlock() - Remove the block from the allocation list
//	Index::TouchBlock() - Move the block to the front of the allocation list
//
// Other Modules Called:
//	RecordFile - To provide storage
//	Location - The stuff indexed
//	LeafIndexBlock - A block within this
//	Error - For exception classes
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           1/2/96      jd_2     Initial version
//
//
// IMPLEMENTATION NOTES:
//	Index is the chief class of a set of classes that support indexing.
//	Taken together, these classes constitute a B-Tree Index that maps
//	string ID's to Location objects (the Location objects indicate the
//	location of Documents with the Indexed ID's).  
//
//	Any good text on database implementation will describe B-Trees.  
//	The text used for this implementation is _Principles_of_Database_
//	_Systems_ by Jeffrey D. Ullman, published in 1982 by Computer Science 
//	Press, Inc. (1803 Research Boulevard, Rockville, Maryland 20850).
//	ISBN 0-914894-36-6  QA76.9.D3U44 1983
//
//	A B-Tree is a tree that is kept in balance by its insertion 
//	and deletion operations.  Each node in a B-Tree can have multiple 
//	entries.  A B-tree mapping the first 8 presidents to their order
//	might contain the following nodes:
//
//	Node 0:
//		Adams		Node 1
//		Jefferson	Node 2
//		Van Buren	Node 3
//
//	Node 1:
//		Adams		Order 2
//		Adams		Order 6
//		Jackson		Order 7
//
//	Node 2:
//		Jefferson	Order 3
//		Madison		Order 4
//		Monroe		Order 5
//
//	Node 3:
//		Van Buren	Order 8
//		Washington	Order 1
//		----------      -------
//		
//	An inner (non-leaf) node contains the first key for each of its 
//	children nodes plus the location of that node.  Each child node 
//	contains up to three keys plus the indexed location data associated 
//	with those keys.
//
//	When inserting data to a binary tree, first the leaf node for the
//	data is found.  Then, if there is room for a new entry, it is added
//	to the leaf.  If the new entry would be the first entry for the node,
//	the node's parent is updated to reflect the new first entry.
//
//	If there is no room for the new entry, the current node is split
//	with half of the entries going to a new sibling node.  The parent
//	is updated to point to the new sibling node.  If this makes the
//	parent too large, it too is split, and its parent is updated to
//	point to the new parent node.  (If the parent is the root node, this
//	causes the creation of a new root node holding the parent and
//	its new sibling as children.)
//	
//	When deleting an entry, the leaf node for the entry is first found.
//	The entry is then deleted from the leaf node.  If this reduces
//	the size of the leaf node to less then half the allowable size,
//	then it is balanced with its sibling node.  If the sibling contains
//	room for all of the leaf node's entries, then they are moved to the
//	sibling and the leaf node is removed (thus causing the removal
//	of its pointer from the parent node, recursively invoking the
//	remove-and-balance procedure).  If there is not enough room in
//	the sibling for this node, then entries are moved from the sibling
//	to this leaf node until the two nodes have the same number of entries.
//
//	Finding an entry in the node is a straight-forward process of 
//	walking the tree, always picking the appropriate branch to walk
//	down based on the target ID.  Once the appropriate leaf node is
//	found, the entry can be identified based on its ID.
//
//	This implementation occasionally varies this standard B-Tree approach.
//	These variants should be clear from the code and comments.
//
//	In this implementation, several other classes support the
//	B-Tree.  In particular, the following classes are involved:
//
//		Index - Encompasses the entire B-Tree, keeping a pointer to
//				the root node of the tree.
//		IndexBlock - A node on the Tree
//			InnerIndexBlock - An inner node.
//			LeafIndexBlock - A leaf node
//		IndexEntry - A data slot, mapping an ID to data or another
//				node
//			InnerIndexEntry - Maps an ID to a node,
//					used in an InnerIndexBlock
//			LeafIndexEntry - Maps an ID to a Location,
//					used in a LeafIndexBlock
//		RecordFile - Used to provide persistent storage of IndexBlocks
//		Location - The data that is indexed
//


#include <iostream.h>			// for I/O
#include <strstream.h>			// for string I/O
#include <stddef.h>			// for NULL

#include "cmcIndex.h" 			// declaration for this
#include "cmcRecordFile.h"		// for storage of the Index
#include "cmcLocation.h"		// items indexed
#include "cmcInnerIndexBlock.h"		// type of data in Index tree
#include "cmcLeafIndexBlock.h"		// type of data in Index tree
#include "cmcError.h"			// for exception classes


// the following is the max number of blocks to allow allocated for the tree at
//    any time.  This does not include the root block.
const int Index::MaxAllocatedBlocks = 100;



//
// Function Name:	Index::Index()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Construct an Index to map to one that already
//			exists in persistent storage.
//
// Parameters:		Name - Name of the Index
//
// Return Value:	None
//
Index::Index(const char *Name) :
	AllocatedBlockCount(0),
	TreeFile(GetFilename(Name)),
	pIndexRoot(NULL),
	pAllocatedHead(NULL),
	pAllocatedTail(NULL)
{
    int HeaderLength;			// length of header read in
    char *Header = NULL;		// the header

    try
    {
    	// get in the top offset from the header
    	if((Header = TreeFile.ReadHeader(HeaderLength)) == NULL)
	    throw LogiconErrors::Read("Index::Index(char*)");
    
    	istrstream Input(Header, HeaderLength);	// stream version of input

	// read the header information
    	Input >> TopOffset;

	// test that read
	if(!Input)
	    throw LogiconErrors::Read("Index::Index(char *)");

    	delete[] Header;
    }
    catch(...)
    {
	if(Header != NULL)
	    delete[] Header;
	throw;
    }
}




//
// Function Name:	Index::Index()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Construct a new Index, setting it up in
//			persistent storage
//
// Parameters:		Name - Name of the Index
//			int - A flag here to indicate this should be newly 
//				created
//
// Return Value:	None
//
Index::Index(const char *Name, int) :
	AllocatedBlockCount(0),
	TreeFile(GetFilename(Name), IndexBlock::Size),
	pIndexRoot(NULL),
	TopOffset(0),
	pAllocatedHead(NULL),
	pAllocatedTail(NULL)
{
    try
    {
	// create an index root
    	if((pIndexRoot = new LeafIndexBlock(*this)) == NULL)
	    throw LogiconErrors::MemoryAllocation("Index::Index(char*, int)");

	// unlink the root from the free list
	CheckOutBlock(*pIndexRoot);

	// store its offset
	TopOffset = pIndexRoot->GetOffset();

	// make a header for this
	CreateHeader();

    	// synchronize files
    	pIndexRoot->Sync();
    	TreeFile.Sync();
    }
    catch(...)
    {
	TreeFile.MarkForDelete();
	if(pIndexRoot != NULL)
	    delete pIndexRoot;
	throw;
    }
}



//
// Function Name:	Index::~Index()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Destructor
//
// Parameters:		None
//
// Return Value:	None
//
Index::~Index()
{
    TreeFile.Sync();

    // free the header block if necessary
    if(pIndexRoot != NULL)
	delete pIndexRoot;
}



//
// Function Name:	Index::Destroy()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Destroy an Index.
//
// Parameters:		Name - name of index to destroy
//
// Return Value:	None
//
void Index::Destroy(const char *Name)
{
    RecordFile::Destroy(GetFilename(Name));
}


//
// Function Name:	Index::Find()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Find an item in the Index
//
// Parameters:		Id - ID of item to find
//
// Return Value:	Location block of the item
//
const Location &Index::Find(const char *Id)
{
    return FindIndexBlock(Id).FindId(Id);
}



//
// Function Name:	Index::Remove()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Remove an item from the Index
//
// Parameters:		Id - ID of the item to remove
//			Place - Location in which it occurs
//
// Return Value:	None
//
void Index::Remove(const char *Id, const Location &Place)
{
    FindIndexBlock(Id, Place).Remove(Id, Place);
}



//
// Function Name:	Index::AddId()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Add an item to the Index
//
// Parameters:		Id - ID of the item to add
//			Place - Indexed location to store for the item
//
// Return Value:	None
//
void Index::AddId(const char *Id, Location &Place)
{
    FindIndexBlock(Id, Place).AddId(Id, Place);
}



//
// Function Name:	Index::Sync()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Synchronize the persistent version of the Index with
//			the in-memory version
//
// Parameters:		None
//
// Return Value:	None
//
void Index::Sync()
{
    // if the IndexRoot is present, sync the Index tree
    if(pIndexRoot != NULL)
	pIndexRoot->Sync();

    // Write out the header
    CreateHeader();

    // synchronize everything out there
    TreeFile.Sync();
}




//
// Function Name:	Index::FindIndexBlock()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Find the block of the Index that would contain
//			a specific ID
//
// Parameters:		Id - The ID to look for
//
// Return Value:	The IndexBlock that holds it
//
IndexBlock &Index::FindIndexBlock(const char *Id)
{
    // if the top block is not read in...
    if(pIndexRoot == NULL)
    {
	pIndexRoot = ReadIndexBlock(*this, NULL, TopOffset);
	CheckOutBlock(*pIndexRoot);
    }

    return pIndexRoot->FindIndexBlock(Id);
}


//
// Function Name:	Index::FindIndexBlock()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Find the block of the Index that would contain
//			a specific ID
//
// Parameters:		Id - The ID to look for
//			Location - Data associated with it
//
// Return Value:	The IndexBlock that holds it
//
IndexBlock &Index::FindIndexBlock(const char *Id, const Location &Place)
{
    // if the top block is not read in...
    if(pIndexRoot == NULL)
    {
	pIndexRoot = ReadIndexBlock(*this, NULL, TopOffset);
	CheckOutBlock(*pIndexRoot);
    }

    return pIndexRoot->FindIndexBlock(Id, Place);
}




//
// Function Name:	Index::GetFilename()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Construct the Index filename given its Index name.
//
// Parameters:		BaseName - the name of the Index
//
// Return Value:	The name of the file
//
const char *Index::GetFilename(const char *BaseName)
{
    ostrstream Output;			// output stream
    static char *Filename = NULL;	// to hold the filename

    // if there is a filename, delete it
    if(Filename != NULL)
	delete[] Filename;

    // put the file stuff out
    Output << BaseName << ".idx" << '\0';

    // check Output for errors
    if(!Output)
	throw LogiconErrors::Write("Index::GetFilename(char *)");

    // get the filename from the buffer
    Filename = Output.str();

    return Filename;
}



//
// Function Name:	Index::SetIndexRoot()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Store the root IndexBlock for the Index.
//			This does not delete the old root - the caller of
//			this is responsible to ensure that it does not
//			become a dangling reference.
//
// Parameters:		pNewRoot - The new value of the Root
//
// Return Value:	The previous index root.
//
IndexBlock *Index::SetIndexRoot(IndexBlock *pNewRoot)
{
    IndexBlock *pResult = pIndexRoot;		// return the current top index

    CheckInBlock(*pIndexRoot);
    pIndexRoot = pNewRoot;
    CheckOutBlock(*pIndexRoot);
    TopOffset = pNewRoot->GetOffset();

    return pResult;
}



//
// Function Name:	Index::GetIndexFile()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Get the IndexFile for this Index.
//
// Parameters:		None
//
// Return Value:	The TreeFile component
//
RecordFile &Index::GetIndexFile()
{
    return TreeFile;
}



//
// Function Name:	Index::CreateHeader()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Create the header for this Index and store it out.
//
// Parameters:		None
//
// Return Value:	None
//
void Index::CreateHeader()
{
    ostrstream Output;				// place to put header info
    char *Header = NULL;			// the header string version

    try
    {
    	// note location of current Top Index
    	Output << TopOffset;

    	// check Output for errors
    	if(!Output)
	    throw LogiconErrors::Write("Index::GetFilename(char *)");

    	Header = Output.str();

    	TreeFile.WriteHeader(Header, Output.pcount());

    	delete[] Header;
    }
    catch(...)
    {
	delete[] Header;
	throw;
    }
}




//
// Function Name:	Index::Display()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Do a debug-dump of this Index to an output
//			stream.
//
// Parameters:		Output - The stream to dump to
//
// Return Value:	None
//
void Index::Display(ostream &Output)
{
#if 0
    Output << "\n";

    Output << "Allhead: " << pAllocatedHead << endl;
    Output << "Alltail: " << pAllocatedTail << endl;

    for(IndexBlock *pBlock = pAllocatedHead; 
	pBlock != NULL; 
	pBlock = pBlock->GetNextAllocated())
    {
	Output << "Block: " << pBlock << endl;
	Output << "Next:  " << pBlock->GetNextAllocated() << endl;
	Output << "Prev:  " << pBlock->GetPreviousAllocated() << endl;
    }

    Output << endl;
#endif

    // read it in if necessary
    if(pIndexRoot == NULL)
    {
	pIndexRoot = ReadIndexBlock(*this, NULL, TopOffset);
    	CheckOutBlock(*pIndexRoot);
    }

    Output << "Top block: " << TopOffset << "\nAllocatedBlockCount: " 
		<< AllocatedBlockCount << "\n";
    Output << "Allocation head: " << pAllocatedHead << "\nAllocated tail: "
		<< pAllocatedTail << "\n";

    pIndexRoot->Display(Output, 0);

    if(!Output)
	throw LogiconErrors::Write("Index::Display(ostream&)");
}



//
// Function Name:	Index::Display()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Do a debug-dump of this Index to cout.
//
// Parameters:		None
//
// Return Value:	None
//
void Index::Display()
{
    Display(cout);
}



//
// Function Name:	Index::CheckInBlock()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Add the block to the list of all allocated blocks
//			in the Index.
//
// Parameters:		Block - The block to add on
//
// Return Value:	None
//
void Index::CheckInBlock(IndexBlock &Block)
{
    // if necessary, clear a block for this
    if(++AllocatedBlockCount > MaxAllocatedBlocks)
    {
	// clear a block
	pAllocatedTail->GetParent().ClearBlock(*pAllocatedTail);
    }

    if(pAllocatedHead != NULL)
    {
	pAllocatedHead->SetPreviousAllocated(&Block);
    }
    else
    {
	pAllocatedTail = &Block;
    }

    Block.SetNextAllocated(pAllocatedHead);
    Block.SetPreviousAllocated(NULL);
    pAllocatedHead = &Block;
}


//
// Function Name:	Index::CheckOutBlock()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Remove the block from the list of all allocated blocks
//			in the Index.
//
// Parameters:		Block - The block to remove
//
// Return Value:	None
//
void Index::CheckOutBlock(IndexBlock &Block)
{
    // set PreviousAllocated in next block to PreviousAllocated in this block
    if(Block.GetNextAllocated() != NULL)
	Block.GetNextAllocated()->SetPreviousAllocated(Block.GetPreviousAllocated());

    // set NextAllocated in prev block to NextAllocated in this block
    if(Block.GetPreviousAllocated() != NULL)
	Block.GetPreviousAllocated()->SetNextAllocated(Block.GetNextAllocated());

    // update allocated head as needed
    if(pAllocatedHead == &Block)
    {
    	pAllocatedHead = Block.GetNextAllocated();
    }

    // update allocated tail as needed
    if(pAllocatedTail == &Block)
    {
	pAllocatedTail = Block.GetPreviousAllocated();
    }

    // clear the next and previous in the block
    Block.SetNextAllocated(NULL);
    Block.SetPreviousAllocated(NULL);

    // decrement the count of allocated blocks
    --AllocatedBlockCount;
}



//
// Function Name:	Index::TouchBlock()
//
// CSCI/CSC/CSU:	CMC/Index
//
// Description:		Move the block to the front of the list of all 
//			allocated blocks in the Index, thus marking it as
//			the most-recently-used block.
//
// Parameters:		Block - The block to remove
//
// Return Value:	None
//
void Index::TouchBlock(IndexBlock &Block)
{
    CheckOutBlock(Block);
    CheckInBlock(Block);
}




