//
// Module Name:		cmcIndex.h
//
// CSCI/CSC/CSU:	CMC/Index
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//		RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	.h for the Index.  An Index maintains a mapping from a string
//	ID to a volume number and offset.
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#ifndef _cmcIndex_h
#define _cmcIndex_h

#include <iostream.h>			// to allow I/O of this

class Index;				// predeclaration
#ifdef __GNUG__
class IndexBlock;
class InnerIndexBlock;
class LeafIndexBlock;
#endif

#include "cmcIndexBlock.h"		// nodes on the B-Tree
#ifdef __GNUG__
#include "cmcInnerIndexBlock.h"		// nodes on the B-Tree
#include "cmcLeafIndexBlock.h"		// nodes on the B-Tree
#endif
#include "cmcRecordFile.h"		// to maintain storage of this
#include "cmcLocation.h"		// for return values

// Index is a persistent representation of a B-Tree
class Index
{
  private:
	static const int MaxAllocatedBlocks;
					// maximum number of blocks to allow 
					//	active
	int AllocatedBlockCount;	// count of blocks read in
	RecordFile TreeFile;		// File containing B-Tree
	IndexBlock *pIndexRoot;		// Top index in B-Tree
	int TopOffset;			// Offset to TopBlock in file

	IndexBlock *pAllocatedHead;	// head of the allocated block list
	IndexBlock *pAllocatedTail;	// tail of the allocated block list

	IndexBlock &FindIndexBlock(const char *Id);
					// find the block for Id
	IndexBlock &FindIndexBlock(const char *Id, const Location &Place);
					// find the block for Id + Place
	static const char *GetFilename(const char *Name);	
					// find the Filename for index Name
	IndexBlock *SetIndexRoot(IndexBlock *pNewRoot);
					// set the pIndexRoot component
					// do NOT delete the current pIndexRoot
					// Return the current pIndexRoot
	void CreateHeader(void);	// create and write out header block

	friend class IndexBlock;	// allow IndexBlocks access to this
	friend class InnerIndexBlock;		
	friend class LeafIndexBlock;	

  public:
	Index(const char *Name); 	// Construct an Index from the named
					//  file.
	Index(const char *Name, int); 	// construct a new Index with the named
					//  file.  The int parameter is here
					//  only to distinguish it from the
					//  other constructor.
	~Index(void);			// Destructor
	static void Destroy(const char *Name);
					// Deletes the Index file
	const Location &Find(const char *BaseId);
					// Find Location corresponding to
					// the given Base ID
	void Remove(const char *Id, const Location &Place);
					// Remove the given item from the Index
	void AddId(const char *BaseId, Location &Place);
					// Add an entry mapping BaseId to 
					// LocalId
	RecordFile &GetIndexFile(void); // get the TreeFile component
	void Sync(void);		// synchronize the output
	void Display(ostream &Output);  // a debug function that displays the
					//   Index to Output
	void Display(void);		// a debug function that displays the
					//   Index to cout

	void CheckInBlock(IndexBlock &Block);
					// note that a block has been allocated
	void CheckOutBlock(IndexBlock &Block);
					// note that a block has been freed
	void TouchBlock(IndexBlock &Block);
					// note that a block has been touched
};

#endif
