//
// Module Name:		cmcIndexBlock.cc
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	An IndexBlock is a node on the Index's B-Tree.  For details on
//	its place in the B-Tree, see the Implementation Notes comments
//	in the cmcIndex.cc file.
//
// Function List:
//	IndexBlock::IndexBlock() - This has 2 constructors
//	IndexBlock::~IndexBlock() - Destructor
//	IndexBlock::FindId() - Find data for a specified ID
//	IndexBlock::Remove() - Remove data for an ID
//	IndexBlock::GetParent() - Get the parent to this node
//	IndexBlock::SetParent() - Set the parent to this node
//	IndexBlock::AddId() - Add a data item to the node
//	IndexBlock::GetId() - Get the ID of the node
//	IndexBlock::GetLocation() - Get the Location of the node
//	IndexBlock::Length() - Get the number of entries in the node
//	IndexBlock::GetOffset() - Get the offset to the node
//	IndexBlock::MarkForDelete() - Mark this to be deleted
//	IndexBlock::Transfer() - Transfer data from another node to this
//	IndexBlock::EqualizeSize() - Balance the size of this with another
//	IndexBlock::AddEntry() - Add an entry to this node
//	IndexBlock::HandleOverflow() - Handle an overflowing block
//	IndexBlock::PutEntry() - Place an entry in a specific location in this
//	IndexBlock::GetIndex() - Get the Index of which this is a part
//	ReadIndexBlock() - Read an IndexBlock from file
//	IndexBlock::GetPreviousAllocated() - Get previous in allocated list
//	IndexBlock::SetPreviousAllocated() - Set previous in allocated list
//	IndexBlock::GetNextAllocated() - Get next in allocated list
//	IndexBlock::SetNextAllocated() - Set next in allocated list
//
// Other Modules Called:
//	RecordFile - To store these
//	Location - Indexed data
//	Index - This is part of an Index
//	InnerIndexBlock - A subclass of this
//	LeafIndexBlock - A subclass of this
//	IndexEntry - Kept in this node
//	Error - For exception classes
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           1/2/96      jd_2     Initial version
//

#include <stddef.h>			// for NULL
#include <iostream.h>			// for I/O
#include <strstream.h>			// for string I/O
#include <string.h>			// for string operations

#include "cmcIndex.h"			// this is part of an Index
#include "cmcIndexBlock.h"		// declaration for this
#include "cmcRecordFile.h"		// for storage of this
#include "cmcLocation.h"		// indexed data
#include "cmcInnerIndexBlock.h"		// subclass to this
#include "cmcLeafIndexBlock.h"		// subclass to this
#include "cmcError.h"			// for exception classes



// Size is the total size of an IndexBlock
const int IndexBlock::Size = 1024;


//
// Function Name:	IndexBlock::IndexBlock()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Construct a new IndexBlock that is not currently
//			stored in PersistentStorage.  Allocate a record
//			for it in the Index's file.
//
// Parameters:		NewIndex - The Index in which this block appears
//
// Return Value:	None
//
IndexBlock::IndexBlock(Index &NewIndex) :
	pNextAllocated(NULL),
	pPreviousAllocated(NULL),
	pParent(NULL),
	ParentIndex(NewIndex),
	Offset(ParentIndex.GetIndexFile().AllocateRecord()),
	MarkedForDelete(0),
	Changed(1)
{
    NewIndex.CheckInBlock(*this);
}



//
// Function Name:	IndexBlock::IndexBlock()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Construct an IndexBlock for an IndexBlock that is
//			already stored in persistent storage.
//
// Parameters:		NewIndex - the Index in which the block appears
//			pNewParent - The parent node to this 
//			NewOffset - Offset to the block in the RecordFile
//
// Return Value:
//
IndexBlock::IndexBlock(Index &NewIndex, InnerIndexBlock *pNewParent, 
		int NewOffset):
	pNextAllocated(NULL),
	pPreviousAllocated(NULL),
	pParent(pNewParent),
	ParentIndex(NewIndex),
	Offset(NewOffset),
	MarkedForDelete(0),
	Changed(0)
{
    NewIndex.CheckInBlock(*this);
}



//
// Function Name:	IndexBlock::~IndexBlock()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Destruct an IndexBlock.
//
// Parameters:		None
//
// Return Value:	None
//
IndexBlock::~IndexBlock()
{
    // check it out at the parent
    ParentIndex.CheckOutBlock(*this);

    // if we should delete this persistently...
    if(MarkedForDelete)
    {
	// delete it from the TreeFile
	ParentIndex.GetIndexFile().FreeRecord(Offset);
    }
}




//
// Function Name:	IndexBlock::FindId()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Find an entry in this block.  This function is
//			only valid for LeafIndexBlocks.  Therefore, this
//			throws an exception.
//
// Parameters:		char * - here to satisfy calling format
//
// Return Value:	NULL (never actually returned - this throws an 
//				exception)
//
const Location &IndexBlock::FindId(const char *) const
{
    throw LogiconErrors::InvalidObject("IndexBlock::FindId(char *)");

    // the following is here to satisfy the compiler: 
    //     it will never be run because of the unconditional throw above
    return *(Location *)NULL;
}




//
// Function Name:	IndexBlock::Remove()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Remove an entry from this block.  This function is
//			only valid for LeafIndexBlocks and therefore this
//			throws an exception.
//
// Parameters:		char * - here to satisy calling format
//			Location & - here to satisfy calling format
//
// Return Value:	NULL - returned to satisfy calling format
//
void IndexBlock::Remove(const char*, const Location&)
{
    throw LogiconErrors::InvalidObject("IndexBlock::Remove(char *)");
}




//
// Function Name:	IndexBlock::GetParent()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Get the parent block to this block.
//
// Parameters:		None
//
// Return Value:	The parent block
//
InnerIndexBlock &IndexBlock::GetParent()
{
    if(pParent == NULL)
	throw LogiconErrors::NotFound("IndexBlock::GetParent()");

    return *pParent;
}




//
// Function Name:	IndexBlock::SetParent()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Set the Parent of this node to something else.
//
// Parameters:		NewParent - new parent node value
//
// Return Value:	None
//
void IndexBlock::SetParent(InnerIndexBlock &NewParent)
{
    pParent = &NewParent;
}



//
// Function Name:	IndexBlock::SetParent()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Set the Parent to NULL
//
// Parameters:		None
//
// Return Value:	None
//
void IndexBlock::SetParent()
{
    pParent = NULL;
}




//
// Function Name:	IndexBlock::AddId()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Add an ID to this block.  This is only valid
//			for LeafIndexBlocks and this therefore throws
//			an exception.
//
// Parameters:		char *, Location& - here to satisfy calling format
//
// Return Value:
//
void IndexBlock::AddId(const char*, const Location&)
{
    throw LogiconErrors::InvalidObject("IndexBlock::AddId(char *, Location&)");
}




//
// Function Name:	IndexBlock::GetId()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Get the "ID" of this node.  The ID of a node is
//			the ID of its first data entry.
//
// Parameters:		None
//
// Return Value:	The ID of the node
//
const char *IndexBlock::GetId() const
{
    if(Entries.Length() == 0)
	throw LogiconErrors::NotFound("IndexBlock::GetId()");
    else
	return Entries.Nth(0).GetId();
}



//
// Function Name:	IndexBlock::GetLocation()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Get the Location of this node.  The Location of a node 
//			is the Location of its first data entry.
//
// Parameters:		None
//
// Return Value:	The Location of the node
//
const Location &IndexBlock::GetLocation() const
{
    return Entries.Nth(0).GetLocation();
}




//
// Function Name:	IndexBlock::Length()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Return the number of entries in this node.
//
// Parameters:		None
//
// Return Value:	The entry count.
//
int IndexBlock::Length() const
{
    return Entries.Length();
}


//
// Function Name:	IndexBlock::GetOffset()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Get the Offset of this node.
//
// Parameters:		None
//
// Return Value:	The Offset component.
//
int IndexBlock::GetOffset() const
{
    return Offset;
}



//
// Function Name:	IndexBlock::MarkForDelete()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Mark this so that it is deleted from persistent
//			storage when it is destructed.
//
// Parameters:		None
//
// Return Value:	None
//
void IndexBlock::MarkForDelete()
{
    MarkedForDelete = 1;
}




//
// Function Name:	IndexBlock::Transfer()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Transfer all entries from a source node to this node.
//
// Parameters:		Source - the node to get stuff from
//
// Return Value:	None
//
void IndexBlock::Transfer(IndexBlock &Source)
{
    // check that the subtypes are the same
    if(GetType() != Source.GetType())
	throw LogiconErrors::Parameter("IndexBlock::Transer(IndexBlock&)");

    // get everything from source to here - do it backwards because
    //     Length() will be changing
    for(int i = Source.Entries.Length(); i > 0; --i)
	PutEntry(Entries.Length(), Source.Entries.PopNth(0));

    // note the nodes have changed
    Changed = 1;
    Source.Changed = 1;
}



//
// Function Name:	IndexBlock::EqualizeSize()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Balance the number of entries between this node
//			and its neighbor (i.e., a sibling node).
//			If either had their first entry changed, 
//			update the parent to reflect the
//			change.
//
// Parameters:		Neighbor - the neighbor node of this
//
// Return Value:	None
//
void IndexBlock::EqualizeSize(IndexBlock &Neighbor)
{
    if(GetType() != Neighbor.GetType())
	throw LogiconErrors::Parameter("IndexBlock::Transer(IndexBlock&)");

    // note what the Id and Location for Neighbor was coming into this
    const char *OldId = Neighbor.GetId();
    const Location OldLocation(Neighbor.GetLocation());

    // if the Neighbor is not greater than this...
    int Comparison = strcmp(OldId, GetId());
    if(Comparison < 0 || (Comparison == 0 && OldLocation < GetLocation()))
    {
	// reverse the call
	Neighbor.EqualizeSize(*this);
    }
    else
    {
    	// if this one is longer then the Neighbor
    	if(Entries.Length() > Neighbor.Entries.Length())
    	{
    	    // check that the subtypes are the same
    	    // do as long as sizes differ by more than one
    	    while((Entries.Length() - Neighbor.Entries.Length()) > 1)
    	    {
	    	// get the last item from this and add it to the right neighbor
	    	Neighbor.PutEntry(0, Entries.Pop());
    	    }
    	}
    	else
    	{
    	    // check that the subtypes are the same
    	    // do as long as sizes differ by more than one
    	    while((Neighbor.Entries.Length() - Entries.Length()) > 1)
    	    {
	    	// get the first item from Neighbor and add it here
	    	PutEntry(Entries.Length(), Neighbor.Entries.PopNth(0));
    	    }

    	}

    	// propagate the change in Neighbor upwards
    	if(pParent != NULL)
	    pParent->ChangeId(OldId, 
	 		      OldLocation, 
			      Neighbor.GetId(),
			      Neighbor.GetLocation());

    	// note the nodes have changed
    	Changed = 1;
    	Neighbor.Changed = 1;
    }
}



//
// Function Name:	IndexBlock::AddEntry()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Add a new entry to this block.  If that makes this
//			block too large, then construct a new sibling block
//			for this and add it as a new child to the parent of
//			this.  Adding it as a child to the parent is
//			recursive and could cause the splitting up of the
//			parent of this.
//
// Parameters:		pData - The entry to add
//
// Return Value:	None
//
void IndexBlock::AddEntry(IndexEntry *pData)
{
    // Find the location to use
    int TargetOffset = -1;              // place to put item
    for(int i = 0; i < Entries.Length() && TargetOffset == -1; ++i)
    {
        // if it should go here...
	int Comparison = strcmp(Entries.Nth(i).GetId(), pData->GetId());
        if(Comparison > 0 || 
		(Comparison == 0 && 
			Entries.Nth(i).GetLocation() > pData->GetLocation()))
        {
            TargetOffset = i;
        }
    }    

    // if not found, should be end of list
    if(TargetOffset == -1)
        TargetOffset = Entries.Length();

    // if this now becomes the first entry and there is a parent...
    if(TargetOffset == 0 && pParent != NULL)
    {
	// change the ID for this in the parent
	pParent->ChangeId(Entries.Nth(0).GetId(),
			  Entries.Nth(0).GetLocation(),
			  pData->GetId(),
			  pData->GetLocation());
    }

    // put the item at the found location
    Entries.Insert(TargetOffset, pData);

    //
    // Add a new block if necessary
    //
    if(Entries.Length() > MaxLength())
    {
	HandleOverflow();
    }

    Changed = 1;
}




//
// Function Name:	IndexBlock::HandleOverflow()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Handle the situation when too many entries have been
//			added to a block.  Handle this by moving the excess
//			entries to a neighbor of this if possible.  If not
//			possible, split this into two blocks.
//
// Parameters:		None
//
// Return Value:	None
//
void IndexBlock::HandleOverflow()
{
    IndexBlock *pSibling;		// sibling to this block

    // find the sibling to this block
    if(pParent == NULL)
	pSibling = NULL;
    else
    {
	try
	{
	    pSibling = &pParent->FindSibling(this);
	}
	catch(LogiconErrors::OutOfBounds)
	{
	    pSibling = NULL;
	}
    }

    // if there is a sibling and it has room for these...
    if(pSibling != NULL && pSibling->Length() < MaxLength())
    {
	// balance with that sibling
	EqualizeSize(*pSibling);
    }
    else
    {
        // we must create a new block for half of the overflow entries
        IndexBlock *pNewBlock = CreateNewBlock();

        // put in half of the entries
        int TargetLength = Entries.Length() / 2;

	// must count down so that we don't use Entries.Length() as endpoint
	//    (it is a moving target)
        for(int j = Entries.Length(); j > TargetLength; --j)
        {
	    pNewBlock->PutEntry(pNewBlock->Entries.Length(), 
				Entries.PopNth(TargetLength));
        }
	pNewBlock->Changed = 1;
 
        // add the new block to the parent
        if(pParent != NULL)
            pParent->Insert(pNewBlock);
        else
        {
            // This is the top of the Index: construct a new parent for this
            InnerIndexBlock *pNewParent =
                        InnerIndexBlock::Construct(ParentIndex);
 
            // add in this and the new sibling
            pNewParent->Insert(this);
            pNewParent->Insert(pNewBlock);
  
            // record this new parent
            ParentIndex.SetIndexRoot(pNewParent);
        }
    }
}




//
// Function Name:	IndexBlock::PutEntry()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Add an entry to this node at some specified
//			location.
//
// Parameters:		TargetOffset - The place to add the entry
//			pData - The entry to add
//
// Return Value:	None
//
void IndexBlock::PutEntry(int TargetOffset, IndexEntry *pData)
{
    Entries.Insert(TargetOffset, pData);
}





//
// Function Name:	IndexBlock::GetIndex()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Get the parent Index of this node.
//
// Parameters:		None
//
// Return Value:	The ParentIndex component.
//
Index &IndexBlock::GetIndex() const
{
    return ParentIndex;
}




//
// Function Name:	IndexBlock::ReadIndexBlock()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Read an IndexBlock from file, constructing the
//			appropriate subclass of IndexBlock (depending
//			on what is actually stored).
//
// Parameters:		NewParentIndex - The parent index to the new block
//			pNewParent - the parent block to the new block
//			Offset - Offset to the read-block in the RecordFile
//
// Return Value:
//
IndexBlock *ReadIndexBlock(Index &NewParentIndex, InnerIndexBlock *pNewParent, 
			int Offset)
{
    char *Data;				// to hold the data
    int Length;				// to hold its length
    IndexBlock *pResult = NULL;		// data results

    // read in the Data
    Data = NewParentIndex.GetIndexFile().ReadData(Offset, Length);

    try
    {
    	// link an input stream to it
    	istrstream Input(Data, Length);

    	// Get out the type
    	IndexBlock::IndexBlockType Type;		// the Type

    	int FileLocation = Input.tellg();
    	if(!Input.read((char *)&Type, sizeof(Type)))
    	{
	    throw LogiconErrors::Read(
		"ReadIndexBlock(Index&, InnerIndexBlock*, int)");
        }
    	Input.seekg(FileLocation);

    	// do based on type
    	switch(Type)
    	{
	    case IndexBlock::InnerBlock:
	    	pResult = InnerIndexBlock::Construct(NewParentIndex, pNewParent,
				Offset, Input);
	    	break;

	    case IndexBlock::LeafBlock:
	    	pResult = new LeafIndexBlock(NewParentIndex, pNewParent, 
				Offset, Input);
	    	break;

	    default:
	    	throw LogiconErrors::Format(
		    "ReadIndexBlock(Index&, InnerIndexBlock*, int)");
    	}

    	delete[] Data;
    }
    catch(...)
    {
	delete[] Data;
	throw;
    }

    return pResult;
}



//
// Function Name:	IndexBlock::GetPreviousAllocated()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Get the previously-allocated block pointer to
//			this block.
//
// Parameters:		None
//
// Return Value:	The pPreviousAllocated member
//
IndexBlock *IndexBlock::GetPreviousAllocated()
{
    return pPreviousAllocated;
}


//
// Function Name:	IndexBlock::SetPreviousAllocated()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Set the previously-allocated block pointer to
//			this block.
//
// Parameters:		pNewPrev - Value to set of the pointer
//
// Return Value:	None
//
void IndexBlock::SetPreviousAllocated(IndexBlock *pNewPrev)
{
    pPreviousAllocated = pNewPrev;
}




//
// Function Name:	IndexBlock::GetNextAllocated()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Get the next-allocated block pointer to
//			this block.
//
// Parameters:		None
//
// Return Value:	The pNextAllocated member
//
IndexBlock *IndexBlock::GetNextAllocated()
{
    return pNextAllocated;
}


//
// Function Name:	IndexBlock::SetNextAllocated()
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Description:		Set the next-allocated block pointer to
//			this block.
//
// Parameters:		pNewPrev - Value to set of the pointer
//
// Return Value:	None
//
void IndexBlock::SetNextAllocated(IndexBlock *pNewPrev)
{
    pNextAllocated = pNewPrev;
}









