//
// Module Name:		cmcIndexBlock.h
//
// CSCI/CSC/CSU:	CMC/IndexBlock
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//		RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	.h for the IndexBlock.  IndexBlock is an abstract class representing
//	blocks of an Index.
//
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#ifndef _cmcIndexBlock_h
#define _cmcIndexBlock_h

#include <iostream.h>			// to allow I/O of this

class IndexBlock;                       // Incomplete Declaration

#include "cmcLocation.h"		// data stored within this
#include "cmcIndex.h"			// item in which this is kept
#include "cmcSet.h"			// to allow sets of data
#include "cmcIndexEntry.h"		// data kept in the block

class InnerIndexBlock;			// incomplete declaration:
					// will not compile correctly otherwise
					// as InnerIndexBlock is-a IndexBlock
#ifdef __GNUG__
class LeafIndexBlock;
#endif

// IndexBlock is a B-Tree block used in the Index file
// This is a virtual class: children will indiate if this is an inner or leaf
//       node.
class IndexBlock
{
  private:
	IndexBlock *pNextAllocated;	// next block on the allocation list
	IndexBlock *pPreviousAllocated;	// previous block on the allocation list

  protected:
	Index &ParentIndex;		// parent index to this
	InnerIndexBlock *pParent;	// parent of this
	int IdLength;			// length of an Id
	int Offset;			// offset to this in file
	int MarkedForDelete;		// has this been marked for deletion?
	Set<IndexEntry> Entries;	// entries in this block
	int Changed;			// flag to show if this has changed

	void AddEntry(IndexEntry *pData);
					// Add a data entry to the IndexBlock
	virtual void PutEntry(int TargetOffset, IndexEntry *pData);
					// put entry to position TargetOffset
	void HandleOverflow(void);	// handle overflow in the block

  public:
	enum IndexBlockType {InnerBlock, LeafBlock};
					// types of IndexBlock

	static const int Size;		// length of an IndexBlock

      	IndexBlock(Index &NewIndex); 	// construct empty new block
	IndexBlock(Index &NewIndex, 
		InnerIndexBlock *pNewParent, 
		int NewOffset);		// construct block by reading it in
					// do NOT take ownership of pNewParent
	virtual ~IndexBlock(void);	// destructor

	virtual const Location &FindId(const char *Id) const;	
					// locate block for an Id
	virtual void Remove(const char *Id, const Location &Place);
					// remove an Id
	virtual void AddId(const char *Id, const Location &Place);
					// add an Id to the block
	
	virtual void Sync(void) = 0;	// synchronize this to file
	virtual IndexBlock &FindIndexBlock(const char *Id) = 0;
					// find the base block for the Id
	virtual IndexBlock &FindIndexBlock(const char *Id, 
					   const Location &Place) = 0;
					// find the base block for the Item
	virtual int MaxLength(void) const = 0;
					// get the max number of entries herein
	virtual IndexBlockType GetType(void) const = 0;
					// get the type of IndexBlock
	virtual IndexBlock *CreateNewBlock(void) const = 0;
					// create a new IndexBlock of this type
	virtual void Display(ostream &Output, int Indent) = 0; 
					// a debug function to display the block

	const char *GetId(void) const; 	// get the Id for this block
	const Location &GetLocation(void) const;
					// get the Location for this block
	int Length(void) const;		// get the length of this
	int GetOffset(void) const;	// get offset of this block
	void MarkForDelete(void);	// mark this for delete on destruction
	void Transfer(IndexBlock &Source);
					// transfer data from pSource to this
	void EqualizeSize(IndexBlock &Target);
					// make this and pTarget of same size
	InnerIndexBlock &GetParent(void);
					// get the parent of this
	void SetParent(InnerIndexBlock &NewParent);
					// note the parent of this
	void SetParent(void);		// note there is no parent for this
	Index &GetIndex(void) const;	// get the Index of this

	IndexBlock *GetNextAllocated(void);
					// get the next allocated block: 
					//  do NOT return ownership of the block
	IndexBlock *GetPreviousAllocated(void);
					// get the previous allocated block:
					//  do NOT return ownership of the block
	void SetNextAllocated(IndexBlock *pNextAllocated);
					// set the next allocated pointer
					//  do NOT take ownership of the block
	void SetPreviousAllocated(IndexBlock *pNextAllocated);
					// set the previous allocated pointer
					//  do NOT take ownership of the block
};


IndexBlock *ReadIndexBlock(Index&, InnerIndexBlock *pNewParent, int Offset);
					// Read in a new index block from file:
					//   do NOT take ownership of pNewParent

#endif
