//
// Module Name:		cmcInnerIndexBlock.cc
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for InnerIndexBlock.  An InnerIndexBlock is an internal node
//	on an Index's B-Tree.  For more details on the B-Tree, see the
//	Implementation Notes comment in cmcIndex.cc.
//
// Function List:
//	InnerIndexBlock::InnerIndexBlock() - There are three constructors for
//			this
//	InnerIndexBlock::Construct() - There are three constructor surrogates 
//			for this
//	InnerIndexBlock::~InnerIndexBlock() - Destructor
//	InnerIndexBlock::ReadBlock() - Read in the details of a block from an
//			input stream
//	InnerIndexBlock::Sync() - Synchronize the block with disk
//	InnerIndexBlock::FindIndexBlock() - Find the block with a specific Id
//	InnerIndexBlock::MaxLength() - Get size of this block type
//	InnerIndexBlock::GetType() - Get the type of this block
//	InnerIndexBlock::CreateNewBlock() - Create a block of this type
//	InnerIndexBlock::Balance() - Balance the children of this node
//	InnerIndexBlock::Insert() - Insert an entry in this
//	InnerIndexBlock::ChangeId() - Change the ID of an entry in this
//	InnerIndexBlock::PutEntry() - Put an entry to this
//	InnerIndexBlock::Display() - Do a debug display of this
//	InnerIndexBlock::FindSibling() - Find the sibling of a block
//	InnerIndexBlock::ClearBlock() - Clear out a block from memory
//
// Other Modules Called:
//	RecordFile - to read and write from
//	Location - Data passed around
//	Index - This is in an Index
//	Misc - To get string I/O functions
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           1/2/96      jd_2     Initial version
//


#include <stddef.h>			// for NULL
#include <iostream.h>			// for I/O
#include <strstream.h>			// for string I/O
#include <string.h>			// for string functions

#include "cmcInnerIndexBlock.h"		// contains declaration of this
#include "cmcRecordFile.h"		// for storage
#include "cmcLocation.h"		// data stored herein
#include "cmcIndex.h"			// this is part of Index
#include "cmcMisc.h"			// For string I/O
#include "cmcError.h"			// for exception classes

const int InnerIndexBlock::HighBlockThreshold = 25;// max entries per block
const int InnerIndexBlock::LowBlockThreshold = 13;	// min entries per block

//const int InnerIndexBlock::HighBlockThreshold = 3;	
							// max entries per block
//const int InnerIndexBlock::LowBlockThreshold = 2;	
							// min entries per block


//
// Function Name:	InnerIndexBlock::InnerIndexBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Construct a brand-new InnerIndexBlock.
//
// Parameters:		NewIndex - The parent Index to this
//
// Return Value:	None
//
InnerIndexBlock::InnerIndexBlock(Index &NewIndex) :
	IndexBlock(NewIndex)
{
}


//
// Function Name:	InnerIndexBlock::InnerIndexBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Construct an InnerIndexBlock by reading it from
//			an input stream.
//
// Parameters:		NewIndex - The Index that this block will be in
//			pNewParent - The parent block to this block
//			NewOffset - the Offset to this block
//			Input - the Input stream containing the block data
//
// Return Value:
//
InnerIndexBlock::InnerIndexBlock(Index &NewIndex, InnerIndexBlock *pNewParent,
		int NewOffset, istream &Input) :
	IndexBlock(NewIndex, pNewParent, NewOffset)
{
    ReadBlock(Input);
}



//
// Function Name:	InnerIndexBlock::InnerIndexBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Construct an InnerIndexBlock by reading it in from
//			a RecordFile.
//
// Parameters:		NewIndex - Index in which this will appear
//			pNewParent - Parent block to this block
//			NewOffset - Offset to the block in the RecordFile
//
// Return Value:	None
//
InnerIndexBlock::InnerIndexBlock(Index &NewIndex, InnerIndexBlock *pNewParent,
		int NewOffset) :
	IndexBlock(NewIndex, pNewParent, NewOffset)
{
    int Length;				// block length
    char *Block = NULL;			// data block

    try
    {
    	// read in the block from file
    	Block = NewIndex.GetIndexFile().ReadData(Offset, Length);

    	istrstream Input(Block, Length);	// stream version of this

    	ReadBlock(Input);

    	delete[] Block;
    } 
    catch(...)
    {
	if(Block != NULL)
	    delete[] Block;
	throw;
    }
}



//
// Function Name:	InnerIndexBlock::Construct()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Construct a new InnerIndexBlock.  (InnerIndexBlock
//			forces construction through this function so that
//			it will be guaranteed to be allocated with new and
//			not on the stack.)
//
// Parameters:		NewIndex - The parent index to this
//
// Return Value:	The new block
//
InnerIndexBlock *InnerIndexBlock::Construct(Index &NewIndex)
{
    return new InnerIndexBlock(NewIndex);
}



//
// Function Name:	InnerIndexBlock::Construct()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Construct a new InnerIndexBlock by reading it from
//			a stream.  (InnerIndexBlock forces construction 
//			through this function so that it will be guaranteed 
//			to be allocated with new and not on the stack.)
//
// Parameters:		NewIndex - The parent index to this
//			pNewParent - The parent block to this
//			NewOffset - The offset to this in the RecordFile
//			Input - The stream to read from
//
// Return Value:	The new block
//
InnerIndexBlock *InnerIndexBlock::Construct(Index &NewIndex, 
		InnerIndexBlock *pNewParent, int NewOffset, istream &Input)
{
    return new InnerIndexBlock(NewIndex, pNewParent, NewOffset, Input);
}



//
// Function Name:	InnerIndexBlock::Construct()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Construct an InnerIndexBlock by reading it from the
//			RecordFile.  (InnerIndexBlock forces construction 
//			through this function so that it will be guaranteed 
//			to be allocated with new and not on the stack.)
//
// Parameters:		NewIndex - The parent index to this
//			pNewParent - Parent block to this
//			NewOffset - Offset to this in the RecordFile
//
// Return Value:	The new block
//
InnerIndexBlock *InnerIndexBlock::Construct(Index &NewIndex, 
		InnerIndexBlock *pNewParent, int NewOffset)
{
    return new InnerIndexBlock(NewIndex, pNewParent, NewOffset);
}


//
// Function Name:	InnerIndexBlock::~InnerIndexBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Destructor
//
// Parameters:		None
//
// Return Value:	None
//
InnerIndexBlock::~InnerIndexBlock()
{
}



//
// Function Name:	InnerIndexBlock::ReadBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Read in the contents of a block from an input stream.
//
// Parameters:		Input - the input stream
//
// Return Value:	None
//
void InnerIndexBlock::ReadBlock(istream &Input)
{
    int Count;				// count of inputs
    IndexBlockType Filler;		// place holder to read

    Input.read((char *)&Filler, sizeof(Filler));
    Input.read((char *)&Count, sizeof(Count));

    if(!Input)
	throw LogiconErrors::Read("InnerIndexBlock::ReadBlock(istream&)");

    // do for all entries
    for(int i = 0; i < Count; ++ i)
    {
	InnerIndexEntry *pEntry = new InnerIndexEntry(Input);

	// check allocation
	if(pEntry == NULL)
		throw LogiconErrors::MemoryAllocation(
				"InnerIndexBlock::ReadBlock(istream&)");

	// store it away
	Entries.Push(pEntry);
    }
}




//
// Function Name:	InnerIndexBlock::Sync()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Synchronize this to disk.
//
// Parameters:		None
//
// Return Value:	None
//
void InnerIndexBlock::Sync()
{
    ostrstream Output;			// to hold output
    int Length = Entries.Length();	// number of entries
    IndexBlockType Data = InnerBlock;	// data to write out

    // if there have been changes...
    if(Changed)
    {
    	Output.write((char *)&Data, sizeof(Data));
    	Output.write((char *)&Length, sizeof(Length));

    	for(int i = 0; i < Entries.Length(); ++ i)
    	{
	    Entries.Nth(i).Write(Output);
    	}

    	// check errors
    	if(!Output)
	    throw LogiconErrors::Write("InnerIndexBlock::Sync()");

    	char *Header = Output.str();

	try
	{
    	    ParentIndex.GetIndexFile().WriteData(Offset, Header, 
						Output.pcount());
    	}
	catch(...)
	{	
	    delete[] Header;
	    throw;
	}

	delete[] Header;

	Changed = 0;
    }

    // recurse on the children of this
    for(int j = 0; j < Entries.Length(); ++j)
    {
	Entries.Nth(j).Sync();
    }
}



//
// Function Name:	InnerIndexBlock::FindIndexBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Find the block that contains a given key.  Choose
//			the entry for the key and recursively invoke
//			FindIndexBlock for that entry's block.
//
// Parameters:		Id - The Id to look for
//
// Return Value:	The block that was found
//
IndexBlock &InnerIndexBlock::FindIndexBlock(const char *Id) 
{
    IndexBlock *pResults = NULL;		// results of this

    // check all blocks for it
    for(int i = 1; pResults == NULL && i < Entries.Length(); ++i)
    {
	// if this one is past the target...
	if(strcmp(Entries.Nth(i).GetId(), Id) > 0)
	{
	    // get the one that was not past the target
	    pResults = &Entries.Nth(i - 1).CastToInnerIndexEntry().
			    GetBlock(*this).FindIndexBlock(Id);
	}
    }

    // if none found, should be the last one
    if(pResults == NULL)
        pResults = &Entries.Nth(Entries.Length() - 1).CastToInnerIndexEntry().
			GetBlock(*this).FindIndexBlock(Id);

    return *pResults;
}



//
// Function Name:	InnerIndexBlock::FindIndexBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Find the block that contains a given key and data
//			for that key.  Choose the entry for the key and 
//			recursively invoke FindIndexBlock for that entry's 
//			block.
//
//			This allows for the possibility that multiple
//			blocks might contain the same keys (which would
//			result with duplicate keys).  In that case, this
//			needs to keep searching all blocks that might
//			contain the target item.
//
// Parameters:		Id - The Id to look for
//			Place - The indexed data to check
//
// Return Value:	The block that was found
//
IndexBlock &InnerIndexBlock::FindIndexBlock(const char *Id, 
					    const Location &Place) 
{
    IndexBlock *pResults = NULL;		// results of this

    // check all blocks for it
    for(int i = 1; pResults == NULL && i < Entries.Length(); ++i)
    {
	// if this one is past the target...
	int Comparison = strcmp(Entries.Nth(i).GetId(), Id);
	if(Comparison > 0
		|| (Comparison == 0 && Entries.Nth(i).GetLocation() > Place))
	{
	    // get the one that was not past the target
	    pResults = &Entries.Nth(i - 1).CastToInnerIndexEntry().
			    GetBlock(*this).FindIndexBlock(Id, Place);
	}
    }

    // if none found, should be the last one
    if(pResults == NULL)
        pResults = &Entries.Nth(Entries.Length() - 1).CastToInnerIndexEntry().
			GetBlock(*this).FindIndexBlock(Id, Place);

    return *pResults;
}



//
// Function Name:	InnerIndexBlock::MaxLength()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Return the maximum size of a InnerIndexBlock.
//
// Parameters:		None
//
// Return Value:	The max allowable size.
//
int InnerIndexBlock::MaxLength() const
{
    return HighBlockThreshold;
}



//
// Function Name:	InnerIndexBlock::GetType()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Get the type of this.
//
// Parameters:		None
//
// Return Value:	InnerBlock
//
IndexBlock::IndexBlockType InnerIndexBlock::GetType() const
{
    return InnerBlock;
}




//
// Function Name:	InnerIndexBlock::CreateNewBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Create a new block of this type.
//
// Parameters:		None
//
// Return Value:	A new block of this type.
//
IndexBlock *InnerIndexBlock::CreateNewBlock() const
{
    return Construct(ParentIndex);
}



//
// Function Name:	InnerIndexBlock::Balance()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Balance around one specific child of this, selecting
//			a neighbor of it and attempting to balance the
//			sizes of it and that neighbor.
//
// Parameters:		BaseBlock - The block to balance around
//
// Return Value:	None
//
void InnerIndexBlock::Balance(IndexBlock &BaseBlock)
{
    int TargetOffset = -1;	// the node to balance
    
    // Find node to balance
    for(int i = 0; i < Entries.Length(); ++i)
    {
	if(Entries.Nth(i).CastToInnerIndexEntry().GetOffset() ==
		BaseBlock.GetOffset())
	{
	    TargetOffset = i;
	    break;
	}
    }

    // if not found...
    if(TargetOffset < 0)
	throw LogiconErrors::NotFound("InnerIndexBlock::Balance(char *)");

    IndexBlock *pTarget;		// node to balance
    IndexBlock *pHigherNeighbor;	// it's neighbor
    int HigherNeighborOffset;		// offset to that neighbor

    // select a neighbor to balance with
    if(TargetOffset == 0)
    {
	pTarget = &BaseBlock;
	pHigherNeighbor = 
		&Entries.Nth(1).CastToInnerIndexEntry().GetBlock(*this);
	HigherNeighborOffset = 1;
    }
    else
    {
	pHigherNeighbor = &BaseBlock;
	HigherNeighborOffset = TargetOffset;
	TargetOffset --;	
	pTarget = 
	    &Entries.Nth(TargetOffset).CastToInnerIndexEntry().GetBlock(*this);
    }

    // if these two can be merged together...
    if(pTarget->Length() + pHigherNeighbor->Length() <= pTarget->MaxLength())
    {
	// merge the neighbor into the target
	pTarget->Transfer(*pHigherNeighbor);
	pHigherNeighbor->MarkForDelete();
	Entries.Delete(HigherNeighborOffset);

	// if there are now too few nodes left...
	if(Entries.Length() < LowBlockThreshold && pParent != NULL)
	{
	    // balance the parent for this
	    pParent->Balance(*this);
	}
	else if(Entries.Length() == 1 && pParent == NULL)
	{
	    // eliminate this as the root and set the new root to the child
	    IndexBlock *pNewRoot = 
			Entries.Nth(0).CastToInnerIndexEntry().
				ExtractBlock(*this);
	    pNewRoot->SetParent();
	    ParentIndex.SetIndexRoot(pNewRoot);
	    MarkForDelete();

	    // the following is dangerous: do not add code after it
	    delete this;
	}
    }
    else
    {
	// balance things between these two nodes
	pTarget->EqualizeSize(*pHigherNeighbor);
    }
}



//
// Function Name:	InnerIndexBlock::Insert()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Insert a block as a child to this.
//
// Parameters:		pBlock - The new child.
//
// Return Value:	None
//
void InnerIndexBlock::Insert(IndexBlock *pBlock)
{
    //
    // Insert the item in the appropriate location
    //

    // note the new parent of the new block
    pBlock->SetParent(*this);

    // construct an Entry for this
    InnerIndexEntry *pEntry = new InnerIndexEntry(pBlock);

    if(pEntry == NULL)
	throw LogiconErrors::MemoryAllocation(
			"InnerIndexBlock::Insert(IndexBlock*)");

    AddEntry(pEntry);

}





//
// Function Name:	InnerIndexBlock::ChangeId()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Change references to one Id to a new Id, recursively
//			changing it in the parent if necessary.
//
// Parameters:		OldId - The old ID to change
//			OldLocation - The old Location to change
//			NewId - the ID to change it to
//			NewLocation - the Location to change to
//
// Return Value:	None
//
void InnerIndexBlock::ChangeId(const char *OldId,
			       const Location &OldLocation,
	  		       const char *NewId,
			       const Location &NewLocation)
{
    int PercolateUp = 0;			// do we do parent as well?

    // check all items to see if must change
    for(int i = 0; i < Entries.Length(); ++i)
    {
	// if this item must change...
	if(strcmp(OldId, Entries.Nth(i).GetId()) == 0 && 
		OldLocation == Entries.Nth(i).GetLocation())
	{
	    // change it
	    Entries.Nth(i).SetId(NewId);
	    Entries.Nth(i).SetLocation(NewLocation);

	    // if this is the first item, percolate up
	    if(i == 0)
		PercolateUp = 1;
	 
	    Changed = 1;
	}
    }

    // if must percolate up...
    if(PercolateUp && pParent != NULL)
	pParent->ChangeId(OldId, OldLocation, NewId, NewLocation);
}



//
// Function Name:	InnerIndexBlock::PutEntry()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Add an entry to the block, changing the parent of
//			that entry to the current node.
//
// Parameters:		TargetOffset - Place in the list to add it to
//			pNewEntry - Entry to add
//
// Return Value:	None
//
void InnerIndexBlock::PutEntry(int TargetOffset, IndexEntry *pNewEntry)
{
    Entries.Insert(TargetOffset, pNewEntry);
    pNewEntry->CastToInnerIndexEntry().GetBlock(*this).SetParent(*this);
}



//
// Function Name:	InnerIndexBlock::Display()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Do a debug-display of this node
//
// Parameters:		Output - Stream to send the display to
//			IndentLevel - Number of spaces to indent the display
//
// Return Value:	None
//
void InnerIndexBlock::Display(ostream &Output, int IndentLevel)
{
    Indent(Output, IndentLevel);
    Output << "Block offset: " << Offset << "\n";
    Indent(Output, IndentLevel);
    Output << "Block ptr: " << this << "\n";
    Indent(Output, IndentLevel);
    Output << "Next allocated: " << GetNextAllocated() << "\n";
    Indent(Output, IndentLevel);
    Output << "Prev allocated: " << GetPreviousAllocated() << "\n";
    
    Indent(Output, IndentLevel);
    if(pParent != NULL)
    	Output << "Parent offset: " << pParent->GetOffset() << "\n";
    else
	Output << "No parent\n";

    int i;			// loop index

    for(i = 0; i < Entries.Length(); ++i)
    {
	Indent(Output, IndentLevel);
	Output << Entries.Nth(i).GetId() << " <" 
		<< Entries.Nth(i).GetLocation().GetVolume() << ","
		<< Entries.Nth(i).GetLocation().GetOffset() << "> = offset " << 
		Entries.Nth(i).CastToInnerIndexEntry().GetOffset() << "\n";
    }
    Output << "\n";

    for(i = 0; i < Entries.Length(); ++i)
    {
	if(Entries.Nth(i).CastToInnerIndexEntry().IsBlockLoaded())
	{
	    Entries.Nth(i).CastToInnerIndexEntry().GetBlock(*this).
			Display(Output, IndentLevel + 1);
	    Output << "\n";
	}
	else
	{
	    Indent(Output, IndentLevel + 1);
	    Output << "Entry " << i << " is not loaded\n";
	}
    }

    if(!Output)
	throw LogiconErrors::Write("InnerIndexBlock::Display(ostream&, int)");
}



//
// Function Name:	InnerIndexBlock::FindSibling()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Find the sibling of a given current block.
//
// Parameters:		pBlock - the block to find the sibling of
//
// Return Value:	The sibling block.
//
IndexBlock &InnerIndexBlock::FindSibling(const IndexBlock *pBlock) 
{
    int BlockOffset = -1;			// the offset to the found block

    // find the base entry
    for(int i = 0; i < Entries.Length(); ++i)
    {
	if(Entries.Nth(i).CastToInnerIndexEntry().GetOffset() == 
				pBlock->GetOffset())
	{
	    BlockOffset = i;
	    break;
	}
    }

    // if not found...
    if(BlockOffset < 0)
	throw LogiconErrors::NotFound(
			"InnerIndexBlock::FindSibling(IndexBlock*)");

    int SiblingOffset;			// offset to the Sibling of this

    // return the sibling to this (if there is one)
    if(BlockOffset > 0)
	SiblingOffset = BlockOffset - 1;
    else if(BlockOffset < Entries.Length() - 1)
	SiblingOffset = BlockOffset + 1;
    else
	throw LogiconErrors::OutOfBounds(
			"InnerIndexBlock::FindSibling(IndexBlock*)");

    return Entries.Nth(SiblingOffset).CastToInnerIndexEntry().GetBlock(*this);
}



//
// Function Name:	InnerIndexBlock::ClearBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Description:		Clear out a specified child of this block by
//			causing it to be removed from volatile memory.
//
// Parameters:		TargetBlock - The block to remove
//
// Return Value:	None
//
void InnerIndexBlock::ClearBlock(IndexBlock &TargetBlock)
{
    int TargetOffset = -1;			// offset of entry to use
    int i;
    // find the block to use
    for(i = 0; i < Entries.Length(); ++i)
    {
	if(TargetBlock.GetOffset() 
		== Entries.Nth(i).CastToInnerIndexEntry().GetOffset())
	{
	    TargetOffset = i;
	    break;
	}
    }

    // if not found...
    if(TargetOffset < 0)
	throw LogiconErrors::NotFound(
			"InnerIndexBlock::RemoveBlock(IndexBlock&)");

    Entries.Nth(i).CastToInnerIndexEntry().ClearBlock();
}
