//
// Module Name:		cmcInnerIndexBlock.h
//
// CSCI/CSC/CSU:	CMC/InnerIndexBlock
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//		RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	.h for InnerIndexBlock.  InnerIndexBlock is a type of IndexBlock
//	that represents inner nodes of the Index tree.  (I.E., not leaf
//	nodes.)  As such, each InnerIndexBlock maps a set of ID's to a
//	set of other blocks containing more mappings for those ID's.
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#ifndef _cmcInnerIndexBlock_h
#define _cmcInnerIndexBlock_h

#include <iostream.h>			// to allow I/O of this

#ifdef __GNUG__
class IndexBlock;                  // forward decl.
#endif
class InnerIndexBlock;                  // Incomplete Declaration

#ifdef __GNUG__
#include "cmcIndex.h"		// base class for this
#endif
#include "cmcIndexBlock.h"		// base class for this
#include "cmcInnerIndexEntry.h"		// data kept in this

// InnerIndexBlock is a B-Tree block used in the Index file that contains
// nodes.
//
// Constructors are private to ensure that these are only allocated with new.
class InnerIndexBlock : public IndexBlock
{
    private:
	static const int HighBlockThreshold;	
					// max number of entries herein
	static const int LowBlockThreshold;
					// min number of entries herein

      	InnerIndexBlock(Index &NewIndex);
					// construct empty new block
	InnerIndexBlock(Index &NewIndex, InnerIndexBlock *pNewParent, 
			int Offset, istream &Input);
					// construct block by reading it
					// do NOT take ownership of pNewParent
	InnerIndexBlock(Index &NewIndex, InnerIndexBlock *pNewParent, 
			int NewOffset);
					// construct block by reading it in
					// do NOT take ownership of pNewParent
	void ReadBlock(istream &Input);	// read block from input

    protected:
	virtual void PutEntry(int TargetOffset, IndexEntry *pNewEntry);
					// add new entry to the block

    public:
	static InnerIndexBlock *Construct(Index &NewIndex);
					// construct one of these
	static InnerIndexBlock *Construct(Index &NewIndex, 
		InnerIndexBlock *pNewParent, int Offset, istream &Input);
					// construct by reading in
					// do NOT take ownership of pNewParent
	static InnerIndexBlock *Construct(Index &NewIndex, 
		InnerIndexBlock *pNewParent, int NewOffset);
					// construct one of these
					// do NOT take ownership of pNewParent
	virtual ~InnerIndexBlock(void);	// destructor

	virtual void Sync(void);	// synchronize this to file
	virtual IndexBlock &FindIndexBlock(const char *Id);
					// find the base block for the Id
	virtual IndexBlock &FindIndexBlock(const char *Id, 
					   const Location &Place);
					// find the base block for the Item
	virtual int MaxLength(void) const;
					// get the max number of entries herein
	virtual IndexBlockType GetType(void) const;
					// get the type of this
	virtual IndexBlock *CreateNewBlock(void) const;
					// create a new block in this's image
	virtual void Display(ostream &Output, int Indent);
					// display the block to Output
	void Balance(IndexBlock &BaseBlock);
					// balance out members around Id
	void Insert(IndexBlock *pBlock);
					// insert the new block
	void ChangeId(const char *OldId, const Location &OldLocation,
			const char *NewId, const Location &NewLocation);
					// change an Id from OldId to NewId
	IndexBlock &FindSibling(const IndexBlock *pBlock);
					// find the sibling to a given block
	void ClearBlock(IndexBlock &Block);
					// clear the given block from memory
};

#endif
