//
// Module Name:		cmcInnerIndexEntry.cc
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for InnerIndexEntry.  An InnerIndexEntry is a data entry into 
//	an InnerIndexBlock.  It is part of the Index family of classes.  For
//	more details, see the Implementation Notes comment in cmcIndex.cc.
//
// Function List:
//	InnerIndexEntry::InnerIndexEntry() - There are 2 constructors to this
//	InnerIndexEntry::~InnerIndexEntry() - destructor
//	InnerIndexEntry::Sync() - Sync this to disk
//	InnerIndexEntry::Write() - Write this to an output stream
//	InnerIndexEntry::GetOffset() - Get the offset to this thing's block
//	InnerIndexEntry::ClearBlock() - Clear block from volatile memory
//	InnerIndexEntry::IsBlockLoaded() - Test if block is in memory
//	InnerIndexEntry::GetBlock() - Get the data block in this
//	InnerIndexEntry::ExtractBlock() - Extract the data block from this
//	InnerIndexEntry::CastToInnerIndexEntry() - Downcast function
//
// Other Modules Called:
//	Misc - for string I/O
//	Index - This is part of Index
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           1/2/96      jd_2     Initial version
//

#include <iostream.h>			// for I/O
#include <stddef.h>			// for NULL

#include "cmcInnerIndexEntry.h"		// declarations for this
#include "cmcMisc.h"			// for string I/O
#include "cmcIndex.h"			// this is part of an Index
#include "cmcError.h"			// for exception classes


//
// Function Name:	InnerIndexEntry::InnerIndexEntry()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Construct by reading from stream
//
// Parameters:		Input - stream to read from
//
// Return Value:	None
//
InnerIndexEntry::InnerIndexEntry(istream &Input) : 
	IndexEntry(),
	pBlock(NULL)
{
    char *NewId = new char[DocumentIdLength+1];		// to hold the Id

    if(NewId == NULL)
	throw LogiconErrors::MemoryAllocation(
		"InnerIndexEntry::InnerIndexEntry(istream&)");

    try
    {
    	// get the Id
    	Input.read(NewId, DocumentIdLength);
    	NewId[DocumentIdLength] = '\0';
    	SetId(NewId);

	// get the location
	int Volume;			// data volume
	int DataOffset;			// data offset

	Input.read((char*)&Volume, sizeof(Volume));
	Input.read((char*)&DataOffset, sizeof(DataOffset));
	SetLocation(Location(Volume, DataOffset));

    	// get the offset
    	Input.read((char *)&Offset, sizeof(Offset));

	if(!Input)
	    throw LogiconErrors::Read(
			"InnerIndexEntry::InnerIndexEntry(istream&)");
    }
    catch(...)
    {
	delete[] NewId;
	throw;
    }
}



//
// Function Name:	InnerIndexEntry::InnerIndexEntry()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Construct given the block referred to
//
// Parameters:		pNewBlock - the Block in question
//
// Return Value:	None
//
InnerIndexEntry::InnerIndexEntry(IndexBlock *pNewBlock) :
		IndexEntry(pNewBlock->GetId(), pNewBlock->GetLocation()),
		pBlock(pNewBlock),
		Offset(pNewBlock->GetOffset())
{
}





//
// Function Name:	InnerIndexEntry::~InnerIndexEntry()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Destructor
//
// Parameters:		None
//
// Return Value:	None
//
InnerIndexEntry::~InnerIndexEntry()
{
    if(pBlock != NULL)
	delete pBlock;
}




//
// Function Name:	InnerIndexEntry::Sync()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Sync this to disk.
//
// Parameters:		None
//
// Return Value:	None
//
void InnerIndexEntry::Sync()
{
    // if the block is not null, synchronize it
    if(pBlock != NULL)
	pBlock->Sync();
}



//
// Function Name:	InnerIndexEntry::Write()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Write this to an output stream
//
// Parameters:		Output - Stream to write to
//
// Return Value:	None
//
void InnerIndexEntry::Write(ostream &Output) const
{
    // write the Id
    Output.write(GetId(), DocumentIdLength);

    // write the Location
    int I;			// to hold temporary data

    I = GetLocation().GetVolume();
    Output.write((char *)&I, sizeof(I));
    I = GetLocation().GetOffset();
    Output.write((char *)&I, sizeof(I));

    // write the offset
    Output.write((char *)&Offset, sizeof(Offset));

    if(!Output)
	throw LogiconErrors::Write("InnerIndexEntry::Write(ostream&)");
}



//
// Function Name:	InnerIndexEntry::GetOffset()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Get the offset of the block this refers to
//
// Parameters:		None
//
// Return Value:	The Offset component
//
int InnerIndexEntry::GetOffset() const
{
    return Offset;
}




//
// Function Name:	InnerIndexEntry::ClearBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Clear out the volatile memory copy of the Entry's
//			block.
//
// Parameters:		None
//
// Return Value:	None
//
void InnerIndexEntry::ClearBlock()
{
    // destruct the block if it is here...
    if(pBlock != NULL)
    {
	pBlock->Sync();
	delete pBlock;
	pBlock = NULL;
    }
}



//
// Function Name:	InnerIndexEntry::IsBlockLoaded()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Test if this Entry's block has already been loaded
//			into volatile memory.
//
// Parameters:		None
//
// Return Value:	1 if loaded
//			0 if not
//
int InnerIndexEntry::IsBlockLoaded() const
{
    return pBlock != NULL;
}


//
// Function Name:	InnerIndexEntry::GetBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Get the block associated with this
//
// Parameters:		Parent - The block in which this appears
//
// Return Value:	The Read-in block
//
IndexBlock &InnerIndexEntry::GetBlock(InnerIndexBlock &Parent) 
{
    // if block is not available...
    if(pBlock == NULL)
    {
	// get it
	pBlock = ReadIndexBlock(Parent.GetIndex(), &Parent, Offset);
    }
    else
    {
	// touch it
	Parent.GetIndex().TouchBlock(*pBlock);
    }

    return *pBlock;
}




//
// Function Name:	InnerIndexEntry::ExtractBlock()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Get the block from this, passing ownership of it
//			out.
//
// Parameters:		Parent - the block in which this entry appears
//
// Return Value:	The block from this
//
IndexBlock *InnerIndexEntry::ExtractBlock(InnerIndexBlock &Parent)
{
    IndexBlock *pResults;		// return results

    // read in the data if need be
    if(pBlock == NULL)
    	(void)GetBlock(Parent);

    // note the results
    pResults = pBlock;

    // clear the current data
    pBlock = NULL;

    return pResults;
}



//
// Function Name:	InnerIndexEntry::CastToInnerIndexEntry()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Downcast function
//
// Parameters:		None
//	
// Return Value:	This entry.
//
InnerIndexEntry &InnerIndexEntry::CastToInnerIndexEntry()
{
    return *this;
}



//
// Function Name:	InnerIndexEntry::CastToInnerIndexEntry()
//
// CSCI/CSC/CSU:	CMC/InnerIndexEntry
//
// Description:		Downcast function
//
// Parameters:		None
//	
// Return Value:	This entry.
//
const InnerIndexEntry &InnerIndexEntry::CastToInnerIndexEntry() const
{
    return *this;
}

