//
// Module Name:		cmcLeafIndexBlock.cc
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Contain the code for a LeafIndexBlock.  This is a leaf node for
//	the Index B-Tree.  For more details, see the Implementation Note
//	comment in cmcIndex.cc.
//
// Function List:
//	LeafIndexBlock::LeafIndexBlock() - There are three constructors herein
//	LeafIndexBlock::~LeafIndexBlock() - destructor
//	LeafIndexBlock::ReadBlock() - Read data from the block from an input 
//			stream
//	LeafIndexBlock::FindId() - Find the entry for a given ID
//	LeafIndexBlock::Remove() - Remove the entry for a given ID
//	LeafIndexBlock::AddId() - Add an entry for an ID
//	LeafIndexBlock::FindIndexBlock() - Find the block for an ID
//	LeafIndexBlock::Sync() - Synchronize to persistent storage
//	LeafIndexBlock::MaxLength() - Get the number of entries allowable 
//			herein
//	LeafIndexBlock::GetType() - Get the type of block that this is
//	LeafIndexBlock::CreateNewBlock() - Create another block like this one
//	LeafIndexBlock::Display() - Display this to debug output
//
// Other Modules Called:
//	RecordFile - For storage
//	Location - stored data
//	Index - This is part of Index
//	LeafIndexEntry - This holds one of these
//	InnerIndexBlock - The parent block to this
//	Misc - For string I/O
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           1/2/96      jd_2     Initial version
//


#include <stddef.h>			// for NULL
#include <iostream.h>			// for I/O
#include <strstream.h>			// for string I/O
#include <string.h>			// for string functions

#include "cmcLeafIndexBlock.h"		// for declaration of this
#include "cmcRecordFile.h"		// for storage
#include "cmcLocation.h"		// stored data
#include "cmcIndex.h"			// this is part of an Index
#include "cmcLeafIndexEntry.h"		// this stores these
#include "cmcInnerIndexBlock.h"		// the parent to this
#include "cmcMisc.h"			// for string I/O
#include "cmcError.h"			// for exception classes


const int LeafIndexBlock::HighBlockThreshold = 25;	// max entries per block
const int LeafIndexBlock::LowBlockThreshold = 13;	// min entries per block

//const int LeafIndexBlock::HighBlockThreshold = 3;	// max entries per block
//const int LeafIndexBlock::LowBlockThreshold = 2;	// min entries per block

//
// Function Name:	LeafIndexBlock::LeafIndexBlock()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Construct a brand-new LeafIndexBlock.
//
// Parameters:		NewIndex - Index in which this appears
//
// Return Value:	None
//
LeafIndexBlock::LeafIndexBlock(Index &NewIndex) :
	IndexBlock(NewIndex)
{
}


//
// Function Name:	LeafIndexBlock::LeafIndexBlock()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Construct a LeafIndexBlock by reading it from
//			an input stream.
//
// Parameters:		NewIndex - Index in which this appears
//			pNewParent - Parent block to this
//			NewOffset - Offset to this
//			Input - The input stream
//
// Return Value:	None
//
LeafIndexBlock::LeafIndexBlock(Index &NewIndex, InnerIndexBlock *pNewParent, 
			int NewOffset, istream &Input) :
		IndexBlock(NewIndex, pNewParent, NewOffset)
{
    ReadBlock(Input);
}


//
// Function Name:	LeafIndexBlock::LeafIndexBlock()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Construct a LeafIndexBlock by reading it from
//			the Index's RecordFile
//
// Parameters:		NewIndex - The Index in question
//			pNewParent - parent block to this
//			NewOffset - Offset to the block in the file
//
// Return Value:	None
//
LeafIndexBlock::LeafIndexBlock(Index &NewIndex, InnerIndexBlock *pNewParent,
		int NewOffset) :
	IndexBlock(NewIndex, pNewParent, NewOffset)
{
    int Length;				// block length
    char *Block;			// data block

    // read in the block from file
    Block = ParentIndex.TreeFile.ReadData(Offset, Length);

    try
    {
    	istrstream Input(Block, Length);	// stream version of this

    	ReadBlock(Input);

    	delete[] Block;
    }
    catch(...)
    {
	delete[] Block;
	throw;
    }
}



//
// Function Name:	LeafIndexBlock::~LeafIndexBlock()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Destructor
//
// Parameters:		None
//
// Return Value:	None
//
LeafIndexBlock::~LeafIndexBlock()
{
}



//
// Function Name:	LeafIndexBlock::ReadBlock()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Read the contents of this block from an input stream.
//
// Parameters:		Input - the Input stream
//
// Return Value:	None
//
void LeafIndexBlock::ReadBlock(istream &Input)
{
    int Count;				// count of inputs
    IndexBlockType Filler;		// filler to read to 

    Input.read((char *)&Filler, sizeof(Filler));
    Input.read((char *)&Count, sizeof(Count));

    // do for all entries
    for(int i = 0; i < Count; ++ i)
    {
	Entries.Push(new LeafIndexEntry(Input));
    }

    // do error checking...
    if(!Input)
	throw LogiconErrors::Read("LeafIndexBlock::ReadBlock()");
}



//
// Function Name:	LeafIndexBlock::FindId()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Find the Location for some ID in the block.
//
// Parameters:		Id - The ID to find
//
// Return Value:	The found Location
//
const Location &LeafIndexBlock::FindId(const char *Id) const
{
    const Location *pResult = NULL;		// to hold the found location

    for(int i = 0; i < Entries.Length() && pResult == NULL; ++i)
    {
	if(strcmp(Entries.Nth(i).GetId(), Id) == 0)
	{
	    pResult = &Entries.Nth(i).CastToLeafIndexEntry().GetLocation();
	}
    }

    // if not found...
    if(pResult == NULL)
	throw LogiconErrors::NotFound("LeafIndexBlock::FindId(char *)");

    return *pResult;
}



//
// Function Name:	LeafIndexBlock::Remove()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Remove the entry for some ID from the block
//
// Parameters:		Id - The ID in question
//			Place - The location of it
//
// Return Value:	None
//
void LeafIndexBlock::Remove(const char *Id, const Location &Place)
{
    int Target = -1;			// target block to remove

    // mark this as changed
    Changed = 1;

    // find the block to remove
    for(int i = 0; i < Entries.Length() && Target < 0; ++i)
    {
	if(strcmp(Entries.Nth(i).GetId(), Id) == 0
		&& Entries.Nth(i).CastToLeafIndexEntry().GetLocation() == Place)
	    Target = i;
    }

    // if not found...
    if(Target < 0)
	throw LogiconErrors::NotFound("LeafIndexBlock::Remove(char *)");

    // if this is the first entry in the block...
    if(Target == 0 && Entries.Length() > 1 && pParent != NULL)
    {
	// note the change in the parent
	pParent->ChangeId(Entries.Nth(0).GetId(), 
			  Entries.Nth(0).GetLocation(),
			  Entries.Nth(1).GetId(),
			  Entries.Nth(1).GetLocation());
    }

    // remove the entry
    Entries.Delete(Target);

    // if there are now too few entries in this block...
    if(Entries.Length() < LowBlockThreshold && pParent != NULL)
    {
	pParent->Balance(*this);
    }
}



//
// Function Name:	LeafIndexBlock::AddId()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Add an ID-Location mapping to the block
//
// Parameters:		Id - the ID to add
//			Place - Its location
//
// Return Value:	None
//
void LeafIndexBlock::AddId(const char *Id, const Location &Place)
{
    // construct an Entry for this
    LeafIndexEntry *pEntry = new LeafIndexEntry(Id, Place);

    if(pEntry == NULL)
	throw LogiconErrors::MemoryAllocation(
			"LeafIndexBlock::AddId(char*, Location&)");

    // add it in
    AddEntry(pEntry);
}



//
// Function Name:	LeafIndexBlock::FindIndexBlock()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Find the block that contains some ID.  If it gets
//			this far in the tree, this is the block in question.
//
// Parameters:		Id - the ID in question
//	
// Return Value:	This block.
//
IndexBlock &LeafIndexBlock::FindIndexBlock(const char *Id) 
{
    return *this;
}




//
// Function Name:	LeafIndexBlock::FindIndexBlock()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Find the block that contains some ID and data.
//
// Parameters:		Id - the ID in question
//			Place - the Location of the data to find
//	
// Return Value:	This block.
//
IndexBlock &LeafIndexBlock::FindIndexBlock(const char *Id, 
					   const Location &Place) 
{
    return *this;
}



//
// Function Name:	LeafIndexBlock::Sync()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Synchronize the entry to file.
//
// Parameters:		None
//
// Return Value:	None
//
void LeafIndexBlock::Sync()
{
    ostrstream Output;			// to hold output
    int Length = Entries.Length();	// number of entries
    IndexBlockType Data = LeafBlock;	// writable data

    // if this has changed since last sync()
    if(Changed)
    {
    	Output.write((char *)&Data, sizeof(Data));
    	Output.write((char *)&Length, sizeof(Length));

    	for(int i = 0; i < Entries.Length(); ++ i)
    	{
	    Entries.Nth(i).Write(Output);
    	}

    	// check errors
    	if(!Output)
	    throw LogiconErrors::Write("LeafIndexBlock::Sync()");

    	char *Header = Output.str();

	try
	{
    	    ParentIndex.TreeFile.WriteData(Offset, Header, Output.pcount());
    	    delete[] Header;
	}
	catch(...)
	{
	    delete[] Header;
	    throw;
	}
    }

    Changed = 0;
}





//
// Function Name:	LeafIndexBlock::MaxLength()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Get the number of entries allowed in this block.
//
// Parameters:		None
//
// Return Value:	The count of entries
//
int LeafIndexBlock::MaxLength() const
{
    return HighBlockThreshold;
}



//
// Function Name:	LeafIndexBlock::GetType()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Get the type of this.
//
// Parameters:		None
//
// Return Value:	LeafBlock
//
IndexBlock::IndexBlockType LeafIndexBlock::GetType() const
{
    return LeafBlock;
}



//
// Function Name:	LeafIndexBlock::CreateNewBlock()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Create a new block of this type.
//
// Parameters:		None
//
// Return Value:	The new block.
//
IndexBlock *LeafIndexBlock::CreateNewBlock() const
{
    IndexBlock *pResult = new LeafIndexBlock(ParentIndex);

    if(pResult == NULL)
	throw LogiconErrors::MemoryAllocation(
			"LeafIndexBlock::CreateNewBlock()");

    return pResult;
}





//
// Function Name:	LeafIndexBlock::Display()
//
// CSCI/CSC/CSU:	CMC/LeafIndexBlock
//
// Description:		Display this in a debug output format.
//
// Parameters:		Output - The stream to display to
//			IndentLevel - The level to indent it to
//
// Return Value:	None
//
void LeafIndexBlock::Display(ostream &Output, int IndentLevel)
{
    Indent(Output, IndentLevel); 
    Output << "Block offset: " << Offset << "\n"; 
    Indent(Output, IndentLevel);
    Output << "Block ptr: " << this << "\n";
    Indent(Output, IndentLevel);
    Output << "Next allocated: " << GetNextAllocated() << "\n";
    Indent(Output, IndentLevel);
    Output << "Prev allocated: " << GetPreviousAllocated() << "\n";

    Indent(Output, IndentLevel);
    if(pParent == NULL)
	Output << "No parent\n";
    else
	Output << "Parent offset: " << pParent->GetOffset() << "\n";
     
    for(int i = 0; i < Entries.Length(); ++i) 
    { 
        Indent(Output, IndentLevel);   
        Output << Entries.Nth(i).GetId() << " = <" <<  
                Entries.Nth(i).CastToLeafIndexEntry().GetLocation().GetVolume()
					<< ", " <<
                Entries.Nth(i).CastToLeafIndexEntry().GetLocation().GetOffset()
					<< ">\n";
    } 

    if(!Output)
	throw LogiconErrors::Write("LeafIndexBlock::Display(ostream&, int)");
}

