//
// Module Name:		cmcObjectReferenceVal.cc
//
// CSCI/CSC/CSU:	CMC/ObjectReferenceValue
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Includes code for ObjectReferenceValue.  An ObjectReferenceValue
//	is a type of AttributeValue that contains an ObjectReference.
//
// Function List:
//	ObjectReferenceValue::ObjectReferenceValue() - This has 2 constructors
//	ObjectReferenceValue::~ObjectReferenceValue() - The destructor
//	ObjectReferenceValue::GetValue() - Get the value of this
//	ObjectReferenceValue::TypeOf() - get the type of AttributeValue that
//			this is
//	ObjectReferenceValue::Copy() - Produce a copy of this
//	ObjectReferenceValue::WriteToStream() - Write this to a stream
//	ObjectReferenceValue::operator==() - Test equality of this
//	ObjectReferenceValue::GetClass() - Get the string label of this class
//	ObjectReferenceValue::CastToObjectReferenceValue() - Downcast function
//
// Other Modules Called:
//	AttributeValueType - to get type of AttributeValues and ObjectReferences
//	ObjectReference - The type of thing herein
//	CollectionReference - One of the possible ObjectReferences
//	DocumentReference - One of the possible ObjectReferences
//	AttributeReference - One of the possible ObjectReferences
//	AnnotationReference - One of the possible ObjectReferences
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#include <iostream.h>			// for I/O
#include <stddef.h>			// for NULL

#include "cmcObjectReferenceVal.h"	// for declaration of this
#include "cmcAttributeValueType.h"	// for TypeOf() return value
#include "cmcCollectionReference.h"	// one possible reference
#include "cmcDocumentReference.h"	// one possible reference
#include "cmcAnnotationReference.h"	// one possible reference
#include "cmcAttributeReference.h"	// one possible reference
#include "cmcError.h"			// for exception classes

// ClassLabel is the string name of this class
const char *ObjectReferenceValue::ClassLabel = "ObjectReferenceValue";



//
// Function Name:	ObjectReferenceValue::ObjectReferenceValue()
//
// CSCI/CSC/CSU:	CMC/ObjectReferenceValue
//
// Description:		Construct given the reference to contain
//
// Parameters:		pNewValue - the reference in question
//
// Return Value:	None
//
ObjectReferenceValue::ObjectReferenceValue(ObjectReference &NewValue)
	: pValue(NewValue.Copy())
{
}


//
// Function Name:	ObjectReferenceValue::ObjectReferenceValue()
//
// CSCI/CSC/CSU:	CMC/ObjectReferenceValue
//
// Description:		Construct given an input stream containing this.
//			There isn't any format to this beyond what the
//			ObjectReference's have.
//
// Parameters:		Type - Type of ObjectReference contained
//			Input - The stream in question
//
// Return Value:	None
//
ObjectReferenceValue::ObjectReferenceValue(AttributeValueType Type, 
						istream &Input)
{
    switch(Type)
    {
	case COLLECTION_REFERENCE:
	    pValue = new CollectionReference(Input);
	    break;
	case DOCUMENT_REFERENCE:
	    pValue = new DocumentReference(Input);
	    break;
	case ANNOTATION_REFERENCE:
	    pValue = new AnnotationReference(Input);
	    break;
	case ATTRIBUTE_REFERENCE:
	    pValue = new AttributeReference(Input);
	    break;
	default:
	    pValue = NULL;
	    throw LogiconErrors::Format(
"ObjectReferenceValue::ObjectReferenceValue(AttributeValueType, istream&)");
	    break;
    }

    if(pValue == NULL)
	throw LogiconErrors::MemoryAllocation(
"ObjectReferenceValue::ObjectReferenceValue(AttributeValueType, istream&)");
}



//
// Function Name:	ObjectReferenceValue::~ObjectReferenceValue()
//
// CSCI/CSC/CSU:	CMC/ObjectReferenceValue
//
// Description:		Destructor
//
// Parameters:		None
//
// Return Value:	None
//
ObjectReferenceValue::~ObjectReferenceValue()
{
    if(pValue != NULL)
    	delete pValue;
}



//
// Function Name:	ObjectReferenceValue::GetValue()
//
// CSCI/CSC/CSU:	CMC/ObjectReferenceValue
//
// Description:		Get the value of this.
//
// Parameters:		None
//
// Return Value:	pValue component
//
ObjectReference &ObjectReferenceValue::GetValue() const
{
    return *pValue;
}


//
// Function Name:	ObjectReferenceValue::TypeOf()
//
// CSCI/CSC/CSU:	CMC/ObjectReferenceValue
//
// Description:		Get the type of this.
//
// Parameters:		None
//
// Return Value:	Type of the contained value.
//
AttributeValueType ObjectReferenceValue::TypeOf() const
{
    return pValue->TypeOf();
}



//
// Function Name:	ObjectReferenceValue::Copy()
//
// CSCI/CSC/CSU:	CMC/ObjectReferenceValue
//
// Description:		Produce a copy of this
//
// Parameters:		None
//
// Return Value:	The new copy.
//
AttributeValue *ObjectReferenceValue::Copy() const
{
    AttributeValue *pCopy = new ObjectReferenceValue(*pValue);

    // check allocation
    if(pCopy == NULL)
	throw LogiconErrors::MemoryAllocation("ObjectReferenceValue::Copy()");

    return pCopy;
}



//
// Function Name:	ObjectReferenceValue::WriteToStream()
//
// CSCI/CSC/CSU:	CMC/ObjectReferenceValue
//
// Description:		Write this to stream in the format specified in
//			ObjectReferenceValue(istream&).
//
// Parameters:		Output - The stream to write to
//
// Return Value:	None
//
void ObjectReferenceValue::WriteToStream(ostream &Output) const
{
    Output << *pValue;

    if(!Output)
	throw LogiconErrors::Write(
		"ObjectReferenceValue::WriteToStream(ostream&)");
}



//
// Function Name:	ObjectReferenceValue::operator==
//
// CSCI/CSC/CSU:	CMC/ObjectReferenceValue
//
// Description:		Test if this is equal to another AttributeValue
//
// Parameters:		A - The AttributeValue
//
// Return Value:	0 if equal
//			1 if not
//
int ObjectReferenceValue::operator==(const AttributeValue &A) const
{
    int Result = 0;			// defaults to no match

    // if they do match...
    if(A.TypeOf() == TypeOf() && 
		*pValue == *((ObjectReferenceValue*)&A)->pValue)
    {
	Result = 1;
    }

    return Result;
}






//
// Function Name:	ObjectReferenceValue::GetClass()
//
// CSCI/CSC/CSU:	CMC/ObjectReferenceValue
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *ObjectReferenceValue::GetClass() const
{
    return ClassLabel;
}




//
// Function Name:	ObjectReferenceValue::CastToObjectReferenceValue()
//
// CSCI/CSC/CSU:	cmc/ObjectReferenceValue
//
// Description:		Downcast this as a ObjectReferenceValue
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
ObjectReferenceValue &ObjectReferenceValue::CastToObjectReferenceValue()
{
    return *this;
}



//
// Function Name:	ObjectReferenceValue::CastToObjectReferenceValue()
//
// CSCI/CSC/CSU:	cmc/ObjectReferenceValue
//
// Description:		Downcast this as a ObjectReferenceValue when this is 
//			const.
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
const ObjectReferenceValue &ObjectReferenceValue::CastToObjectReferenceValue() 
			const
{
    return *this;
}




