//
// Module Name:		cmcPersistentCollection.cc
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for PersistentCollection.  A PersistentCollection is actually
//	a shell around a RealPersistentCollection.  This shell is maintained
//	so that a given persistent collection can be kept open multiple times
//	by the same process.  The shell is managed by using a ReferenceCache.
//
// Function List:
//      PersistentCollection::PersistentCollection() - There are 3 
//                      constructors for this
//      PersistentCollection::~PersistentCollection() - Destructor
//      PersistentCollection::Length() - Get number of Documents herein
//      PersistentCollection::RemoveDocument() - Remove a specified Document
//      PersistentCollection::GetDocument() - Retrieve a specified Document
//      PersistentCollection::FirstDocument() - Get the first Document in 
//                      the Collection
//      PersistentCollection::NextDocument() - Get the next Document after 
//                      the last returned by FirstDocument() or NextDocument()
//      PersistentCollection::GetByExternalId() - Get a Document given its 
//                      external ID
//      PersistentCollection::Sync() - Synchronize the persistent version of
//                      a Document with its in-memory version
//      PersistentCollection::Destroy() - Destroy a PersistentCollection
//      PersistentCollection::Store() - Store a Document in the 
//                      PersistentCollection
//      PersistentCollection::Update() - Update a Document in the 
//                      PersistentCollection
//      PersistentCollection::AnnotateCollection() - Annotate Documents in 
//                      the Collection
//      PersistentCollection::PutAttribute() - Add an Attribute to the 
//                      Collection
//      PersistentCollection::GetAttribute() - Get an Attribute of the 
//                      Collection
//      PersistentCollection::GetAttributes() - Get the Attributes of the 
//                      Collection
//      PersistentCollection::HasAttributes() - See if a Collection has a 
//                      specified set of Attributes
//      PersistentCollection::IsAttributesChanged() - Test if this has been 
//                      recently changed
//      PersistentCollection::ClearAttributesChanged() - Clear the changed 
//                      indicator for this
//      PersistentCollection::GetName() - Get name of this
//      PersistentCollection::NextDocument(Iterator&) - Get next document of
//                              this
//
// Other Modules Called:
//	RealPersistentCollection - to do all the heavy lifting
//	ReferenceCache - to keep references cached
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           2/22/96     jd_2     Initial version
//


#include "cmcPersistentCollection.h"		// defines for this
#include "cmcReferenceCache.h"			// to hold references to reals

ReferenceCache RealCollections;			// cache of actual collections


//
// Function Name:	PersistentCollection::PersistentCollection()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Construct a new PersistentCollection.
//
// Parameters:		NewName - The name of the PersistentCollection
//			NewAttributes - Attributes of it
//			VolumeCount - Number of Volumes for it to contain
//			BlockSize - Size of a data block in it
//
// Return Value:	None
//
PersistentCollection::PersistentCollection(const char *NewName, 
	const Set<Attribute> &NewAttributes,
	int NewVolumeCount, int BlockSize) :
		PersistentCollectionStub(NewName),
		pIterator(NULL)
{
    // try to construct this anew: it will fail if the thing is already present
    if((pCollection = new RealPersistentCollection(NewName, NewAttributes, 
			NewVolumeCount, BlockSize)) == NULL)
    {
	throw LogiconErrors::MemoryAllocation(
		"PersistentCollection::PersistentCollection(char*, ...)");
    }

    // add it to the reference cache
    RealCollections.AddReference(pCollection);
}


//
// Function Name:	PersistentCollection::PersistentCollection()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Open an existing PersistentCollection
//
// Parameters:		NewName - name of it
//			NewAttributes - Attributes of it
//
// Return Value:	None
//
PersistentCollection::PersistentCollection(const char *NewName, 
	const Set<Attribute> &NewAttributes) :
		PersistentCollectionStub(NewName),
		pIterator(NULL)
{
    // try to construct this anew: it will fail if the thing is already present
    if((pCollection = new RealPersistentCollection(NewName, NewAttributes)) 
				== NULL)
    {
	throw LogiconErrors::MemoryAllocation(
		"PersistentCollection::PersistentCollection(char*, ...)");
    }

    // add it to the reference cache
    RealCollections.AddReference(pCollection);
}



//
// Function Name:	PersistentCollection::PersistentCollection()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Open an existing PersistentCollection
//
// Parameters:		NewName - name of it
//
// Return Value:	None
//
PersistentCollection::PersistentCollection(const char *NewName) : 
		PersistentCollectionStub(NewName),
		pIterator(NULL)
{
    Reference *pReference;			// the reference
    
    // look for it in the reference cache
    if((pReference = RealCollections.GetReference(NewName)) != NULL)
	pCollection = &pReference->CastToRealPersistentCollection();
    else
    {
	// not in the cache: construct it
	if((pCollection = new RealPersistentCollection(NewName)) == NULL)
	    throw LogiconErrors::MemoryAllocation(
		"PersistentCollection::PersistentCollection(char*)");

	// add it to the cache
    	RealCollections.AddReference(pCollection);
    }
}



//
// Function Name:	PersistentCollection::~PersistentCollection()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Destruct a PersistentCollection
//
// Parameters:		None
//
// Return Value:	None
//
PersistentCollection::~PersistentCollection()
{
    RealCollections.DropReference(*pCollection);

    // delete the iterator
    if(pIterator != NULL)
	delete pIterator;
}



//
// Function Name:	PersistentCollection::Length()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Get the number of Documents in the Collection by
//			summing the number of Documents in all of its Volumes.
//
// Parameters:		None
//
// Return Value:	The number of Documents herein
//
int PersistentCollection::Length() const
{
    return pCollection->Length();
}



//
// Function Name:	PersistentCollection::RemoveDocument()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Remove a specified Document from the Collection.
//
// Parameters:		Id - The ID of the Document to remove
//
// Return Value:	None
//
void PersistentCollection::RemoveDocument(const char *Id)
{
    pCollection->RemoveDocument(Id);
}



//
// Function Name:	PersistentCollection::GetDocument()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Get a Document from the Collection.
//
// Parameters:		Id - The ID of the Document to get.
//
// Return Value:	The fetched Document.
//
Document *PersistentCollection::GetDocument(const char *Id)
{
    return pCollection->GetDocument(Id);
}




//
// Function Name:	PersistentCollection::FirstDocument()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Get the first Document in the Collection.  Do this
//			by resetting all Volumes and starting the NextDocument()
//			walk-through at the first volume.
//
// Parameters:		None
//
// Return Value:	The Document
//			NULL if there is none
//
Document *PersistentCollection::FirstDocument()
{
    // if the iterator exists, delete it
    if(pIterator != NULL)
	delete pIterator;

    // construct a new iterator
    if((pIterator = new RealPersistentCollection::Iterator) == NULL)
	throw LogiconErrors::MemoryAllocation(
			"PersistentCollection::FirstDocument()");

    return pCollection->NextDocument(*pIterator);
}



//
// Function Name:	PersistentCollection::NextDocument()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Get the next Document after the last one retrieved by
//			FirstDocument or NextDocument.  This
//			maintains a member indicating which is the current
//			Volume being walked through.  This gets the next
//			Document in that current Volume.  (The Volume
//			supports a walkthrough through itself.)  If there
//			is no next Document in the Volume, this moves onto
//			the next Volume.
//
// Parameters:		None
//
// Return Value:	The next Document
//			NULL if there is none
//
Document *PersistentCollection::NextDocument()
{
    // if there is no iterator...
    if(pIterator == NULL)
	throw LogiconErrors::InvalidObject(
			"PersistentCollection::NextDocument()");

    return pCollection->NextDocument(*pIterator);
}



//
// Function Name:	PersistentCollection::GetByExternalId()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Get a Document given it's external ID
//
// Parameters:		ExternalId - The ID in question
//
// Return Value:	The Document
//			NULL if not found
//
Document *PersistentCollection::GetByExternalId(const char *ExternalId)
{
    return pCollection->GetByExternalId(ExternalId);
}



//
// Function Name:	PersistentCollection::AnnotateCollection()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Annotate the Documents in a Collection.  
//
// Parameters:		Which - Collection indicating which Documents to 
//				annotate
//			Annotator - The annotator in question
//
// Return Value:	None
//
void PersistentCollection::AnnotateCollection(Collection &Target, 
					      const char *Annotator)
{
    pCollection->AnnotateCollection(Target, Annotator);
}



//
// Function Name:	PersistentCollection::Sync()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Synchronize the PersistentCollection with its
//			persistent representation.
//
// Parameters:		None
//
// Return Value:	None
//
void PersistentCollection::Sync()
{
    pCollection->Sync();
}



//
// Function Name:	PersistentCollection::Destroy()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Destroy a PersistentCollection by destroying all
//			of its Volumes and its Index.  This must first
//			open the collection in order to determine how 
//			many Volumes it has.
//
// Parameters:		Name - Collection to destroy
//
// Return Value:	None
//
void PersistentCollection::Destroy(const char *NewName)
{
    RealPersistentCollection::Destroy(NewName);
}






//
// Function Name:	PersistentCollection::PutAttribute()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Add an Attribute to a Collection.  All Attributes
//			for the Collection are kept by the Collection's
//			first Volume.
//
// Parameters:		AttrName - Name of the Attribute
//			Value - New value of the Attribute
//
// Return Value:	None
//
void PersistentCollection::PutAttribute(const char *NewName, 
					const AttributeValue &Value)
{
    pCollection->PutAttribute(NewName, Value);
}



//
// Function Name:	PersistentCollection::GetAttribute()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Pass this call to the Volume's GetAttribute() function.
//
// Parameters:		AttributeName - Name of Attribute to get
//
// Return Value:	Its value
//
const AttributeValue *PersistentCollection::GetAttribute(const char *AttrName)
				const
{
    return pCollection->GetAttribute(AttrName);
}



//
// Function Name:	PersistentCollection::GetAttributes()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Pass this to the first Volume.
//
// Parameters:		None
//
// Return Value:	The set of Attributes for this
//
Set<Attribute> &PersistentCollection::GetAttributes()
{
    return pCollection->GetAttributes();
}



//
// Function Name:	PersistentCollection::GetAttributes()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Pass this to the first Volume: this is for const
//			attributes only.
//
// Parameters:		None
//
// Return Value:	The set of Attributes for this
//
const Set<Attribute> &PersistentCollection::GetAttributes() const
{
    return pCollection->GetAttributes();
}



//
// Function Name:	PersistentCollection::HasAttributes()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Pass this to the first Volume.
//
// Parameters:		pBaseAttributes() - Attributes to test
//
// Return Value:	1 if it has them
//			0 otherwise
//
int PersistentCollection::HasAttributes(Set<Attribute> &TestAttributes) const
{
    return pCollection->HasAttributes(TestAttributes);
}



//
// Function Name:	PersistentCollection::IsAttributesChanged()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Test whether this has been changed since it was
//			constructed or ClearChanged was called.
//
// Parameters:		None
//
// Return Value:	1 if it has been changed
//			0 if not
//
int PersistentCollection::IsAttributesChanged() const
{
    return pCollection->IsAttributesChanged();
}



//
// Function Name:	PersistentCollection::ClearAttributesChanged()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Clear the changed indicators of this and its member
//			objects.
//
// Parameters:		None
//
// Return Value:	None
//
void PersistentCollection::ClearAttributesChanged()
{
    pCollection->ClearAttributesChanged();
}



//
// Function Name:	PersistentCollection::Store()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Store a Document in the Collection.  
//
// Parameters:		StoredDocument - the Document to store
//
// Return Value:	None
//
void PersistentCollection::Store(Document &Document)
{
    pCollection->Store(Document);
}



//
// Function Name:	PersistentCollection::Update()
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Description:		Update a Document's representation in the Collection.  
//
// Parameters:		UpdateDocument - the Document to update
//
// Return Value:	None
//
void PersistentCollection::Update(Document &Document)
{
    pCollection->Update(Document);
}

