//
// Module Name:		cmcRealPersistentCollection.cc
//
// CSCI/CSC/CSU:	CMC/PersistentCollection
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	This contains the code for RealPersistentCollection.  A 
//	RealPersistentCollection is a type of Collection that is stored in
//	persistent storage.  It has two major parts: the ExternalIdIndex,
//	which is an Index that maps Document ExternalId's to their location
//	in the Collection, and the Volumes, which contain the actual 
//	Documents.
//
//	A RealPersistentCollection is configurable by its parameters when
//	first created.  There are two major configuration parameters:
//	the count of Volumes and the size of a data block in the Volume.
//	For heavy-weight Collections containing entire Documents, the
//	block offset should be the size of a data block as read from the
//	disk.  For light-weight Collections (those containing only Document
//	references), you may wish to decrease this to avoid wasting storage.
//	Count of Volumes should be selected based on expected size of the
//	Collection.  Very large Collections should be spread over multiple
//	Volumes.  Most other Collections should be put into only one Volume.
//	The defaults for Collection construction parameters are set for
//	light-weight Collections on the grounds that those will be created
//	more often.
//
// Function List:
//	RealPersistentCollection::RealPersistentCollection() - There are 2 
//			constructors for this
//	RealPersistentCollection::~RealPersistentCollection() - Destructor
//	RealPersistentCollection::Length() - Get number of Documents herein
//	RealPersistentCollection::RemoveDocument() - Remove a specified Document
//	RealPersistentCollection::GetDocument() - Retrieve a specified Document
//	RealPersistentCollection::FirstDocument() - Get the first Document in 
//			the Collection
//	RealPersistentCollection::NextDocument() - Get the next Document after 
//			the last returned by FirstDocument() or NextDocument()
//	RealPersistentCollection::GetByExternalId() - Get a Document given its 
//			external ID
//	RealPersistentCollection::Sync() - Synchronize the persistent version of
//			a Document with its in-memory version
//	RealPersistentCollection::Destroy() - Destroy a RealPersistentCollection
//	RealPersistentCollection::Store() - Store a Document in the 
//			RealPersistentCollection
//	RealPersistentCollection::Update() - Update a Document in the 
//			RealPersistentCollection
//	RealPersistentCollection::AnnotateCollection() - Annotate Documents in 
//			the Collection
//	RealPersistentCollection::PutAttribute() - Add an Attribute to the 
//			Collection
//	RealPersistentCollection::GetAttribute() - Get an Attribute of the 
//			Collection
//	RealPersistentCollection::GetAttributes() - Get the Attributes of the 
//			Collection
//	RealPersistentCollection::HasAttributes() - See if a Collection has a 
//			specified set of Attributes
//	RealPersistentCollection::IsAttributesChanged() - Test if this has been 
//			recently changed
//	RealPersistentCollection::ClearAttributesChanged() - Clear the changed 
//			indicator for this
// 	RealPersistentCollection::GetName() - Get name of this
//	RealPersistentCollection::NextDocument(Iterator&) - Get next document of
//				this
//
// Other Modules Called:
//	DocumentId - To parse Document ID's
//	Set - to hold sets of Attributes and Volumes
//	Volume - To store data
//	Index - To index data
//	AttributeValue - To hold values of Attributes
//	Attribute - To store information about the Collection
//	Document - What is stored herein
//	Collection - This is a subclass of Collection
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//

#include <stddef.h>			// for NULL
#include <string.h>			// for string functions

#include "cmcRealPersistentCollection.h"// for declaration of this
#include "cmcDocumentId.h"		// for a DocumentId structure
#include "cmcError.h"			// for exception classes

// the following are defaults.
// These sizes are meant to be appropriate for Document list-style collections.
const int RealPersistentCollection::DefaultVolumeCount = 1;
const int RealPersistentCollection::DefaultBlockSize = 256;


//
// Function Name:	RealPersistentCollection::RealPersistentCollection()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Construct a new RealPersistentCollection.
//
// Parameters:		NewName - The name of the RealPersistentCollection
//			NewAttributes - Attributes of it
//			VolumeCount - Number of Volumes for it to contain
//			BlockSize - Size of a data block in it
//
// Return Value:	None
//
RealPersistentCollection::RealPersistentCollection(const char *NewName, 
	const Set<Attribute> &NewAttributes, int VolumeCount, int BlockSize) : 
			PersistentCollectionStub(NewName, Set<Attribute>()),
			ExternalIdIndex(NewName, 1),
			pCurrentIterator(NULL)
{
    // construct new volumes for this
    for(int i = 0; i < VolumeCount; ++i)
    {
	Volume *pNew;		// the new volume

	if(i == 0)
	    pNew = new Volume(*this, GetName(), NewAttributes, i, BlockSize, 
				VolumeCount);
	else
	    pNew = new Volume(*this, GetName(), Set<Attribute>(), i, BlockSize, 
				VolumeCount);

	// test the allocation
	if(pNew == NULL)
	    throw LogiconErrors::MemoryAllocation(
	"RealPersistentCollection::RealPersistentCollection(char*,...)");

	try
	{
	    Volumes.Push(pNew);
	}
	catch(...)
	{
	    delete pNew;
	    throw;
	}
    }

    // sync the new collection
    Sync();
}



//
// Function Name:	RealPersistentCollection::RealPersistentCollection()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Open an existing RealPersistentCollection
//
// Parameters:		NewName - name of it
//
// Return Value:	None
//
RealPersistentCollection::RealPersistentCollection(const char *NewName) :
		PersistentCollectionStub(NewName, Set<Attribute>()),
		ExternalIdIndex(NewName),
		pCurrentIterator(NULL)
{
    // get the first volume
    Volume *pFirst = new Volume(*this, GetName(), 0);

    // test allocation
    if(pFirst == NULL)
	throw LogiconErrors::MemoryAllocation(
	    "RealPersistentCollection::RealPersistentCollection(char *)");

    int VolumeCount;			// count of volumes

    try
    {
    	// get data from it
    	VolumeCount = pFirst->GetVolumeCount();

    	// read all volumes
    	Volumes.Push(pFirst);
    }
    catch(...)
    {
	delete pFirst;
	throw;
    }

    for(int j = 1; j < VolumeCount; ++j)
    {
	Volume *pVolume = new Volume(*this, GetName(), j);

	if(pVolume == NULL)
	    throw LogiconErrors::MemoryAllocation(
	    "RealPersistentCollection::RealPersistentCollection(char *)");

	Volumes.Push(pVolume);
    }

    // sync the collection
    Sync();
}



//
// Function Name:	RealPersistentCollection::~RealPersistentCollection()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Destruct a RealPersistentCollection
//
// Parameters:		None
//
// Return Value:	None
//
RealPersistentCollection::~RealPersistentCollection(void)
{
    Sync();
    if(pCurrentIterator != NULL)
	delete pCurrentIterator;
}





//
// Function Name:	RealPersistentCollection::Length()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Get the number of Documents in the Collection by
//			summing the number of Documents in all of its Volumes.
//
// Parameters:		None
//
// Return Value:	The number of Documents herein
//
int RealPersistentCollection::Length(void) const
{
    int Result = 0;			// the total length

    // add up lengths of all volumes
    for(int i = 0; i < Volumes.Length(); ++i)
	Result += Volumes.Nth(i).Length();

    return Result;
}



//
// Function Name:	RealPersistentCollection::RemoveDocument()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Remove a specified Document from the Collection.
//
// Parameters:		Id - The ID of the Document to remove
//
// Return Value:	None
//
void RealPersistentCollection::RemoveDocument(const char *Id)
{
    char *ExtId;			// external Id to remove
    DocumentId DocId(Id);		// the ID to use

    try
    {
    	// remove it
    	if((ExtId = Volumes.Nth(DocId.GetVolume()).
			RemoveDocument(DocId.GetOffset(), Id)) != NULL)
    	{
	    try
	    {
	    	ExternalIdIndex.Remove(ExtId, 
				Location(DocId.GetVolume(), DocId.GetOffset()));
	    	delete[] ExtId;
	    }
	    catch(...)
	    {
	    	delete[] ExtId;
	    	throw;
	    }
	}
    }
    catch(LogiconErrors::NotFound)
    {
	// ignore this error
    }
}



//
// Function Name:	RealPersistentCollection::GetDocument()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Get a Document from the Collection.
//
// Parameters:		Id - The ID of the Document to get.
//
// Return Value:	The fetched Document.
//
Document *RealPersistentCollection::GetDocument(const char *Id)
{
    DocumentId DocId(Id);		// the ID to use
    Document *pResults;			// the found results

    try
    {
    	// Get it
    	pResults = Volumes.Nth(DocId.GetVolume()).
			GetDocument(DocId.GetOffset(), Id);
    }
    catch(LogiconErrors::NotFound)
    {
	pResults = NULL;
    }
	
    return pResults;
}



//
// Function Name:	RealPersistentCollection::FirstDocument()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Get the first Document in the Collection.  Do this
//			by resetting all Volumes and starting the NextDocument()
//			walk-through at the first volume.
//
// Parameters:		None
//
// Return Value:	The Document
//			NULL if there is none
//
Document *RealPersistentCollection::FirstDocument(void)
{
    // delete the old iterator if need be
    if(pCurrentIterator != NULL)
	delete pCurrentIterator;

    // construct a new iterator
    if((pCurrentIterator = new Iterator) == NULL)
    {
	throw LogiconErrors::MemoryAllocation(
		"RealPersistentCollection::FirstDocument(void)");
    }

    // iterate based on it
    return NextDocument(*pCurrentIterator);
}




//
// Function Name:	RealPersistentCollection::NextDocument()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Get the next Document after the last one retrieved by
//			FirstDocument or NextDocument.  This
//			maintains a member indicating which is the current
//			Volume being walked through.  This gets the next
//			Document in that current Volume.  (The Volume
//			supports a walkthrough through itself.)  If there
//			is no next Document in the Volume, this moves onto
//			the next Volume.
//
// Parameters:		None
//
// Return Value:	The next Document
//			NULL if there is none
//
Document *RealPersistentCollection::NextDocument(void)
{
    // if there is no iterator...
    if(pCurrentIterator == NULL)
	throw LogiconErrors::InvalidObject(
			"RealPersistentCollection::NextDocument(void)");

    // iterate based on it
    return NextDocument(*pCurrentIterator);
}



//
// Function Name:	RealPersistentCollection::GetByExternalId()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Get a Document given it's external ID
//
// Parameters:		ExternalId - The ID in question
//
// Return Value:	The Document
//			NULL if not found
//
Document *RealPersistentCollection::GetByExternalId(const char* ExternalId)
{
    Document *pResult = NULL;			// result to this call

    try
    {
    	// find the offset for this  
    	const Location &DocumentLocation = ExternalIdIndex.Find(ExternalId);

	pResult = Volumes.Nth(DocumentLocation.GetVolume()).
		GetDocument(DocumentLocation.GetOffset());
    }
    catch(LogiconErrors::NotFound)
    {
	// do nothing: if not found, we want to return NULL
    }

    return pResult;
}




//
// Function Name:	RealPersistentCollection::Sync()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Synchronize the RealPersistentCollection with its
//			persistent representation.
//
// Parameters:		None
//
// Return Value:	None
//
void RealPersistentCollection::Sync(void)
{
    // sync all volumes
    for(int i = 0; i < Volumes.Length(); ++i)
	Volumes.Nth(i).Sync();

    ExternalIdIndex.Sync();
}

//
// Function Name:	RealPersistentCollection::Destroy()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Destroy a RealPersistentCollection by destroying all
//			of its Volumes and its Index.  This must first
//			open the collection in order to determine how 
//			many Volumes it has.
//
// Parameters:		Name - Collection to destroy
//
// Return Value:	None
//
void RealPersistentCollection::Destroy(const char *Name)
{
    int VolumeNo = 0;			// number of current volume to destroy
    int Done = 0;			// are we done working?

    // destroy the Volumes
    while(!Done)
    {
	// destroy volumes as long as they are there
	try
	{
	    // destory a volume
	    Volume::Destroy(Name, VolumeNo);
	    VolumeNo++;
	}
	catch (LogiconErrors::Delete)
	{	
	    // if there was a failure on the first volume, pass it on
	    // else, assume that we have destroyed all volumes
	    if(VolumeNo == 0)
		throw;
	    else
		Done = 1;
	}
    }

    // destroy the Index
    Index::Destroy(Name);
}




//
// Function Name:	RealPersistentCollection::Store()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Store a Document in the Collection, assigning it a
//			new ID.  
//
//			Then add the ExternalId of this into the
//			ExternalIdIndex.
//
// Parameters:		StoredDocument - the Document to store
//
// Return Value:	None
//
void RealPersistentCollection::Store(Document &StoredDocument)
{
    // Select least populated Volume to put this into

    int VolumeNumber = 0;		// default VolumeNumber
    int VolumeSize = Volumes.Nth(0).Length();
					// size of the current volume

    // do for all other volumes
    for(int i = 1; i < Volumes.Length(); ++i)
    {
	if(Volumes.Nth(i).Length() < VolumeSize)
	{
	    VolumeNumber = i;
	    VolumeSize = Volumes.Nth(i).Length();
	}
    }

    // store the Document in the volume
    Volumes.Nth(VolumeNumber).Store(StoredDocument);

    // if there is an ExternalId to set...
    if(StoredDocument.GetExternalId() != NULL)
    {
	DocumentId Id(StoredDocument.GetId());	// id of this
        Location DocLocation(Id.GetVolume(), Id.GetOffset());
						// location of Document

        ExternalIdIndex.AddId(StoredDocument.GetExternalId(), DocLocation);
    }
}



//
// Function Name:	RealPersistentCollection::Update()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Update a Document's representation in the Collection,
//			replacing its old representation.
//
//			Then add the ExternalId of this into the
//			ExternalIdIndex.
//
// Parameters:		UpdatedDocument - the Document to store
//
// Return Value:	None
//
void RealPersistentCollection::Update(Document &UpdatedDocument)
{
    // check to make sure the Document has an ID
    if(UpdatedDocument.GetId() == NULL)
    {
	throw LogiconErrors::NotFound(
		"RealPersistentCollection::Update(Document&)");
    }

    // create a DocumentId from this
    DocumentId Id(UpdatedDocument.GetId());

    // use the existing location
    char *OldExternalId = Volumes.Nth(Id.GetVolume()).
				Store(UpdatedDocument, Id.GetOffset());

    try
    {
	// if there is an old ExternalId...
	if(OldExternalId != NULL)
	{
       	    // remove it
    	    ExternalIdIndex.Remove(OldExternalId,
			Location(Id.GetVolume(), 
				 Id.GetOffset()));
    	    delete[] OldExternalId;
    	    OldExternalId = NULL;
	}
    }
    catch(...)
    {
    	if(OldExternalId != NULL)
    	    delete[] OldExternalId;
    	throw;
    }

    // if there is an ExternalId to set...
    if(UpdatedDocument.GetExternalId() != NULL)
    {
    	Location DocLocation(Id.GetVolume(), Id.GetOffset());
						// location of Document

	ExternalIdIndex.AddId(UpdatedDocument.GetExternalId(), DocLocation);
    }
}





//
// Function Name:	RealPersistentCollection::AnnotateCollection()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Annotate the Documents in a Collection.  
//
// Parameters:		Which - Collection indicating which Documents to 
//				annotate
//			Annotator - The annotator in question
//
// Return Value:	None
//
void RealPersistentCollection::AnnotateCollection(Collection &Which,
			const char *Annotator)
{
    Iterator Walkthrough;		// to iterate through documents
    Document *pDocument;		// document to work with

    // do for all documents
    while((pDocument = NextDocument(Walkthrough)) != NULL)
    {
	Document *pTester;		// to test its appearance with

	// if this is in the Which collection...
	if((pTester = GetDocument(pDocument->GetId())) != NULL)
	{
	    // annotate it
	    pDocument->Annotate(Annotator);
	    pDocument->Sync();
	    delete pTester;
	}

	delete pDocument;
    }
}



//
// Function Name:	RealPersistentCollection::PutAttribute()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Add an Attribute to a Collection.  All Attributes
//			for the Collection are kept by the Collection's
//			first Volume.
//
// Parameters:		AttrName - Name of the Attribute
//			Value - New value of the Attribute
//
// Return Value:	None
//
void RealPersistentCollection::PutAttribute(const char *AttrName, 
			const AttributeValue &Value)
{
    Volumes.Nth(0).PutAttribute(AttrName, Value);
}




//
// Function Name:	RealPersistentCollection::GetAttribute()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Pass this call to the Volume's GetAttribute() function.
//
// Parameters:		AttributeName - Name of Attribute to get
//
// Return Value:	Its value
//
const AttributeValue *RealPersistentCollection::GetAttribute(
			const char *AttributeName) const
{
    return Volumes.Nth(0).GetAttribute(AttributeName);
}



//
// Function Name:	RealPersistentCollection::GetAttributes()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Pass this to the first Volume.
//
// Parameters:		None
//
// Return Value:	The set of Attributes for this
//
const Set<Attribute> &RealPersistentCollection::GetAttributes() const
{
    return Volumes.Nth(0).GetAttributes();
}


//
// Function Name:	RealPersistentCollection::GetAttributes()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Pass this to the first Volume: this is for const
//			attributes only.
//
// Parameters:		None
//
// Return Value:	The set of Attributes for this
//
Set<Attribute> &RealPersistentCollection::GetAttributes() 
{
    return Volumes.Nth(0).GetAttributes();
}



//
// Function Name:	RealPersistentCollection::HasAttributes()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Pass this to the first Volume.
//
// Parameters:		pBaseAttributes() - Attributes to test
//
// Return Value:	1 if it has them
//			0 otherwise
//
int RealPersistentCollection::HasAttributes(Set<Attribute> &BaseAttributes) 
				const
{
    return Volumes.Nth(0).HasAttributes(BaseAttributes);
}



//
// Function Name:	RealPersistentCollection::IsAttributesChanged()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Test whether this has been changed since it was
//			constructed or ClearChanged was called.
//
// Parameters:		None
//
// Return Value:	1 if it has been changed
//			0 if not
//
int RealPersistentCollection::IsAttributesChanged() const
{
    return Volumes.Nth(0).IsAttributesChanged();
}



//
// Function Name:	RealPersistentCollection::ClearAttributesChanged()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Clear the changed indicators of this and its member
//			objects.
//
// Parameters:		None
//
// Return Value:	None
//
void RealPersistentCollection::ClearAttributesChanged()
{
    Volumes.Nth(0).ClearAttributesChanged();
}



//
// Function Name:	RealPersistentCollection::GetName()
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Return the name of the PersistentCollection.
//
// Parameters:		None
//
// Return Value:	The name string.
//
const char *RealPersistentCollection::GetName() const
{
    return Collection::GetName();
}



//
// Function Name:	RealPersistentCollection::NextDocument(Iterator&)
//
// CSCI/CSC/CSU:	CMC/RealPersistentCollection
//
// Description:		Get the next document as indicated by the passed
//			in iterator.
//
// Parameters:		CurrentIterator - the Iterator guiding the walkthrough
//
// Return Value:	The next Document
//			NULL if there is none
//
Document *RealPersistentCollection::NextDocument(Iterator &CurrentIterator) 
{
    Document *pResults;			// results of this function

    // if there are no more volumes...
    if(CurrentIterator.VolumeNumber >= Volumes.Length())
    {
	pResults = NULL;
    }
    else
    {
	// if there is no volume iterator...
	if(CurrentIterator.pVolumeIterator == NULL)
	{
	    // make one
	    if((CurrentIterator.pVolumeIterator = new Volume::Iterator) == NULL)
	    {
		throw LogiconErrors::MemoryAllocation(
			"RealPersistentCollection::NextDocument(Iterator&)");
	    }
	}

	// iterate on this
	if((pResults = Volumes.Nth(CurrentIterator.VolumeNumber).
			NextDocument(*CurrentIterator.pVolumeIterator)) == NULL)
	{
	    // Try the next Volume
	    delete CurrentIterator.pVolumeIterator;
	    CurrentIterator.pVolumeIterator = NULL;
	    CurrentIterator.VolumeNumber++;
	    pResults = NextDocument(CurrentIterator);
	}
    }

    return pResults;
}

