//
// Module Name:		cmcRecordFile.cc
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for the RecordFile class.  A RecordFile is a file broken 
//	into a set of fixed length records that supports operations to allocate
//	and free records.  In addition, it will write data blocks starting
//	at a specified record, spilling over to other records as necessary
//	(allocating those records to the new entry).  The RecordFile also
//	keeps a header, which is maintained in allocated records just
//	as is any piece of data.
//
//	The length of a record can vary across RecordFiles, but within
//	a given RecordFile it is fixed according to an input parameter
//	when the record is first created.
//
//	The record file has the following layout:
//
//		<Header block>		0 - 1023
//		<Fat block 0>		1024 - 2047
//		<Record 0>		2048 - 2047 + RecordSize
//		<Record 1>		2048+RecordSize - 2047 + 2*RecordSize
//		...
//		<Record 8191>		2048 + 8191*RecordSize - 
//							2047 + 8192*RecordSize 
//		<Fat block 1>		2048 + 8192 * RecordSize -
//							3071 + 8192 * RecordSize
//		<Record 8192>		3072 + 8192 * RecordSize -
//							2071 + 8193 * RecordSize
//		...
//
//	In other words, the file starts with a header block that contains some
//	useful information (location of the first header record, number of total
//	records in the file, record size, etc).  Following that are a series
//	of record groups.  Each record group contains a 1 K FAT block and
//	8192 records.  The FAT block has one bit for each data record.  The
//	bit is set if the data record is already allocated and cleared if
//	the record is available.  The RecordFile caches the current FAT
//	block in memory for quick access.  When allocating a new record, 
//	the RecordFile attempts to allocate a record from the current FAT
//	block.  If that is not possible, it checks all existing FAT blocks
//	to see if some record is available for reuse.  If there are no such
//	records available, the RecordFile allocates a new record.
//
//	The first 8 bytes of each record are used by the RecordFile.
//	The first 4 bytes are used to hold the offset of the next record
//	following this one containing data.  This is -1 if this is the
//	last record for this block of data.  The next 4 bytes are used in
//	the first record for data to hold the total length of the stored
//	data.  In subsequent records for the same data, this is set to 
//	NthRecord to indicate this is not the first record.
//
//	Within a given FAT block (which is an integer array), the bit
//	for the Rth record is found by:
//	
//		FAT[R / 32] & (1 << RecordNo % 32)
//
//	In addition, the RecordFile maintains a lock on itself.  Whenever an
//	action is taken that would write data to the file, a write lock is
//	acquired on the corresponding file.  This lock is kept for as long as
//	the file allocation table is pending (in other words, as long as
//	there is buffered data that has not been written to file).  In addition,
//	if the file is not locked, the FAT table is not kept in memory.
//
// Function List:
//	Min() - Get the minimum of two values
//	RecordFile::RecordFile() - This has 2 constructors
//	RecordFile::~RecordFile() - Destructor
//	RecordFile::Destroy() - Destroy a RecordFile
//	RecordFile::AllocateRecord() - Allocate some record from the file
//	RecordFile::MarkAllocated() - Mark a specific record as allocated in
//			the file
//	RecordFile::FreeRecord() - Free some record in the file
//	RecordFile::WriteData() - Write out a data block to the file
//	RecordFile::ReadData() - Read a data block from the file
//	RecordFile::FreeData() - Free data in the file
//	RecordFile::WriteHeader() - Write a header to the file
//	RecordFile::ReadHeader() - Read the header from the file
//	RecordFile::Sync() - Synchronize the file to disk
//	RecordFile::GetNextRecord() - Get the next record in a walkthrough
//	RecordFile::ReadFatBlock() - Read in some specific FAT block
//	RecordFile::AllocateFromCurrentFat() - Allocate record from current 
//			FAT block if possible
//	RecordFile::GetFilename() - Construct a filename for this
//	RecordFile::GetLockFilename() - Construct lock filename for this
//	RecordFile::FatBlockNo() - Get the FAT block number for a given 
//			record
//	RecordFile::DataRecordOffset() - Find offset to a record in the file
//	RecordFile::CheckRecord() - See if a specified block is available in
//			the FAT
//	RecordFile::DataRecordSize() - Calculate space in the record 
//			available for data
//	RecordFile::MarkForDelete() - Mark this record file to be deleted on
//			destruction
//	RecordFile::Lock() - Lock the RecordFile
//	RecordFile::Unlock() - Unlock the RecordFile
//	RecordFile::FlushFat() - Flush the FAT block to disk
//	GetRecordFileDirectory() - Get the directory name of this
//
// Other Modules Called:
//	Error - Used for exceptions
//
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           12/15/95    jd_2     Initial version
//   1.5     09/24/96    bholmes  Suspect out-of-sync DataFileOpen flag is
//                                 causing DataFile to never be unlocked.
//                                 Elim. unneccesary flag and use fstream's.
//   1.6     03/06/97    bholmes  Unlock DataFile before closing it.
//

#include <errno.h>              // for errno 
#include <iostream.h>		// for I/O
#include <syslog.h>                 // For temp lock debugging...
#include <fstream.h>		// for file I/O
#include <stdlib.h>		// for standard C functions
#include <stdio.h>		// for standard C functions
#include <string.h>		// for string functions
#include <fcntl.h>		// for lock functions

#include "cmcRecordFile.h"	// for declaration of this
#include "cmcError.h"		// for exception classes

// Temp... added to debug Lock exception                  bholmes
void SetUpSyslog()
{
   static int initialized = 0;
   if (!initialized)
   {
      openlog("prides", (LOG_PID | LOG_CONS), LOG_LOCAL0);
      initialized = 1;
   }
}             //End of temp code...
// End of temporary debugging code....

const int RecordFile::FileHeaderLength = 1024;
						// length of header
const int RecordFile::FatBlockSize = 1024;	// size of FAT block
const int RecordFile::NoRecord = - 1;		// flag for no record
const int RecordFile::NthRecord = - 2;		// flag if not first record
const int RecordFile::HeaderRecordOffset = 4;
						// Offset to header block in
						//    first block
const int RecordFile::TotalCountOffset = 8;	// Offset to block count in 
						//    first block
const int RecordFile::RecordsPerFatBlock = 8 * FatBlockSize;
						// number of records per FAT

static const char *LockFileSuffix = ".lck";	// suffix for lock file

// the following gets the name of the directory where stuff is put
const char *GetRecordFileDirectory(); 	// directory where stuff is put


//
// Function Name:	Min()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Find the minimum of two inputs
//
// Parameters:		A, B - inputs to test
//
// Return Value:	The minimum of the two
//
static inline int Min(const int A, const int B)
{
    if(A < B)
	return A;
    else
	return B;
}


//
// Function Name:	RecordFile::RecordFile()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Construct a RecordFile by opening an existing file
//			and reading in necessary information from its
//			header block.
//
// Parameters:		NewName - Name of the RecordFile to construct.
//
// Return Value:	None
//
RecordFile::RecordFile(const char *NewName) : 
	Name(NULL),
//	DataFile(GetFilename(NewName), ios::in|ios::out|ios::nocreate),  //holmes r1.3
	LockFile(GetLockFilename(NewName), ios::in|ios::out),
	CurrentFatOffset(NoRecord), 
	TotalRecords(-1),
	FatPending(0),
	MarkedForDelete(0)
{
    try
    {
        SetUpSyslog();            //Temporary for debugging lock problem
	// test that the lockfile is okay
	if(!LockFile)
	{
	    throw LogiconErrors::Open("RecordFile::RecordFile(char*)");
	}

    	// store name
    	if((Name = new char[strlen(NewName) + 1]) == NULL)
	    throw LogiconErrors::MemoryAllocation(
			"RecordFile::RecordFile(char*)");
    	strcpy(Name, NewName);

	// open the data file
	OpenDataFile( ios::in|ios::out|ios::nocreate );  //holmes r1.3

    	// read in the record size
    	if(!DataFile.read((char *)&RecordSize, sizeof(int)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Read("RecordFile::RecordFile(char *)");
	}

	// close the file
	CloseDataFile();
    }
    catch(...)
    {
	if(Name != NULL)
	    delete[] Name;
	throw;
    }
}



//
// Function Name:	RecordFile::RecordFile()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Construct a new RecordFile by creating its file
//			and setting up its values to appropriate initial values,
//			writing them to the header block.
//
// Parameters:		NewName - Name of the new RecordFile
//			NewRecordSize - Size of Records in the new file
//
// Return Value:
//
RecordFile::RecordFile(const char *NewName, int NewRecordSize) :
	Name(NULL),
	LockFile(GetLockFilename(NewName), ios::in|ios::out),
	RecordSize(NewRecordSize), 
	CurrentFatOffset(NoRecord), 
	TotalRecords(0),
	FatPending(0),
	MarkedForDelete(0)
{
    try
    {
        SetUpSyslog();            //Temporary for debugging lock problem
    	// store name
    	if((Name = new char[strlen(NewName) + 1]) == NULL)
	    throw LogiconErrors::MemoryAllocation(
			"RecordFile::RecordFile(char*, int)");
    	strcpy(Name, NewName);

	// open the data file
	OpenDataFile(ios::noreplace);

    	// write out the record size
    	if(!DataFile.write((char *)&RecordSize, sizeof(int)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Write(
			"RecordFile::RecordFile(char*, int)");
	}

    	// write out the header offset
    	if(!DataFile.write((char *)&NoRecord, sizeof(int)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Write(
			"RecordFile::RecordFile(char*, int)");
	}

    	// write out the size
    	if(!DataFile.write((char *)&TotalRecords, sizeof(int)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Write(
			"RecordFile::RecordFile(char*, int)");
	}

    	// close the data file
	CloseDataFile();
    }
    catch(...)
    {
	if(Name != NULL)
	    delete[] Name;
	throw;
    }
}



//
// Function Name:	RecordFile::~RecordFile()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Destruct the RecordFile by first synchronizing it
//			with its disk representation.
//
// Parameters:		None
//
// Return Value:	None
//
RecordFile::~RecordFile()
{
    // synchronize its output
    Sync();

    // close it if necessary
    CloseDataFile();

    // destroy it if necessary
    if(MarkedForDelete)
	Destroy(Name);

    // delete its name
    delete[] Name;
}





//
// Function Name:	RecordFile::Destroy()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Destroy a RecordFile by deleting its file.
//
// Parameters:		Name - Name of record file to destroy.
//
// Return Value:	None
//
void RecordFile::Destroy(const char *Name)
{
    if(remove(GetFilename(Name)) != 0)
	throw LogiconErrors::Delete("RecordFile::Destroy(char*)");
    remove(GetLockFilename(Name));
}



//
// Function Name:	RecordFile::MarkAsAllocated()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Mark some record as allocated.
//
// Parameters:		RecordNo - Number of the record to mark
//
// Return Value:	The record number
//
int RecordFile::MarkAsAllocated(int RecordNo)
{
    int WordNo = (RecordNo % RecordsPerFatBlock) / 32;	// word in block
    int BitNo = RecordNo % 32;				// bit in byte

    // open the file
    OpenDataFile();

    // get and allocate the target block
    ReadFatBlock(FatBlockNo(RecordNo));
    CurrentFatBlock[WordNo] |= 1 << BitNo;

    // flush out the FAT changes
    FlushFat();

    return(RecordNo);
}


//
// Function Name:	RecordFile::AllocateRecord()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Allocate a new record in the RecordFile when any
//			record will do.  Try to allocate from the current
//			FAT.  If that is not possible, try all FAT blocks.
//
// Parameters:		None
//
// Return Value:	The new record number.
//
int RecordFile::AllocateRecord()
{
    int RecordNo = NoRecord;		// found result

    // open the file
    OpenDataFile();

    // if there is no current FAT block...
    if(CurrentFatOffset < 0)
    {
	// get one
	ReadFatBlock(0);
    }

    // Try the current FAT block for a data block
    if((RecordNo = AllocateFromCurrentFat()) < 0)
    {
	// not found: must look elsewhere

	int FatTested = CurrentFatOffset;	// FAT block already tested
	int FatIndex = 0;			// FAT block to test

	while(RecordNo < 0)
	{
	    // if we have not checked this one out yet...
	    if(FatIndex != FatTested)
	    {
		ReadFatBlock(FatIndex);
		RecordNo = AllocateFromCurrentFat();
	    }

	    ++FatIndex;
	}
    }

    // if the allocated record is greater then greatest allocated thus far...
    if(RecordNo >= GetTotalRecords())
    {
	// update the count for it
	TotalRecords = RecordNo + 1;

	// write it out
	if(!DataFile.seekp(TotalCountOffset))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Seek("RecordFile::AllocateRecord(void)");
	}
	if(!DataFile.write((char*)&TotalRecords, sizeof(int)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Write("RecordFile::AllocateRecord(void)");
	}
    }

    return RecordNo;
}



//
// Function Name:	RecordFile::FreeRecord()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Free a specified record.
//
// Parameters:		RecordNo - Record to free
//
// Return Value:	None
//
void RecordFile::FreeRecord(int RecordNo)
{
    int WordNo = (RecordNo % RecordsPerFatBlock) / 32;	// word in block
    int BitNo = RecordNo % 32;				// bit in byte

    // open the file
    OpenDataFile();

    // get and free the FAT bit
    ReadFatBlock(FatBlockNo(RecordNo));
    CurrentFatBlock[WordNo] &= ~(1 << BitNo);
}



//
// Function Name:	RecordFile::WriteData()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Write a data buffer to the RecordFile.  Chain the
//			records of that buffer together if it goes over
//			one record in length.
//
// Parameters:		RecordNo - Place to write the buffer
//			Data - the buffer
//			Length - Size of the buffer
//
// Return Value:	None
//
void RecordFile:: WriteData(int RecordNo, const char *Data, int Length)
{
    int NewRecord = NoRecord;	// offset of next record to use
    int FirstRecord = 1;	// flag to indicate if this is the first record

    // open the file
    OpenDataFile();

    // make sure the record is marked allocated
    MarkAsAllocated(RecordNo);

    // do as long as there is stuff to write
    while(Length > 0)
    {
	// determine next record to use
	if(Length > DataRecordSize())
	    NewRecord = AllocateRecord();
	else
	    NewRecord = NoRecord;

    	// position to location
    	if(!DataFile.seekp(DataRecordOffset(RecordNo)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Seek(
			"RecordFile::WriteData(int, char*, int)");
	}

	// write out the next record
	if(!DataFile.write((char *)&NewRecord, sizeof(int)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Write(
			"RecordFile::WriteData(int, char*, int)");
	}

	// if this is the first record, write out the entire length
	if(FirstRecord)
	{
    	    if(!DataFile.write((char *)&Length, sizeof(int)))
    	    {
		DataFile.clear(0);
	    	throw LogiconErrors::Write(
			"RecordFile::WriteData(int, char*, int)");
	    }
	}
	else
	{
	    // write out the Nth record flag
    	    if(!DataFile.write((char *)&NthRecord, sizeof(int)))
    	    {
		DataFile.clear(0);
	    	throw LogiconErrors::Write(
			"RecordFile::WriteData(int, char*, int)");
	    }
	}

	// write out data record
	if(!DataFile.write(Data, Min(Length, DataRecordSize())))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Write(
			"RecordFile::WriteData(int, char*, int)");
	}

	// prepare for next record of stuff
	RecordNo = NewRecord;
        Length -= DataRecordSize();
	Data += DataRecordSize();
	if(FirstRecord)
	{
	    FirstRecord = 0;
	}
    }

    FlushFat();
}



//
// Function Name:	RecordFile::ReadData()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Read in a data buffer from the file, following
//			next-chains as necessary.
//
// Parameters:		RecordNo - place to start reading
//			Length - place to put the read length
//
// Return Value:	The read buffer
//
char *RecordFile::ReadData(int RecordNo, int &Length)
{
    char *Data;				// read-in data
    char *ReadLocation;			// place to read to
    int RemainingLength;		// amount to read in all records
    int Filler;				// to take up space

    // open the file
    OpenDataFile();

    // Read in the length
    if(!DataFile.seekg(DataRecordOffset(RecordNo) + sizeof(int)))
    {
	DataFile.clear(0);
	throw LogiconErrors::Seek("RecordFile::ReadData(int, int&)");
    }
    if(!DataFile.read((char *)&Length, sizeof(int)))
    {
	DataFile.clear(0);
	throw LogiconErrors::Read("RecordFile::ReadData(int, int&)");
    }

    // allocate a buffer for it
    if((Data = new char[Length + 1]) == NULL)
	throw LogiconErrors::MemoryAllocation(
			"RecordFile::ReadData(int, int&)");

    try
    {
    	// prepare to read
    	RemainingLength = Length;
    	ReadLocation = Data;

    	// go read stuff
    	while(RemainingLength > 0 && RecordNo != NoRecord)
    	{
	    // position to the next record
	    if(!DataFile.seekg(DataRecordOffset(RecordNo)))
    	    {
		DataFile.clear(0);
	    	throw LogiconErrors::Seek(
				"RecordFile::ReadData(int, int&)");
	    }

	    // read the next record offset
	    if(!DataFile.read((char *)&RecordNo, sizeof(int)))
    	    {
		DataFile.clear(0);
	    	throw LogiconErrors::Read(
				"RecordFile::ReadData(int, int&)");
	    }

	    // skip over total length
    	    if(!DataFile.read((char *)&Filler, sizeof(int)))
    	    {
		DataFile.clear(0);
	    	throw LogiconErrors::Read(
				"RecordFile::ReadData(int, int&)");
	    }

	    // read data
	    if(!DataFile.read(ReadLocation, 
			Min(RemainingLength, DataRecordSize())))
    	    {
		DataFile.clear(0);
	   	throw LogiconErrors::Read(
				"RecordFile::ReadData(int, int&)");
	    }

	    // setup to read next stuff
	    ReadLocation += DataRecordSize();
	    RemainingLength -= DataRecordSize();
    	}
    }
    catch(...)
    {
	delete[] Data;
	throw;
    }

    Data[Length] = '\0';

    return Data;
}




//
// Function Name:	RecordFile::FreeData()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Free all data starting with a specified record.
//
// Parameters:		RecordNo - The place to start freeing
//
// Return Value:	None
//
void RecordFile::FreeData(int RecordNo)
{
    int NextRecordNo = NoRecord;			// next record to read

    // lock the file
    OpenDataFile();

    // do as long as there is a record
    while(RecordNo != NoRecord)
    {
	// get the next record
	if(!DataFile.seekg(DataRecordOffset(RecordNo)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Seek("RecordFile::FreeData(int)");
	}
	if(!DataFile.read((char *)&NextRecordNo, sizeof(int)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Read("RecordFile::FreeData(int)");
	}

	FreeRecord(RecordNo);

	RecordNo = NextRecordNo;
    }
}


//
// Function Name:	RecordFile::WriteHeader()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Write the header to file, chaining it if necessary.
//
// Parameters:		Header - The header.
//			HeaderLen - its length
//
// Return Value:	None
//
void RecordFile::WriteHeader(const char *Header, int HeaderLen)
{
    int NewHeaderRecord;		// record in which to store header

    // lock the file
    OpenDataFile();

    // allocate a record to use for the new header
    NewHeaderRecord = AllocateRecord();

    // write out the header record
    WriteData(NewHeaderRecord, Header, HeaderLen);

    // clear the old header
    FreeData(GetHeaderRecordOffset());

    // Store the header indicators
    if(!DataFile.seekp(HeaderRecordOffset))
    {
	DataFile.clear(0);
	throw LogiconErrors::Seek("RecordFile::WriteHeader(char*, int)");
    }
    if(!DataFile.write((char *)&NewHeaderRecord, sizeof(int)))
    {
	DataFile.clear(0);
	throw LogiconErrors::Write("RecordFile::WriteHeader(char*, int)");
    }

    // flush everything out
    FlushFat();
    if(!DataFile.flush())
    {
	DataFile.clear(0);
	throw LogiconErrors::Write("RecordFile::WriteHeader(char*, int)");
    }
}



//
// Function Name:	RecordFile::ReadHeader()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Read the header from file.
//
// Parameters:		HeaderLength - place to put length of header
//
// Return Value:	The header
//
char *RecordFile::ReadHeader(int &HeaderLength)
{
    return(ReadData(GetHeaderRecordOffset(), HeaderLength));
}




//
// Function Name:	RecordFile::Sync()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Sync the file with it's persistent version.  Write
//			out the current FAT block if it has been changed
//			since it was written.
//
// Parameters:		None
//
// Return Value:	None
//
void RecordFile::Sync()
{
    // close the data file
    CloseDataFile();
}



//
// Function Name:	RecordFile::GetNextRecord()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Get the offset to the next record that starts 
//			a data buffer.  Do this by moving past the NextRecord
//			indicator and finding the next record that is both
//			allocated and has a length in it (thus is not 
//			a non-first record).
//
// Parameters:		CurrentRecord - iterator to use in this walk
//
// Return Value:	The next record offset
//			NoRecord if there is not one
//
int RecordFile::GetNextRecord(Iterator &CurrentRecord)
{
    int FoundRecord = NoRecord;		// next record found
    int RecordFlag;			// flag to show record status

    // open the file
    OpenDataFile();

    int HeaderRecord = GetHeaderRecordOffset();
					// the header record offset 

    // do until something is found
    while(FoundRecord == NoRecord 
		&& CurrentRecord.RecordNumber < GetTotalRecords())
    {
	// if this one is set...
	if(CurrentRecord.RecordNumber != HeaderRecord
		&& CheckRecord(CurrentRecord.RecordNumber))
	{
	    // see if it is a first-record
	    if(!DataFile.seekg(DataRecordOffset(CurrentRecord.RecordNumber) 
			+ sizeof(int)))
    	    {
		DataFile.clear(0);
		throw LogiconErrors::Seek(
				"RecordFile::GetNextRecord(void)");
	    }
	    if(!DataFile.read((char *)&RecordFlag, sizeof(int)))
    	    {
		DataFile.clear(0);
		throw LogiconErrors::Read(
				"RecordFile::GetNextRecord(void)");
	    }
	    if(RecordFlag != NthRecord)
	    	FoundRecord = CurrentRecord.RecordNumber;
	}
	CurrentRecord.RecordNumber++;
    }

    return FoundRecord;
}




//
// Function Name:	RecordFile::ReadFatBlock()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Read in a specific FAT block.
//
// Parameters:		BlockNo - Block to read in
//
// Return Value:	None
//
void RecordFile::ReadFatBlock(int BlockNo)
{
    // open the file if this happens
    OpenDataFile();

    // if it must be read in...
    if(BlockNo != CurrentFatOffset)
    {
    	// if there is already one read in...
    	if(CurrentFatOffset != NoRecord)
	    FlushFat();

    	// position to the record and write
    	if(!DataFile.seekg(FatBlockOffset(BlockNo)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Seek("RecordFile::ReadFatBlock(int)");
	}
	if(!DataFile.read((char *)CurrentFatBlock, FAT_BLOCK_SIZE))
	{
	    DataFile.clear(0);
	    memset(CurrentFatBlock, 0, FAT_BLOCK_SIZE);
	}
    }

    CurrentFatOffset = BlockNo;
}



//
// Function Name:	RecordFile::AllocateFromCurrentFat()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Attempt to allocate a record from the current
//			FAT block.
//
// Parameters:		None
//
// Return Value:	The allocated record number if one is available.
//			-1 if not.
//
int RecordFile::AllocateFromCurrentFat()
{
    int WordNo;			// number of current byte
    int RecordNo = -1;		// resulting record

    // find a byte with a free space
    for(WordNo = 0; 
	WordNo < FAT_BLOCK_SIZE / 4 && CurrentFatBlock[WordNo] 
					== (unsigned int)0xffffffff;
	++WordNo)
    {}

    // if any is found...
    if(WordNo < FAT_BLOCK_SIZE / 4)
    {
	int BitNo;			// to walk through bits

	// pick a bit
	for(BitNo = 0; 
		BitNo < 32 && (CurrentFatBlock[WordNo] & (1 << BitNo)) != 0;
		++BitNo)
	{}

	// set the bit
	CurrentFatBlock[WordNo] |= (1 << BitNo);

	// allocate a record from that bit
	RecordNo = CurrentFatOffset * RecordsPerFatBlock + WordNo * 32 + BitNo;

	FatPending = 1;
    }

    return RecordNo;
}



//
// Function Name:	RecordFile::GetFilename()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Get the name of a file to hold this RecordFile.
//
// Parameters:		Name - The base RecordFile name.
//
// Return Value:	The corresponding filename.
//
const char *RecordFile::GetFilename(const char *Name)
{
    static char FileName[1000];		// to hold the result

    strcpy(FileName, GetRecordFileDirectory());
    strcat(FileName, "/");
    strcat(FileName, Name);

    cerr << "GetFilename returning filename: " << FileName ;
    return FileName;
}



//
// Function Name:	RecordFile::GetLockFilename()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Get the name of a file to hold this RecordFile's
//			lockfile.
//
// Parameters:		Name - The base RecordFile name.
//
// Return Value:	The corresponding filename.
//
const char *RecordFile::GetLockFilename(const char *Name)
{
    static char FileName[1000];		// to hold the result

    strcpy(FileName, GetRecordFileDirectory());
    strcat(FileName, "/");
    strcat(FileName, Name);
    strcat(FileName, LockFileSuffix);

    return FileName;
}



//
// Function Name:	RecordFile::FatBlockNo()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Find the FAT block that will contain the bit for a
//			given record.
//
// Parameters:		RecordNo - The record in question.
//
// Return Value:	The FAT block offset.
//
int RecordFile::FatBlockNo(int RecordNo) const
{
    return RecordNo / RecordsPerFatBlock;
}




//
// Function Name:	RecordFile::DataRecordOffset()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Get the offset to a specified record in the file.
//
// Parameters:		RecordNo - The record to find
//
// Return Value:	Its offset in the file.
//
int RecordFile::DataRecordOffset(int RecordNo) const
{
    return FileHeaderLength 
		+ (1 + RecordNo / RecordsPerFatBlock) * FatBlockSize 
		+ RecordNo * RecordSize;
}



//
// Function Name:	RecordFile::FatBlockOffset()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Get the offset to a specified FAT block in the file.
//
// Parameters:		BlockNo - the FAT block number
//
// Return Value:	Its file offset.
//
int RecordFile::FatBlockOffset(int BlockNo)
{
    return FileHeaderLength 
		+ BlockNo * FatBlockSize 
		+ BlockNo * RecordSize * RecordsPerFatBlock;    
}



//
// Function Name:	RecordFile::CheckRecord()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		See if a specified record is available in the FAT.
//
// Parameters:		RecordNo - the record to test
//
// Return Value:	the state of the FAT bit
//
int RecordFile::CheckRecord(int RecordNo)
{
    int WordNo = (RecordNo % RecordsPerFatBlock) / 32;	// byte in block
    int BitNo = RecordNo % 32;				// bit in byte

    ReadFatBlock(FatBlockNo(RecordNo));

    return (CurrentFatBlock[WordNo] & (1 << BitNo)) != 0;
}



//
// Function Name:	RecordFile::DataRecordSize()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Determine the space available for data in a record
//			(i.e., minus the RecordFile overhead)
//
// Parameters:		None
//
// Return Value:	The data record size.
//
int RecordFile::DataRecordSize() const
{
    return RecordSize - 2 * sizeof(int);
}



//
// Function Name:	RecordFile::MarkForDelete()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Mark this RecordFile to be deleted on destruction
//
// Parameters:		None
//
// Return Value:	None
//
void RecordFile::MarkForDelete() 
{
    MarkedForDelete = 1;
}



//
// Function Name:	RecordFile::OpenDataFile()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Open the RecordFile's DataFile if it is not open.
//
// Parameters:		int Flags - flags to use on open
//
// Return Value:	None
//
void RecordFile::OpenDataFile(int Flags)
{
    // if it is not open...
    if (!DataFile.rdbuf()->is_open())
    {
	// open the file
	DataFile.open(GetFilename(Name), ios::in|ios::out|Flags);
	if(!DataFile)
	{
	    throw LogiconErrors::Open("RecordFile::OpenDataFile(int)");
	}

        // Using advisory locking, wait until I get exclusive access
        // to the LockFile:
	flock_t lock;			// to hold lock information
	// setup type of lock to unlock the entire file
	lock.l_type = F_WRLCK;
	lock.l_whence = 0;
	lock.l_start = 0;
	lock.l_len = 0;

	// do the lock
        syslog(LOG_DEBUG, "  Locking %s.lck\n", Name);
RETRY_LOCK:                             //Quick and (very) dirty...
        errno = 0;
	int rc = fcntl(LockFile.rdbuf()->fd(), F_SETLKW, &lock);
	if(rc == -1)
	{
            syslog(LOG_ERR, "Error locking %s: errno=%d from fcntl(%d, F_SETLKW/F_WRLCK, %p),"
                   " which means: %m\n", Name, errno, LockFile.rdbuf()->fd(), &lock);
            if (errno == EINTR) goto RETRY_LOCK;
            DataFile.close();
	    throw LogiconErrors::Lock("RecordFile::OpenDataFile(int)");
	}
    }
}



//
// Function Name:	RecordFile::CloseDataFile()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Close a DataFile for this.
//
// Parameters:		None
//
// Return Value:	None
//
void RecordFile::CloseDataFile()
{
    	// Always unlock the LockFile, just in case the lock is out of 
        // synch with the data file open state.  Won't hurt to unlock
        // if we haven't got a lock.

    	flock_t lock;			// to hold lock information

    	// setup type of lock to unlock the entire file
    	lock.l_type = F_UNLCK;
    	lock.l_whence = 0;
    	lock.l_start = 0;
    	lock.l_len = 0;

    	// do the unlock
        syslog(LOG_DEBUG, "Unlocking %s.lck\n", Name);
RETRY_UNLOCK:                           //Quick and (very) dirty...
        errno = 0;
    	int rc = fcntl(LockFile.rdbuf()->fd(), F_SETLKW, &lock);
    	if(rc == -1)
    	{
            syslog(LOG_ERR, "Error unlocking %s: errno=%d from fcntl(%d, F_SETLKW/F_UNLCK, %p),"
                   " which means: %m\n", Name, errno, LockFile.rdbuf()->fd(), &lock);

	    LockFile.clear(0);
            if (errno == EINTR) goto RETRY_UNLOCK;
	    throw LogiconErrors::Lock("RecordFile::CloseDataFile(void)");
    	}
    
    	// clear the FAT block
    	CurrentFatOffset = NoRecord;


    if(DataFile.rdbuf()->is_open())
    {
    	DataFile.close();
    }
}



//
// Function Name:	RecordFile::GetHeaderRecordOffset()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Get the offset to the header record in the file.
//
// Parameters:		None
//
// Return Value:	The header offset
//
int RecordFile::GetHeaderRecordOffset()
{
    int HeaderRecord;			// the header record offset

    OpenDataFile();

    // position to the offset
    if(!DataFile.seekp(HeaderRecordOffset))
    {
	DataFile.clear(0);
	throw LogiconErrors::Seek(
			"RecordFile::GetHeaderRecordOffset()");
    }

    // read the offset
    if(!DataFile.read((char *)&HeaderRecord, sizeof(int)))
    {
	DataFile.clear(0);
	throw LogiconErrors::Read(
			"RecordFile::GetHeaderRecordOffset()");
    }

    return HeaderRecord;
}




//
// Function Name:	RecordFile::GetTotalRecords()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Get the count of the total records in the file.
//
// Parameters:		None
//
// Return Value:	The total record count
//
int RecordFile::GetTotalRecords()
{
    // if it is not present...
    if(TotalRecords == -1)
    {
	OpenDataFile();

	// position to the offset
    	if(!DataFile.seekp(TotalCountOffset))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Seek("RecordFile::GetTotalRecords()");
	}

	// read the offset
    	if(!DataFile.read((char *)&TotalRecords, sizeof(int)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Read("RecordFile::GetTotalRecords()");
	}
    }

    return TotalRecords;
}



//
// Function Name:	RecordFile::FlushFat()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Flush the buffered FAT block to disk.
//
// Parameters:		None
//
// Return Value:	None
//
void RecordFile::FlushFat()
{
    // if there is a current FAT block...
    if(CurrentFatOffset != NoRecord && FatPending)
    {
    	// Open the file
	OpenDataFile();

	// write out the FAT block
	if(!DataFile.seekp(FatBlockOffset(CurrentFatOffset)))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Seek("RecordFile::FlushFat(void)");
	}
	if(!DataFile.write((char *)CurrentFatBlock, FAT_BLOCK_SIZE))
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Write("RecordFile::FlushFat(void)");
	}

	// sync the Datafile to output
	if(!DataFile.flush())
    	{
	    DataFile.clear(0);
	    throw LogiconErrors::Write("RecordFile::FlushFat(void)");
	}
    }
}




//
// Function Name:	RecordFile::DumpFat()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Dump the fat to cout for debug purposes.
//
// Parameters:		None
//
// Return Value:	None
//
void RecordFile::DumpFat()
{
    ReadFatBlock(0);

    for(int i = 0; i < FAT_BLOCK_SIZE * 8; ++i)
    {
	if(CheckRecord(i))
	{
	    cout << "  " << i << " allocated\n";
	}
    }
    cout.flush();
}



//
// Function Name:	RecordFile::DumpRecords()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Dump the records chain to cout for debug purposes.
//
// Parameters:		None
//
// Return Value:	None
//
void RecordFile::DumpRecords()
{
    Iterator WalkthroughIterator;		// to walkthrough with
    int recno;					// to walkthrough with

    while((recno = GetNextRecord(WalkthroughIterator)) != NoRecord)
    {
	cout << "Start item\n";
	while(recno != NoRecord)
	{
	    cout << "\t" << recno << "\n";
            if(!DataFile.seekg(DataRecordOffset(recno)))
    	    {
		DataFile.clear(0);
            	throw LogiconErrors::Seek("RecordFile::FreeData(int)");
	    }
            if(!DataFile.read((char *)&recno, sizeof(int)))
    	    {
		DataFile.clear(0);
            	throw LogiconErrors::Read("RecordFile::FreeData(int)");
	    }
	}
    }

    cout << "Start header item\n";
    recno = GetHeaderRecordOffset();
    while(recno != NoRecord)
    {
	cout << "\t" << recno << "\n";
        if(!DataFile.seekg(DataRecordOffset(recno)))
    	{
	    DataFile.clear(0);
            throw LogiconErrors::Seek("RecordFile::FreeData(int)");
	}
        if(!DataFile.read((char *)&recno, sizeof(int)))
    	{
	    DataFile.clear(0);
            throw LogiconErrors::Read("RecordFile::FreeData(int)");
	}
    }

    cout.flush();
}




static const char *DirectoryVariable = "TIP_DATA_DIRECTORY";
						// environment variable 
						//	with directory name
// static const char *DefaultDirectory = "/export/home3/user-data";
static const char *DefaultDirectory = "./data";
						// default data directory
						// 	for files


static char *Directory = NULL;		// the directory name

//
// Function Name:	GetRecordFileDirectory()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Get the name of the directory for holding record files.
//
// Parameters:		None
//
// Return Value:	The directory name.
//
const char *GetRecordFileDirectory()
{

    // if there is no directory...
    if(Directory == NULL)
    {
	const char *Value;			// value to use

	// see if the environment variable is out there...
 	if((Value = getenv(DirectoryVariable)) == NULL)
	{
	    // use the default
	    Value = DefaultDirectory;
	}

	// store the value
	SetRecordFileDirectory( Value);
    }

    return Directory;
}

//
// Function Name:	SetRecordFileDirectory()
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Description:		Set the name of the directory for holding record files.
//
// Parameters:		char* Value - The directory name.
//
// Return Value:	None.
//
void SetRecordFileDirectory( const char* Value ) {
    delete Directory;
    Directory = new char[ strlen(Value) + 1 ];
    if ( ! Directory ) {
        throw LogiconErrors::MemoryAllocation("SetRecordFileDirectory()");
    }
    cerr << "in SetRecordFileDirectory(" << Value << ")" << endl;
    strcpy( Directory, Value );
}
