//
// Module Name:		cmcRecordFile.h
//
// CSCI/CSC/CSU:	CMC/RecordFile
//
// Author:		Ken Kolda
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//		RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	This includes the class definition for RecordFile.  A RecordFile
//	is a file that stores data in blocks.  The RecordFile maintains
//	links between those blocks to store large data buffers.
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           11/28/95    kolda    Initial version
//

#ifndef _cmcRecordFile_h
#define _cmcRecordFile_h

#include <fstream.h>			// fstream.h provides files

#define FAT_BLOCK_SIZE		1024	// size of a FAT block in bytes


extern const char* GetRecordFileDirectory();
extern void SetRecordFileDirectory( const char* Value );


// RecordFile is used to store CMC Volumes
class RecordFile
{
    private:
	static const int NoRecord;	// flag to indicate no block
	static const int NthRecord;	// flag to indicate this is not first
					// block in a piece of data
	static const int HeaderRecordOffset;
					// place to write header offset
	static const int TotalCountOffset;	
					// place to write total offset
	static const int RecordsPerFatBlock;
					// number of bits in a FAT block
	static const char RecordFileDirectory[];
					// file in which records are kept
	static const int FileHeaderLength;
					// length of a file header
	static const int FatBlockSize;	// size of a Fat block

	char *Name;			// name of this
	fstream DataFile;		// Physical stream for this RecordFile
	int DataFileOpen;		// flag showing if file is open
	fstream LockFile;		// Physical stream for lockfile for this
	int RecordSize;			// Size of data file records
	int CurrentFatOffset;		// Offset to Current FAT block
	unsigned int CurrentFatBlock[FAT_BLOCK_SIZE / 4];
					// Current FAT block
	int TotalRecords;		// max block allocated in the file
	int FatPending;			// flag if FAT changes have been made
	int MarkedForDelete;		// flag if has been marked for delete

	void ReadFatBlock(int RecordNo);// Reads the RecordNo'th FAT into
					// the CurrentFatBlock component
	int AllocateFromCurrentFat(void);
					// Allocates a free block from the
					// current FAT
	static const char *GetFilename(const char *Name);
					// get filename given volume name
	static const char *GetLockFilename(const char *Name);
					// get lock filename given volume name
	int FatBlockNo(int RecordNo) const;
					// get fat block that a block is in
	int DataRecordOffset(int RecordNo) const;
					// get the offset to a block in the file
	int FatBlockOffset(int RecordNo);
					// get the offset to a FAT block
	int CheckRecord(int RecordNo); 	// see if a block has been set
	int DataRecordSize() const;	// get the size of data in a record
	void OpenDataFile(int Flags = 0);
					// open the associated data file
	void CloseDataFile(); 		// close the associated data file
	int GetTotalRecords(void);	// get total count of records
	int GetHeaderRecordOffset(void);// get offset to the header record
	void FlushFat(void);		// flush the fat block to file

  public:
	class Iterator			// for iterating through file
	{
	    private:
		int RecordNumber;		// last record number tested
		friend RecordFile;		// allow RecordFile access to it
	
	    public:
		Iterator(void) { RecordNumber = 0; }
						// constructor
	};

	RecordFile(const char *Name);
					// Constructor for existing file
	RecordFile(const char *Name, int NewRecordSize);
					// Constructor for new file
					// NOTE: 8 bytes of each record are used
					//   by the RecordFile.  Adjust for this
					//   when selecting a NewRecordSize.
	virtual ~RecordFile(void);		// Destructor
	static void Destroy(const char *Name);
					// Deletes the name file
	int MarkAsAllocated(int RecordNo);
					// Marks RecordNo as used 
	int AllocateRecord(void); 	// Allocates new block in Record File,
					// returns index of block
	void FreeRecord(int RecordNo);	// Frees the given Record
	void WriteData(int RecordNo, const char *Data, int Length);
					// Writes the given data starting 
					// with block RecordNo
	char *ReadData(int RecordNo, int &Length);
					// Reads the given Record from disk
					// Put its length in Length
	void FreeData(int RecordNo);	// free the data at RecordNo
	void WriteHeader(const char *Header, int HeaderLen);
					// Writes a header to the RecordFile
	char *ReadHeader(int &HeaderLength);
					// Read the RecordFile's header
					// Put its length in HeaderLength
	void Sync(void);		// Flushes data to file
	int GetNextRecord(Iterator &Current);
					// Gets the next block offset
	void MarkForDelete(void);	// mark this for deletion

	void DumpFat(void);		// display the FAT
	void DumpRecords(void);
};



#endif
