//
// Module Name:		cmcReferenceCache.cc
//
// CSCI/CSC/CSU:	CMC/ReferenceCache
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for the ReferenceCache.  A ReferenceCache maintains a set
//	of references to allow them to be shared.  It maintains for each
//	item a count of references to it, deleting the item if
//	the count reaches 0.
//
// Function List:
// 	ReferenceCache::GetReference(char*) - Get reference from cache
//	ReferenceCache::DropReference(Reference&) - Drop reference to cache item
//	ReferenceCache::AddReference(Reference*) - add reference to cache
//	ReferenceCache::FindOffset(char*) - find offset to item in cache
//	ReferenceCache::FindOffset(Reference&) - find offset to item in cache
//
// Other Modules Called:
//	Set - to hold items
//	Reference - things in the cache
//	Error - for error classes
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           2/22/96     jd_2     Initial version
//

#include <string.h>			// for string operations

#include "cmcReferenceCache.h"		// includes for this
#include "cmcError.h"			// for error classes



//
// Function Name:	ReferenceCache::GetReference(char*)
//
// CSCI/CSC/CSU:	CMC/ReferenceCache
//
// Description:		Get a reference to some named item from the Cache.
//			Note the reference is gotten.  Do NOT return ownership
//			of the returned item.  Return NULL if not found
//
// Parameters:		Name - name of item to get reference of
//
// Return Value:	The reference gotten
//			NULL if not found
//
Reference *ReferenceCache::GetReference(const char *Name)
{
    // find the item
    int Offset = FindOffset(Name);

    // if not found...
    if(Offset < 0)
	return NULL;
    else
    {
    	// increment its references
    	Container.Nth(Offset).IncrementReferenceCount();

    	// return it
    	return &Container.Nth(Offset);
    }
}




//
// Function Name:	ReferenceCache::DropReference(Reference&)
//
// CSCI/CSC/CSU:	CMC/ReferenceCache
//
// Description:		Drop a reference to a specific item.  Decrement
//			its reference count and, if it becomes 0, remove
//			it from the cache.
//
// Parameters:		Target - The item to drop
//
// Return Value:	None
//
void ReferenceCache::DropReference(Reference &Target)
{
    // decrement its references
    Target.DecrementReferenceCount();

    // if there are no more references to it...
    if(Target.GetReferenceCount() <= 0)
    {
	int Offset = FindOffset(Target);	// offset to examine

	// if not found...
	if(Offset < 0)
	{
	    throw LogiconErrors::NotFound(
		"ReferenceCache::DropReference(Reference&)");
	}

    	Container.Delete(FindOffset(Target));
    }
}




//
// Function Name:	ReferenceCache::AddReference(Reference*)
//
// CSCI/CSC/CSU:	CMC/ReferenceCache
//
// Description:		Add a new item to the cache, incrementing its
//			reference count.
//
// Parameters:		pNewContents - New reference
//
// Return Value:	None
//
void ReferenceCache::AddReference(Reference *pNewContents)
{
    // see if it is already present
    if(FindOffset(pNewContents->GetName()) >= 0)
    {
	throw LogiconErrors::AlreadyPresent(
			"ReferenceCache::AddReference(Reference*)");
    }

    // do the work
    pNewContents->IncrementReferenceCount();
    Container.Push(pNewContents);
}



//
// Function Name:	ReferenceCache::FindOffset(char*)
//
// CSCI/CSC/CSU:	CMC/ReferenceCache
//
// Description:		Find the offset to a specific item in the cache.
//
// Parameters:		Name - name of item to find
//
// Return Value:	Its offset
//			-1 if not found
//
int ReferenceCache::FindOffset(const char *Name)
{
    int Offset = -1;			// offset to data

    // do for all entries...
    for(int i = 0; i < Container.Length(); ++i)
    {
	// if this is the one...
	if(strcmp(Container.Nth(i).GetName(), Name) == 0)
	{
	    Offset = i;
	    break;
	}
    }

    return Offset;
}




//
// Function Name:	ReferenceCache::FindOffset(Reference&)
//
// CSCI/CSC/CSU:	CMC/ReferenceCache
//
// Description:		Find a reference to a specific item in the cache.
//
// Parameters:		Target - Reference to find
//
// Return Value:	Offset to it
//			-1 if not found
//
int ReferenceCache::FindOffset(const Reference &Target)
{
    int Offset = -1;			// offset to data

    // do for all entries...
    for(int i = 0; i < Container.Length(); ++i)
    {
	// if this is the one...
	if(&Container.Nth(i) == &Target)
	{
	    Offset = i;
	    break;
	}
    }

    return Offset;
}
