//
// Module Name:		cmcSequenceValue.cc
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	This contains the code for the SequenceValue class.  A SequenceValue
//	is a type of AttributeValue that contains a sequence of 
//	other AttributeValues.
//
// Function List:
//	SequenceValue::SequenceValue() - This contains 2 constructors
//	SequenceValue::~SequenceValue() - Destructor
//	SequenceValue::GetValue() - Get the sequence of values for this
//	SequenceValue::TypeOf() - Get the type of this value
//	SequenceValue::Copy() - Produce a copy of this
//	SequenceValue::WriteToStream() - Write a text version of this to a 
//			stream
//	SequenceValue::IsChanged() - Test if this has been recently changed
//	SequenceValue::ClearChanged() - Clear the changed indicator for this
//	SequenceValue::operator== - See if this is equal to another 
//			AttributeValue
//	SequenceValue::GetClass() - Get the class label of this
//	SequenceValue::CastToSequenceValue() - Downcast function
//
// Other Modules Called:
//	Set<> - Keeps the AttributeValues in this
//	AttributeValue - This both is an AttributeValue and holds them
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           11/17/95    jd_2     Initial version
//

#include <iostream.h>			// for I/O
#include <stddef.h>			// for NULL

#include "cmcSequenceValue.h"		// for declaration of this
#include "cmcSet.h"			// to hold the sequence values
#include "cmcAttributeValue.h"		// the sequenced items
#include "cmcError.h"			// for exception classes


// ClassLabel is the string name of this class
const char *SequenceValue::ClassLabel = "SequenceValue";

//
// Function Name:	SequenceValue::SequenceValue()
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Construct a SequenceValue given a set of
//			AttributeValues for it to contain.
//
// Parameters:		NewValue - The AttributeValues for this to have
//
// Return Value:	None
//
SequenceValue::SequenceValue(const Set<AttributeValue> &NewValue) :
	Changed(0)
{
    // fill it in
    for(int i = 0; i < NewValue.Length(); ++i)
    {
	Value.Push(NewValue.Nth(i).Copy());
    }
}



//
// Function Name:	SequenceValue::SequenceValue()
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Construct a SequenceValue from a text representation
//			of it on a stream.  The format is as follows:
//
//				<SequenceValue AttributeValueType code>
//				<count of values> space
//				<Value 1> ... <Value N>
//
// Parameters:		Input - the stream to read from
//
// Return Value:	None
//
SequenceValue::SequenceValue(istream &Input) : Changed(0)
{
    int count;			// count of values to read
    AttributeValueType Type;	// type of the value
    char ch;			// character to skip

    // skip the type
    Input >> Type;

    // get the count of values
    Input >> count;
    Input.get(ch);

    // check Input state
    if(!Input)
	throw LogiconErrors::Read("SequenceValue::SequenceValue(istream&)");

    // put out the values
    for(int i = 0; i<count; ++i)
    {
	AttributeValue *p = ReadAttributeValue(Input);	// value read in 

	if(p != NULL)
	    Value.Push(p);
    }
}


//
// Function Name:	SequenceValue::~SequenceValue()
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Destruct a SequenceValue
//
// Parameters:		None
//
// Return Value:	None
//
SequenceValue::~SequenceValue()
{
}



//
// Function Name:	SequenceValue::GetValue()
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Get the set of Attributes in this.
//
// Parameters:		None
//
// Return Value:	The Value component
//
const Set<AttributeValue> &SequenceValue::GetValue() const
{
    return Value;
}




//
// Function Name:	SequenceValue::GetValue()
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Get the set of Attributes in this for non-constant 
//			stuff.
//
// Parameters:		None
//
// Return Value:	The Value component
//
Set<AttributeValue> &SequenceValue::GetValue()
{
    return Value;
}



//
// Function Name:	SequenceValue::TypeOf()
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Return the type of this AttributeValue
//
// Parameters:		None
//
// Return Value:	Return Sequence
//
AttributeValueType SequenceValue::TypeOf() const
{
    return SEQUENCE;
}



//
// Function Name:	SequenceValue::Copy()
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Produce a copy of this value
//
// Parameters:		None
//
// Return Value:	A new SequenceValue containing a copy of this
//
AttributeValue *SequenceValue::Copy() const
{
    AttributeValue *pCopy = new SequenceValue(Value);

    // test allocation
    if(pCopy == NULL)
	throw LogiconErrors::MemoryAllocation("SequenceValue::Copy(void)");

    return pCopy;
}



//
// Function Name:	SequenceValue::WriteToStream()
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Write a text representation of this to a stream in
//			the format specified under 
//			SequenceValue::SequenceValue(istream&).
//
// Parameters:		Output - the stream to write to
//
// Return Value:	None
//
void SequenceValue::WriteToStream(ostream &Output) const
{
    Output << TypeOf() << Value.Length() << ' ';
    for(int i = 0; i < Value.Length(); ++i)
    {
	Output << Value.Nth(i);
    }

    if(!Output)
	throw LogiconErrors::Write("SequenceValue::WriteToStream(ostream&)");
}



//
// Function Name:	SequenceValue::operator==
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Test if this is equal to another AttributeValue,
//			testing that it is a Sequence and contains the
//			same AttributeValues as does this.
//
// Parameters:		Comparer - AttributeValue to test against
//
// Return Value:	1 if they are equal
//			0 if not
//
int SequenceValue::operator==(const AttributeValue &Comparer) const
{
    // test type of comparer
    if(Comparer.TypeOf() != SEQUENCE)
	return 0;

    // if they do not have the same number of items, they are not equal
    if(Value.Length() != Comparer.CastToSequenceValue().Value.Length())
	return 0;

    // test all attributes in this
    for(int i = 0; i < Value.Length(); ++i)
    {
	if(Value.Nth(i) != Comparer.CastToSequenceValue().Value.Nth(i))
	    return 0;
    }

    // if we are here, it matches
    return 1;
}






//
// Function Name:	SequenceValue::IsChanged()
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Test whether this has been changed since it was
//			constructed or ClearChanged was called.
//
// Parameters:		None
//
// Return Value:	1 if it has been changed
//			0 if not
//
int SequenceValue::IsChanged() const
{
    // don't do anything if we know it has been changed...
    if(!Changed)
    {
	// test the set
	if(Value.IsChanged())
	{
	    // this violates const-ness of Changed
	    // This is strictly a performance optimization and does not
	    //		violate Changed's conceptual const-ness
	    ((SequenceValue*)this)->Changed = 1;
	}

	// test all set members
	for(int i = 0; !Changed && i < Value.Length(); ++i)
	    if(Value.Nth(i).IsChanged())
	    {
	    	// this violates const-ness of Changed
	    	// This is strictly a performance optimization and does not
	    	//	violate Changed's conceptual const-ness
	    	((SequenceValue*)this)->Changed = 1;
	    }
    }

    return Changed;
}



//
// Function Name:	SequenceValue::ClearChanged()
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Clear the changed indicators of this and its member
//			objects.
//
// Parameters:		None
//
// Return Value:	None
//
void SequenceValue::ClearChanged()
{
    // clear local changed flag
    Changed = 0;

    // clear changed flag of all Values and the Value set
    Value.ClearChanged();
    for(int i = 0; i < Value.Length(); ++i)
	Value.Nth(i).ClearChanged();
}





//
// Function Name:	SequenceValue::GetClass()
//
// CSCI/CSC/CSU:	CMC/SequenceValue
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *SequenceValue::GetClass() const
{
    return ClassLabel;
}



//
// Function Name:	SequenceValue::CastToSequenceValue()
//
// CSCI/CSC/CSU:	cmc/SequenceValue
//
// Description:		Downcast this as a SequenceValue
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
SequenceValue &SequenceValue::CastToSequenceValue()
{
    return *this;
}



//
// Function Name:	SequenceValue::CastToSequenceValue()
//
// CSCI/CSC/CSU:	cmc/SequenceValue
//
// Description:		Downcast this as a SequenceValue when this is const.
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
const SequenceValue &SequenceValue::CastToSequenceValue() const
{
    return *this;
}



