//
// Module Name:		cmcSet.cc
//
// CSCI/CSC/CSU:	CMC/Set
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Contain code for the Set<X> class.  This is a template class
//	that maintains sets of objects.  It corresponds to the TIPSTER
//	xset.
//
// Function List:
//	Set::Set() - constructor
//	Set::~Set() - destructor
//	Set::Nth() - Get the Nth item from the set
//	Set::Push() - Add an item to the set
//	Set::Pop() - Pop an item off the set
//	Set::PopNth() - Pops Nth item from the set
//	Set::Length() - Get the length of the set
//	Set::Sort() - Sort the items in the set
//	Set::Delete() - Delete the Nth item from the set
//	Set::Insert() - Insert an item to a specific location in the set
//	Set::Replace() - Replace an item at a specific location in the set
//	Set::Expand() - Expand internal storage for this
//	Set::IsChanged() - See if the set has been changed
//	Set::ClearChanged() - Clear the changed status
//	Set::GetClass() - Get the class label for this
//
// Other Modules Called:
//
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//  1.1       11/20/95    jd_2     Initial version
//  1.2       07/17/96    kolda    Replace now sets changed flag to TRUE
//

#include <stdlib.h>		// to get qsort()
#include <stddef.h>		// to get NULL
#include <strstream.h>		// to get string stream operations
#include <iostream.h>		// to get basic stream operations

#include "cmcSet.h"		// .h for this
#include "cmcError.h"		// contains exception classes

#ifdef __GNUG__
#include "cmcAnnotation.h"
#include "cmcAnnotationReference.h"
#include "cmcAnnotationSet.h"
#include "cmcAttribute.h"
#include "cmcAttributeReference.h"
#include "cmcAttributeValue.h"
#include "cmcAttributeValueType.h"
#include "cmcAttributedObject.h"
#include "cmcByteSequence.h"
#include "cmcCollection.h"
#include "cmcCollectionReference.h"
#include "cmcDetectionNeed.h"
#include "cmcDetectionNeedCollection.h"
#include "cmcDetectionQuery.h"
#include "cmcDocMgrObject.h"
#include "cmcDocument.h"
#include "cmcDocumentId.h"
#include "cmcDocumentReference.h"
#include "cmcError.h"
#include "cmcIndex.h"
#include "cmcIndexBlock.h"
#include "cmcIndexEntry.h"
#include "cmcInnerIndexBlock.h"
#include "cmcInnerIndexEntry.h"
#include "cmcLeafIndexBlock.h"
#include "cmcLeafIndexEntry.h"
#include "cmcLocation.h"
#include "cmcMisc.h"
#include "cmcObjectReference.h"
#include "cmcObjectReferenceVal.h"
#include "cmcPersistentCollection.h"
#include "cmcPersistentCollectionStub.h"
#include "cmcPersistentObject.h"
#include "cmcRealPersistentCollection.h"
#include "cmcRecordFile.h"
#include "cmcReference.h"
#include "cmcReferenceCache.h"
#include "cmcSecSurrogate.h"
#include "cmcSequenceValue.h"
#include "cmcSet.h"
#include "cmcSpan.h"
#include "cmcStringValue.h"
#include "cmcTipsterCSet.h"
#include "cmcVolatileCollection.h"
#include "cmcVolume.h"
#include "docdisplay.h"
#endif

#define SetSizeQuantum 100	// number of elements to put in a set

#ifndef __GNUG__
template <class X> const char *Set<X>::ClassLabel = "Set";
#endif
				// name of this class

//
// Function Name:	Set<X>::Set()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Construct a Set by allocating space for its data.
//
// Parameters:		None
//
// Return Value:	None
//
template <class X> Set<X>::Set() : 
	ppData(NULL), 
	Size(0), 
	AllocatedSize(0), 
	Changed(0)
{
    // allocate a block of space in data
    if((ppData = new X *[SetSizeQuantum]) == NULL)
	throw LogiconErrors::MemoryAllocation("Set<X>::Set(void)");

    // initialize sizes
    Size = 0;
    AllocatedSize = SetSizeQuantum;

#ifdef __GNUG__
    ClassLabel = "Set";
#endif
}

//
// Function Name:	Set<X>::~Set()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Destruct a set by freeing space for its members.
//
// Parameters:		None
//
// Return Value:	None
//
template <class X> Set<X>::~Set()
{
    int i;		// loop index

    // free all members in ppData
    for(i=0; i<Size; ++i)
	delete ppData[i];

    delete[] ppData;
}


//
// Function Name:	Set<X>::Nth()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Return the Nth element from the Set.
//
// Parameters:		N - Offset of item to get
//
// Return Value:	The Nth item
//
template <class X> X &Set<X>::Nth(int N) const
{
    if(N >= Size)
	throw LogiconErrors::OutOfBounds("Set<X>::Nth(int)");
    else
	return *ppData[N];
}



//
// Function Name:	Set<X>::Push()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Add Item to the set, making it the last item in the
//			set.
//
// Parameters:		pItem - The item to add
//
// Return Value:	None
//
template <class X> void Set<X>::Push(X *pItem)
{
    if(Size == AllocatedSize)
	Expand();

    ppData[Size] = pItem;
    Size++;
    Changed = 1;
}



//
// Function Name:	Set<X>::Pop()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Pop the last item from the set, removing it from
//			the set.
//
// Parameters:		None
//
// Return Value:	The last item
//
template <class X> X *Set<X>::Pop(void)
{
    if(Size <= 0)
	throw LogiconErrors::OutOfBounds("Set<X>::Pop()");
    else
    {
	Changed = 1;
	return(ppData[--Size]);
    }
}




//
// Function Name:	Set<X>::PopNth()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Pop the Nth item from a set
//
// Parameters:		N - The N in question
//
// Return Value:	The value of the Nth item
//
template <class X> X *Set<X>::PopNth(int N)
{
    if(N >= Size)
	throw LogiconErrors::OutOfBounds("Set<X>::PopNth(int)");
    else
    {
    	X *pResults = NULL;			// to hold data results

	// store the Nth item
	pResults = ppData[N];

	// Move all down by one
	for(int i = N + 1; i < Size; ++i)
	    ppData[i - 1] = ppData[i];
	Size--;
    
	Changed = 1;

	return pResults;
    }
}



//
// Function Name:	Set<X>::Length()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Return the number of items in the set.
//
// Parameters:		None
//
// Return Value:	Number of items in the set.
//
template <class X> int Set<X>::Length() const
{
    return Size;
}



//
// Function Name:	Set<X>::Sort()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Sort the elements in the set.
//
// Parameters:		pComparisonFunction - Function to use to compare two
//				set items
//
// Return Value:	None
//
template <class X> void Set<X>::Sort(int (*pComparisonFunction)(X**, X**))
{
    // do this by calling qsort()
    qsort(ppData, Size, sizeof(X *), 
	(int(*)(const void*, const void*))pComparisonFunction);
    Changed = 1;
}


//
// Function Name:	Set<X>::Delete()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Delete the Nth item from the set.
//
// Parameters:		N - the number of the item to delete.
//
// Return Value:	None
//
template <class X> void Set<X>::Delete(int N)
{
    if(N >= Size)
	throw LogiconErrors::OutOfBounds("Set<X>::Delete(int)");
    else
    {
	// delete the item
	delete ppData[N];

	// move all subsequent items up one space
	for(int i = N + 1; i < Size; ++i)
	    ppData[i-1] = ppData[i];
	Size--;
	Changed = 1;
    }
}




//
// Function Name:	Set<X>::Insert()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Insert an item to a specific location in a set
//
// Parameters:		N - Location to insert to
//			pItem - Item to insert
//
// Return Value:	None
//
template <class X> void Set<X>::Insert(int N, X *pItem)
{
    // as long as this is in the set...
    if(N > Size)
	throw LogiconErrors::OutOfBounds("Set<X>::Insert(int, X*)");
    else
    {
	// make sure there is room for this
	if(Size == AllocatedSize)
	    Expand();

	// move all items after N down a space to make room for N
	for(int i = Size; i > N; --i)
	{
	    ppData[i] = ppData[i - 1];
	}

	// insert the new item at N
	ppData[N] = pItem;
	++Size;

	Changed = 1;
    }
}




//
// Function Name:	Set<X>::Replace()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Replace an item at a specific location in the set
//
// Parameters:		N - Location to replace
//			pItem - Item to insert
//
// Return Value:	None
//
template <class X> void Set<X>::Replace(int N, X *pItem)
{
    // if this is in the set...
    if(N > Size)
	throw LogiconErrors::OutOfBounds("Set<X>::Replace(int, X*)");

    // out with the old...
    delete ppData[N];

    // in with the new...
    ppData[N] = pItem;

    Changed = 1;
}




//
// Function Name:	Set<X>::Expand()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Expand a set, allocating more internal space for it
//
// Parameters:		None
//
// Return Value:	None
//
template <class X> void Set<X>::Expand()
{
    // expand the size of ppData
    X **ppNewData;
    if((ppNewData = new X *[AllocatedSize + SetSizeQuantum]) == NULL)
	throw LogiconErrors::MemoryAllocation("Set<X>::Expand()");

    // move data to the expanded array
    for(int i = 0; i<Size; ++i)
    {
	ppNewData[i] = ppData[i];
    }

    // out with the old array and in with the new
    delete[] ppData;
    ppData = ppNewData;

    // update allocation size
    AllocatedSize += SetSizeQuantum;
}



//
// Function Name:	Set<X>::IsChanged()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Return the state of the Changed flag.
//
// Parameters:		None
//
// Return Value:	Changed
//
template <class X> int Set<X>::IsChanged() const
{
    return Changed;
}




//
// Function Name:	Set<X>::ClearChanged()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Clear the Changed flag.
//
// Parameters:		None
//
// Return Value:	None
//
template <class X> void Set<X>::ClearChanged()
{
    Changed = 0;
}



//
// Function Name:	Set<X>::GetClass()
//
// CSCI/CSC/CSU:	CMC/Set
//
// Description:		Get the text label for this class
//
// Parameters:		None
//
// Return Value:	The label
//
template <class X> const char *Set<X>::GetClass() const
{
    return ClassLabel;
}

#ifdef __GNUG__
template class Set<Document>;
template class Set<Span>;
template class Set<Attribute>;
template class Set<AttributeValue>;
template class Set<Annotation>;
template class Set<Volume>;
template class Set<SpanPlace>;
template class Set<IndexEntry>;
template class Set<Reference>;
#endif
