//
// Module Name:		cmcSpan.cc
//
// CSCI/CSC/CSU:	CMC/Span
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Contain the code for the Span class.  A Span designates a start
//	and end offset.
//
// Function List:
//	Span::Span() - There are 2 constructors here.
//	Span::GetStart() - Return the stored start of the Span
//	Span::GetEnd() - Return the end of the Span
//	Span::WriteToStream() - Write the Span to an output stream
//	operator<< - Write the Span to an output stream.
//	Span::GetClass() - Get class label of this
//	Span::CastToSpan() - Downcast function
//
// Other Modules Called:
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           11/17/95    jd_2     Initial version
//

#include "cmcSpan.h"		// for declaration of this
#include "cmcError.h"		// for exception classes

// ClassLabel is the string name of this class
const char *Span::ClassLabel = "Span";


//
// Function Name:	Span::Span()
//
// CSCI/CSC/CSU:	CMC/Span
//
// Description:		Construct a Span given its start and end.
//
// Parameters:		NewStart - start offset
//			NewEnd - end offset
//
// Return Value:	None
//
Span::Span(int NewStart, int NewEnd) : Start(NewStart), End(NewEnd)
{
}


//
// Function Name:	Span::Span()
//
// CSCI/CSC/CSU:	CMC/Span
//
// Description:		Construct a Span given a stream containing a text
//			representation of it in the following format:
//
//				<start offset> space <end offset>
//
// Parameters:		Input - the input stream
//
// Return Value:
//
Span::Span(istream &Input)
{
    Input >> Start >> End;

    if(!Input)
	throw LogiconErrors::Read("Span::Span(istream&)");
}




//
// Function Name:	Span::GetStart()
//
// CSCI/CSC/CSU:	CMC/Span
//
// Description:		Get the start offset of a Span
//
// Parameters:		None
//
// Return Value:	The Start component
//
Span::GetStart() const
{
    return Start;
}



//
// Function Name:	Span::GetEnd()
//
// CSCI/CSC/CSU:	CMC/Span
//
// Description:		Get the end offset of a Span
//
// Parameters:		None
//
// Return Value:	The End component
//
Span::GetEnd() const
{
    return End;
}


//
// Function Name:	Span::WriteToStream()
//
// CSCI/CSC/CSU:	CMC/Span
//
// Description:		Write a Span to an output stream in the format
//			specified under Span::Span(istream&).
//
//			This exists in addition to operator<< so that
//			a subclass could override it.
//
// Parameters:		Output - Stream to write to
//
// Return Value:	None
//
void Span::WriteToStream(ostream &Output) const
{
    Output << Start << ' ' << End;

    if(!Output)
	throw LogiconErrors::Write("Span::WriteToStream(ostream&)");
}



//
// Function Name:	operator<<
//
// CSCI/CSC/CSU:	CMC/Span
//
// Description:		Write a span to an output stream in the standard
//			C++ style.  
//
// Parameters:		Output - stream to write to
//			Data - Span to write
//
// Return Value:	The Output stream
//
ostream &operator<<(ostream &Output, Span &Data)
{
    Data.WriteToStream(Output);
    return Output;
}




//
// Function Name:	Span::GetClass()
//
// CSCI/CSC/CSU:	CMC/Span
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *Span::GetClass() const
{
    return ClassLabel;
}



//
// Function Name:	Span::CastToSpan()
//
// CSCI/CSC/CSU:	CMC/Span
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
Span &Span::CastToSpan()
{
    return *this;
}




//
// Function Name:	Span::CastToSpan()
//
// CSCI/CSC/CSU:	CMC/Span
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const Span &Span::CastToSpan() const
{
    return *this;
}


