//
// Module Name:		cmcStringValue.cc
//
// CSCI/CSC/CSU:	CMC/StringValue
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for the StringValue class.  A StringValue is a type of
//	AttributeValue that contains a string as its value.
//
// Function List:
//	StringValue::StringValue() - There are 2 constructors here
//	StringValue::~StringValue() - Destructor
//	StringValue::GetValue() - Get the value of this
//	StringValue::TypeOf() - Return the type of AttributeValue that this is
//	StringValue::Copy() - Produce a copy of this
//	StringValue::WriteToStream() - Write the value to an output stream
//	StringValue::operator== - Test equality with another AttributeValue
//	StringValue::GetClass() - Get class label of this
//	StringValue::CastToStringValue() - Downcast function
//
// Other Modules Called:
//	AttributeValue - This is a subclass of AttributeValue
//	Misc - This uses the Misc functions to write out the string
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           11/17/95    jd_2     Initial version
//

#include <iostream.h>			// for I/O
#include <string.h>			// for string functions

#include "cmcStringValue.h"		// for declaration of this
#include "cmcAttributeValue.h"		// for superclass of this
#include "cmcAttributeValueType.h"	// for TypeOf() return value
#include "cmcMisc.h"			// for string I/O
#include "cmcError.h"			// for exception classes

// ClassLabel is the string name of this class
const char *StringValue::ClassLabel = "StringValue";


//
// Function Name:	StringValue::StringValue()
//
// CSCI/CSC/CSU:	CMC/StringValue
//
// Description:		Construct a StringValue given the string to put in
//			it.
//
// Parameters:		NewValue - String to put in the StringValue.
//
// Return Value:	None
//
StringValue::StringValue(const char *NewValue)
{
    if((Value = new char[strlen(NewValue) + 1]) == NULL)
	throw LogiconErrors::MemoryAllocation(
			"StringValue::StringValue(char *)");
    strcpy(Value, NewValue);
}



//
// Function Name:	StringValue::StringValue()
//
// CSCI/CSC/CSU:	CMC/StringValue
//
// Description:		Construct a StringValue by reading it from an input
//			stream in the following format:
//
//				<StringValue AttributeValueType>
//				<The string in WriteString() format>
//
// Parameters:		Input - Stream to read from
//
// Return Value:	None
//
StringValue::StringValue(istream &Input)
{
    int length;			// length of the string
    char ch;			// character to get
    AttributeValueType Type;	// the gotten type

    // get the type
    Input >> Type;

    if(!Input)
	throw LogiconErrors::Read("StringValue::StringValue(istream&)");

    // read the string
    if((Value = ReadString(Input)) == NULL)
    {
	// string must be empty: store a null string
	if((Value = new char[1]) == NULL)
	    throw LogiconErrors::MemoryAllocation(
			"StringValue::StringValue(istream&)");
	strcpy(Value, "");
    }
}



//
// Function Name:	StringValue::~StringValue()
//
// CSCI/CSC/CSU:	CMC/StringValue
//
// Description:		Destruct a StringValue.
//
// Parameters:		None
//
// Return Value:	None
//
StringValue::~StringValue()
{
    if(Value != NULL)
    	delete[] Value;
}



//
// Function Name:	StringValue::GetValue()
//
// CSCI/CSC/CSU:	CMC/StringValue
//
// Description:		Get the string value of this.
//
// Parameters:		None
//
// Return Value:	The Value component.
//
const char *StringValue::GetValue() const
{
    return Value;
}



//
// Function Name:	StringValue::TypeOf()
//
// CSCI/CSC/CSU:	CMC/StringValue
//
// Description:		Get the type of AttributeValue that this is.
//
// Parameters:		None
//
// Return Value:	String
//
AttributeValueType StringValue::TypeOf() const
{
    return STRING;
}



//
// Function Name:	StringValue::Copy()
//
// CSCI/CSC/CSU:	CMC/StringValue
//
// Description:		Produce a copy of this.
//
// Parameters:		None
//
// Return Value:	The copy.
//
AttributeValue *StringValue::Copy() const
{
    AttributeValue *pCopy = new StringValue(Value);

    if(pCopy == NULL)
	throw LogiconErrors::MemoryAllocation("StringValue::Copy(void)");

    return pCopy;
}



//
// Function Name:	StringValue::WriteToStream()
//
// CSCI/CSC/CSU:	CMC/StringValue
//
// Description:		Write this value to an output stream in the format
//			described under StringValue::StringValue(istream &)
//
// Parameters:		Output - The output stream
//
// Return Value:	None
//
void StringValue::WriteToStream(ostream &Output) const
{
    Output << TypeOf();

    if(!Output)
	throw LogiconErrors::Write("StringValue::WriteToStream(ostream&)");

    WriteString(Value, Output);
}



//
// Function Name:	StringValue::operator==
//
// CSCI/CSC/CSU:	CMC/StringValue
//
// Description:		Test whether this is equal to another AttributeValue.
//
// Parameters:		Comparer - AttributeValue to test against
//
// Return Value:	1 if they are equal
//			0 if not
//
int StringValue::operator==(const AttributeValue &Comparer) const
{
    if(Comparer.TypeOf() != STRING 
	|| strcmp(Value, ((StringValue*)&Comparer)->Value) != 0)
	return 0;
    else
	return 1;
}




//
// Function Name:	StringValue::GetClass()
//
// CSCI/CSC/CSU:	CMC/StringValue
//
// Description:		Get a string description of the Class.
//
// Parameters:		None
//
// Return Value:	The string label of the Class.
//
const char *StringValue::GetClass() const
{
    return ClassLabel;
}




//
// Function Name:	StringValue::CastToStringValue()
//
// CSCI/CSC/CSU:	cmc/StringValue
//
// Description:		Downcast this as a StringValue
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
StringValue &StringValue::CastToStringValue()
{
    return *this;
}



//
// Function Name:	StringValue::CastToStringValue()
//
// CSCI/CSC/CSU:	cmc/StringValue
//
// Description:		Downcast this as a StringValue when this is const.
//
// Parameters:		None
//
// Return Value:	This as a sequence value.
//
const StringValue &StringValue::CastToStringValue() const
{
    return *this;
}




