//
// Module Name:		cmcTipsterCSet.cc
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for the TipsterCSet class.  TipsterCSet is used by the
//	C binding to store sets.  The C binding needs a generic Set
//	type that can be the input to any of the Set operations, thus
//	allowing overloading on the various Set types.  The TipsterCSet
//	provides this by supporting the set types used in the Tipster
//	architecture.
//	
//	This does mean that types of sets not directly mentioned in the
//	TIPSTER architecture are not supported by this binding.  Oh well.
//
// Function List:
//	TipsterCSet::TipsterCSet() - This contains two constructors for each 
//			supported set type
//	TipsterCSet::~TipsterCSet() - Destructor
//	TipsterCSet::Length() - Get number of items in the set
//	TipsterCSet::Nth() - Get the Nth item from the set
//	TipsterCSet::Pop() - Pop the last item from the set
//	TipsterCSet::Push() - Add an item to the set
//	TipsterCSet::GetStringSet() - Get the set of strings from this
//	TipsterCSet::GetAttributeSet() - Get the set of Attributes from this
//	TipsterCSet::GetAttributeValueSet() - Get the set of AttributeValues 
//			from this
//	TipsterCSet::GetSpanSet() - Get the set of Spans from this
//	TipsterCSet::GetClass() - Get the class label for this
//	TipsterCSet::CastToTipsterCSet() - Downcast function
//
// Other Modules Called:
//	Attribute - This can hold Attributes
//	AttributeValue - This can hold AttributeValues
//	Span - This can hold Spans
//	Set - To hold items
//	Error - Exception classes
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           1/2/96      jd_2     Initial version
//

#include <string.h>			// for string functions

#include "cmcTipsterCSet.h"		// declaration of this
#include "cmcAttribute.h"		// this can hold Attributes
#include "cmcAttributeValue.h"		// this can hold AttributeValues
#include "cmcSpan.h"			// this can hold Spans
#include "cmcSet.h"			// this maps to Sets
#include "cmcError.h"			// for exception classes

// ClassLabel is the name of this class
const char *TipsterCSet::ClassLabel = "TipsterCSet";


//
// Function Name:	TipsterCSet::TipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Construct a TipsterCSet by taking over a
//			string Set.
//
// Parameters:		pNewData - The set to take over
//
// Return Value:	None
//
TipsterCSet::TipsterCSet(Set<char *>* pNewData)
{
    pStringSet = pNewData;
    pAttributeSet = NULL;
    pAttributeValueSet = NULL;
    pSpanSet = NULL;
    pSecSurrogateSet = NULL;
}



//
// Function Name:	TipsterCSet::TipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Construct a TipsterCSet by taking over an
//			Attribute Set.
//
// Parameters:		pNewData - The set to take over
//
// Return Value:	None
//
TipsterCSet::TipsterCSet(Set<Attribute>* pNewData)
{
    pStringSet = NULL;
    pAttributeSet = pNewData;
    pAttributeValueSet = NULL;
    pSpanSet = NULL;
    pSecSurrogateSet = NULL;
}




//
// Function Name:	TipsterCSet::TipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Construct a TipsterCSet by taking over an
//			AttributeValue set.
//
// Parameters:		pNewData - The set to take over
//
// Return Value:	None
//
TipsterCSet::TipsterCSet(Set<AttributeValue>* pNewData)
{
    pStringSet = NULL;
    pAttributeSet = NULL;
    pAttributeValueSet = pNewData;
    pSpanSet = NULL;
    pSecSurrogateSet = NULL;
}

//
// Function Name:	TipsterCSet::TipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Construct a TipsterCSet by taking over an
//			SecSurrogate set.
//
// Parameters:		pNewData - The set to take over
//
// Return Value:	None
//
TipsterCSet::TipsterCSet(Set<SecSurrogate>* pNewData)
{
    pStringSet = NULL;
    pAttributeSet = NULL;
    pAttributeValueSet = NULL;
    pSpanSet = NULL;
    pSecSurrogateSet = pNewData;
}



//
// Function Name:	TipsterCSet::TipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Construct a TipsterCSet by taking over a
//			Span Set.
//
// Parameters:		pNewData - The set to take over
//
// Return Value:	None
//
TipsterCSet::TipsterCSet(Set<Span>* pNewData)
{
    pStringSet = NULL;
    pAttributeSet = NULL;
    pAttributeValueSet = NULL;
    pSpanSet = pNewData;
    pSecSurrogateSet = NULL;
}


//
// Function Name:	TipsterCSet::TipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Construct a TipsterCSet by copying a
//			string Set.
//
// Parameters:		NewData - the string Set to copy.
//
// Return Value:	None
//
TipsterCSet::TipsterCSet(Set<char *> &NewData)
{
    pSpanSet = NULL;
    pAttributeSet = NULL;
    pAttributeValueSet = NULL;
    pSecSurrogateSet = NULL;
    if((pStringSet = new Set<char*>) == NULL)
	throw LogiconErrors::MemoryAllocation(
		"TipsterCSet::TipsterCSet(Set<char*>&)");
	
    try
    {
    	for(int i=0; i<NewData.Length(); ++i)
    	{
	    char *pString = new char[strlen(NewData.Nth(i)) + 1];
	    if(pString == NULL)
		throw LogiconErrors::MemoryAllocation(
			"TipsterCSet::TipsterCSet(Set<char*>&)");
	    try
	    {
		pStringSet->Push(&pString);
	    }
	    catch (...)
	    {
		delete[] pString;
		throw;
	    }
	}
    }
    catch (...)
    {
	delete pStringSet;
	throw;
    }
}



//
// Function Name:	TipsterCSet::TipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Construct a TipsterCSet by copying an
//			Attribute Set.
//
// Parameters:		NewData - the string Set to copy.
//
// Return Value:	None
//
TipsterCSet::TipsterCSet(Set<Attribute> &NewData)
{
    pStringSet = NULL;
    pAttributeValueSet = NULL;
    pSpanSet = NULL;
    pSecSurrogateSet = NULL;

    if((pAttributeSet = new Set<Attribute>) == NULL)
	throw LogiconErrors::MemoryAllocation(
		"TipsterCSet::TipsterCSet(Set<Attribute>&)");
	
    try
    {
    	for(int i=0; i<NewData.Length(); ++i)
    	{
	    pAttributeSet->Push(NewData.Nth(i).Copy());
	}
    }
    catch (...)
    {
	delete pAttributeSet;
	throw;
    }
}




//
// Function Name:	TipsterCSet::TipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Construct a TipsterCSet by copying an
//			AttributeValue Set.
//
// Parameters:		NewData - the string Set to copy.
//
// Return Value:	None
//
TipsterCSet::TipsterCSet(Set<AttributeValue> &NewData)
{
    pStringSet = NULL;
    pAttributeSet = NULL;
    pSpanSet = NULL;
    pSecSurrogateSet = NULL;

    if((pAttributeValueSet = new Set<AttributeValue>) == NULL)
	throw LogiconErrors::MemoryAllocation(
		"TipsterCSet::TipsterCSet(Set<AttributeValue>&)");
	
    try
    {
    	for(int i=0; i<NewData.Length(); ++i)
    	{
	    pAttributeValueSet->Push(NewData.Nth(i).Copy());
	}
    }
    catch (...)
    {
	delete pAttributeValueSet;
	throw;
    }
}



//
// Function Name:	TipsterCSet::TipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Construct a TipsterCSet by copying a
//			Span Set.
//
// Parameters:		NewData - the string Set to copy.
//
// Return Value:	None
//
TipsterCSet::TipsterCSet(Set<Span> &NewData)
{
    pStringSet = NULL;
    pAttributeSet = NULL;
    pAttributeValueSet = NULL;
    pSecSurrogateSet = NULL;
    if((pSpanSet = new Set<Span>) == NULL)
	throw LogiconErrors::MemoryAllocation(
		"TipsterCSet::TipsterCSet(Set<Span>&)");
	
    try
    {
    	for(int i=0; i<NewData.Length(); ++i)
    	{
	    Span *pSpan = new Span(NewData.Nth(i));
	    if(pSpan == NULL)
		throw LogiconErrors::MemoryAllocation(
			"TipsterCSet::TipsterCSet(Set<Span>&)");
	    try
	    {
		pSpanSet->Push(pSpan);
	    }
	    catch (...)
	    {
		delete pSpan;
		throw;
	    }
	}
    }
    catch (...)
    {
	delete pSpanSet;
	throw;
    }
}




//
// Function Name:	TipsterCSet::~TipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Destructor.
//
// Parameters:		None
//
// Return Value:	None
//
TipsterCSet::~TipsterCSet()
{
    if(pStringSet != NULL)
	delete pStringSet;
    if(pAttributeSet != NULL)
	delete pAttributeSet;
    if(pAttributeValueSet != NULL)
	delete pAttributeValueSet;
    if(pSpanSet != NULL)
	delete pSpanSet;
    if(pSecSurrogateSet != NULL)
	delete pSecSurrogateSet;
}



//
// Function Name:	TipsterCSet::Length()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Get the number of items in the set.
//
// Parameters:		None
//
// Return Value:	Length of the set.
//
int TipsterCSet::Length() const
{
    if(pStringSet != NULL)
	return pStringSet->Length();
    else if(pAttributeSet != NULL)
	return pAttributeSet->Length();
    else if(pAttributeValueSet != NULL)
	return pAttributeValueSet->Length();
    else if(pSpanSet != NULL)
	return pSpanSet->Length();
    else if(pSecSurrogateSet != NULL)
	return pSecSurrogateSet->Length();
    else
	throw LogiconErrors::InvalidObject("TipsterCSet::Length()");
}



//
// Function Name:	TipsterCSet::Nth()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Return the Nth item from the set
//
// Parameters:		N - The N in question
//
// Return Value:	The Nth item
//
void *TipsterCSet::Nth(int N) const
{
    if(pStringSet != NULL)
	return (void*)pStringSet->Nth(N);
    else if(pAttributeSet != NULL)
	return (void*)&pAttributeSet->Nth(N);
    else if(pAttributeValueSet != NULL)
	return (void*)&pAttributeValueSet->Nth(N);
    else if(pSpanSet != NULL)
	return (void*)&pSpanSet->Nth(N);
    else if(pSecSurrogateSet != NULL)
	return (void*)pSecSurrogateSet->Nth(N).GetData();
    else
	throw LogiconErrors::InvalidObject("TipsterCSet::Nth(int)");
}





//
// Function Name:	TipsterCSet::Pop()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Pop the end item from the set
//
// Parameters:		None
//
// Return Value:	The last item in the set
//
void *TipsterCSet::Pop()
{
    if(pStringSet != NULL)
	return (void*)*pStringSet->Pop();
    else if(pAttributeSet != NULL)
	return (void*)pAttributeSet->Pop();
    else if(pAttributeValueSet != NULL)
	return (void*)pAttributeValueSet->Pop();
    else if(pSpanSet != NULL)
	return (void*)pSpanSet->Pop();
    else if(pSecSurrogateSet != NULL)
    {
	// get the surrogate
	SecSurrogate *pSurrogate = pSecSurrogateSet->Pop();

	// get the data from it
	void *pData = pSurrogate->ExtractData();

	// surrogate is no longer needed
	delete pSurrogate;

	return pData;
    }
    else
	throw LogiconErrors::InvalidObject("TipsterCSet::Pop()");
}



//
// Function Name:	TipsterCSet::Push()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Add an item to the set
//
// Parameters:		pItem - the Item to add
//
// Return Value:	None
//
void TipsterCSet::Push(void *pItem)
{
    if(pStringSet != NULL)
	pStringSet->Push((char **)&pItem);
    else if(pAttributeSet != NULL)
	pAttributeSet->Push((Attribute *)pItem);
    else if(pAttributeValueSet != NULL)
	pAttributeValueSet->Push((AttributeValue *)pItem);
    else if(pSpanSet != NULL)
	pSpanSet->Push((Span *)pItem);
    else if(pSecSurrogateSet != NULL)
    {
	// construct a surrogate around this
	SecSurrogate *pData = new SecSurrogate(pItem);

	if(pData == NULL)
	    throw LogiconErrors::MemoryAllocation("TipsterCSet::Push(void*)");
	pSecSurrogateSet->Push(pData);
    }
    else
	throw LogiconErrors::InvalidObject("TipsterCSet::Push(void*)");
}




//
// Function Name:	TipsterCSet::GetStringSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Get the StringSet component of the set.
//
// Parameters:		None
//
// Return Value:	The pStringSet component
//
Set<char *> &TipsterCSet::GetStringSet(void) const
{
    if(pStringSet == NULL)
	throw LogiconErrors::InvalidObject("TipsterCSet::GetStringSet()");

    return *pStringSet;
}


//
// Function Name:	TipsterCSet::GetAttributeSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Get the AttributeSet component of the set.
//
// Parameters:		None
//
// Return Value:	The pAttributeSet component
//
Set<Attribute> &TipsterCSet::GetAttributeSet(void) const
{
    if(pAttributeSet == NULL)
	throw LogiconErrors::InvalidObject("TipsterCSet::GetAttributeSet()");

    return *pAttributeSet;
}

//
// Function Name:	TipsterCSet::GetAttributeValueSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Get the AttributeValueSet component of the set.
//
// Parameters:		None
//
// Return Value:	The pAttributeValueSet component
//
Set<AttributeValue> &TipsterCSet::GetAttributeValueSet(void) const
{
    if(pAttributeValueSet == NULL)
	throw 
	    LogiconErrors::InvalidObject("TipsterCSet::GetAttributeValueSet()");

    return *pAttributeValueSet;
}

//
// Function Name:	TipsterCSet::GetSecSurrogateSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Get the SecSurrogateSet component of the set.
//
// Parameters:		None
//
// Return Value:	The pSecSurrogateSet component
//
Set<SecSurrogate> &TipsterCSet::GetSecSurrogateSet(void) const
{
    if(pSecSurrogateSet == NULL)
	throw 
	    LogiconErrors::InvalidObject("TipsterCSet::GetSecSurrogateSet()");

    return *pSecSurrogateSet;
}



//
// Function Name:	TipsterCSet::GetSpanSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Get the SpanSet component of the set.
//
// Parameters:		None
//
// Return Value:	The pSpanSet component
//
Set<Span> &TipsterCSet::GetSpanSet(void) const
{
    if(pSpanSet == NULL)
	throw LogiconErrors::InvalidObject("TipsterCSet::GetSpanSet()");

    return *pSpanSet;
}


//
// Function Name:	TipsterCSet::GetClass()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Get the Class label of this
//
// Parameters:		None
//
// Return Value:	The ClassLabel for this
//
const char *TipsterCSet::GetClass(void) const
{
    return ClassLabel;
}




//
// Function Name:	TipsterCSet::CastToTipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
TipsterCSet &TipsterCSet::CastToTipsterCSet()
{
    return *this;
}




//
// Function Name:	TipsterCSet::CastToTipsterCSet()
//
// CSCI/CSC/CSU:	CMC/TipsterCSet
//
// Description:		Downcast this to the current type.
//
// Parameters:		None
//
// Return Value:	*this
//
const TipsterCSet &TipsterCSet::CastToTipsterCSet() const
{
    return *this;
}


