//
// Module Name:		cmcVolatileCollection.h
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	Code for the VolatileCollection class.  A VolatileCollection is
//	a type of Collection that exists only in volatile memory.  It
//	keeps its Documents in a Set<Document> called Contents.
//
// Function List:
//	VolatileCollection::VolatileCollection() - Constructor
//	VolatileCollection::~VolatileCollection() - Destructor
//	VolatileCollection::Length() - Return the number of Documents in this 
//			Collection
//	VolatileCollection::RemoveDocument() - Remove a Document from the 
//			Collection
//	VolatileCollection::GetDocument() - Get a Document from the Collection
//	VolatileCollection::FirstDocument() - Return the first Document from 
//			the Collection
//	VolatileCollection::NextDocument() - Return the next Document from 
//			the Collection
//	VolatileCollection::GetByExternalId() - Get a Document given its 
//			ExternalId
//	VolatileCollection::AnnotateCollection() - Annotate a Collection
//	VolatileCollection::Store() - Store a Document in this Collection
//	VolatileCollection::Update() - Update a Document in this Collection
//	VolatileCollection::MakePersistent() - Make a persistent copy of this
//	VolatileCollection::MakeId() - Construct an ID for a new Document
//
// Other Modules Called:
//	Set - to hold Documents
//	Attribute - to hold Attributes of this
//	Document - Items stored herein
//	Collection - parent to this
//	PersistentCollection - to construct it in MakePersistent()
//	DocumentId - to construct DocumentId's
//	Error - Used for exceptions
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           1/2/96      jd_21    Initial version
//    1.2    9/23/96     BHolmes  During RemoveDocument(), fixup NextDocumentIndex
//                                 for iteration.
//

#include <sys/types.h>			// for basic system types
#include <time.h>			// for the time() function
#include <stddef.h>			// for NULL
#include <string.h>			// for string functions
#include <strstream.h>			// for string I/O
#include <time.h>			// to get current time

#include "cmcVolatileCollection.h"	// contains declarations for this
#include "cmcSet.h"			// for Document set stuff
#include "cmcAttribute.h"		// this has Attributes
#include "cmcDocument.h"		// what is kept herein
#include "cmcCollection.h"		// parent class to this
#include "cmcPersistentCollection.h"	// to construct one of these
#include "cmcDocumentId.h"		// to manipulate document ID's
#include "cmcError.h"			// for exception classes


//
// Function Name:	VolatileCollection::VolatileCollection()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Construct a new VolatileCollection
//
// Parameters:		NewAttributes - Attributes for this Collection
//
// Return Value:	None
//
VolatileCollection::VolatileCollection(const Set<Attribute> &NewAttributes) 
	: Collection(NewAttributes),
	  NextDocumentIndex(0),
	  NextDocumentId(0),
	  BasePtrValue((int)this)
{
}



//
// Function Name:	VolatileCollection::~VolatileCollection()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection()
//
// Description:		Destructor for this
//
// Parameters:		None
//
// Return Value:	None
//
VolatileCollection::~VolatileCollection()
{
}



//
// Function Name:	VolatileCollection::Length()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Return the number of Documents in the Volatile
//			Collection.
//
// Parameters:		None
//
// Return Value:	Number of Documents in the Contents set
//
int VolatileCollection::Length() const
{
    return Contents.Length();
}



//
// Function Name:	VolatileCollection::RemoveDocument()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Remove a Document from the VolatileCollection
//
// Parameters:		Id - ID of the Document to remove
//
// Return Value:	None
//
void VolatileCollection::RemoveDocument(const char *Id)
{
    int DocumentOffset;		// Offset of the document to remove

    if((DocumentOffset = FindDocument(Id)) >= 0)
    {
	Contents.Delete(DocumentOffset);
        if (DocumentOffset < NextDocumentIndex)
            NextDocumentIndex--;  //Account for Delete's moving of elements
    }
}



//
// Function Name:	VolatileCollection::GetDocument()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Retrieve a Document from the VolatileCollection
//
// Parameters:		Id - ID of the Document to retrieve
//
// Return Value:	The Document
//
Document *VolatileCollection::GetDocument(const char *Id)
{
    int DocumentOffset = FindDocument(Id);	// offset to document to use

    if(DocumentOffset < 0)
	return NULL;
    else
	return Contents.Nth(DocumentOffset).Copy();
}



//
// Function Name:	VolatileCollection::FirstDocument()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Get the First document from the VolatileCollection,
//			restarting the walk-through of the Collection.
//
// Parameters:		None
//
// Return Value:	The first Document
//			NULL if there is none
//
Document *VolatileCollection::FirstDocument()
{
    NextDocumentIndex = 0;

    return NextDocument();
}



//
// Function Name:	VolatileCollection::NextDocument()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Get the next Document following that returned
//			by the last call to NextDocument() or FirstDocument().
//			This maintains an internal counter of the
//			previously-viewed Document.
//
// Parameters:		None
//
// Return Value:	The next Document
//
Document *VolatileCollection::NextDocument()
{
    Document *pResults = NULL;		// results pointer

    // if there is a next Document...
    if(NextDocumentIndex < Contents.Length())
	pResults = Contents.Nth(NextDocumentIndex++).Copy();

    return pResults;
}



//
// Function Name:	VolatileCollection::GetByExternalId()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Retrieve a Document given its ExternalId
//
// Parameters:		ExternalId - The ExternalId in question
//
// Return Value:	The Document
//
Document *VolatileCollection::GetByExternalId(const char *ExternalId)
{
    Document *pResults = NULL;			// results pointer

    // check all Documents
    for(int i = 0; i < Contents.Length() && pResults == NULL; ++i)
    {
	if(strcmp(Contents.Nth(i).GetExternalId(), ExternalId) == 0)
	    pResults = Contents.Nth(i).Copy();
    }

    return pResults;
}




//
// Function Name:	VolatileCollection::AnnotateCollection()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Annotate all Documents in the Collection
//
// Parameters:		Which - If the Document is also in this Collection,
//				Annotate it
//			char *AnnotatorName - Name of Annotator to use
//
// Return Value:	None
//
void VolatileCollection::AnnotateCollection(Collection &Which, 
		const char *AnnotatorName)
{
    // do for all Documents in this collection...
    for(int i = 0; i < Contents.Length(); ++i)
    {
	Document *p;			// temp document pointer

	// if the Document is also in pWhich...
	if((p = Which.GetDocument(Contents.Nth(i).GetId())) != NULL)
	{
	    Contents.Nth(i).Annotate(AnnotatorName);
	    delete p;
	}
    }
}




//
// Function Name:	VolatileCollection::Store()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Store a new Document to the VolatileCollection.
//
// Parameters:		NewDocument - Document to store
//
// Return Value:	None
//
void VolatileCollection::Store(Document &NewDocument)
{
    // create a new ID for this
    NewDocument.SetId(MakeId());
    Contents.Push(NewDocument.Copy());
}




//
// Function Name:	VolatileCollection::Update()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Update a Document in the the VolatileCollection.
//
// Parameters:		UpdateDocument - Document to update
//
// Return Value:	None
//
void VolatileCollection::Update(Document &UpdateDocument)
{
    // find the Document offset
    int DocumentOffset = FindDocument(UpdateDocument.GetId());

    if(DocumentOffset < 0)
	Store(UpdateDocument);
    else
	Contents.Replace(DocumentOffset, UpdateDocument.Copy());
}





//
// Function Name:	VolatileCollection::MakePersistent()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Construct a PersistentCollection containing
//			all Documents from this Collection.
//
// Parameters:		NewName - Name of the new PersistentCollection to
//				construct
//
// Return Value:	The new PersistentCollection
//
PersistentCollection *VolatileCollection::MakePersistent(const char *NewName) 
			const
{
    PersistentCollection *pResults;		// the made collection

    // construct the new Collection
    if((pResults = new PersistentCollection(NewName, GetAttributes())) == NULL)
	throw LogiconErrors::MemoryAllocation(
			"VolatileCollection::MakePersistent(char *)");

    try
    {
    	// for all Documents in this Collection...
    	for(int i = 0; i < Contents.Length(); ++i)
    	{
	    pResults->Store(Contents.Nth(i));
    	}

    	// synchronize the new Collection
    	pResults->Sync();
    }
    catch(...)
    {
	delete pResults;
	throw;
    }

    return pResults;
}




//
// Function Name:	VolatileCollection::MakeId()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Construct a DocumentId for a new Document
//
// Parameters:		None
//
// Return Value:	The new ID
//
const char *VolatileCollection::MakeId()
{
    static char Id[100];		// to hold the ID

    // use pointer as volume number and NextDocumentId as number
    DocumentId DocId(BasePtrValue, NextDocumentId++, time(0));

    strcpy(Id, DocId.GetIdText());

    return Id;
}





//
// Function Name:	VolatileCollection::FindDocument()
//
// CSCI/CSC/CSU:	CMC/VolatileCollection
//
// Description:		Find offset to a Document given its ID
//
// Parameters:		Id - Document to find
//
// Return Value:	The offset to it
//			-1 if not found
//
int VolatileCollection::FindDocument(const char *Id) const
{
    int Results = -1;			// found results

    // loop through all Documents looking for it
    for(int i = 0; i < Contents.Length(); ++i)
    {
	// see if this is the one...
	if(strcmp(Id, Contents.Nth(i).GetId()) == 0)
	{
	    Results = i;
	    break;
	}
    }

    return Results;
}
