//
// Module Name:		cmcVolume.cc
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Author:		Joe Dzikiewicz
//			Logicon ITG
//			2100 Washington Boulevard
//			Arlington, VA 22204
//
// Security:		Unclassified
//
//			RESTRICTED RIGHTS LEGEND
//
// Use, duplication, or disclosure by the Government is subject to
// restrictions as set forth in subdivision (b) (3) (iii) of the Rights
// in Technical Data and Computer Software clause at 52.227-7013.
//
//	Logicon, Inc., 2100 Washington Blvd., Arlington, VA 22204-5703
//
// Description:
//	All code for Volume.  A Volume stores Documents as part of a 
//	Collection, providing persistent storage for those Documents.
//	Documents can be retrieved from the Volume given their offset
//	(as kept as part of the Document's ID).
//
// Function List:
//	Volume::Volume() - This contains 2 constructors
//	Volume::~Volume() - Destructor for this
//	Volume::GetVolumeCount() - Get VolumeCount component of this
//	Volume::Length() - Get number of Documents in the Volume
//	Volume::RemoveDocument() - Remove a Document from the Volume
//	Volume::NextDocument() - Get the next Document from the Volume
//	Volume::Store() - Store a Document to the Volume
//	Volume::GetFilename() - Get a Filename to hold a Volume
//	Volume::WriteHeader() - Write the Volume's header to file
//	Volume::ReadHeader() - Read the Volume's header from file
//	Volume::Sync() - Synchronize the Volume to disk
//	Volume::Destroy() - Destroy a Volume
//	Volume::PutAttribute() - Add an Attribute to the Volume
//	Volume::Cleanup() - Free memory allocated for the Volume
//
// Other Modules Called:
//	Set - to hold Attributes
//	Attribute - Volume has Attributes
//	RecordFile - to hold Volume's data
//	Document - Kept in the Volume
//	DocumentId - to construct ID's for Documents
//	Error - For exception classes
//
// Maintenance History
// Version	Date	Userid		Description of changes
// =======  =========== ======== ===========================================
//           1/2/96      jd_2     Initial version
//

#include <strstream.h>			// for string I/O
#include <iostream.h>			// for I/O
#include <string.h>			// for string functions
#include <stddef.h>			// for NULL
#include <sys/types.h>			// for system types
#include <time.h>			// to get the time

#include "cmcVolume.h"			// for declaration of this
#include "cmcSet.h"			// for a constructor
#include "cmcAttribute.h"		// for Attributes of this
#include "cmcRecordFile.h"		// for the storage of this
#include "cmcDocument.h"		// for what is stored
#include "cmcDocumentId.h"		// for constructing Document ID's
#include "cmcError.h"			// for exception classes

//
// Function Name:	Volume::Volume()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Construct a Volume to map to an already existing
//			Volume.
//
// Parameters:		NewParent - Collection in which this Volume is a part
//			NewName - Name of the Volume
//			NewVolumeNo - Number of Volume that this is
//
// Return Value:	None
//
Volume::Volume(Collection &NewParent, const char *NewName, int NewVolumeNo) : 
	Parent(NewParent),
	File(GetFilename(NewName, NewVolumeNo)),
	DocumentCount(0),
	VolumeCount(0),
	Name(NULL),
	VolumeNo(NewVolumeNo),
	HeaderChanged(0)
{
    try
    {
    	// save the input parameters
    	if((Name = new char [strlen(NewName) + 1]) == NULL)
	    throw LogiconErrors::MemoryAllocation(
			"Volume::Volume(Collection&, char*, int)");
    	strcpy(Name, NewName);

    	// read in the header
    	ReadHeader();
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}



//
// Function Name:	Volume::Volume()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Construct a brand-new Volume that does not yet
//			exist in persistent storage.
//
// Parameters:		NewParent - Parent Collection to this Volume
//			NewName - Name of this Volume
//			NewVolumeNo - number of this Volume
//			RecordSize - size of a record in this Volume
//			NewVolumeCount - Total number of volumes assocated with
//				this
//
// Return Value:	None
//
Volume::Volume(Collection &NewParent, const char *NewName, 
		const Set<Attribute> &NewAttributes, int NewVolumeNo, 
		int RecordSize, int NewVolumeCount)
	: AttributedObject(NewAttributes),
	  Parent(NewParent),
	  File(GetFilename(NewName, NewVolumeNo), RecordSize),
	  DocumentCount(0), 
	  VolumeCount(NewVolumeCount), 
	  Name(NULL),
	  VolumeNo(NewVolumeNo),
	  HeaderChanged(1)
{
    try
    {
        // save the input parameters
    	if((Name = new char [strlen(NewName) + 1]) == NULL)
	    throw LogiconErrors::MemoryAllocation(
  "Volume::Volume(Collection&, char*, Set<Attribute>&, int, int, int)");
    	strcpy(Name, NewName);

    	// construct and save the header and write everything out
    	Sync();
    }
    catch(...)
    {
	Cleanup();
	throw;
    }
}






//
// Function Name:	Volume::~Volume()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Destructor for a Volume
//
// Parameters:		None
//
// Return Value:	None
//
Volume::~Volume()
{
    Cleanup();
}




//
// Function Name:	Volume::GetVolumeCount()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Return the VolumeCount component
//
// Parameters:		None
//
// Return Value:	The VolumeCount component
//
int Volume::GetVolumeCount() const
{
    return VolumeCount;
}



//
// Function Name:	Volume::Length()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Get the number of Documents in this Volume
//
// Parameters:		None
//
// Return Value:	The DocumentCount component
//
int Volume::Length() const
{
    return DocumentCount;
}



//
// Function Name:	Volume::RemoveDocument()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Remove a Document from the Volume
//
// Parameters:		Offset - Offset to the Document in the Volume
//			Id - ID that the document should have
//
// Return Value:	External ID of the removed Document
//
char *Volume::RemoveDocument(int Offset, const char *Id)
{
    Document *pDocument = GetDocument(Offset, Id);
						// the Document there
    char *ExtId = NULL;				// external ID of Document

    try
    {
    	// verify the Document is gotten and is correct
    	if(pDocument == NULL)
	    throw LogiconErrors::NotFound("Volume::RemoveDocument(int)");

    	// get the ExternalId of the Document
    	if(pDocument->GetExternalId() != NULL)
    	{
            if((ExtId = new char[strlen(pDocument->GetExternalId()) + 1]) 
				== NULL)
	    	throw LogiconErrors::MemoryAllocation(
			"Volume::RemoveDocument(int)");
	    strcpy(ExtId, pDocument->GetExternalId());
    	}

    	// free the data storage
    	File.FreeData(Offset);

    	delete pDocument;
    	pDocument = NULL;

    	--DocumentCount;
    	HeaderChanged = 1;

    	return ExtId;
    }
    catch(...)
    {
	if(ExtId != NULL)
	    delete[] ExtId;
	if(pDocument != NULL)
	    delete pDocument;
	throw;
    }
}



//
// Function Name:	Volume::GetDocument()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Get a specific Document from the Volume
//
// Parameters:		Offset - Offset to the desired Document
//			Id - ID that the Document should have
//
// Return Value:	The Document
//
Document *Volume::GetDocument(int Offset, const char *Id)
{
    char *Data = NULL;				// buffer of document
    int DataLength;				// length of Data
    Document *pNewDocument = NULL;		// the Document

    try
    {
    	// verify offset 
    	if(Offset < 0)
	    throw LogiconErrors::Parameter("Volume::GetDocument(int)");

    	// read in the data from this
    	Data = File.ReadData(Offset, DataLength);
	File.Sync();

    	// construct the corresponding Document
    	if((pNewDocument = 
		new Document(Parent, istrstream(Data, DataLength))) == NULL)
	    throw LogiconErrors::MemoryAllocation("Volume::GetDocument()");

	// check that it is correct
	if(Id != NULL && strcmp(pNewDocument->GetId(), Id) != 0)
	{
	    throw LogiconErrors::NotFound("Volume::GetDocument(int, char*)");
	}

	// delete the Data buffer
	delete[] Data;

    	return pNewDocument;
    }
    catch(LogiconErrors::Read)
    {
	if(Data != NULL)
	    delete[] Data;
	if(pNewDocument != NULL)
	    delete pNewDocument;
	throw LogiconErrors::NotFound("Volume::GetDocument(int, char*)");
    }
    catch(...)
    {
	if(Data != NULL)
	    delete[] Data;
	if(pNewDocument != NULL)
	    delete pNewDocument;
	throw;
    }
}





//
// Function Name:	Volume::NextDocument()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Get the next Document in a walk-through of the
//			Volume
//
// Parameters:		Walkthrough - The iterator to use
//
// Return Value:	The next Document
//			NULL if there is none
//
Document *Volume::NextDocument(Iterator &Walkthrough)
{
    int Offset = -1;			// next offset to get
    Document *pResults;			// Document to get

    // get the next offset
    if((Offset = File.GetNextRecord(Walkthrough.Value)) >= 0)
	pResults = GetDocument(Offset);
    else
    {
	pResults = NULL;
	File.Sync();
    }

    return pResults;
}




//
// Function Name:	Volume::Store()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Store a Document in the Volume to any old place, 
//			assigning it a new DocumentId
//
// Parameters:		Doc - The Document to store
//
// Return Value:	None
//
void Volume::Store(Document &Doc)
{
    ostrstream Output;			// output stream to hold Document
    int RecordNo = -1;			// block to place Document
    char *Data = NULL;			// the data

    try
    {
    	// allocate a block for this
    	RecordNo = File.AllocateRecord();

    	// set the Document's ID
    	DocumentId DocId(VolumeNo, RecordNo, time(0));
    	Doc.SetId(DocId.GetIdText());

    	// output the Document
    	Doc.WriteToStream(Output);

	// get the data
	Data = Output.str();

    	// write out the data
    	File.WriteData(RecordNo, Data, Output.pcount());

    	// update document count for this
    	++DocumentCount;
    	HeaderChanged = 1;

	// free stuff
	delete[] Data;
	Data = NULL;
    }
    catch(...)
    {
	if(RecordNo >= 0)
	    File.FreeRecord(RecordNo);
	if(Data != NULL)
	    delete[] Data;
	throw;
    }
}



//
// Function Name:	Volume::Store()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Store a Document to some specific place in the
//			Volume (most likely because this is an update
//			to an existing Document)
//
// Parameters:		Doc - the Document to store
//			RecordNo - the place to store it
//
// Return Value:	The ExternalId of the old version of the Document
//
char *Volume::Store(Document &Doc, int RecordNo)
{
    ostrstream Output;			// output stream to hold Document
    char *pResult = NULL;		// return result
    char *Data = NULL;			// data to store

    try
    {
    	// output the Document
    	Doc.WriteToStream(Output);

    	// free the current data
    	pResult = RemoveDocument(RecordNo, Doc.GetId());

    	// allocate the block again
    	File.MarkAsAllocated(RecordNo);

	// get the data
	Data = Output.str();

    	// write out the data
    	File.WriteData(RecordNo, Data, Output.pcount());

    	// update doc count for this to offset its decrement in RemoveDocument
    	++DocumentCount;
    	HeaderChanged = 1;

	delete[] Data;
	Data = NULL;

    	return pResult;
    }
    catch(...)
    {
	if(pResult != NULL)
	    delete[] pResult;
	if(Data != NULL)
	    delete[] Data;
	throw;
    }
}



//
// Function Name:	Volume::GetFilename()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Generate a filename for a specific Volume
//
// Parameters:		CollectionName - Name of associated Collection
//			VolumeNumber - number of this Volume
//
// Return Value:	The filename
//
const char *Volume::GetFilename(const char *CollectionName, int VolumeNumber)
{
    ostrstream Output;			// to hold buffer version of it
    static char *Filename;		// to hold the filename

    if(Filename != NULL)
    {
	delete[] Filename;
	Filename = NULL;
    }

    Output << CollectionName << '.' << VolumeNumber << '\0';

    if(!Output)
	throw LogiconErrors::Write("Volume::GetFilename(char*, int)");

    Filename = Output.str();

    return Filename;
}



//
// Function Name:	Volume::WriteHeader()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Write out the header of the Volume
//
// Parameters:		None
//
// Return Value:	None
//
void Volume::WriteHeader()
{
    ostrstream Output;		// to hold header output

    if(HeaderChanged || IsAttributesChanged())
    {
    	// write out volume, document, and Attribute counts
    	Output << VolumeCount << ' ' << DocumentCount << ' ' 
			<< GetAttributes().Length();

    	// do for all attributes...
    	for(int i = 0; i < GetAttributes().Length(); ++i)
    	{
	    // write out the i'th attribute
	    Output << ' ' << GetAttributes().Nth(i);
    	}

	if(!Output)
	    throw LogiconErrors::Write("Volume::WriteHeader(void)");

    	char *Header = Output.str();		// string version of header

	try
	{
    	    File.WriteHeader(Header, Output.pcount());
	    delete[] Header;
	    ClearAttributesChanged();
	}
	catch(...)
	{
	    if(Header != NULL)
		delete[] Header;
	    throw;
	}
    }

    HeaderChanged = 0;
}




//
// Function Name:	Volume::ReadHeader()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Read in the header of the Volume, getting the elements
//			of the Volume from it.
//
// Parameters:		None
//
// Return Value:	None
//
void Volume::ReadHeader()
{
    char *Header = NULL;	// header of the Volume
    int HeaderLen;		// length of the Header
    int AttributeCount;		// count of Attributes

    try
    {
    	// get information from the header
    	Header = File.ReadHeader(HeaderLen);

    	istrstream HeaderInput(Header, HeaderLen);  // input stream for header

    	if(!(HeaderInput >> VolumeCount >> DocumentCount >> AttributeCount))
	    throw LogiconErrors::Read("Volume::ReadHeader(void)");

    	// do as long as there is stuff to read 
    	for(int i = 0; i < AttributeCount; ++i)
    	{
	    char ch;			// to hold read-in stuff

	    // read space filler
	    if(!HeaderInput.get(ch))
	    	throw LogiconErrors::Read("Volume::ReadHeader(void)");

	    // read in the Attribute
	    Attribute A(HeaderInput);
	    PutAttribute(A.GetName(), A.GetValue());
    	}

    	delete[] Header;

    	HeaderChanged = 0;
    }
    catch(...)
    {
	if(Header != NULL)
	    delete[] Header;
	throw;
    }
}



//
// Function Name:	Volume::Sync()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Synchronize the Volume to disk.
//
// Parameters:		None
//
// Return Value:	None
//
void Volume::Sync()
{
    // write out the header
    WriteHeader();

    // synchronize the RecordFile
    File.Sync();
}



//
// Function Name:	Volume::Destroy()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Destroy a Volume.
//
// Parameters:		Name - name of the Volume
//			Number - number of the Volume
//
// Return Value:	None
//
void Volume::Destroy(const char *Name, int Number)
{
    RecordFile::Destroy(GetFilename(Name, Number));
}



//
// Function Name:	Volume::PutAttribute()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Add an Attribute to the Volume by invoking the
//			AttributeObject::PutAttribute() function and noting
//			that the Volume has had header information changed.
//
// Parameters:		AttributeName - Name of the Attribute in question
//			AttributeValue - Value of the Attribute
//
// Return Value:	None
//
void Volume::PutAttribute(const char *AttributeName, 
			  const AttributeValue &Value)
{
    HeaderChanged = 1;

    AttributedObject::PutAttribute(AttributeName, Value);
}



//
// Function Name:	Volume::Cleanup()
//
// CSCI/CSC/CSU:	CMC/Volume
//
// Description:		Clear all memory allocated by this
//
// Parameters:		None
//
// Return Value:	None
//
void Volume::Cleanup()
{
    if(Name != NULL)
    {
	delete[] Name;
	Name = NULL;
    }
}
